@@ -1,8 +1,8 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2015-2016 Mario Luzeiro <mrluzeiro@ua.pt>
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2015-2022 Mario Luzeiro <mrluzeiro@ua.pt>
+ * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -33,10 +33,19 @@
 #include <3d_math.h>
 #include "3d_fastmath.h"
 #include <geometry/geometry_utils.h>
-#include <convert_to_biu.h>
 #include <pgm_base.h>
 #include <settings/settings_manager.h>
 #include <wx/log.h>
+#include <advanced_config.h>
+
+
+#define DEFAULT_BOARD_THICKNESS pcbIUScale.mmToIU( 1.6 )
+#define DEFAULT_COPPER_THICKNESS pcbIUScale.mmToIU( 0.035 ) // for 35 um
+// The solder mask layer (and silkscreen) thickness
+#define DEFAULT_TECH_LAYER_THICKNESS pcbIUScale.mmToIU( 0.025 )
+// The solder paste thickness is chosen bigger than the solder mask layer
+// to be sure is covers the mask when overlapping.
+#define SOLDERPASTE_LAYER_THICKNESS pcbIUScale.mmToIU( 0.04 )
 
 
 CUSTOM_COLORS_LIST   BOARD_ADAPTER::g_SilkscreenColors;
@@ -53,6 +62,9 @@ KIGFX::COLOR4D       BOARD_ADAPTER::g_DefaultSolderPaste;
 KIGFX::COLOR4D       BOARD_ADAPTER::g_DefaultSurfaceFinish;
 KIGFX::COLOR4D       BOARD_ADAPTER::g_DefaultBoardBody;
 
+// To be used in Raytracing render to create bevels on layer items
+float                g_BevelThickness3DU = 0.0f;
+
 static bool          g_ColorsLoaded = false;
 
 /**
@@ -67,26 +79,24 @@ const wxChar *BOARD_ADAPTER::m_logTrace = wxT( "KI_TRACE_EDA_CINFO3D_VISU" );
 
 
 BOARD_ADAPTER::BOARD_ADAPTER() :
+        m_Cfg( nullptr ),
+        m_IsBoardView( true ),
+        m_MousewheelPanning( true ),
+        m_IsPreviewer( false ),
         m_board( nullptr ),
         m_3dModelManager( nullptr ),
+        m_renderSettings( nullptr ),
         m_colors( nullptr ),
         m_layerZcoordTop(),
         m_layerZcoordBottom()
 {
     wxLogTrace( m_logTrace, wxT( "BOARD_ADAPTER::BOARD_ADAPTER" ) );
 
-    m_gridType = GRID3D_TYPE::NONE;
-    m_antiAliasingMode = ANTIALIASING_MODE::AA_8X;
-    m_drawFlags.resize( FL_LAST, false );
-
     if( PgmOrNull() )
         m_colors = Pgm().GetSettingsManager().GetColorSettings();
 
-    m_renderEngine = RENDER_ENGINE::OPENGL;
-    m_materialMode = MATERIAL_MODE::NORMAL;
-
-    m_boardPos = wxPoint();
-    m_boardSize = wxSize();
+    m_boardPos = VECTOR2I();
+    m_boardSize = VECTOR2I();
     m_boardCenter = SFVEC3F( 0.0f );
 
     m_boardBoundingBox.Reset();
@@ -95,12 +105,14 @@ BOARD_ADAPTER::BOARD_ADAPTER() :
     m_throughHoleOds.Clear();
     m_throughHoleAnnularRings.Clear();
 
-    m_copperLayersCount = -1;
-    m_epoxyThickness3DU = 0.0f;
-    m_copperThickness3DU  = 0.0f;
-    m_nonCopperLayerThickness3DU = 0.0f;
-    m_solderPasteLayerThickness3DU = 0.0f;
+    m_copperLayersCount = 2;
+
     m_biuTo3Dunits = 1.0;
+    m_boardBodyThickness3DU        = DEFAULT_BOARD_THICKNESS      * m_biuTo3Dunits;
+    m_frontCopperThickness3DU      = DEFAULT_COPPER_THICKNESS     * m_biuTo3Dunits;
+    m_backCopperThickness3DU       = DEFAULT_COPPER_THICKNESS     * m_biuTo3Dunits;
+    m_nonCopperLayerThickness3DU   = DEFAULT_TECH_LAYER_THICKNESS * m_biuTo3Dunits;
+    m_solderPasteLayerThickness3DU = SOLDERPASTE_LAYER_THICKNESS  * m_biuTo3Dunits;
 
     m_trackCount = 0;
     m_viaCount = 0;
@@ -109,27 +121,6 @@ BOARD_ADAPTER::BOARD_ADAPTER() :
     m_averageHoleDiameter = 0.0f;
     m_averageTrackWidth = 0.0f;
 
-    SetFlag( FL_USE_REALISTIC_MODE, true );
-    SetFlag( FL_FP_ATTRIBUTES_NORMAL, true );
-    SetFlag( FL_SHOW_BOARD_BODY, true );
-    SetFlag( FL_CLIP_SILK_ON_VIA_ANNULUS, false );
-    SetFlag( FL_FP_ATTRIBUTES_NORMAL, true );
-    SetFlag( FL_FP_ATTRIBUTES_NORMAL_INSERT, true );
-    SetFlag( FL_FP_ATTRIBUTES_VIRTUAL, true );
-    SetFlag( FL_ZONE, true );
-    SetFlag( FL_SILKSCREEN, true );
-    SetFlag( FL_SOLDERMASK, true );
-    SetFlag( FL_SUBTRACT_MASK_FROM_SILK, false );
-
-    SetFlag( FL_RENDER_OPENGL_COPPER_THICKNESS, true );
-    SetFlag( FL_RENDER_OPENGL_AA_DISABLE_ON_MOVE, false );
-    SetFlag( FL_RENDER_OPENGL_THICKNESS_DISABLE_ON_MOVE, false );
-    SetFlag( FL_RENDER_OPENGL_VIAS_DISABLE_ON_MOVE, false );
-    SetFlag( FL_RENDER_OPENGL_HOLES_DISABLE_ON_MOVE, false );
-
-    SetFlag( FL_USE_SELECTION, true );
-    SetFlag( FL_HIGHLIGHT_ROLLOVER_ITEM, true );
-
     m_BgColorBot         = SFVEC4F( 0.4, 0.4, 0.5, 1.0 );
     m_BgColorTop         = SFVEC4F( 0.8, 0.8, 0.9, 1.0 );
     m_BoardBodyColor     = SFVEC4F( 0.4, 0.4, 0.5, 0.9 );
@@ -146,67 +137,55 @@ BOARD_ADAPTER::BOARD_ADAPTER() :
     m_frontPlatedPadPolys = nullptr;
     m_backPlatedPadPolys = nullptr;
 
-    // Avoid raytracing options not initialized:
-    m_RtShadowSampleCount = 0;
-    m_RtReflectionSampleCount = 0;
-    m_RtRefractionSampleCount = 0;
-
-    m_RtSpreadShadows = 0.0;
-    m_RtSpreadReflections = 0.0;
-    m_RtSpreadRefractions = 0.0;
-
-    m_RtRecursiveReflectionCount = 0;
-    m_RtRecursiveRefractionCount = 0;
-
     if( !g_ColorsLoaded )
     {
 #define ADD_COLOR( list, r, g, b, a, name ) \
-    list.push_back( CUSTOM_COLOR_ITEM( r/255.0, g/255.0, b/255.0, a, name ) )
+    list.emplace_back( r/255.0, g/255.0, b/255.0, a, name )
 
         ADD_COLOR( g_SilkscreenColors, 245, 245, 245, 1.0, NotSpecifiedPrm() ); // White
-        ADD_COLOR( g_SilkscreenColors,  20,  51,  36, 1.0, "Green" );
-        ADD_COLOR( g_SilkscreenColors, 181,  19,  21, 1.0, "Red" );
-        ADD_COLOR( g_SilkscreenColors,   2,  59, 162, 1.0, "Blue" );
-        ADD_COLOR( g_SilkscreenColors,  11,  11,  11, 1.0, "Black" );
-        ADD_COLOR( g_SilkscreenColors, 245, 245, 245, 1.0, "White" );
-        ADD_COLOR( g_SilkscreenColors,  32,   2,  53, 1.0, "Purple" );
-        ADD_COLOR( g_SilkscreenColors, 194,  195,  0, 1.0, "Yellow" );
+        ADD_COLOR( g_SilkscreenColors,  20,  51,  36, 1.0, wxT( "Green" ) );
+        ADD_COLOR( g_SilkscreenColors, 181,  19,  21, 1.0, wxT( "Red" ) );
+        ADD_COLOR( g_SilkscreenColors,   2,  59, 162, 1.0, wxT( "Blue" ) );
+        ADD_COLOR( g_SilkscreenColors,  11,  11,  11, 1.0, wxT( "Black" ) );
+        ADD_COLOR( g_SilkscreenColors, 245, 245, 245, 1.0, wxT( "White" ) );
+        ADD_COLOR( g_SilkscreenColors,  32,   2,  53, 1.0, wxT( "Purple" ) );
+        ADD_COLOR( g_SilkscreenColors, 194,  195,  0, 1.0, wxT( "Yellow" ) );
 
         ADD_COLOR( g_MaskColors,  20,  51,  36, 0.83, NotSpecifiedPrm() ); // Green
-        ADD_COLOR( g_MaskColors,  20,  51,  36, 0.83, "Green" );
-        ADD_COLOR( g_MaskColors,  91, 168,  12, 0.83, "Light Green" );
-        ADD_COLOR( g_MaskColors,  13, 104,  11, 0.83, "Saturated Green" );
-        ADD_COLOR( g_MaskColors, 181,  19,  21, 0.83, "Red" );
-        ADD_COLOR( g_MaskColors, 210,  40,  14, 0.83, "Light Red" );
-        ADD_COLOR( g_MaskColors, 239,  53,  41, 0.83, "Red/Orange" );
-        ADD_COLOR( g_MaskColors,   2,  59, 162, 0.83, "Blue" );
-        ADD_COLOR( g_MaskColors,  54,  79, 116, 0.83, "Light Blue 1" );
-        ADD_COLOR( g_MaskColors,  61,  85, 130, 0.83, "Light Blue 2" );
-        ADD_COLOR( g_MaskColors,  21,  70,  80, 0.83, "Green/Blue" );
-        ADD_COLOR( g_MaskColors,  11,  11,  11, 0.83, "Black" );
-        ADD_COLOR( g_MaskColors, 245, 245, 245, 0.83, "White" );
-        ADD_COLOR( g_MaskColors,  32,   2,  53, 0.83, "Purple" );
-        ADD_COLOR( g_MaskColors, 119,  31,  91, 0.83, "Light Purple" );
-        ADD_COLOR( g_MaskColors, 194,  195,  0, 0.83, "Yellow" );
+        ADD_COLOR( g_MaskColors,  20,  51,  36, 0.83, wxT( "Green" ) );
+        ADD_COLOR( g_MaskColors,  91, 168,  12, 0.83, wxT( "Light Green" ) );
+        ADD_COLOR( g_MaskColors,  13, 104,  11, 0.83, wxT( "Saturated Green" ) );
+        ADD_COLOR( g_MaskColors, 181,  19,  21, 0.83, wxT( "Red" ) );
+        ADD_COLOR( g_MaskColors, 210,  40,  14, 0.83, wxT( "Light Red" ) );
+        ADD_COLOR( g_MaskColors, 239,  53,  41, 0.83, wxT( "Red/Orange" ) );
+        ADD_COLOR( g_MaskColors,   2,  59, 162, 0.83, wxT( "Blue" ) );
+        ADD_COLOR( g_MaskColors,  54,  79, 116, 0.83, wxT( "Light Blue 1" ) );
+        ADD_COLOR( g_MaskColors,  61,  85, 130, 0.83, wxT( "Light Blue 2" ) );
+        ADD_COLOR( g_MaskColors,  21,  70,  80, 0.83, wxT( "Green/Blue" ) );
+        ADD_COLOR( g_MaskColors,  11,  11,  11, 0.83, wxT( "Black" ) );
+        ADD_COLOR( g_MaskColors, 245, 245, 245, 0.83, wxT( "White" ) );
+        ADD_COLOR( g_MaskColors,  32,   2,  53, 0.83, wxT( "Purple" ) );
+        ADD_COLOR( g_MaskColors, 119,  31,  91, 0.83, wxT( "Light Purple" ) );
+        ADD_COLOR( g_MaskColors, 194,  195,  0, 0.83, wxT( "Yellow" ) );
 
-        ADD_COLOR( g_PasteColors, 128, 128, 128, 1.0, "Grey" );
-        ADD_COLOR( g_PasteColors,  90,  90,  90, 1.0, "Dark Grey" );
-        ADD_COLOR( g_PasteColors, 213, 213, 213, 1.0, "Silver" );
+        ADD_COLOR( g_PasteColors, 128, 128, 128, 1.0, wxT( "Grey" ) );
+        ADD_COLOR( g_PasteColors,  90,  90,  90, 1.0, wxT( "Dark Grey" ) );
+        ADD_COLOR( g_PasteColors, 213, 213, 213, 1.0, wxT( "Silver" ) );
 
-        ADD_COLOR( g_FinishColors, 184, 115,  50, 1.0, "Copper" );
-        ADD_COLOR( g_FinishColors, 178, 156,   0, 1.0, "Gold" );
-        ADD_COLOR( g_FinishColors, 213, 213, 213, 1.0, "Silver" );
-        ADD_COLOR( g_FinishColors, 160, 160, 160, 1.0, "Tin" );
+        ADD_COLOR( g_FinishColors, 184, 115,  50, 1.0, wxT( "Copper" ) );
+        ADD_COLOR( g_FinishColors, 178, 156,   0, 1.0, wxT( "Gold" ) );
+        ADD_COLOR( g_FinishColors, 213, 213, 213, 1.0, wxT( "Silver" ) );
+        ADD_COLOR( g_FinishColors, 160, 160, 160, 1.0, wxT( "Tin" ) );
 
-        ADD_COLOR( g_BoardColors,  51,  43,  22, 0.83, "FR4 natural, dark" );
-        ADD_COLOR( g_BoardColors, 109, 116,  75, 0.83, "FR4 natural" );
-        ADD_COLOR( g_BoardColors, 252, 252, 250, 0.90, "PTFE natural" );
-        ADD_COLOR( g_BoardColors, 205, 130,   0, 0.68, "Polyimide" );
-        ADD_COLOR( g_BoardColors,  92,  17,   6, 0.90, "Phenolic natural" );
-        ADD_COLOR( g_BoardColors, 146,  99,  47, 0.83, "Brown 1" );
-        ADD_COLOR( g_BoardColors, 160, 123,  54, 0.83, "Brown 2" );
-        ADD_COLOR( g_BoardColors, 146,  99,  47, 0.83, "Brown 3" );
-        ADD_COLOR( g_BoardColors, 213, 213, 213,  1.0, "Aluminum" );
+        ADD_COLOR( g_BoardColors,  51,  43,  22, 0.83, wxT( "FR4 natural, dark" ) );
+        ADD_COLOR( g_BoardColors, 109, 116,  75, 0.83, wxT( "FR4 natural" ) );
+        ADD_COLOR( g_BoardColors, 252, 252, 250, 0.90, wxT( "PTFE natural" ) );
+        ADD_COLOR( g_BoardColors, 205, 130,   0, 0.68, wxT( "Polyimide" ) );
+        ADD_COLOR( g_BoardColors,  92,  17,   6, 0.90, wxT( "Phenolic natural" ) );
+        ADD_COLOR( g_BoardColors, 146,  99,  47, 0.83, wxT( "Brown 1" ) );
+        ADD_COLOR( g_BoardColors, 160, 123,  54, 0.83, wxT( "Brown 2" ) );
+        ADD_COLOR( g_BoardColors, 146,  99,  47, 0.83, wxT( "Brown 3" ) );
+        ADD_COLOR( g_BoardColors, 213, 213, 213,  1.0, wxT( "Aluminum" ) );
 
         g_DefaultBackgroundTop = COLOR4D(  0.80, 0.80, 0.90, 1.0 );
         g_DefaultBackgroundBot = COLOR4D(  0.40, 0.40, 0.50, 1.0 );
@@ -242,50 +221,37 @@ bool BOARD_ADAPTER::Is3dLayerEnabled( PCB_LAYER_ID aLayer ) const
     {
     case B_Adhes:
     case F_Adhes:
-        return GetFlag( FL_ADHESIVE );
+        return m_Cfg->m_Render.show_adhesive;
 
     case B_Paste:
     case F_Paste:
-        return GetFlag( FL_SOLDERPASTE );
+        return m_Cfg->m_Render.show_solderpaste;
 
     case B_SilkS:
     case F_SilkS:
-        return GetFlag( FL_SILKSCREEN );
+        return m_Cfg->m_Render.show_silkscreen;
 
     case B_Mask:
     case F_Mask:
-        return GetFlag( FL_SOLDERMASK );
+        return m_Cfg->m_Render.show_soldermask;
 
     case Dwgs_User:
     case Cmts_User:
-        if( GetFlag( FL_USE_REALISTIC_MODE ) )
-            return false;
-
-        return GetFlag( FL_COMMENTS );
+        return !m_Cfg->m_Render.realistic && m_Cfg->m_Render.show_comments;
 
     case Eco1_User:
     case Eco2_User:
-        if( GetFlag( FL_USE_REALISTIC_MODE ) )
-            return false;
-
-        return GetFlag( FL_ECO );
+        return !m_Cfg->m_Render.realistic && m_Cfg->m_Render.show_eco;
 
     case Edge_Cuts:
-        if( GetFlag( FL_SHOW_BOARD_BODY ) || GetFlag( FL_USE_REALISTIC_MODE ) )
-            return false;
-
-        return true;
+        return !m_Cfg->m_Render.realistic && !m_Cfg->m_Render.show_board_body;
 
     case Margin:
-        if( GetFlag( FL_USE_REALISTIC_MODE ) )
-            return false;
-
-        return true;
+        return !m_Cfg->m_Render.realistic;
 
     case B_Cu:
     case F_Cu:
-        return m_board ? m_board->IsLayerVisible( aLayer ) || GetFlag( FL_USE_REALISTIC_MODE )
-                       : true;
+        return !m_board || m_board->IsLayerVisible( aLayer ) || m_Cfg->m_Render.realistic;
 
     default:
         // the layer is an internal copper layer, used the visibility
@@ -294,45 +260,31 @@ bool BOARD_ADAPTER::Is3dLayerEnabled( PCB_LAYER_ID aLayer ) const
 }
 
 
-bool BOARD_ADAPTER::GetFlag( DISPLAY3D_FLG aFlag ) const
-{
-    wxASSERT( aFlag < FL_LAST );
-
-    return m_drawFlags[aFlag];
-}
-
-
-void BOARD_ADAPTER::SetFlag( DISPLAY3D_FLG aFlag, bool aState )
-{
-    wxASSERT( aFlag < FL_LAST );
-
-    m_drawFlags[aFlag] = aState;
-}
-
-
 bool BOARD_ADAPTER::IsFootprintShown( FOOTPRINT_ATTR_T aFPAttributes ) const
 {
+    if( m_IsPreviewer )     // In panel Preview, footprints are always shown, of course
+        return true;
+
+    if( aFPAttributes & FP_EXCLUDE_FROM_POS_FILES )
+    {
+        if( !m_Cfg->m_Render.show_footprints_not_in_posfile )
+            return false;
+    }
+
     if( aFPAttributes & FP_SMD )
-        return GetFlag( FL_FP_ATTRIBUTES_NORMAL_INSERT );
-    else if( aFPAttributes & FP_THROUGH_HOLE )
-        return GetFlag( FL_FP_ATTRIBUTES_NORMAL );
-    else
-        return GetFlag( FL_FP_ATTRIBUTES_VIRTUAL );
+        return m_Cfg->m_Render.show_footprints_insert;
+
+    if( aFPAttributes & FP_THROUGH_HOLE )
+        return m_Cfg->m_Render.show_footprints_normal;
+
+    return m_Cfg->m_Render.show_footprints_virtual;
 }
 
 
-// !TODO: define the actual copper thickness by user from board stackup
-#define COPPER_THICKNESS Millimeter2iu( 0.035 )   // for 35 um
-// The solder mask layer (and silkscreen) thickness
-#define TECH_LAYER_THICKNESS Millimeter2iu( 0.025 )
-// The solder paste thickness is chosen bigger than the solder mask layer
-// to be sure is covers the mask when overlapping.
-#define SOLDERPASTE_LAYER_THICKNESS Millimeter2iu( 0.04 )
-
 int BOARD_ADAPTER::GetHolePlatingThickness() const noexcept
 {
     return m_board ? m_board->GetDesignSettings().GetHolePlatingThickness()
-                   : 0.035 * PCB_IU_PER_MM;
+                   : DEFAULT_COPPER_THICKNESS;
 }
 
 
@@ -348,7 +300,7 @@ unsigned int BOARD_ADAPTER::GetCircleSegmentCount( int aDiameterBIU ) const
 {
     wxASSERT( aDiameterBIU > 0 );
 
-    return GetArcToSegmentCount( aDiameterBIU / 2, ARC_HIGH_DEF, 360.0 );
+    return GetArcToSegmentCount( aDiameterBIU / 2, ARC_HIGH_DEF, FULL_CIRCLE );
 }
 
 
@@ -361,34 +313,28 @@ void BOARD_ADAPTER::InitSettings( REPORTER* aStatusReporter, REPORTER* aWarningR
 
     wxString msg;
 
-    const bool succeedToGetBoardPolygon = createBoardPolygon( &msg );
+    const bool haveOutline = createBoardPolygon( &msg );
 
     if( aWarningReporter )
     {
-        if( !succeedToGetBoardPolygon )
+        if( !haveOutline )
             aWarningReporter->Report( msg, RPT_SEVERITY_WARNING );
         else
             aWarningReporter->Report( wxEmptyString );
     }
 
-    // Calculates the board bounding box (board outlines + items)
-    // to ensure any item, even outside the board outlines can be seen
-    bool boardEdgesOnly = true;
-
-    if( ( m_board && m_board->IsFootprintHolder() ) || !GetFlag( FL_USE_REALISTIC_MODE )
-            || !succeedToGetBoardPolygon )
-    {
-        boardEdgesOnly = false;
-    }
-
-    EDA_RECT bbbox;
+    BOX2I bbbox;
 
     if( m_board )
-        bbbox = m_board->ComputeBoundingBox( boardEdgesOnly );
+    {
+        bbbox = m_board->ComputeBoundingBox( !m_board->IsFootprintHolder()
+                                                 && m_Cfg->m_Render.realistic
+                                                 && haveOutline );
+    }
 
     // Gives a non null size to avoid issues in zoom / scale calculations
     if( ( bbbox.GetWidth() == 0 ) && ( bbbox.GetHeight() == 0 ) )
-        bbbox.Inflate( Millimeter2iu( 10 ) );
+        bbbox.Inflate( pcbIUScale.mmToIU( 10 ) );
 
     m_boardSize = bbbox.GetSize();
     m_boardPos  = bbbox.Centre();
@@ -407,20 +353,54 @@ void BOARD_ADAPTER::InitSettings( REPORTER* aStatusReporter, REPORTER* aWarningR
     // Calculate the conversion to apply to all positions.
     m_biuTo3Dunits = RANGE_SCALE_3D / std::max( m_boardSize.x, m_boardSize.y );
 
-    m_epoxyThickness3DU = m_board
-                            ? m_board->GetDesignSettings().GetBoardThickness() * m_biuTo3Dunits
-                            : 1.6 * PCB_IU_PER_MM * m_biuTo3Dunits;
+    m_boardBodyThickness3DU        = DEFAULT_BOARD_THICKNESS      * m_biuTo3Dunits;
+    m_frontCopperThickness3DU      = DEFAULT_COPPER_THICKNESS     * m_biuTo3Dunits;
+    m_backCopperThickness3DU       = DEFAULT_COPPER_THICKNESS     * m_biuTo3Dunits;
+    m_nonCopperLayerThickness3DU   = DEFAULT_TECH_LAYER_THICKNESS * m_biuTo3Dunits;
+    m_solderPasteLayerThickness3DU = SOLDERPASTE_LAYER_THICKNESS  * m_biuTo3Dunits;
 
-    // !TODO: use value defined by user (currently use default values by ctor
-    m_copperThickness3DU         = COPPER_THICKNESS     * m_biuTo3Dunits;
-    m_nonCopperLayerThickness3DU = TECH_LAYER_THICKNESS * m_biuTo3Dunits;
-    m_solderPasteLayerThickness3DU = SOLDERPASTE_LAYER_THICKNESS * m_biuTo3Dunits;
+    g_BevelThickness3DU = pcbIUScale.mmToIU( ADVANCED_CFG::GetCfg().m_3DRT_BevelHeight_um / 1000.0 ) * m_biuTo3Dunits;
+
+    if( m_board )
+    {
+        const BOARD_DESIGN_SETTINGS& bds = m_board->GetDesignSettings();
+
+        if( bds.GetStackupDescriptor().GetCount() )
+        {
+            int thickness = 0;
+
+            for( BOARD_STACKUP_ITEM* item : bds.GetStackupDescriptor().GetList() )
+            {
+                switch( item->GetType() )
+                {
+                case BS_ITEM_TYPE_DIELECTRIC:
+                    for( int sublayer = 0; sublayer < item->GetSublayersCount(); sublayer++ )
+                        thickness += item->GetThickness( sublayer );
+                    break;
+
+                case BS_ITEM_TYPE_COPPER:
+                    if( item->GetBrdLayerId() == F_Cu )
+                        m_frontCopperThickness3DU = item->GetThickness() * m_biuTo3Dunits;
+                    else if( item->GetBrdLayerId() == B_Cu )
+                        m_backCopperThickness3DU = item->GetThickness() * m_biuTo3Dunits;
+                    else if( item->IsEnabled() )
+                        thickness += item->GetThickness();
+
+                    break;
+
+                default:
+                    break;
+                }
+            }
+
+            m_boardBodyThickness3DU = thickness * m_biuTo3Dunits;
+        }
+    }
 
     // Init  Z position of each layer
     // calculate z position for each copper layer
     // Zstart = -m_epoxyThickness / 2.0 is the z position of the back (bottom layer) (layer id = 31)
     // Zstart = +m_epoxyThickness / 2.0 is the z position of the front (top layer) (layer id = 0)
-    // all unused copper layer z position are set to 0
 
     //  ____==__________==________==______ <- Bottom = +m_epoxyThickness / 2.0,
     // |                                  |   Top = Bottom + m_copperThickness
@@ -432,24 +412,26 @@ void BOARD_ADAPTER::InitSettings( REPORTER* aStatusReporter, REPORTER* aWarningR
 
     for( layer = 0; layer < m_copperLayersCount; ++layer )
     {
-        m_layerZcoordBottom[layer] = m_epoxyThickness3DU / 2.0f -
-                                     (m_epoxyThickness3DU * layer / (m_copperLayersCount - 1) );
+        // This approximates internal layer positions (because we're treating all the dielectric
+        // layers as having the same thickness).  But we don't render them anyway so it doesn't
+        // really matter.
+        m_layerZcoordBottom[layer] = m_boardBodyThickness3DU / 2.0f -
+                                     (m_boardBodyThickness3DU * layer / (m_copperLayersCount - 1) );
 
         if( layer < (m_copperLayersCount / 2) )
-            m_layerZcoordTop[layer] = m_layerZcoordBottom[layer] + m_copperThickness3DU;
+            m_layerZcoordTop[layer] = m_layerZcoordBottom[layer] + m_frontCopperThickness3DU;
         else
-            m_layerZcoordTop[layer] = m_layerZcoordBottom[layer] - m_copperThickness3DU;
+            m_layerZcoordTop[layer] = m_layerZcoordBottom[layer] - m_backCopperThickness3DU;
     }
 
     #define layerThicknessMargin 1.1
     const float zpos_offset = m_nonCopperLayerThickness3DU * layerThicknessMargin;
 
-    // Fill remaining unused copper layers and back layer zpos
-    // with -m_epoxyThickness / 2.0
+    // Fill remaining unused copper layers and back layer zpos with -m_boardBodyThickness / 2.0
     for( ; layer < MAX_CU_LAYERS; layer++ )
     {
-        m_layerZcoordBottom[layer] = -(m_epoxyThickness3DU / 2.0f);
-        m_layerZcoordTop[layer]    = -(m_epoxyThickness3DU / 2.0f) - m_copperThickness3DU;
+        m_layerZcoordBottom[layer] = -( m_boardBodyThickness3DU / 2.0f );
+        m_layerZcoordTop[layer]    = -( m_boardBodyThickness3DU / 2.0f ) - m_backCopperThickness3DU;
     }
 
     // This is the top of the copper layer thickness.
@@ -559,7 +541,7 @@ void BOARD_ADAPTER::InitSettings( REPORTER* aStatusReporter, REPORTER* aWarningR
         auto findColor =
                 []( const wxString& aColorName, const CUSTOM_COLORS_LIST& aColorSet )
                 {
-                    if( aColorName.StartsWith( "#" ) )
+                    if( aColorName.StartsWith( wxT( "#" ) ) )
                     {
                         return KIGFX::COLOR4D( aColorName );
                     }
@@ -605,32 +587,7 @@ void BOARD_ADAPTER::InitSettings( REPORTER* aStatusReporter, REPORTER* aWarningR
 
             case BS_ITEM_TYPE_DIELECTRIC:
             {
-                KIGFX::COLOR4D  layerColor = COLOR4D::UNSPECIFIED;
-                const wxString& materialName = stackupItem->GetMaterial();
-
-                if( materialName.StartsWith( "FR4" ) )
-                {
-                    layerColor = findColor( "FR4 natural", g_BoardColors );
-                }
-                else if( materialName.IsSameAs( "PTFE" )
-                      || materialName.IsSameAs( "Teflon" ) )
-                {
-                    layerColor = findColor( "PTFE natural", g_BoardColors );
-                }
-                else if( materialName.IsSameAs( "Polyimide" )
-                      || materialName.IsSameAs( "Kapton" ) )
-                {
-                    layerColor = findColor( "Polyimide", g_BoardColors );
-                }
-                else if( materialName.IsSameAs( "Al" ) )
-                {
-                    layerColor = findColor( "Aluminum", g_BoardColors );
-                }
-                else    // A default color value for unknown dielectric material
-                        // (i.e. an exotic name entered by hand)
-                {
-                    layerColor = findColor( "FR4 natural", g_BoardColors );
-                }
+                KIGFX::COLOR4D layerColor = findColor( colorName, g_BoardColors );
 
                 if( bodyColor == COLOR4D( 0, 0, 0, 0 ) )
                     bodyColor = layerColor;
@@ -653,25 +610,25 @@ void BOARD_ADAPTER::InitSettings( REPORTER* aStatusReporter, REPORTER* aWarningR
 
         const wxString& finishName = stackup.m_FinishType;
 
-        if( finishName.EndsWith( "OSP" ) )
+        if( finishName.EndsWith( wxT( "OSP" ) ) )
         {
-            m_CopperColor = to_SFVEC4F( findColor( "Copper", g_FinishColors ) );
+            m_CopperColor = to_SFVEC4F( findColor( wxT( "Copper" ), g_FinishColors ) );
         }
-        else if( finishName.EndsWith( "IG" )
-              || finishName.EndsWith( "gold" ) )
+        else if( finishName.EndsWith( wxT( "IG" ) )
+              || finishName.EndsWith( wxT( "gold" ) ) )
         {
-            m_CopperColor = to_SFVEC4F( findColor( "Gold", g_FinishColors ) );
+            m_CopperColor = to_SFVEC4F( findColor( wxT( "Gold" ), g_FinishColors ) );
         }
-        else if( finishName.StartsWith( "HAL" )
-              || finishName.StartsWith( "HASL" )
-              || finishName.EndsWith( "tin" )
-              || finishName.EndsWith( "nickel" ) )
+        else if( finishName.StartsWith( wxT( "HAL" ) )
+              || finishName.StartsWith( wxT( "HASL" ) )
+              || finishName.EndsWith( wxT( "tin" ) )
+              || finishName.EndsWith( wxT( "nickel" ) ) )
         {
-            m_CopperColor = to_SFVEC4F( findColor( "Tin", g_FinishColors ) );
+            m_CopperColor = to_SFVEC4F( findColor( wxT( "Tin" ), g_FinishColors ) );
         }
-        else if( finishName.EndsWith( "silver" ) )
+        else if( finishName.EndsWith( wxT( "silver" ) ) )
         {
-            m_CopperColor = to_SFVEC4F( findColor( "Silver", g_FinishColors ) );
+            m_CopperColor = to_SFVEC4F( findColor( wxT( "Silver" ), g_FinishColors ) );
         }
         else
         {
@@ -714,7 +671,7 @@ bool BOARD_ADAPTER::createBoardPolygon( wxString* aErrorMsg )
             return false;
         }
 
-        int chainingEpsilon = Millimeter2iu( 0.02 );  // max dist from one endPt to next startPt
+        int chainingEpsilon = pcbIUScale.mmToIU( 0.02 ); // max dist from one endPt to next startPt
 
         success = BuildFootprintPolygonOutlines( m_board, m_board_poly,
                                                  m_board->GetDesignSettings().m_MaxError,
@@ -745,14 +702,14 @@ float BOARD_ADAPTER::GetFootprintZPos( bool aIsFlipped ) const
 {
     if( aIsFlipped )
     {
-        if( GetFlag( FL_SOLDERPASTE ) )
+        if( m_Cfg->m_Render.show_solderpaste )
             return m_layerZcoordBottom[B_SilkS];
         else
             return m_layerZcoordBottom[B_Paste];
     }
     else
     {
-        if( GetFlag( FL_SOLDERPASTE ) )
+        if( m_Cfg->m_Render.show_solderpaste )
             return m_layerZcoordTop[F_SilkS];
         else
             return m_layerZcoordTop[F_Paste];
@@ -780,3 +737,16 @@ SFVEC4F BOARD_ADAPTER::GetColor( const COLOR4D& aColor ) const
 {
     return SFVEC4F( aColor.r, aColor.g, aColor.b, aColor.a );
 }
+
+
+SFVEC2F BOARD_ADAPTER::GetSphericalCoord( int i ) const
+{
+    SFVEC2F sphericalCoord =
+            SFVEC2F( ( m_Cfg->m_Render.raytrace_lightElevation[i] + 90.0f ) / 180.0f,
+                       m_Cfg->m_Render.raytrace_lightAzimuth[i] / 180.0f );
+
+    sphericalCoord.x = glm::clamp( sphericalCoord.x, 0.0f, 1.0f );
+    sphericalCoord.y = glm::clamp( sphericalCoord.y, 0.0f, 2.0f );
+
+    return sphericalCoord;
+}
