@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2015-2016 Mario Luzeiro <mrluzeiro@ua.pt>
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -55,14 +55,12 @@
  */
 const wxChar* EDA_3D_CANVAS::m_logTrace = wxT( "KI_TRACE_EDA_3D_CANVAS" );
 
-const float EDA_3D_CANVAS::m_delta_move_step_factor = 0.7f;
-
 
 // A custom event, used to call DoRePaint during an idle time
-wxDEFINE_EVENT( wxEVT_REFRESH_CUSTOM_COMMAND, wxEvent);
+wxDEFINE_EVENT( wxEVT_REFRESH_CUSTOM_COMMAND, wxEvent );
 
 
-BEGIN_EVENT_TABLE( EDA_3D_CANVAS, wxGLCanvas )
+BEGIN_EVENT_TABLE( EDA_3D_CANVAS, HIDPI_GL_3D_CANVAS )
     EVT_PAINT( EDA_3D_CANVAS::OnPaint )
 
     // mouse events
@@ -79,7 +77,6 @@ BEGIN_EVENT_TABLE( EDA_3D_CANVAS, wxGLCanvas )
 
     // other events
     EVT_ERASE_BACKGROUND( EDA_3D_CANVAS::OnEraseBackground )
-    //EVT_REFRESH_CUSTOM_COMMAND( ID_CUSTOM_EVENT_1, EDA_3D_CANVAS::OnRefreshRequest )
     EVT_CUSTOM(wxEVT_REFRESH_CUSTOM_COMMAND, ID_CUSTOM_EVENT_1, EDA_3D_CANVAS::OnRefreshRequest )
 
     EVT_CLOSE( EDA_3D_CANVAS::OnCloseWindow )
@@ -89,32 +86,30 @@ END_EVENT_TABLE()
 
 EDA_3D_CANVAS::EDA_3D_CANVAS( wxWindow* aParent, const int* aAttribList,
                               BOARD_ADAPTER& aBoardAdapter, CAMERA& aCamera,
-                              S3D_CACHE* a3DCachePointer )
-        : HIDPI_GL_CANVAS( aParent, wxID_ANY, aAttribList, wxDefaultPosition, wxDefaultSize,
-                           wxFULL_REPAINT_ON_RESIZE ),
-          m_eventDispatcher( nullptr ),
-          m_parentStatusBar( nullptr ),
-          m_parentInfoBar( nullptr ),
-          m_glRC( nullptr ),
-          m_is_opengl_initialized( false ),
-          m_is_opengl_version_supported( true ),
-          m_mouse_is_moving( false ),
-          m_mouse_was_moved( false ),
-          m_camera_is_moving( false ),
-          m_render_pivot( false ),
-          m_camera_moving_speed( 1.0f ),
-          m_strtime_camera_movement( 0 ),
-          m_animation_enabled( true ),
-          m_moving_speed_multiplier( 3 ),
-          m_boardAdapter( aBoardAdapter ),
-          m_camera( aCamera ),
-          m_3d_render( nullptr ),
-          m_opengl_supports_raytracing( true ),
-          m_render_raytracing_was_requested( false ),
-          m_accelerator3DShapes( nullptr ),
-          m_currentRollOverItem( nullptr )
+                              S3D_CACHE* a3DCachePointer ) :
+        HIDPI_GL_3D_CANVAS( aCamera, aParent, wxID_ANY, aAttribList, wxDefaultPosition,
+                            wxDefaultSize, wxFULL_REPAINT_ON_RESIZE ),
+        m_eventDispatcher( nullptr ),
+        m_parentStatusBar( nullptr ),
+        m_parentInfoBar( nullptr ),
+        m_glRC( nullptr ),
+        m_is_opengl_initialized( false ),
+        m_is_opengl_version_supported( true ),
+        m_editing_timeout_timer( this, wxID_HIGHEST + 1 ),
+        m_redraw_trigger_timer( this, wxID_HIGHEST + 2 ),
+        m_render_pivot( false ),
+        m_camera_moving_speed( 1.0f ),
+        m_strtime_camera_movement( 0 ),
+        m_animation_enabled( true ),
+        m_moving_speed_multiplier( 3 ),
+        m_boardAdapter( aBoardAdapter ),
+        m_3d_render( nullptr ),
+        m_opengl_supports_raytracing( true ),
+        m_render_raytracing_was_requested( false ),
+        m_accelerator3DShapes( nullptr ),
+        m_currentRollOverItem( nullptr )
 {
-    wxLogTrace( m_logTrace, "EDA_3D_CANVAS::EDA_3D_CANVAS" );
+    wxLogTrace( m_logTrace, wxT( "EDA_3D_CANVAS::EDA_3D_CANVAS" ) );
 
     m_editing_timeout_timer.SetOwner( this );
     Connect( m_editing_timeout_timer.GetId(), wxEVT_TIMER,
@@ -132,12 +127,18 @@ EDA_3D_CANVAS::EDA_3D_CANVAS( wxWindow* aParent, const int* aAttribList,
     wxASSERT( m_3d_render_raytracing != nullptr );
     wxASSERT( m_3d_render_opengl != nullptr );
 
-    auto busy_indicator_factory = []() { return std::make_unique<WX_BUSY_INDICATOR>(); };
+    auto busy_indicator_factory =
+            []()
+            {
+                return std::make_unique<WX_BUSY_INDICATOR>();
+            };
 
     m_3d_render_raytracing->SetBusyIndicatorFactory( busy_indicator_factory );
     m_3d_render_opengl->SetBusyIndicatorFactory( busy_indicator_factory );
 
-    RenderEngineChanged();
+    // We always start with the opengl engine (raytracing is avoided due to very
+    // long calculation time)
+    m_3d_render = m_3d_render_opengl;
 
     m_boardAdapter.SetColorSettings( Pgm().GetSettingsManager().GetColorSettings() );
 
@@ -167,7 +168,7 @@ EDA_3D_CANVAS::EDA_3D_CANVAS( wxWindow* aParent, const int* aAttribList,
 
 EDA_3D_CANVAS::~EDA_3D_CANVAS()
 {
-    wxLogTrace( m_logTrace, "EDA_3D_CANVAS::~EDA_3D_CANVAS" );
+    wxLogTrace( m_logTrace, wxT( "EDA_3D_CANVAS::~EDA_3D_CANVAS" ) );
 
     delete m_accelerator3DShapes;
     m_accelerator3DShapes = nullptr;
@@ -208,13 +209,13 @@ void EDA_3D_CANVAS::OnCloseWindow( wxCloseEvent& event )
 
 void EDA_3D_CANVAS::OnResize( wxSizeEvent& event )
 {
-    this->Request_refresh();
+    Request_refresh();
 }
 
 
 bool  EDA_3D_CANVAS::initializeOpenGL()
 {
-    wxLogTrace( m_logTrace, "EDA_3D_CANVAS::initializeOpenGL" );
+    wxLogTrace( m_logTrace, wxT( "EDA_3D_CANVAS::initializeOpenGL" ) );
 
     const GLenum err = glewInit();
 
@@ -228,13 +229,13 @@ bool  EDA_3D_CANVAS::initializeOpenGL()
     }
     else
     {
-        wxLogTrace( m_logTrace, "EDA_3D_CANVAS::initializeOpenGL Using GLEW version %s",
+        wxLogTrace( m_logTrace, wxT( "EDA_3D_CANVAS::initializeOpenGL Using GLEW version %s" ),
                     FROM_UTF8( (char*) glewGetString( GLEW_VERSION ) ) );
     }
 
     wxString version = FROM_UTF8( (char *) glGetString( GL_VERSION ) );
 
-    wxLogTrace( m_logTrace, "EDA_3D_CANVAS::%s OpenGL version string %s.",
+    wxLogTrace( m_logTrace, wxT( "EDA_3D_CANVAS::%s OpenGL version string %s." ),
                 __WXFUNCTION__, version );
 
     // Extract OpenGL version from string.  This method is used because prior to OpenGL 2,
@@ -250,7 +251,7 @@ bool  EDA_3D_CANVAS::initializeOpenGL()
 
         tmp = tokenizer.GetNextToken();
 
-        tokenizer.SetString( tmp, wxString( "." ) );
+        tokenizer.SetString( tmp, wxString( wxT( "." ) ) );
 
         if( tokenizer.HasMoreTokens() )
             tokenizer.GetNextToken().ToLong( &major );
@@ -260,7 +261,7 @@ bool  EDA_3D_CANVAS::initializeOpenGL()
 
         if( major < 2 || ( ( major == 2 ) && ( minor < 1 ) ) )
         {
-            wxLogTrace( m_logTrace, "EDA_3D_CANVAS::%s OpenGL ray tracing not supported.",
+            wxLogTrace( m_logTrace, wxT( "EDA_3D_CANVAS::%s OpenGL ray tracing not supported." ),
                         __WXFUNCTION__ );
 
             if( GetParent() )
@@ -274,7 +275,8 @@ bool  EDA_3D_CANVAS::initializeOpenGL()
 
         if( ( major == 1 ) && ( minor < 5 ) )
         {
-            wxLogTrace( m_logTrace, "EDA_3D_CANVAS::%s OpenGL not supported.", __WXFUNCTION__ );
+            wxLogTrace( m_logTrace, wxT( "EDA_3D_CANVAS::%s OpenGL not supported." ),
+                        __WXFUNCTION__ );
 
             m_is_opengl_version_supported = false;
         }
@@ -327,10 +329,10 @@ void EDA_3D_CANVAS::DisplayStatus()
     {
         wxString msg;
 
-        msg.Printf( "dx %3.2f", m_camera.GetCameraPos().x );
+        msg.Printf( wxT( "dx %3.2f" ), m_camera.GetCameraPos().x );
         m_parentStatusBar->SetStatusText( msg, static_cast<int>( EDA_3D_VIEWER_STATUSBAR::X_POS ) );
 
-        msg.Printf( "dy %3.2f", m_camera.GetCameraPos().y );
+        msg.Printf( wxT( "dy %3.2f" ), m_camera.GetCameraPos().y );
         m_parentStatusBar->SetStatusText( msg, static_cast<int>( EDA_3D_VIEWER_STATUSBAR::Y_POS ) );
     }
 }
@@ -351,7 +353,7 @@ void EDA_3D_CANVAS::DoRePaint()
     // SwapBuffer requires the window to be shown before calling
     if( !IsShownOnScreen() )
     {
-        wxLogTrace( m_logTrace, "EDA_3D_CANVAS::DoRePaint !IsShown" );
+        wxLogTrace( m_logTrace, wxT( "EDA_3D_CANVAS::DoRePaint !IsShown" ) );
         m_is_currently_painting.clear();
         return;
     }
@@ -369,8 +371,7 @@ void EDA_3D_CANVAS::DoRePaint()
 
     // !TODO: implement error reporter
     INFOBAR_REPORTER   warningReporter( m_parentInfoBar );
-    STATUSBAR_REPORTER activityReporter( m_parentStatusBar,
-                                         (int) EDA_3D_VIEWER_STATUSBAR::ACTIVITY );
+    STATUSBAR_REPORTER activityReporter( m_parentStatusBar, EDA_3D_VIEWER_STATUSBAR::ACTIVITY );
 
     unsigned strtime = GetRunningMicroSecs();
 
@@ -432,11 +433,11 @@ void EDA_3D_CANVAS::DoRePaint()
     {
         m_3d_render = m_3d_render_opengl;
         m_render_raytracing_was_requested = false;
-        m_boardAdapter.SetRenderEngine( RENDER_ENGINE::OPENGL );
+        m_boardAdapter.m_Cfg->m_Render.engine = RENDER_ENGINE::OPENGL;
     }
 
     // Check if a raytacing was requested and need to switch to raytracing mode
-    if( m_boardAdapter.GetRenderEngine() == RENDER_ENGINE::OPENGL )
+    if( m_boardAdapter.m_Cfg->m_Render.engine == RENDER_ENGINE::OPENGL )
     {
         const bool was_camera_changed = m_camera.ParametersChanged();
 
@@ -485,7 +486,7 @@ void EDA_3D_CANVAS::DoRePaint()
 
             bool reloadRaytracingForCalculations = false;
 
-            if( m_boardAdapter.GetRenderEngine() == RENDER_ENGINE::OPENGL
+            if( m_boardAdapter.m_Cfg->m_Render.engine == RENDER_ENGINE::OPENGL
                     && m_3d_render_opengl->IsReloadRequestPending() )
             {
                 reloadRaytracingForCalculations = true;
@@ -521,6 +522,14 @@ void EDA_3D_CANVAS::DoRePaint()
         render_pivot( curtime_delta_s, scale );
     }
 
+#if defined( KICAD_USE_3DCONNEXION )
+    if( m_render3dmousePivot )
+    {
+        const float scale = glm::min( m_camera.GetZoom(), 1.0f );
+        render3dmousePivot( scale );
+    }
+#endif
+
     // "Swaps the double-buffer of this window, making the back-buffer the
     //  front-buffer and vice versa, so that the output of the previous OpenGL
     //  commands is displayed on the window."
@@ -577,75 +586,23 @@ void EDA_3D_CANVAS::OnEvent( wxEvent& aEvent )
 
 void EDA_3D_CANVAS::OnEraseBackground( wxEraseEvent& event )
 {
-    wxLogTrace( m_logTrace, "EDA_3D_CANVAS::OnEraseBackground" );
+    wxLogTrace( m_logTrace, wxT( "EDA_3D_CANVAS::OnEraseBackground" ) );
     // Do nothing, to avoid flashing.
 }
 
 
 void EDA_3D_CANVAS::OnMouseWheel( wxMouseEvent& event )
 {
-    bool mouseActivity = false;
+    wxLogTrace( m_logTrace, wxT( "EDA_3D_CANVAS::OnMouseWheel" ) );
 
-    wxLogTrace( m_logTrace, "EDA_3D_CANVAS::OnMouseWheel" );
+    OnMouseWheelCamera( event, m_boardAdapter.m_MousewheelPanning );
 
-    if( m_camera_is_moving )
-        return;
-
-    float delta_move = m_delta_move_step_factor * m_camera.GetZoom();
-
-    if( m_boardAdapter.GetFlag( FL_MOUSEWHEEL_PANNING ) )
-        delta_move *= 0.01f * event.GetWheelRotation();
-    else
-        if( event.GetWheelRotation() < 0 )
-            delta_move = -delta_move;
-
-    // mousewheel_panning enabled:
-    //      wheel           -> pan;
-    //      wheel + shift   -> horizontal scrolling;
-    //      wheel + ctrl    -> zooming;
-    // mousewheel_panning disabled:
-    //      wheel + shift   -> vertical scrolling;
-    //      wheel + ctrl    -> horizontal scrolling;
-    //      wheel           -> zooming.
-
-    if( m_boardAdapter.GetFlag( FL_MOUSEWHEEL_PANNING ) && !event.ControlDown() )
-    {
-        if( event.GetWheelAxis() == wxMOUSE_WHEEL_HORIZONTAL || event.ShiftDown() )
-            m_camera.Pan( SFVEC3F( -delta_move, 0.0f, 0.0f ) );
-        else
-            m_camera.Pan( SFVEC3F( 0.0f, -delta_move, 0.0f ) );
-
-        mouseActivity = true;
-    }
-    else if( event.ShiftDown() && !m_boardAdapter.GetFlag( FL_MOUSEWHEEL_PANNING ) )
-    {
-        m_camera.Pan( SFVEC3F( 0.0f, -delta_move, 0.0f ) );
-        mouseActivity = true;
-    }
-    else if( event.ControlDown() && !m_boardAdapter.GetFlag( FL_MOUSEWHEEL_PANNING ) )
-    {
-        m_camera.Pan( SFVEC3F( delta_move, 0.0f, 0.0f ) );
-        mouseActivity = true;
-    }
-    else
-    {
-        mouseActivity = m_camera.Zoom( event.GetWheelRotation() > 0 ? 1.1f : 1/1.1f );
-    }
-
-    // If it results on a camera movement
-    if( mouseActivity )
+    if( m_mouse_was_moved )
     {
         DisplayStatus();
         Request_refresh();
-
-        m_mouse_is_moving = true;
-        m_mouse_was_moved = true;
-
         restart_editingTimeOut_Timer();
     }
-
-    // Update the cursor current mouse position on the camera
-    m_camera.SetCurMousePosition( GetNativePosition( event.GetPosition() ) );
 }
 
 
@@ -672,44 +629,37 @@ void EDA_3D_CANVAS::OnMagnify( wxMouseEvent& event )
 
 void EDA_3D_CANVAS::OnMouseMove( wxMouseEvent& event )
 {
-    //wxLogTrace( m_logTrace, wxT( "EDA_3D_CANVAS::OnMouseMove" ) );
-
     if( m_camera_is_moving )
         return;
 
-    const wxSize&  nativeWinSize  = GetNativePixelSize();
-    const wxPoint& nativePosition = GetNativePosition( event.GetPosition() );
+    OnMouseMoveCamera( event );
 
-    m_camera.SetCurWindowSize( nativeWinSize );
-
-    if( event.Dragging() )
+    if( m_mouse_was_moved )
     {
-        if( event.LeftIsDown() )            // Drag
-            m_camera.Drag( nativePosition );
-        else if( event.MiddleIsDown() )     // Pan
-            m_camera.Pan( nativePosition );
-
-        m_mouse_is_moving = true;
-        m_mouse_was_moved = true;
-
-        // orientation has changed, redraw mesh
         DisplayStatus();
         Request_refresh();
+        // *Do not* reactivate the timer here during the mouse move command:
+        // OnMiddleUp() will do it at the end of mouse drag/move command
     }
 
-    m_camera.SetCurMousePosition( nativePosition );
-
-    if( !event.Dragging() && m_boardAdapter.GetRenderEngine() == RENDER_ENGINE::OPENGL )
+    if( !event.Dragging() && m_boardAdapter.m_Cfg->m_Render.engine == RENDER_ENGINE::OPENGL )
     {
-        STATUSBAR_REPORTER reporter( m_parentStatusBar,
-                                     static_cast<int>( EDA_3D_VIEWER_STATUSBAR::HOVERED_ITEM ) );
+        STATUSBAR_REPORTER reporter( m_parentStatusBar, EDA_3D_VIEWER_STATUSBAR::HOVERED_ITEM );
+        RAY                mouseRay = getRayAtCurrentMousePosition();
+        BOARD_ITEM*        rollOverItem = m_3d_render_raytracing->IntersectBoardItem( mouseRay );
 
-        RAY mouseRay = getRayAtCurrrentMousePosition();
-
-        BOARD_ITEM *rollOverItem = m_3d_render_raytracing->IntersectBoardItem( mouseRay );
+        auto printNetInfo =
+                []( BOARD_CONNECTED_ITEM* aItem )
+                {
+                    return wxString::Format( _( "Net %s\tNet class %s" ),
+                                             aItem->GetNet()->GetNetname(),
+                                             aItem->GetNet()->GetNetClass()->GetName() );
+                };
 
         if( rollOverItem )
         {
+            wxString msg;
+
             if( rollOverItem != m_currentRollOverItem )
             {
                 m_3d_render_opengl->SetCurrentRollOverItem( rollOverItem );
@@ -722,67 +672,66 @@ void EDA_3D_CANVAS::OnMouseMove( wxMouseEvent& event )
             {
             case PCB_PAD_T:
             {
-                PAD* pad = dynamic_cast<PAD*>( rollOverItem );
+                PAD* pad = static_cast<PAD*>( rollOverItem );
+
+                if( !pad->GetNumber().IsEmpty() )
+                    msg += wxString::Format( _( "Pad %s\t" ), pad->GetNumber() );
+
+                if( pad->IsOnCopperLayer() )
+                    msg += printNetInfo( pad );
 
-                if( pad && pad->IsOnCopperLayer() )
-                {
-                    reporter.Report( wxString::Format( _( "Net %s\tNetClass %s\tPadName %s" ),
-                                                       pad->GetNet()->GetNetname(),
-                                                       pad->GetNet()->GetNetClassName(),
-                                                       pad->GetNumber() ) );
-                }
-            }
                 break;
+            }
 
             case PCB_FOOTPRINT_T:
             {
-                FOOTPRINT* footprint = dynamic_cast<FOOTPRINT*>( rollOverItem );
-
-                if( footprint )
-                    reporter.Report( footprint->GetReference() );
-            }
+                FOOTPRINT* footprint = static_cast<FOOTPRINT*>( rollOverItem );
+                msg += footprint->GetReference();
                 break;
+            }
 
             case PCB_TRACE_T:
             case PCB_VIA_T:
             case PCB_ARC_T:
             {
-                PCB_TRACK* track = dynamic_cast<PCB_TRACK*>( rollOverItem );
-
-                if( track )
-                {
-                    reporter.Report( wxString::Format( _( "Net %s\tNetClass %s" ),
-                                                       track->GetNet()->GetNetname(),
-                                                       track->GetNet()->GetNetClassName() ) );
-                }
-            }
+                PCB_TRACK* track = static_cast<PCB_TRACK*>( rollOverItem );
+                msg += printNetInfo( track );
                 break;
+            }
 
             case PCB_ZONE_T:
             {
-                ZONE* zone = dynamic_cast<ZONE*>( rollOverItem );
+                ZONE* zone = static_cast<ZONE*>( rollOverItem );
 
-                if( zone && zone->IsOnCopperLayer() )
+                if( !zone->GetZoneName().IsEmpty() )
                 {
-                    reporter.Report( wxString::Format( _( "Net %s\tNetClass %s" ),
-                                                       zone->GetNet()->GetNetname(),
-                                                       zone->GetNet()->GetNetClassName() ) );
+                    if( zone->GetIsRuleArea() )
+                        msg += wxString::Format( _( "Rule area %s\t" ), zone->GetZoneName() );
+                    else
+                        msg += wxString::Format( _( "Zone %s\t" ), zone->GetZoneName() );
                 }
-            }
+
+                if( zone->IsOnCopperLayer() )
+                    msg += printNetInfo( zone );
+
                 break;
+            }
 
             default:
                 break;
             }
+
+            reporter.Report( msg );
         }
         else
         {
-            if( m_currentRollOverItem && m_boardAdapter.GetRenderEngine() == RENDER_ENGINE::OPENGL )
+            if( m_currentRollOverItem
+                    && m_boardAdapter.m_Cfg->m_Render.engine == RENDER_ENGINE::OPENGL )
             {
                 m_3d_render_opengl->SetCurrentRollOverItem( nullptr );
                 Request_refresh();
 
-                reporter.Report( "" );
+                reporter.Report( wxEmptyString );
             }
 
             m_currentRollOverItem = nullptr;
@@ -798,7 +747,7 @@ void EDA_3D_CANVAS::OnLeftDown( wxMouseEvent& event )
 
     if( !event.Dragging() && ( m_3d_render_raytracing != nullptr ) )
     {
-        RAY mouseRay = getRayAtCurrrentMousePosition();
+        RAY mouseRay = getRayAtCurrentMousePosition();
 
         BOARD_ITEM *intersectedBoardItem = m_3d_render_raytracing->IntersectBoardItem( mouseRay );
 
@@ -844,8 +793,14 @@ void EDA_3D_CANVAS::OnMiddleUp( wxMouseEvent& event )
 }
 
 
-void EDA_3D_CANVAS::OnTimerTimeout_Editing( wxTimerEvent& /* event */ )
+void EDA_3D_CANVAS::OnTimerTimeout_Editing( wxTimerEvent& aEvent )
 {
+    if( aEvent.GetId() != m_editing_timeout_timer.GetId() )
+    {
+        aEvent.Skip();
+        return;
+    }
+
     m_mouse_is_moving = false;
     m_mouse_was_moved = false;
 
@@ -866,8 +821,14 @@ void EDA_3D_CANVAS::restart_editingTimeOut_Timer()
 }
 
 
-void EDA_3D_CANVAS::OnTimerTimeout_Redraw( wxTimerEvent& event )
+void EDA_3D_CANVAS::OnTimerTimeout_Redraw( wxTimerEvent& aEvent )
 {
+    if( aEvent.GetId() != m_redraw_trigger_timer.GetId() )
+    {
+        aEvent.Skip();
+        return;
+    }
+
     Request_refresh( true );
 }
 
@@ -930,7 +891,7 @@ void EDA_3D_CANVAS::request_start_moving_camera( float aMovingSpeed, bool aRende
 
 void EDA_3D_CANVAS::move_pivot_based_on_cur_mouse_position()
 {
-    RAY mouseRay = getRayAtCurrrentMousePosition();
+    RAY mouseRay = getRayAtCurrentMousePosition();
 
     float hit_t;
 
@@ -954,7 +915,7 @@ bool EDA_3D_CANVAS::SetView3D( int aKeycode )
 
     const float delta_move = m_delta_move_step_factor * m_camera.GetZoom();
     const float arrow_moving_time_speed = 8.0f;
-    bool handled = false;
+    bool        handled = false;
 
     switch( aKeycode )
     {
@@ -1111,7 +1072,10 @@ bool EDA_3D_CANVAS::SetView3D( int aKeycode )
 
 void EDA_3D_CANVAS::RenderEngineChanged()
 {
-    switch( m_boardAdapter.GetRenderEngine() )
+    SETTINGS_MANAGER&       mgr = Pgm().GetSettingsManager();
+    EDA_3D_VIEWER_SETTINGS* cfg = mgr.GetAppSettings<EDA_3D_VIEWER_SETTINGS>();
+
+    switch( cfg->m_Render.engine )
     {
     case RENDER_ENGINE::OPENGL:     m_3d_render = m_3d_render_opengl;     break;
     case RENDER_ENGINE::RAYTRACING: m_3d_render = m_3d_render_raytracing; break;
@@ -1127,13 +1091,13 @@ void EDA_3D_CANVAS::RenderEngineChanged()
 }
 
 
-RAY EDA_3D_CANVAS::getRayAtCurrrentMousePosition()
+RAY EDA_3D_CANVAS::getRayAtCurrentMousePosition()
 {
     SFVEC3F rayOrigin;
     SFVEC3F rayDir;
 
     // Generate a ray origin and direction based on current mouser position and camera
-    m_camera.MakeRayAtCurrrentMousePosition( rayOrigin, rayDir );
+    m_camera.MakeRayAtCurrentMousePosition( rayOrigin, rayDir );
 
     RAY mouseRay;
     mouseRay.Init( rayOrigin, rayDir );
