@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2015-2016 Mario Luzeiro <mrluzeiro@ua.pt>
- * Copyright (C) 2015-2020 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2015-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,6 +30,7 @@
 #include <trigo.h>
 #include <project.h>
 #include <profile.h>        // To use GetRunningMicroSecs or another profiling utility
+#include <fp_lib_table.h>
 #include <eda_3d_canvas.h>
 #include <eda_3d_viewer_frame.h>
 
@@ -276,7 +277,7 @@ OPENGL_RENDER_LIST* RENDER_3D_OPENGL::generateHoles( const LIST_OBJECT2D& aListH
                 break;
 
             default:
-                wxFAIL_MSG( "RENDER_3D_OPENGL::generateHoles: Object type is not implemented" );
+                wxFAIL_MSG( wxT( "RENDER_3D_OPENGL::generateHoles: Object type not implemented" ) );
                 break;
             }
         }
@@ -358,7 +359,7 @@ OPENGL_RENDER_LIST* RENDER_3D_OPENGL::generateLayerList( const BVH_CONTAINER_2D*
             break;
 
         default:
-            wxFAIL_MSG( "RENDER_3D_OPENGL: Object type is not implemented" );
+            wxFAIL_MSG( wxT( "RENDER_3D_OPENGL: Object type is not implemented" ) );
             break;
         }
     }
@@ -374,23 +375,37 @@ OPENGL_RENDER_LIST* RENDER_3D_OPENGL::generateLayerList( const BVH_CONTAINER_2D*
 }
 
 
+OPENGL_RENDER_LIST* RENDER_3D_OPENGL::generateEmptyLayerList( PCB_LAYER_ID aLayerId )
+{
+    float layer_z_bot = 0.0f;
+    float layer_z_top = 0.0f;
+
+    getLayerZPos( aLayerId, layer_z_top, layer_z_bot );
+
+    TRIANGLE_DISPLAY_LIST* layerTriangles = new TRIANGLE_DISPLAY_LIST( 1 );
+
+    // store in a list so it will be latter deleted
+    m_triangles.push_back( layerTriangles );
+
+    return new OPENGL_RENDER_LIST( *layerTriangles, m_circleTexture, layer_z_bot, layer_z_top );
+}
+
+
 OPENGL_RENDER_LIST* RENDER_3D_OPENGL::createBoard( const SHAPE_POLY_SET& aBoardPoly,
                                                    const BVH_CONTAINER_2D* aThroughHoles )
 {
     OPENGL_RENDER_LIST* dispLists = nullptr;
     CONTAINER_2D boardContainer;
-    SHAPE_POLY_SET brd_outlines = aBoardPoly;
 
-    ConvertPolygonToTriangles( brd_outlines, boardContainer, m_boardAdapter.BiuTo3dUnits(),
+    ConvertPolygonToTriangles( aBoardPoly, boardContainer, m_boardAdapter.BiuTo3dUnits(),
                                (const BOARD_ITEM &)*m_boardAdapter.GetBoard() );
 
     const LIST_OBJECT2D& listBoardObject2d = boardContainer.GetList();
 
     if( listBoardObject2d.size() > 0 )
     {
-        // We will set a unitary Z so it will in future used with transformations
-        // since the board poly will be used not only to draw itself but also the
-        // solder mask layers.
+        // We will set a unitary Z so it will in future used with transformations since the
+        // board poly will be used not only to draw itself but also the solder mask layers.
         const float layer_z_top = 1.0f;
         const float layer_z_bot = 0.0f;
 
@@ -451,7 +466,7 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
     // Create Board
     m_board = createBoard( m_boardAdapter.GetBoardPoly(), &m_boardAdapter.GetThroughHoleIds() );
 
-    if( m_boardAdapter.GetFlag( FL_USE_REALISTIC_MODE ) )
+    if( m_boardAdapter.m_Cfg->m_Render.realistic )
     {
         m_antiBoardPolys.RemoveAllContours();
         m_antiBoardPolys.NewOutline();
@@ -466,7 +481,7 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
         m_antiBoard = createBoard( m_antiBoardPolys );
     }
 
-    SHAPE_POLY_SET board_poly_with_holes = m_boardAdapter.GetBoardPoly();
+    SHAPE_POLY_SET board_poly_with_holes = m_boardAdapter.GetBoardPoly().CloneDropTriangulation();
     board_poly_with_holes.BooleanSubtract( m_boardAdapter.GetThroughHoleOdPolys(),
                                            SHAPE_POLY_SET::PM_FAST );
     board_poly_with_holes.BooleanSubtract( m_boardAdapter.GetOuterNonPlatedThroughHolePoly(),
@@ -481,11 +496,13 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
     if( aStatusReporter )
         aStatusReporter->Report( _( "Load OpenGL: holes and vias" ) );
 
-    SHAPE_POLY_SET outerPolyTHT = m_boardAdapter.GetThroughHoleOdPolys();
+    SHAPE_POLY_SET outerPolyTHT = m_boardAdapter.GetThroughHoleOdPolys().CloneDropTriangulation();
 
-    if( m_boardAdapter.GetFlag( FL_USE_REALISTIC_MODE ) )
+    if( m_boardAdapter.m_Cfg->m_Render.realistic )
+    {
         outerPolyTHT.BooleanIntersection( m_boardAdapter.GetBoardPoly(),
                                           SHAPE_POLY_SET::PM_STRICTLY_SIMPLE );
+    }
 
     m_outerThroughHoles = generateHoles( m_boardAdapter.GetThroughHoleOds().GetList(),
                                          outerPolyTHT, 1.0f, 0.0f, false,
@@ -495,8 +512,8 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
             m_boardAdapter.GetThroughHoleViaOds().GetList(),
             m_boardAdapter.GetThroughHoleViaOdPolys(), 1.0f, 0.0f, false );
 
-    if( m_boardAdapter.GetFlag( FL_CLIP_SILK_ON_VIA_ANNULUS ) &&
-        m_boardAdapter.GetFlag( FL_USE_REALISTIC_MODE ) )
+    if( m_boardAdapter.m_Cfg->m_Render.clip_silk_on_via_annulus &&
+        m_boardAdapter.m_Cfg->m_Render.realistic )
     {
         m_outerThroughHoleRings = generateHoles(
                 m_boardAdapter.GetThroughHoleAnnularRings().GetList(),
@@ -515,7 +532,7 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
         float layer_z_bot = 0.0f;
         float layer_z_top = 0.0f;
 
-        for( const auto ii : outerMapHoles )
+        for( const std::pair<const PCB_LAYER_ID, SHAPE_POLY_SET*>& ii : outerMapHoles )
         {
             const PCB_LAYER_ID      layer_id  = ii.first;
             const SHAPE_POLY_SET*   poly      = ii.second;
@@ -527,7 +544,7 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
                                                          layer_z_top, layer_z_bot, false );
         }
 
-        for( const auto ii : innerMapHoles )
+        for( const std::pair<const PCB_LAYER_ID, SHAPE_POLY_SET*>& ii : innerMapHoles )
         {
             const PCB_LAYER_ID      layer_id  = ii.first;
             const SHAPE_POLY_SET*   poly      = ii.second;
@@ -549,7 +566,7 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
 
     const MAP_POLY& map_poly = m_boardAdapter.GetPolyMap();
 
-    for( const auto ii : m_boardAdapter.GetLayerMap() )
+    for( const std::pair<const PCB_LAYER_ID, BVH_CONTAINER_2D*>& ii : m_boardAdapter.GetLayerMap() )
     {
         const PCB_LAYER_ID layer_id = ii.first;
 
@@ -557,13 +574,15 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
             continue;
 
         if( aStatusReporter )
-            aStatusReporter->Report( wxString::Format(
-                                     _( "Load OpenGL layer %d" ), (int)layer_id ) );
+        {
+            aStatusReporter->Report( wxString::Format( _( "Load OpenGL layer %d" ),
+                                                       (int) layer_id ) );
+        }
 
         const BVH_CONTAINER_2D* container2d = ii.second;
 
         SHAPE_POLY_SET polyListSubtracted;
-        SHAPE_POLY_SET* aPolyList = nullptr;
+        SHAPE_POLY_SET* polyList = nullptr;
 
         // Load the vertical (Z axis) component of shapes
 
@@ -571,21 +590,20 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
         {
             polyListSubtracted = *map_poly.at( layer_id );;
 
-            if( m_boardAdapter.GetFlag( FL_USE_REALISTIC_MODE ) )
+            if( m_boardAdapter.m_Cfg->m_Render.realistic )
             {
                 polyListSubtracted.BooleanIntersection( m_boardAdapter.GetBoardPoly(),
                                                         SHAPE_POLY_SET::PM_FAST );
 
-                if( ( layer_id != B_Mask ) && ( layer_id != F_Mask ) )
+                if( layer_id != B_Mask && layer_id != F_Mask )
                 {
                     polyListSubtracted.BooleanSubtract( m_boardAdapter.GetThroughHoleOdPolys(),
                                                         SHAPE_POLY_SET::PM_FAST );
-                    polyListSubtracted.BooleanSubtract(
-                            m_boardAdapter.GetOuterNonPlatedThroughHolePoly(),
-                            SHAPE_POLY_SET::PM_FAST );
+                    polyListSubtracted.BooleanSubtract( m_boardAdapter.GetOuterNonPlatedThroughHolePoly(),
+                                                        SHAPE_POLY_SET::PM_FAST );
                 }
 
-                if( m_boardAdapter.GetFlag( FL_SUBTRACT_MASK_FROM_SILK ) )
+                if( m_boardAdapter.m_Cfg->m_Render.subtract_mask_from_silk )
                 {
                     if( layer_id == B_SilkS && map_poly.find( B_Mask ) != map_poly.end() )
                     {
@@ -600,10 +618,10 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
                 }
             }
 
-            aPolyList = &polyListSubtracted;
+            polyList = &polyListSubtracted;
         }
 
-        OPENGL_RENDER_LIST* oglList = generateLayerList( container2d, aPolyList, layer_id,
+        OPENGL_RENDER_LIST* oglList = generateLayerList( container2d, polyList, layer_id,
                                                          &m_boardAdapter.GetThroughHoleIds() );
 
         if( oglList != nullptr )
@@ -611,12 +629,15 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
 
     }
 
-    if( m_boardAdapter.GetFlag( FL_RENDER_PLATED_PADS_AS_PLATED ) &&
-        m_boardAdapter.GetFlag( FL_USE_REALISTIC_MODE ) )
+    if( m_boardAdapter.m_Cfg->m_Render.renderPlatedPadsAsPlated
+            && m_boardAdapter.m_Cfg->m_Render.realistic )
     {
-        if( m_boardAdapter.GetFrontPlatedPadPolys() )
+        const SHAPE_POLY_SET* frontPlatedPadPolys = m_boardAdapter.GetFrontPlatedPadPolys();
+        const SHAPE_POLY_SET* backPlatedPadPolys = m_boardAdapter.GetBackPlatedPadPolys();
+
+        if( frontPlatedPadPolys )
         {
-            SHAPE_POLY_SET polySubtracted = *m_boardAdapter.GetFrontPlatedPadPolys();
+            SHAPE_POLY_SET polySubtracted = frontPlatedPadPolys->CloneDropTriangulation();
             polySubtracted.BooleanIntersection( m_boardAdapter.GetBoardPoly(),
                                                 SHAPE_POLY_SET::PM_FAST );
             polySubtracted.BooleanSubtract( m_boardAdapter.GetThroughHoleOdPolys(),
@@ -626,11 +647,15 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
 
             m_platedPadsFront = generateLayerList( m_boardAdapter.GetPlatedPadsFront(),
                                                    &polySubtracted, F_Cu );
+
+            // An entry for F_Cu must exist in m_layers or we'll never look at m_platedPadsFront
+            if( m_layers.count( F_Cu ) == 0 )
+                m_layers[F_Cu] = generateEmptyLayerList( F_Cu );
         }
 
-        if( m_boardAdapter.GetBackPlatedPadPolys() )
+        if( backPlatedPadPolys )
         {
-            SHAPE_POLY_SET polySubtracted = *m_boardAdapter.GetBackPlatedPadPolys();
+            SHAPE_POLY_SET polySubtracted = backPlatedPadPolys->CloneDropTriangulation();
             polySubtracted.BooleanIntersection( m_boardAdapter.GetBoardPoly(),
                                                 SHAPE_POLY_SET::PM_FAST );
             polySubtracted.BooleanSubtract( m_boardAdapter.GetThroughHoleOdPolys(),
@@ -640,6 +665,10 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
 
             m_platedPadsBack = generateLayerList( m_boardAdapter.GetPlatedPadsBack(),
                                                   &polySubtracted, B_Cu );
+
+            // An entry for B_Cu must exist in m_layers or we'll never look at m_platedPadsBack
+            if( m_layers.count( B_Cu ) == 0 )
+                m_layers[B_Cu] = generateEmptyLayerList( B_Cu );
         }
     }
 
@@ -652,8 +681,7 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
     if( aStatusReporter )
     {
         // Calculation time in seconds
-        const double calculation_time = (double)( GetRunningMicroSecs() -
-                                                  stats_startReloadTime) / 1e6;
+        double calculation_time = (double)( GetRunningMicroSecs() - stats_startReloadTime) / 1e6;
 
         aStatusReporter->Report( wxString::Format( _( "Reload time %.3f s" ), calculation_time ) );
     }
@@ -674,10 +702,10 @@ void RENDER_3D_OPENGL::addTopAndBottomTriangles( TRIANGLE_DISPLAY_LIST* aDst, co
 }
 
 
-void RENDER_3D_OPENGL::getLayerZPos( PCB_LAYER_ID aLayerID, float& aOutZtop, float& aOutZbot ) const
+void RENDER_3D_OPENGL::getLayerZPos( PCB_LAYER_ID aLayer, float& aOutZtop, float& aOutZbot ) const
 {
-    aOutZbot = m_boardAdapter.GetLayerBottomZPos( aLayerID );
-    aOutZtop = m_boardAdapter.GetLayerTopZPos( aLayerID );
+    aOutZbot = m_boardAdapter.GetLayerBottomZPos( aLayer );
+    aOutZtop = m_boardAdapter.GetLayerTopZPos( aLayer );
 
     if( aOutZtop < aOutZbot )
     {
@@ -732,13 +760,11 @@ void RENDER_3D_OPENGL::generateViasAndPads()
 
     if( m_boardAdapter.GetViaCount() > 0 )
     {
-        const unsigned int reserve_nr_triangles_estimation =
-                m_boardAdapter.GetCircleSegmentCount(
-                        m_boardAdapter.GetAverageViaHoleDiameter() ) * 8 *
-                        m_boardAdapter.GetViaCount();
+        float        averageDiameter = m_boardAdapter.GetAverageViaHoleDiameter();
+        unsigned int averageSegCount = m_boardAdapter.GetCircleSegmentCount( averageDiameter );
+        unsigned int trianglesEstimate = averageSegCount * 8 * m_boardAdapter.GetViaCount();
 
-        TRIANGLE_DISPLAY_LIST* layerTriangleVIA =
-                new TRIANGLE_DISPLAY_LIST( reserve_nr_triangles_estimation );
+        TRIANGLE_DISPLAY_LIST* layerTriangleVIA = new TRIANGLE_DISPLAY_LIST( trianglesEstimate );
 
         // Insert plated vertical holes inside the board
 
@@ -793,17 +819,16 @@ void RENDER_3D_OPENGL::generateViasAndPads()
             {
                 if( pad->GetAttribute() != PAD_ATTRIB::NPTH )
                 {
-                    const wxSize drillsize = pad->GetDrillSize();
-                    const bool   hasHole   = drillsize.x && drillsize.y;
+                    const VECTOR2I drillsize = pad->GetDrillSize();
+                    const bool     hasHole = drillsize.x && drillsize.y;
 
                     if( !hasHole )
                         continue;
 
-                    pad->TransformHoleWithClearanceToPolygon( tht_outer_holes_poly,
-                                                              platingThickness,
-                                                              ARC_HIGH_DEF, ERROR_INSIDE );
-                    pad->TransformHoleWithClearanceToPolygon( tht_inner_holes_poly, 0,
-                                                              ARC_HIGH_DEF, ERROR_INSIDE );
+                    pad->TransformHoleToPolygon( tht_outer_holes_poly, platingThickness,
+                                                 ARC_HIGH_DEF, ERROR_INSIDE );
+                    pad->TransformHoleToPolygon( tht_inner_holes_poly, 0, ARC_HIGH_DEF,
+                                                 ERROR_INSIDE );
                 }
             }
         }
@@ -811,29 +836,27 @@ void RENDER_3D_OPENGL::generateViasAndPads()
         // Subtract the holes
         tht_outer_holes_poly.BooleanSubtract( tht_inner_holes_poly, SHAPE_POLY_SET::PM_FAST );
 
-        if( m_boardAdapter.GetFlag( FL_USE_REALISTIC_MODE ) )
+        if( m_boardAdapter.m_Cfg->m_Render.realistic )
             tht_outer_holes_poly.BooleanSubtract( m_antiBoardPolys, SHAPE_POLY_SET::PM_FAST );
 
         CONTAINER_2D holesContainer;
 
         ConvertPolygonToTriangles( tht_outer_holes_poly, holesContainer,
-                                   m_boardAdapter.BiuTo3dUnits(),
-                                   (const BOARD_ITEM &)*m_boardAdapter.GetBoard() );
+                                   m_boardAdapter.BiuTo3dUnits(), *m_boardAdapter.GetBoard() );
 
-        const LIST_OBJECT2D& listHolesObject2d = holesContainer.GetList();
+        const LIST_OBJECT2D& holes2D = holesContainer.GetList();
 
-        if( listHolesObject2d.size() > 0 )
+        if( holes2D.size() > 0 )
         {
             float layer_z_top, layer_z_bot, dummy;
 
             getLayerZPos( F_Cu, layer_z_top, dummy );
             getLayerZPos( B_Cu, dummy, layer_z_bot );
 
-            TRIANGLE_DISPLAY_LIST* layerTriangles =
-                    new TRIANGLE_DISPLAY_LIST( listHolesObject2d.size() );
+            TRIANGLE_DISPLAY_LIST* layerTriangles = new TRIANGLE_DISPLAY_LIST( holes2D.size() );
 
             // Convert the list of objects(triangles) to triangle layer structure
-            for( const OBJECT_2D* itemOnLayer : listHolesObject2d )
+            for( const OBJECT_2D* itemOnLayer : holes2D )
             {
                 const OBJECT_2D* object2d_A = itemOnLayer;
 
@@ -889,9 +912,9 @@ void RENDER_3D_OPENGL::load3dModels( REPORTER* aStatusReporter )
     if( !m_boardAdapter.GetBoard() )
         return;
 
-    if( !m_boardAdapter.GetFlag( FL_FP_ATTRIBUTES_NORMAL )
-      && !m_boardAdapter.GetFlag( FL_FP_ATTRIBUTES_NORMAL_INSERT )
-      && !m_boardAdapter.GetFlag( FL_FP_ATTRIBUTES_VIRTUAL ) )
+    if( !m_boardAdapter.m_Cfg->m_Render.show_footprints_normal
+          && !m_boardAdapter.m_Cfg->m_Render.show_footprints_insert
+          && !m_boardAdapter.m_Cfg->m_Render.show_footprints_virtual )
     {
         return;
     }
@@ -899,35 +922,55 @@ void RENDER_3D_OPENGL::load3dModels( REPORTER* aStatusReporter )
     // Go for all footprints
     for( const FOOTPRINT* footprint : m_boardAdapter.GetBoard()->Footprints() )
     {
-        for( const FP_3DMODEL& model : footprint->Models() )
+        wxString                libraryName = footprint->GetFPID().GetLibNickname();
+        wxString                footprintBasePath = wxEmptyString;
+
+        if( m_boardAdapter.GetBoard()->GetProject() )
         {
-            if( model.m_Show && !model.m_Filename.empty() )
+            try
+            {
+                // FindRow() can throw an exception
+                const FP_LIB_TABLE_ROW* fpRow =
+                        m_boardAdapter.GetBoard()->GetProject()->PcbFootprintLibs()->FindRow(
+                                libraryName, false );
+
+                if( fpRow )
+                    footprintBasePath = fpRow->GetFullURI( true );
+            }
+            catch( ... )
+            {
+                // Do nothing if the libraryName is not found in lib table
+            }
+        }
+
+        for( const FP_3DMODEL& fp_model : footprint->Models() )
+        {
+            if( fp_model.m_Show && !fp_model.m_Filename.empty() )
             {
                 if( aStatusReporter )
                 {
-                    // Display the short filename of the 3D model loaded:
+                    // Display the short filename of the 3D fp_model loaded:
                     // (the full name is usually too long to be displayed)
-                    wxFileName fn( model.m_Filename );
+                    wxFileName fn( fp_model.m_Filename );
                     aStatusReporter->Report( wxString::Format( _( "Loading %s..." ),
                                                                fn.GetFullName() ) );
                 }
 
-                // Check if the model is not present in our cache map
+                // Check if the fp_model is not present in our cache map
                 // (Not already loaded in memory)
-                if( m_3dModelMap.find( model.m_Filename ) == m_3dModelMap.end() )
+                if( m_3dModelMap.find( fp_model.m_Filename ) == m_3dModelMap.end() )
                 {
                     // It is not present, try get it from cache
                     const S3DMODEL* modelPtr =
-                            m_boardAdapter.Get3dCacheManager()->GetModel( model.m_Filename );
+                            m_boardAdapter.Get3dCacheManager()->GetModel( fp_model.m_Filename, footprintBasePath );
 
                     // only add it if the return is not NULL
                     if( modelPtr )
                     {
-                        MATERIAL_MODE materialMode = m_boardAdapter.GetMaterialMode();
-                        MODEL_3D* ogl_model = new MODEL_3D( *modelPtr, materialMode );
+                        MATERIAL_MODE materialMode = m_boardAdapter.m_Cfg->m_Render.material_mode;
+                        MODEL_3D*     model        = new MODEL_3D( *modelPtr, materialMode );
 
-                        if( ogl_model )
-                            m_3dModelMap[ model.m_Filename ] = ogl_model;
+                        m_3dModelMap[ fp_model.m_Filename ] = model;
                     }
                 }
             }
