@@ -24,11 +24,11 @@
  * or you may write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
-
 #include <confirm.h>
 #include <eda_draw_frame.h>
 #include <kiface_base.h>
 #include <macros.h>
+#include <scoped_set_reset.h>
 #include <settings/app_settings.h>
 #include <trace_helpers.h>
 
@@ -46,12 +46,13 @@
 #include <tool/tool_dispatcher.h>
 #include <tool/tool_manager.h>
 
-#include <widgets/infobar.h>
+#include <widgets/wx_infobar.h>
 
 #include <kiplatform/ui.h>
 
 #include <profile.h>
 
+#include <pgm_base.h>
 
 EDA_DRAW_PANEL_GAL::EDA_DRAW_PANEL_GAL( wxWindow* aParentWindow, wxWindowID aWindowId,
                                         const wxPoint& aPosition, const wxSize& aSize,
@@ -66,7 +67,8 @@ EDA_DRAW_PANEL_GAL::EDA_DRAW_PANEL_GAL( wxWindow* aParentWindow, wxWindowID aWin
         m_options( aOptions ),
         m_eventDispatcher( nullptr ),
         m_lostFocus( false ),
-        m_stealsFocus( true )
+        m_stealsFocus( true ),
+        m_statusPopup( nullptr )
 {
     m_parent = aParentWindow;
     m_MouseCapturedLost = false;
@@ -92,8 +94,17 @@ EDA_DRAW_PANEL_GAL::EDA_DRAW_PANEL_GAL( wxWindow* aParentWindow, wxWindowID aWin
     SwitchBackend( aGalType );
     SetBackgroundStyle( wxBG_STYLE_CUSTOM );
 
-    ShowScrollbars( wxSHOW_SB_ALWAYS, wxSHOW_SB_ALWAYS );
+    if( Pgm().GetCommonSettings()->m_Appearance.show_scrollbars )
+    {
+        ShowScrollbars( wxSHOW_SB_ALWAYS, wxSHOW_SB_ALWAYS );
+    }
+    else
+    {
+        ShowScrollbars( wxSHOW_SB_NEVER, wxSHOW_SB_NEVER );
+    }
+
     EnableScrolling( false, false ); // otherwise Zoom Auto disables GAL canvas
+    KIPLATFORM::UI::SetOverlayScrolling( this, false ); // Prevent excessive repaint on GTK
 
     Connect( wxEVT_SIZE, wxSizeEventHandler( EDA_DRAW_PANEL_GAL::onSize ), nullptr, this );
     Connect( wxEVT_ENTER_WINDOW, wxMouseEventHandler( EDA_DRAW_PANEL_GAL::onEnter ), nullptr,
@@ -115,6 +126,12 @@ EDA_DRAW_PANEL_GAL::EDA_DRAW_PANEL_GAL( wxWindow* aParentWindow, wxWindowID aWin
         wxEVT_MIDDLE_UP,
         wxEVT_MIDDLE_DOWN,
         wxEVT_MIDDLE_DCLICK,
+        wxEVT_AUX1_UP,
+        wxEVT_AUX1_DOWN,
+        wxEVT_AUX1_DCLICK,
+        wxEVT_AUX2_UP,
+        wxEVT_AUX2_DOWN,
+        wxEVT_AUX2_DCLICK,
         wxEVT_MOTION,
         wxEVT_MOUSEWHEEL,
         wxEVT_CHAR,
@@ -133,6 +150,8 @@ EDA_DRAW_PANEL_GAL::EDA_DRAW_PANEL_GAL( wxWindow* aParentWindow, wxWindowID aWin
     m_drawing = false;
     m_drawingEnabled = false;
 
+    m_minRefreshPeriod = 13; // 77 FPS (minus render time) by default
+
     // Set up timer that prevents too frequent redraw commands
     m_refreshTimer.SetOwner( this );
     Connect( m_refreshTimer.GetId(), wxEVT_TIMER,
@@ -173,6 +192,11 @@ void EDA_DRAW_PANEL_GAL::onPaint( wxPaintEvent& WXUNUSED( aEvent ) )
 
 void EDA_DRAW_PANEL_GAL::DoRePaint()
 {
+    if( !m_refreshMutex.try_lock() )
+        return;
+
+    std::lock_guard<std::mutex> lock( m_refreshMutex, std::adopt_lock );
+
     // Repaint the canvas, and fix scrollbar cursors
     // Usually called by a OnPaint event, but because it does not use a wxPaintDC,
     // it can be called outside a wxPaintEvent.
@@ -182,7 +206,11 @@ void EDA_DRAW_PANEL_GAL::DoRePaint()
     if( !IsDialogPreview() && GetParentEDAFrame() && GetParentEDAFrame()->GetScreen() )
         GetParentEDAFrame()->GetScreen()->m_ScrollCenter = GetView()->GetCenter();
 
-    m_viewControls->UpdateScrollbars();
+    if( Pgm().GetCommonSettings()->m_Appearance.show_scrollbars )
+        m_viewControls->UpdateScrollbars();
+
+    if( !m_drawingEnabled )
+        return;
 
     if( !m_gal->IsVisible() )
         return;
@@ -192,11 +220,12 @@ void EDA_DRAW_PANEL_GAL::DoRePaint()
     if( m_drawing )
         return;
 
+    SCOPED_SET_RESET<bool> drawing( m_drawing, true );
+
     ( *m_PaintEventCounter )++;
 
     wxASSERT( m_painter );
 
-    m_drawing = true;
     KIGFX::RENDER_SETTINGS* settings =
             static_cast<KIGFX::RENDER_SETTINGS*>( m_painter->GetSettings() );
 
@@ -212,6 +241,7 @@ void EDA_DRAW_PANEL_GAL::DoRePaint()
     try
     {
         cntUpd.Start();
+
         try
         {
             m_view->UpdateItems();
@@ -227,51 +257,55 @@ void EDA_DRAW_PANEL_GAL::DoRePaint()
 
         cntUpd.Stop();
 
-        cntCtx.Start();
-        int cookie = rand();
-        m_gal->LockContext( cookie );
-        m_gal->BeginDrawing();
-        cntCtx.Stop();
-
-        if( m_view->IsTargetDirty( KIGFX::TARGET_OVERLAY )
-            && !m_gal->HasTarget( KIGFX::TARGET_OVERLAY ) )
+        // GAL_DRAWING_CONTEXT can throw in the dtor, so we need to scope
+        // the full lifetime inside the try block
         {
-            m_view->MarkDirty();
-        }
+            cntCtx.Start();
+            KIGFX::GAL_DRAWING_CONTEXT ctx( m_gal );
+            cntCtx.Stop();
 
-        m_gal->SetClearColor( settings->GetBackgroundColor() );
-        m_gal->SetGridColor( settings->GetGridColor() );
-        m_gal->SetCursorColor( settings->GetCursorColor() );
+            if( m_view->IsTargetDirty( KIGFX::TARGET_OVERLAY )
+                && !m_gal->HasTarget( KIGFX::TARGET_OVERLAY ) )
+            {
+                m_view->MarkDirty();
+            }
 
-        // TODO: find why ClearScreen() must be called here in opengl mode
-        // and only if m_view->IsDirty() in Cairo mode to avoid display artifacts
-        // when moving the mouse cursor
-        if( m_backend == GAL_TYPE_OPENGL )
-            m_gal->ClearScreen();
+            m_gal->SetClearColor( settings->GetBackgroundColor() );
+            m_gal->SetGridColor( settings->GetGridColor() );
+            m_gal->SetCursorColor( settings->GetCursorColor() );
 
-        if( m_view->IsDirty() )
-        {
-            if( m_backend != GAL_TYPE_OPENGL && // Already called in opengl
-                m_view->IsTargetDirty( KIGFX::TARGET_NONCACHED ) )
+            // TODO: find why ClearScreen() must be called here in opengl mode
+            // and only if m_view->IsDirty() in Cairo mode to avoid display artifacts
+            // when moving the mouse cursor
+            if( m_backend == GAL_TYPE_OPENGL )
                 m_gal->ClearScreen();
 
-            m_view->ClearTargets();
+            if( m_view->IsDirty() )
+            {
+                if( m_backend != GAL_TYPE_OPENGL  // Already called in opengl
+                        && m_view->IsTargetDirty( KIGFX::TARGET_NONCACHED ) )
+                {
+                    m_gal->ClearScreen();
+                }
 
-            // Grid has to be redrawn only when the NONCACHED target is redrawn
-            if( m_view->IsTargetDirty( KIGFX::TARGET_NONCACHED ) )
-                m_gal->DrawGrid();
+                m_view->ClearTargets();
 
-            cntRedraw.Start();
-            m_view->Redraw();
-            cntRedraw.Stop();
-            isDirty = true;
+                // Grid has to be redrawn only when the NONCACHED target is redrawn
+                if( m_view->IsTargetDirty( KIGFX::TARGET_NONCACHED ) )
+                    m_gal->DrawGrid();
+
+                cntRedraw.Start();
+                m_view->Redraw();
+                cntRedraw.Stop();
+                isDirty = true;
+            }
+
+            m_gal->DrawCursor( m_viewControls->GetCursorPosition() );
+
+            cntCtxDestroy.Start();
         }
 
-        m_gal->DrawCursor( m_viewControls->GetCursorPosition() );
-
-        cntCtxDestroy.Start();
-        m_gal->EndDrawing();
-        m_gal->UnlockContext( cookie );
+        // ctx goes out of scope here so destructor would be called
         cntCtxDestroy.Stop();
     }
     catch( std::exception& err )
@@ -291,7 +325,6 @@ void EDA_DRAW_PANEL_GAL::DoRePaint()
         }
     }
 
-
     if( isDirty )
     {
         KI_TRACE( traceGalProfile, "View timing: %s %s %s %s %s\n",
@@ -304,7 +337,6 @@ void EDA_DRAW_PANEL_GAL::DoRePaint()
     }
 
     m_lastRefresh = wxGetLocalTimeMillis();
-    m_drawing = false;
 }
 
 
@@ -350,16 +382,16 @@ void EDA_DRAW_PANEL_GAL::Refresh( bool aEraseBackground, const wxRect* aRect )
     // If it has been too long since the last frame (possible depending on platform timer latency),
     // just do a refresh.  Otherwise, start the refresh timer if it hasn't already been started.
     // This ensures that we will render often enough but not too often.
-    if( delta >= MinRefreshPeriod )
+    if( delta >= m_minRefreshPeriod )
     {
         if( !m_pendingRefresh )
             ForceRefresh();
 
-        m_refreshTimer.Start( MinRefreshPeriod, true );
+        m_refreshTimer.Start( m_minRefreshPeriod, true );
     }
     else if( !m_refreshTimer.IsRunning() )
     {
-        m_refreshTimer.Start( ( MinRefreshPeriod - delta ).ToLong(), true );
+        m_refreshTimer.Start( ( m_minRefreshPeriod - delta ).ToLong(), true );
     }
 }
 
@@ -386,10 +418,10 @@ void EDA_DRAW_PANEL_GAL::StartDrawing()
 
 void EDA_DRAW_PANEL_GAL::StopDrawing()
 {
+    m_refreshTimer.Stop();
     m_drawingEnabled = false;
     Disconnect( wxEVT_PAINT, wxPaintEventHandler( EDA_DRAW_PANEL_GAL::onPaint ), nullptr, this );
     m_pendingRefresh = false;
-    m_refreshTimer.Stop();
 }
 
 
@@ -501,6 +533,12 @@ bool EDA_DRAW_PANEL_GAL::SwitchBackend( GAL_TYPE aGalType )
 
     m_gal->SetGridVisibility( grid_visibility );
 
+    if( m_gal->GetSwapInterval() != 0 )
+    {
+        // In theory this could be 0 but then more CPU cycles will be wasted in SwapBuffers
+        m_minRefreshPeriod = 5;
+    }
+
     // Make sure the cursor is set on the new canvas
     SetCurrentCursor( KICURSOR::ARROW );
 
@@ -570,7 +608,6 @@ void EDA_DRAW_PANEL_GAL::onRefreshTimer( wxTimerEvent& aEvent )
     {
         if( m_gal && m_gal->IsInitialized() )
         {
-            m_drawing = false;
             m_pendingRefresh = true;
             Connect( wxEVT_PAINT, wxPaintEventHandler( EDA_DRAW_PANEL_GAL::onPaint ), nullptr,
                      this );
