@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2017 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2013 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,14 +28,17 @@
 #include <dialog_shim.h>
 #include <dialogs/panel_common_settings.h>
 #include <dialogs/panel_mouse_settings.h>
+#include <dialogs/panel_data_collection.h>
 #include <eda_dde.h>
-#include <filehistory.h>
+#include <file_history.h>
 #include <id.h>
 #include <kiface_base.h>
 #include <menus_helpers.h>
+#include <hotkeys_basic.h>
 #include <panel_hotkeys_editor.h>
 #include <paths.h>
 #include <confirm.h>
+#include <panel_pcm_settings.h>
 #include <pgm_base.h>
 #include <settings/app_settings.h>
 #include <settings/common_settings.h>
@@ -50,7 +53,7 @@
 #include <tool/tool_dispatcher.h>
 #include <trace_helpers.h>
 #include <widgets/paged_dialog.h>
-#include <widgets/infobar.h>
+#include <widgets/wx_infobar.h>
 #include <widgets/wx_aui_art_providers.h>
 #include <wx/app.h>
 #include <wx/config.h>
@@ -62,6 +65,7 @@
 #include <kiplatform/ui.h>
 
 #include <functional>
+#include <kiface_ids.h>
 
 wxDEFINE_EVENT( UNITS_CHANGED, wxCommandEvent );
 
@@ -106,6 +110,7 @@ BEGIN_EVENT_TABLE( EDA_BASE_FRAME, wxFrame )
     EVT_MAXIMIZE( EDA_BASE_FRAME::OnMaximize )
 
     EVT_SYS_COLOUR_CHANGED( EDA_BASE_FRAME::onSystemColorChange )
+    EVT_ICONIZE( EDA_BASE_FRAME::onIconize )
 END_EVENT_TABLE()
 
 
@@ -116,14 +121,13 @@ void EDA_BASE_FRAME::commonInit( FRAME_T aFrameType )
     m_infoBar           = nullptr;
     m_settingsManager   = nullptr;
     m_fileHistory       = nullptr;
-    m_hasAutoSave       = false;
-    m_autoSaveState     = false;
-    m_autoSaveInterval  = -1;
+    m_supportsAutoSave  = false;
+    m_autoSavePending   = false;
     m_undoRedoCountMax  = DEFAULT_MAX_UNDO_ITEMS;
-    m_userUnits         = EDA_UNITS::MILLIMETRES;
     m_isClosing         = false;
     m_isNonUserClose    = false;
     m_autoSaveTimer     = new wxTimer( this, ID_AUTO_SAVE_TIMER );
+    m_autoSaveRequired  = false;
     m_mruPath           = PATHS::GetDefaultUserProjectsPath();
     m_frameSize         = defaultSize( aFrameType );
     m_displayIndex      = -1;
@@ -149,21 +153,14 @@ void EDA_BASE_FRAME::commonInit( FRAME_T aFrameType )
 }
 
 
-EDA_BASE_FRAME::EDA_BASE_FRAME( FRAME_T aFrameType, KIWAY* aKiway ) :
-        wxFrame(),
-        TOOLS_HOLDER(),
-        KIWAY_HOLDER( aKiway, KIWAY_HOLDER::FRAME )
-{
-    commonInit( aFrameType );
-}
-
-
-EDA_BASE_FRAME::EDA_BASE_FRAME( wxWindow* aParent, FRAME_T aFrameType,
-                                const wxString& aTitle, const wxPoint& aPos, const wxSize& aSize,
-                                long aStyle, const wxString& aFrameName, KIWAY* aKiway ) :
+EDA_BASE_FRAME::EDA_BASE_FRAME( wxWindow* aParent, FRAME_T aFrameType, const wxString& aTitle,
+                                const wxPoint& aPos, const wxSize& aSize, long aStyle,
+                                const wxString& aFrameName, KIWAY* aKiway,
+                                const EDA_IU_SCALE& aIuScale ) :
         wxFrame( aParent, wxID_ANY, aTitle, aPos, aSize, aStyle, aFrameName ),
         TOOLS_HOLDER(),
-        KIWAY_HOLDER( aKiway, KIWAY_HOLDER::FRAME )
+        KIWAY_HOLDER( aKiway, KIWAY_HOLDER::FRAME ),
+        UNITS_PROVIDER( aIuScale, EDA_UNITS::MILLIMETRES )
 {
     commonInit( aFrameType );
 }
@@ -183,7 +180,7 @@ wxWindow* EDA_BASE_FRAME::findQuasiModalDialog()
     // so we have to look for it separately.
     if( m_ident == FRAME_SCH )
     {
-        wxWindow* cvpcb = wxWindow::FindWindowByName( "CvpcbFrame" );
+        wxWindow* cvpcb = wxWindow::FindWindowByName( wxS( "CvpcbFrame" ) );
 
         if( cvpcb )
             return cvpcb;
@@ -276,21 +273,21 @@ bool EDA_BASE_FRAME::ProcessEvent( wxEvent& aEvent )
     if( Pgm().m_Quitting )
         return true;
 
-    if( !m_isClosing && m_hasAutoSave && IsShown() && IsActive()
-        && m_autoSaveState != isAutoSaveRequired()
-        && m_autoSaveInterval > 0 )
+    if( !m_isClosing && m_supportsAutoSave && IsShown() && IsActive()
+            && m_autoSavePending != isAutoSaveRequired()
+            && GetAutoSaveInterval() > 0 )
     {
-        if( !m_autoSaveState )
+        if( !m_autoSavePending )
         {
             wxLogTrace( traceAutoSave, wxT( "Starting auto save timer." ) );
-            m_autoSaveTimer->Start( m_autoSaveInterval * 1000, wxTIMER_ONE_SHOT );
-            m_autoSaveState = true;
+            m_autoSaveTimer->Start( GetAutoSaveInterval() * 1000, wxTIMER_ONE_SHOT );
+            m_autoSavePending = true;
         }
         else if( m_autoSaveTimer->IsRunning() )
         {
             wxLogTrace( traceAutoSave, wxT( "Stopping auto save timer." ) );
             m_autoSaveTimer->Stop();
-            m_autoSaveState = false;
+            m_autoSavePending = false;
         }
     }
 
@@ -298,29 +295,23 @@ bool EDA_BASE_FRAME::ProcessEvent( wxEvent& aEvent )
 }
 
 
-void EDA_BASE_FRAME::SetAutoSaveInterval( int aInterval )
+int EDA_BASE_FRAME::GetAutoSaveInterval() const
 {
-    m_autoSaveInterval = aInterval;
-
-    if( m_autoSaveTimer->IsRunning() )
-    {
-        if( m_autoSaveInterval > 0 )
-        {
-            m_autoSaveTimer->Start( m_autoSaveInterval * 1000, wxTIMER_ONE_SHOT );
-        }
-        else
-        {
-            m_autoSaveTimer->Stop();
-            m_autoSaveState = false;
-        }
-    }
+    return Pgm().GetCommonSettings()->m_System.autosave_interval;
 }
 
 
 void EDA_BASE_FRAME::onAutoSaveTimer( wxTimerEvent& aEvent )
 {
+    // Don't stomp on someone else's timer event.
+    if( aEvent.GetId() != ID_AUTO_SAVE_TIMER )
+    {
+        aEvent.Skip();
+        return;
+    }
+
     if( !doAutoSave() )
-        m_autoSaveTimer->Start( m_autoSaveInterval * 1000, wxTIMER_ONE_SHOT );
+        m_autoSaveTimer->Start( GetAutoSaveInterval() * 1000, wxTIMER_ONE_SHOT );
 }
 
 
@@ -332,7 +323,7 @@ bool EDA_BASE_FRAME::doAutoSave()
 
 void EDA_BASE_FRAME::OnCharHook( wxKeyEvent& aKeyEvent )
 {
-    wxLogTrace( kicadTraceKeyEvent, "EDA_BASE_FRAME::OnCharHook %s", dump( aKeyEvent ) );
+    wxLogTrace( kicadTraceKeyEvent, wxS( "EDA_BASE_FRAME::OnCharHook %s" ), dump( aKeyEvent ) );
 
     // Key events can be filtered here.
     // Currently no filtering is made.
@@ -423,7 +414,7 @@ void EDA_BASE_FRAME::HandleUpdateUIEvent( wxUpdateUIEvent& aEvent, EDA_BASE_FRAM
 
     // wxMenuItems don't want to be checked unless they actually are checkable, so we have to
     // check to see if they can be and can't just universally apply a check in this event.
-    if( auto menu = dynamic_cast<wxMenu*>( aEvent.GetEventObject() ) )
+    if( wxMenu* menu = dynamic_cast<wxMenu*>( aEvent.GetEventObject() ) )
         canCheck = menu->FindItem( aEvent.GetId() )->IsCheckable();
 
     if( canCheck )
@@ -454,6 +445,17 @@ void EDA_BASE_FRAME::setupUIConditions()
 
 void EDA_BASE_FRAME::ReCreateMenuBar()
 {
+    /**
+     * As of wxWidgets 3.2, recreating the menubar from within an event handler of that menubar
+     * will result in memory corruption on macOS.  In order to minimize the chance of programmer
+     * error causing regressions here, we always wrap calls to ReCreateMenuBar in a CallAfter to
+     * ensure that they do not occur within the same event handling call stack.
+     */
+
+    CallAfter( [=]()
+               {
+                   doReCreateMenuBar();
+               } );
 }
 
 
@@ -535,7 +537,7 @@ void EDA_BASE_FRAME::OnSize( wxSizeEvent& aEvent )
 
     if( m_displayIndex >= 0 && currentDisplay >= 0 && currentDisplay != m_displayIndex )
     {
-        wxLogTrace( traceDisplayLocation, "OnSize: current display changed %d to %d",
+        wxLogTrace( traceDisplayLocation, wxS( "OnSize: current display changed %d to %d" ),
                     m_displayIndex, currentDisplay );
         m_displayIndex = currentDisplay;
         ensureWindowIsOnScreen();
@@ -569,7 +571,7 @@ void EDA_BASE_FRAME::LoadWindowState( const WINDOW_STATE& aState )
     m_frameSize.x = aState.size_x;
     m_frameSize.y = aState.size_y;
 
-    wxLogTrace( traceDisplayLocation, "Config position (%d, %d) with size (%d, %d)",
+    wxLogTrace( traceDisplayLocation, wxS( "Config position (%d, %d) with size (%d, %d)" ),
                 m_framePos.x, m_framePos.y, m_frameSize.x, m_frameSize.y );
 
     // Ensure minimum size is set if the stored config was zero-initialized
@@ -578,15 +580,15 @@ void EDA_BASE_FRAME::LoadWindowState( const WINDOW_STATE& aState )
         m_frameSize = defaultSize( m_ident );
         wasDefault  = true;
 
-        wxLogTrace( traceDisplayLocation, "Using minimum size (%d, %d)",
+        wxLogTrace( traceDisplayLocation, wxS( "Using minimum size (%d, %d)" ),
                     m_frameSize.x, m_frameSize.y );
     }
 
-    wxLogTrace( traceDisplayLocation, "Number of displays: %d", wxDisplay::GetCount() );
+    wxLogTrace( traceDisplayLocation, wxS( "Number of displays: %d" ), wxDisplay::GetCount() );
 
     if( aState.display >= wxDisplay::GetCount() )
     {
-        wxLogTrace( traceDisplayLocation, "Previous display not found" );
+        wxLogTrace( traceDisplayLocation, wxS( "Previous display not found" ) );
 
         // If it isn't attached, use the first display
         // Warning wxDisplay has 2 ctor variants. the parameter needs a type:
@@ -622,11 +624,11 @@ void EDA_BASE_FRAME::LoadWindowState( const WINDOW_STATE& aState )
             upperLeft.y > yLimBottom )
         {
             m_framePos = wxDefaultPosition;
-            wxLogTrace( traceDisplayLocation, "Resetting to default position" );
+            wxLogTrace( traceDisplayLocation, wxS( "Resetting to default position" ) );
         }
     }
 
-    wxLogTrace( traceDisplayLocation, "Final window position (%d, %d) with size (%d, %d)",
+    wxLogTrace( traceDisplayLocation, wxS( "Final window position (%d, %d) with size (%d, %d)" ),
                 m_framePos.x, m_framePos.y, m_frameSize.x, m_frameSize.y );
 
     SetSize( m_framePos.x, m_framePos.y, m_frameSize.x, m_frameSize.y );
@@ -634,7 +636,7 @@ void EDA_BASE_FRAME::LoadWindowState( const WINDOW_STATE& aState )
     // Center the window if we reset to default
     if( m_framePos.x == -1 )
     {
-        wxLogTrace( traceDisplayLocation, "Centering window" );
+        wxLogTrace( traceDisplayLocation, wxS( "Centering window" ) );
         Center();
         m_framePos = GetPosition();
     }
@@ -646,7 +648,7 @@ void EDA_BASE_FRAME::LoadWindowState( const WINDOW_STATE& aState )
     // Maximize if we were maximized before
     if( aState.maximized || ( wasDefault && m_maximizeByDefault ) )
     {
-        wxLogTrace( traceDisplayLocation, "Maximizing window" );
+        wxLogTrace( traceDisplayLocation, wxS( "Maximizing window" ) );
         Maximize();
     }
 
@@ -662,13 +664,13 @@ void EDA_BASE_FRAME::ensureWindowIsOnScreen()
     wxSize    size       = GetWindowSize();
 
     wxLogTrace( traceDisplayLocation,
-                "ensureWindowIsOnScreen: clientArea (%d, %d) w %d h %d", clientSize.x, clientSize.y,
+                wxS( "ensureWindowIsOnScreen: clientArea (%d, %d) w %d h %d" ), clientSize.x, clientSize.y,
                 clientSize.width, clientSize.height );
 
     if( pos.y < clientSize.y )
     {
         wxLogTrace( traceDisplayLocation,
-                    "ensureWindowIsOnScreen: y pos %d below minimum, setting to %d", pos.y,
+                    wxS( "ensureWindowIsOnScreen: y pos %d below minimum, setting to %d" ), pos.y,
                     clientSize.y );
         pos.y = clientSize.y;
     }
@@ -676,7 +678,7 @@ void EDA_BASE_FRAME::ensureWindowIsOnScreen()
     if( pos.x < clientSize.x )
     {
         wxLogTrace( traceDisplayLocation,
-                    "ensureWindowIsOnScreen: x pos %d is off the client rect, setting to %d", pos.x,
+                    wxS( "ensureWindowIsOnScreen: x pos %d is off the client rect, setting to %d" ), pos.x,
                     clientSize.x );
         pos.x = clientSize.x;
     }
@@ -685,7 +687,7 @@ void EDA_BASE_FRAME::ensureWindowIsOnScreen()
     {
         int newWidth = clientSize.width - ( pos.x - clientSize.x );
         wxLogTrace( traceDisplayLocation,
-                    "ensureWindowIsOnScreen: effective width %d above available %d, setting to %d",
+                    wxS( "ensureWindowIsOnScreen: effective width %d above available %d, setting to %d" ),
                     pos.x + size.x, clientSize.width, newWidth );
         size.x = newWidth;
     }
@@ -694,12 +696,12 @@ void EDA_BASE_FRAME::ensureWindowIsOnScreen()
     {
         int newHeight = clientSize.height - ( pos.y - clientSize.y );
         wxLogTrace( traceDisplayLocation,
-                    "ensureWindowIsOnScreen: effective height %d above available %d, setting to %d",
+                    wxS( "ensureWindowIsOnScreen: effective height %d above available %d, setting to %d" ),
                     pos.y + size.y, clientSize.height, newHeight );
         size.y = newHeight;
     }
 
-    wxLogTrace( traceDisplayLocation, "Updating window position (%d, %d) with size (%d, %d)",
+    wxLogTrace( traceDisplayLocation, wxS( "Updating window position (%d, %d) with size (%d, %d)" ),
                 pos.x, pos.y, size.x, size.y );
 
     SetSize( pos.x, pos.y, size.x, size.y );
@@ -710,9 +712,6 @@ void EDA_BASE_FRAME::LoadWindowSettings( const WINDOW_SETTINGS* aCfg )
 {
     LoadWindowState( aCfg->state );
 
-    if( m_hasAutoSave )
-        m_autoSaveInterval = Pgm().GetCommonSettings()->m_System.autosave_interval;
-
     m_perspective = aCfg->perspective;
     m_mruPath = aCfg->mru_path;
 
@@ -722,13 +721,9 @@ void EDA_BASE_FRAME::LoadWindowSettings( const WINDOW_SETTINGS* aCfg )
 
 void EDA_BASE_FRAME::SaveWindowSettings( WINDOW_SETTINGS* aCfg )
 {
-    wxString        text;
-
     if( IsIconized() )
         return;
 
-    wxString baseCfgName = ConfigBaseName();
-
     // If the window is maximized, we use the saved window size from before it was maximized
     if( IsMaximized() )
     {
@@ -748,15 +743,11 @@ void EDA_BASE_FRAME::SaveWindowSettings( WINDOW_SETTINGS* aCfg )
     aCfg->state.maximized = IsMaximized();
     aCfg->state.display   = wxDisplay::GetFromWindow( this );
 
-    wxLogTrace( traceDisplayLocation, "Saving window maximized: %s",
-                IsMaximized() ? "true" : "false" );
-    wxLogTrace( traceDisplayLocation, "Saving config position (%d, %d) with size (%d, %d)",
+    wxLogTrace( traceDisplayLocation, wxS( "Saving window maximized: %s" ),
+                IsMaximized() ? wxS( "true" ) : wxS( "false" ) );
+    wxLogTrace( traceDisplayLocation, wxS( "Saving config position (%d, %d) with size (%d, %d)" ),
                 m_framePos.x, m_framePos.y, m_frameSize.x, m_frameSize.y );
 
-    // TODO(JE) should auto-save in common settings be overwritten by every app?
-    if( m_hasAutoSave )
-        Pgm().GetCommonSettings()->m_System.autosave_interval = m_autoSaveInterval;
-
     // Once this is fully implemented, wxAuiManager will be used to maintain
     // the persistence of the main frame and all it's managed windows and
     // all of the legacy frame persistence position code can be removed.
@@ -782,6 +773,8 @@ void EDA_BASE_FRAME::LoadSettings( APP_SETTINGS_BASE* aCfg )
 
 void EDA_BASE_FRAME::SaveSettings( APP_SETTINGS_BASE* aCfg )
 {
+    wxCHECK( config(), /* void */ );
+
     SaveWindowSettings( GetWindowSettings( aCfg ) );
 
     bool fileOpen = m_isClosing && m_isNonUserClose;
@@ -845,7 +838,7 @@ void EDA_BASE_FRAME::CreateInfoBar()
 #else
     m_infoBar = new WX_INFOBAR( this, &m_auimgr );
 
-    m_auimgr.AddPane( m_infoBar, EDA_PANE().InfoBar().Name( "InfoBar" ).Top().Layer(1) );
+    m_auimgr.AddPane( m_infoBar, EDA_PANE().InfoBar().Name( wxS( "InfoBar" ) ).Top().Layer(1) );
 #endif
 }
 
@@ -860,7 +853,7 @@ void EDA_BASE_FRAME::FinishAUIInitialization()
     m_auimgr.Update();
 
     // We don't want the infobar displayed right away
-    m_auimgr.GetPane( "InfoBar" ).Hide();
+    m_auimgr.GetPane( wxS( "InfoBar" ) ).Hide();
     m_auimgr.Update();
 #endif
 }
@@ -1000,45 +993,195 @@ void EDA_BASE_FRAME::OnKicadAbout( wxCommandEvent& event )
 
 void EDA_BASE_FRAME::OnPreferences( wxCommandEvent& event )
 {
+    ShowPreferences( wxEmptyString, wxEmptyString );
+}
+
+
+void EDA_BASE_FRAME::ShowPreferences( wxString aStartPage, wxString aStartParentPage )
+{
+    wxBeginBusyCursor( wxHOURGLASS_CURSOR );
+
     PAGED_DIALOG dlg( this, _( "Preferences" ), true );
-    wxTreebook* book = dlg.GetTreebook();
+
+    dlg.SetEvtHandlerEnabled( false );
+    wxTreebook*  book = dlg.GetTreebook();
+
+    PANEL_HOTKEYS_EDITOR*   hotkeysPanel = new PANEL_HOTKEYS_EDITOR( this, book, false );
+    KIFACE*                 kiface = nullptr;
+    std::vector<int>        expand;
+
+    Kiway().GetActions( hotkeysPanel->ActionsList() );
 
     book->AddPage( new PANEL_COMMON_SETTINGS( &dlg, book ), _( "Common" ) );
-
     book->AddPage( new PANEL_MOUSE_SETTINGS( &dlg, book ), _( "Mouse and Touchpad" ) );
-
-    PANEL_HOTKEYS_EDITOR* hotkeysPanel = new PANEL_HOTKEYS_EDITOR( this, book, false );
     book->AddPage( hotkeysPanel, _( "Hotkeys" ) );
 
-    wxWindow* viewer3D = nullptr;
+#ifdef KICAD_USE_SENTRY
+    book->AddPage( new PANEL_DATA_COLLECTION( &dlg, book ), _( "Data Collection" ) );
+#endif
 
-    for( unsigned i = 0; i < KIWAY_PLAYER_COUNT;  ++i )
+#define CREATE_PANEL( key ) kiface->CreateKiWindow( book, key, &Kiway() )
+
+    // If a dll is not loaded, the loader will show an error message.
+
+    try
     {
-        KIWAY_PLAYER* frame = dlg.Kiway().Player( (FRAME_T) i, false );
+        kiface = Kiway().KiFACE( KIWAY::FACE_SCH );
 
-        if( frame )
-        {
-            frame->InstallPreferences( &dlg, hotkeysPanel );
+        kiface->GetActions( hotkeysPanel->ActionsList() );
 
-            if( !viewer3D )
-                viewer3D = wxFindWindowByName( QUALIFIED_VIEWER3D_FRAMENAME( frame ) );
-        }
+        if( GetFrameType() == FRAME_SCH_SYMBOL_EDITOR )
+            expand.push_back( book->GetPageCount() );
+
+        book->AddPage( new wxPanel( book ), _( "Symbol Editor" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_SYM_DISP_OPTIONS ), _( "Display Options" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_SYM_EDIT_OPTIONS ), _( "Editing Options" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_SYM_COLORS ), _( "Colors" ) );
+
+        if( GetFrameType() == FRAME_SCH )
+            expand.push_back( book->GetPageCount() );
+
+        book->AddPage( new wxPanel( book ), _( "Schematic Editor" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_SCH_DISP_OPTIONS ), _( "Display Options" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_SCH_EDIT_OPTIONS ), _( "Editing Options" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_SCH_ANNO_OPTIONS ), _( "Annotation Options" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_SCH_COLORS ), _( "Colors" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_SCH_FIELD_NAME_TEMPLATES ),
+                          _( "Field Name Templates" ) );
+    }
+    catch( ... )
+    {
     }
 
-    if( viewer3D )
-        static_cast<EDA_BASE_FRAME*>( viewer3D )->InstallPreferences( &dlg, hotkeysPanel );
+    try
+    {
+        kiface = Kiway().KiFACE( KIWAY::FACE_PCB );
 
-    // The Kicad manager frame is not a player so we have to add it by hand
-    wxWindow* manager = wxFindWindowByName( KICAD_MANAGER_FRAME_NAME );
+        kiface->GetActions( hotkeysPanel->ActionsList() );
 
-    if( manager )
-        static_cast<EDA_BASE_FRAME*>( manager )->InstallPreferences( &dlg, hotkeysPanel );
+        if( GetFrameType() == FRAME_FOOTPRINT_EDITOR )
+            expand.push_back( book->GetPageCount() );
+
+        book->AddPage( new wxPanel( book ), _( "Footprint Editor" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_FP_DISPLAY_OPTIONS ), _( "Display Options" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_FP_EDIT_OPTIONS ), _( "Editing Options" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_FP_COLORS ), _( "Colors" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_FP_DEFAULT_VALUES ), _( "Default Values" ) );
+
+        if( GetFrameType() ==  FRAME_PCB_EDITOR )
+            expand.push_back( book->GetPageCount() );
+
+        book->AddPage( new wxPanel( book ), _( "PCB Editor" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_PCB_DISPLAY_OPTIONS ), _( "Display Options" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_PCB_EDIT_OPTIONS ), _( "Editing Options" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_PCB_COLORS ), _( "Colors" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_PCB_ACTION_PLUGINS ), _( "Action Plugins" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_PCB_ORIGINS_AXES ), _( "Origins & Axes" ) );
+
+        if( GetFrameType() == FRAME_PCB_DISPLAY3D )
+            expand.push_back( book->GetPageCount() );
+
+        book->AddPage( new wxPanel( book ), _( "3D Viewer" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_3DV_DISPLAY_OPTIONS ), _( "General" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_3DV_OPENGL ), _( "Realtime Renderer" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_3DV_RAYTRACING ), _( "Raytracing Renderer" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_3DV_COLORS ), _( "Colors" ) );
+    }
+    catch( ... )
+    {
+    }
+
+    try
+    {
+        kiface = Kiway().KiFACE( KIWAY::FACE_GERBVIEW );
+
+        kiface->GetActions( hotkeysPanel->ActionsList() );
+
+        if( GetFrameType() == FRAME_GERBER )
+            expand.push_back( book->GetPageCount() );
+
+        book->AddPage( new wxPanel( book ), _( "Gerber Viewer" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_GBR_DISPLAY_OPTIONS ), _( "Display Options" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_GBR_COLORS ), _( "Colors" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_GBR_EXCELLON_OPTIONS ), _( "Excellon Options" ) );
+    }
+    catch( ... )
+    {
+    }
+
+    try
+    {
+        kiface = Kiway().KiFACE( KIWAY::FACE_PL_EDITOR );
+        kiface->GetActions( hotkeysPanel->ActionsList() );
+
+        if( GetFrameType() == FRAME_PL_EDITOR )
+            expand.push_back( book->GetPageCount() );
+
+        book->AddPage( new wxPanel( book ), _( "Drawing Sheet Editor" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_DS_DISPLAY_OPTIONS ), _( "Display Options" ) );
+        book->AddSubPage( CREATE_PANEL( PANEL_DS_COLORS ), _( "Colors" ) );
+
+        book->AddPage( new PANEL_PCM_SETTINGS( book ), _( "Plugin and Content Manager" ) );
+    }
+    catch( ... )
+    {
+    }
+
+    // Update all of the action hotkeys. The process of loading the actions through
+    // the KiFACE will only get us the default hotkeys
+    ReadHotKeyConfigIntoActions( wxEmptyString, hotkeysPanel->ActionsList() );
 
     for( size_t i = 0; i < book->GetPageCount(); ++i )
         book->GetPage( i )->Layout();
 
+    for( int page : expand )
+        book->ExpandNode( page );
+
+    if( !aStartPage.IsEmpty() )
+        dlg.SetInitialPage( aStartPage, aStartParentPage );
+
+    dlg.SetEvtHandlerEnabled( true );
+    wxEndBusyCursor();
+
     if( dlg.ShowModal() == wxID_OK )
+    {
+        Pgm().GetSettingsManager().Save();
         dlg.Kiway().CommonSettingsChanged( false, false );
+    }
+
+#undef CREATE_PANEL
+}
+
+
+void EDA_BASE_FRAME::OnDropFiles( wxDropFilesEvent& aEvent )
+{
+    wxString* files = aEvent.GetFiles();
+
+    for( int nb = 0; nb < aEvent.GetNumberOfFiles(); nb++ )
+    {
+        const wxFileName fn = wxFileName( files[nb] );
+        wxString         ext = fn.GetExt();
+
+        // Alias all gerber files as GerberFileExtension
+        if( IsGerberFileExtension( ext ) )
+            ext = GerberFileExtension;
+
+        if( m_acceptedExts.find( ext.ToStdString() ) != m_acceptedExts.end() )
+            m_AcceptedFiles.emplace_back( fn );
+    }
+
+    DoWithAcceptedFiles();
+    m_AcceptedFiles.clear();
+}
+
+
+void EDA_BASE_FRAME::DoWithAcceptedFiles()
+{
+    for( const wxFileName& file : m_AcceptedFiles )
+    {
+        wxString fn = file.GetFullPath();
+        m_toolManager->RunAction( *m_acceptedExts.at( file.GetExt() ), true, &fn );
+    }
 }
 
 
@@ -1062,24 +1205,21 @@ bool EDA_BASE_FRAME::IsWritable( const wxFileName& aFileName, bool aVerbose )
 
     if( fn.IsDir() && !fn.IsDirWritable() )
     {
-        msg.Printf( _( "Insufficient permissions to folder '%s'." ),
-                    fn.GetPath() );
+        msg.Printf( _( "Insufficient permissions to folder '%s'." ), fn.GetPath() );
     }
     else if( !fn.FileExists() && !fn.IsDirWritable() )
     {
-        msg.Printf( _( "Insufficient permissions to save file '%s'." ),
-                    fn.GetFullName(), fn.GetPath() );
+        msg.Printf( _( "Insufficient permissions to save file '%s'." ), fn.GetFullPath() );
     }
     else if( fn.FileExists() && !fn.IsFileWritable() )
     {
-        msg.Printf( _( "Insufficient permissions to save file '%s'." ),
-                    fn.GetFullPath() );
+        msg.Printf( _( "Insufficient permissions to save file '%s'." ), fn.GetFullPath() );
     }
 
     if( !msg.IsEmpty() )
     {
         if( aVerbose )
-            wxMessageBox( msg );
+            DisplayErrorMessage( this, msg );
 
         return false;
     }
@@ -1090,6 +1230,9 @@ bool EDA_BASE_FRAME::IsWritable( const wxFileName& aFileName, bool aVerbose )
 
 void EDA_BASE_FRAME::CheckForAutoSaveFile( const wxFileName& aFileName )
 {
+    if( !IsGUI() )
+        return;
+
     wxCHECK_RET( aFileName.IsOk(), wxT( "Invalid file name!" ) );
 
     wxFileName autoSaveFileName = aFileName;
@@ -1103,13 +1246,13 @@ void EDA_BASE_FRAME::CheckForAutoSaveFile( const wxFileName& aFileName )
     if( !autoSaveFileName.FileExists() )
         return;
 
-    wxString msg = wxString::Format( _(
-            "Well this is potentially embarrassing!\n"
-            "It appears that the last time you were editing the file\n"
-            "%s\n"
-            "it was not saved properly.  Do you wish to restore the last saved edits you made?" ),
-            aFileName.GetFullName()
-        );
+    wxString msg = wxString::Format( _( "Well this is potentially embarrassing!\n"
+                                        "It appears that the last time you were editing\n"
+                                        "%s\n"
+                                        "KiCad exited before saving.\n"
+                                        "\n"
+                                        "Do you wish to open the auto-saved file instead?" ),
+                                        aFileName.GetFullName() );
 
     int response = wxMessageBox( msg, Pgm().App().GetAppDisplayName(), wxYES_NO | wxICON_QUESTION,
                                  this );
@@ -1200,12 +1343,19 @@ PICKED_ITEMS_LIST* EDA_BASE_FRAME::PopCommandFromRedoList( )
 }
 
 
+void EDA_BASE_FRAME::OnModify()
+{
+    m_autoSaveRequired = true;
+}
+
+
 void EDA_BASE_FRAME::ChangeUserUnits( EDA_UNITS aUnits )
 {
     SetUserUnits( aUnits );
     unitsChangeRefresh();
 
     wxCommandEvent e( UNITS_CHANGED );
+    e.SetClientData( this );
     ProcessEventLocally( e );
 }
 
@@ -1284,14 +1434,26 @@ void EDA_BASE_FRAME::onSystemColorChange( wxSysColourChangedEvent& aEvent )
 }
 
 
+void EDA_BASE_FRAME::onIconize( wxIconizeEvent& aEvent )
+{
+    // Call the handler
+    handleIconizeEvent( aEvent );
+
+    // Skip the event.
+    aEvent.Skip();
+}
+
+
 #ifdef _WIN32
 WXLRESULT EDA_BASE_FRAME::MSWWindowProc( WXUINT message, WXWPARAM wParam, WXLPARAM lParam )
 {
     // This will help avoid the menu keeping focus when the alt key is released
     // You can still trigger accelerators as long as you hold down alt
     if( message == WM_SYSCOMMAND )
+    {
         if( wParam == SC_KEYMENU && ( lParam >> 16 ) <= 0 )
             return 0;
+    }
 
     return wxFrame::MSWWindowProc( message, wParam, lParam );
 }
