@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2004-2017 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2008 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 2004-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2004-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,7 +29,7 @@
 #include <core/arraydim.h>
 #include <dialog_shim.h>
 #include <eda_draw_frame.h>
-#include <filehistory.h>
+#include <file_history.h>
 #include <id.h>
 #include <kiface_base.h>
 #include <lockfile.h>
@@ -59,6 +59,7 @@
 #include <wx/snglinst.h>
 #include <dialogs/dialog_grid_settings.h>
 #include <widgets/ui_common.h>
+#include <widgets/search_pane.h>
 #include <wx/dirdlg.h>
 #include <wx/filedlg.h>
 #include <wx/msgdlg.h>
@@ -77,11 +78,12 @@ BEGIN_EVENT_TABLE( EDA_DRAW_FRAME, KIWAY_PLAYER )
     EVT_ACTIVATE( EDA_DRAW_FRAME::onActivate )
 END_EVENT_TABLE()
 
+bool EDA_DRAW_FRAME::m_openGLFailureOccured = false;
 
 EDA_DRAW_FRAME::EDA_DRAW_FRAME( KIWAY* aKiway, wxWindow* aParent, FRAME_T aFrameType,
                                 const wxString& aTitle, const wxPoint& aPos, const wxSize& aSize,
-                                long aStyle, const wxString & aFrameName ) :
-    KIWAY_PLAYER( aKiway, aParent, aFrameType, aTitle, aPos, aSize, aStyle, aFrameName )
+                                long aStyle, const wxString& aFrameName, const EDA_IU_SCALE& aIuScale ) :
+    KIWAY_PLAYER( aKiway, aParent, aFrameType, aTitle, aPos, aSize, aStyle, aFrameName, aIuScale )
 {
     m_socketServer        = nullptr;
     m_mainToolBar         = nullptr;
@@ -90,7 +92,7 @@ EDA_DRAW_FRAME::EDA_DRAW_FRAME( KIWAY* aKiway, wxWindow* aParent, FRAME_T aFrame
     m_auxiliaryToolBar    = nullptr;
     m_gridSelectBox       = nullptr;
     m_zoomSelectBox       = nullptr;
-    m_firstRunDialogSetting = 0;
+    m_searchPane          = nullptr;
     m_undoRedoCountMax    = DEFAULT_MAX_UNDO_ITEMS;
 
     m_canvasType          = EDA_DRAW_PANEL_GAL::GAL_TYPE_NONE;
@@ -100,18 +102,18 @@ EDA_DRAW_FRAME::EDA_DRAW_FRAME( KIWAY* aKiway, wxWindow* aParent, FRAME_T aFrame
     m_currentScreen       = nullptr;
     m_showBorderAndTitleBlock = false;  // true to display reference sheet.
     m_gridColor           = COLOR4D( DARKGRAY );   // Default grid color
-    m_showPageLimits      = false;
     m_drawBgColor         = COLOR4D( BLACK );   // the background color of the draw canvas:
                                                 // BLACK for Pcbnew, BLACK or WHITE for Eeschema
     m_colorSettings       = nullptr;
     m_msgFrameHeight      = EDA_MSG_PANEL::GetRequiredHeight( this );
-    m_userUnits           = EDA_UNITS::MILLIMETRES;
     m_polarCoords         = false;
-    m_findReplaceData     = new wxFindReplaceData( wxFR_DOWN );
+    m_findReplaceData     = std::make_unique<EDA_SEARCH_DATA>();
+
+    SetUserUnits( EDA_UNITS::MILLIMETRES );
 
     m_auimgr.SetFlags( wxAUI_MGR_DEFAULT );
 
-    CreateStatusBar( 8 );
+    CreateStatusBar( 8 )->SetDoubleBuffered( true );
 
     // set the size of the status bar subwindows:
 
@@ -167,6 +169,16 @@ EDA_DRAW_FRAME::EDA_DRAW_FRAME( KIWAY* aKiway, wxWindow* aParent, FRAME_T aFrame
           {
               wxMoveEvent dummy;
               OnMove( dummy );
+
+              // we need to kludge the msg panel to the correct size again
+              // especially important even for first launches as the constructor of the window here usually doesn't
+              // have the correct dpi awareness yet
+              m_frameSize.y += m_msgFrameHeight;
+              m_msgFrameHeight = EDA_MSG_PANEL::GetRequiredHeight( this );
+              m_frameSize.y -= m_msgFrameHeight;
+
+              m_messagePanel->SetPosition( wxPoint( 0, m_frameSize.y ) );
+              m_messagePanel->SetSize( m_frameSize.x, m_msgFrameHeight );
           } );
 #endif
 }
@@ -176,7 +188,7 @@ EDA_DRAW_FRAME::~EDA_DRAW_FRAME()
 {
     delete m_socketServer;
 
-    for( auto socket : m_sockets )
+    for( wxSocketBase* socket : m_sockets )
     {
         socket->Shutdown();
         socket->Destroy();
@@ -192,8 +204,6 @@ EDA_DRAW_FRAME::~EDA_DRAW_FRAME()
     delete m_currentScreen;
     m_currentScreen = nullptr;
 
-    delete m_findReplaceData;
-
     m_auimgr.UnInit();
 
     ReleaseFile();
@@ -274,7 +284,7 @@ void EDA_DRAW_FRAME::ToggleUserUnits()
     }
     else
     {
-        SetUserUnits( m_userUnits == EDA_UNITS::INCHES ? EDA_UNITS::MILLIMETRES
+        SetUserUnits( GetUserUnits() == EDA_UNITS::INCHES ? EDA_UNITS::MILLIMETRES
                                                        : EDA_UNITS::INCHES );
         unitsChangeRefresh();
 
@@ -291,12 +301,41 @@ void EDA_DRAW_FRAME::CommonSettingsChanged( bool aEnvVarsChanged, bool aTextVars
     COMMON_SETTINGS*      settings = Pgm().GetCommonSettings();
     KIGFX::VIEW_CONTROLS* viewControls = GetCanvas()->GetViewControls();
 
-    SetAutoSaveInterval( settings->m_System.autosave_interval );
+    if( m_supportsAutoSave && m_autoSaveTimer->IsRunning() )
+    {
+        if( GetAutoSaveInterval() > 0 )
+        {
+            m_autoSaveTimer->Start( GetAutoSaveInterval() * 1000, wxTIMER_ONE_SHOT );
+        }
+        else
+        {
+            m_autoSaveTimer->Stop();
+            m_autoSavePending = false;
+        }
+    }
 
     viewControls->LoadSettings();
 
     m_galDisplayOptions.ReadCommonConfig( *settings, this );
 
+#ifndef __WXMAC__
+    resolveCanvasType();
+
+    if( m_canvasType != GetCanvas()->GetBackend() )
+    {
+        // Try to switch (will automatically fallback if necessary)
+        GetCanvas()->SwitchBackend( m_canvasType );
+        EDA_DRAW_PANEL_GAL::GAL_TYPE newGAL = GetCanvas()->GetBackend();
+        bool                         success = newGAL == m_canvasType;
+
+        if( !success )
+        {
+            m_canvasType = newGAL;
+            m_openGLFailureOccured = true; // Store failure for other EDA_DRAW_FRAMEs
+        }
+    }
+#endif
+
     // Notify all tools the preferences have changed
     if( m_toolManager )
         m_toolManager->RunAction( ACTIONS::updatePreferences, true );
@@ -322,6 +361,8 @@ void EDA_DRAW_FRAME::UpdateGridSelectBox()
     m_gridSelectBox->Clear();
     wxArrayString gridsList;
 
+    wxCHECK( config(), /* void */ );
+
     GRID_MENU::BuildChoiceList( &gridsList, config(), this );
 
     for( const wxString& grid : gridsList )
@@ -341,6 +382,8 @@ void EDA_DRAW_FRAME::OnUpdateSelectGrid( wxUpdateUIEvent& aEvent )
     if( m_gridSelectBox == nullptr )
         return;
 
+    wxCHECK( config(), /* void */ );
+
     int idx = config()->m_Window.grid.last_size_idx;
     idx = std::max( 0, std::min( idx, (int) m_gridSelectBox->GetCount() - 1 ) );
 
@@ -349,15 +392,49 @@ void EDA_DRAW_FRAME::OnUpdateSelectGrid( wxUpdateUIEvent& aEvent )
 }
 
 
+
+void EDA_DRAW_FRAME::OnUpdateSelectZoom( wxUpdateUIEvent& aEvent )
+{
+    // No need to update the grid select box if it doesn't exist or the grid setting change
+    // was made using the select box.
+    if( m_zoomSelectBox == nullptr )
+        return;
+
+    double zoom = GetCanvas()->GetGAL()->GetZoomFactor();
+
+    wxCHECK( config(), /* void */ );
+
+    const std::vector<double>& zoomList = config()->m_Window.zoom_factors;
+    int curr_selection = m_zoomSelectBox->GetSelection();
+    int new_selection = 0;      // select zoom auto
+    double last_approx = 1e9;   // large value to start calculation
+
+    // Search for the nearest available value to the current zoom setting, and select it
+    for( size_t jj = 0; jj < zoomList.size(); ++jj )
+    {
+        double rel_error = std::fabs( zoomList[jj] - zoom ) / zoom;
+
+        if( rel_error < last_approx )
+        {
+            last_approx = rel_error;
+            // zoom IDs in m_zoomSelectBox start with 1 (leaving 0 for auto-zoom choice)
+            new_selection = jj+1;
+        }
+    }
+
+    if( curr_selection != new_selection )
+        m_zoomSelectBox->SetSelection( new_selection );
+}
+
 void EDA_DRAW_FRAME::PrintPage( const RENDER_SETTINGS* aSettings )
 {
-    wxMessageBox( wxT("EDA_DRAW_FRAME::PrintPage() error") );
+    DisplayErrorMessage( this, wxT("EDA_DRAW_FRAME::PrintPage() error") );
 }
 
 
 void EDA_DRAW_FRAME::OnSelectGrid( wxCommandEvent& event )
 {
-    wxCHECK_RET( m_gridSelectBox, "m_gridSelectBox uninitialized" );
+    wxCHECK_RET( m_gridSelectBox, wxS( "m_gridSelectBox uninitialized" ) );
 
     int idx = m_gridSelectBox->GetCurrentSelection();
 
@@ -388,6 +465,9 @@ void EDA_DRAW_FRAME::OnSelectGrid( wxCommandEvent& event )
 
     UpdateStatusBar();
     m_canvas->Refresh();
+    // Needed on Windows because clicking on m_gridSelectBox remove the focus from m_canvas
+    // (Windows specific
+    m_canvas->SetFocus();
 }
 
 
@@ -405,12 +485,16 @@ void EDA_DRAW_FRAME::OnGridSettings( wxCommandEvent& aEvent )
 
 bool EDA_DRAW_FRAME::IsGridVisible() const
 {
+    wxCHECK( config(), true );
+
     return config()->m_Window.grid.show;
 }
 
 
 void EDA_DRAW_FRAME::SetGridVisibility( bool aVisible )
 {
+    wxCHECK( config(), /* void */ );
+
     config()->m_Window.grid.show = aVisible;
 
     // Update the display with the new grid
@@ -440,6 +524,8 @@ void EDA_DRAW_FRAME::UpdateZoomSelectBox()
     m_zoomSelectBox->Append( _( "Zoom Auto" ) );
     m_zoomSelectBox->SetSelection( 0 );
 
+    wxCHECK( config(), /* void */ );
+
     for( unsigned i = 0;  i < config()->m_Window.zoom_factors.size();  ++i )
     {
         double current = config()->m_Window.zoom_factors[i];
@@ -452,33 +538,9 @@ void EDA_DRAW_FRAME::UpdateZoomSelectBox()
 }
 
 
-void EDA_DRAW_FRAME::OnUpdateSelectZoom( wxUpdateUIEvent& aEvent )
-{
-    if( m_zoomSelectBox == nullptr || m_zoomSelectBox->GetParent() == nullptr )
-        return;
-
-    int current = 0;    // display Auto if no match found
-
-    // check for a match within 1%
-    double zoom = GetCanvas()->GetGAL()->GetZoomFactor();
-
-    for( unsigned i = 0; i < config()->m_Window.zoom_factors.size(); i++ )
-    {
-        if( std::fabs( zoom - config()->m_Window.zoom_factors[i] ) < ( zoom / 100.0 ) )
-        {
-            current = i + 1;
-            break;
-        }
-    }
-
-    if( current != m_zoomSelectBox->GetSelection() )
-        m_zoomSelectBox->SetSelection( current );
-}
-
-
 void EDA_DRAW_FRAME::OnSelectZoom( wxCommandEvent& event )
 {
-    wxCHECK_RET( m_zoomSelectBox, "m_zoomSelectBox uninitialized" );
+    wxCHECK_RET( m_zoomSelectBox, wxS( "m_zoomSelectBox uninitialized" ) );
 
     int id = m_zoomSelectBox->GetCurrentSelection();
 
@@ -488,6 +550,9 @@ void EDA_DRAW_FRAME::OnSelectZoom( wxCommandEvent& event )
     m_toolManager->RunAction( ACTIONS::zoomPreset, true, static_cast<intptr_t>( id ) );
     UpdateStatusBar();
     m_canvas->Refresh();
+    // Needed on Windows because clicking on m_zoomSelectBox remove the focus from m_canvas
+    // (Windows specific
+    m_canvas->SetFocus();
 }
 
 
@@ -515,13 +580,13 @@ void EDA_DRAW_FRAME::AddStandardSubMenus( TOOL_MENU& aToolMenu )
 
     aMenu.AddSeparator( 1000 );
 
-    auto zoomMenu = std::make_shared<ZOOM_MENU>( this );
+    std::shared_ptr<ZOOM_MENU> zoomMenu = std::make_shared<ZOOM_MENU>( this );
     zoomMenu->SetTool( commonTools );
-    aToolMenu.AddSubMenu( zoomMenu );
+    aToolMenu.RegisterSubMenu( zoomMenu );
 
-    auto gridMenu = std::make_shared<GRID_MENU>( this );
+    std::shared_ptr<GRID_MENU> gridMenu = std::make_shared<GRID_MENU>( this );
     gridMenu->SetTool( commonTools );
-    aToolMenu.AddSubMenu( gridMenu );
+    aToolMenu.RegisterSubMenu( gridMenu );
 
     aMenu.AddMenu( zoomMenu.get(), SELECTION_CONDITIONS::ShowAlways, 1000 );
     aMenu.AddMenu( gridMenu.get(), SELECTION_CONDITIONS::ShowAlways, 1000 );
@@ -542,13 +607,11 @@ void EDA_DRAW_FRAME::DisplayConstraintsMsg( const wxString& msg )
 
 void EDA_DRAW_FRAME::DisplayGridMsg()
 {
-    wxString line;
+    wxString msg;
 
-    line.Printf( "grid %s",
-                 MessageTextFromValue( GetUserUnits(), GetCanvas()->GetGAL()->GetGridSize().x,
-                                       false ) );
+    msg.Printf( wxS( "grid %s" ), MessageTextFromValue( GetCanvas()->GetGAL()->GetGridSize().x, false ) );
 
-    SetStatusText( line, 4 );
+    SetStatusText( msg, 4 );
 }
 
 
@@ -556,7 +619,7 @@ void EDA_DRAW_FRAME::DisplayUnitsMsg()
 {
     wxString msg;
 
-    switch( m_userUnits )
+    switch( GetUserUnits() )
     {
     case EDA_UNITS::INCHES:      msg = _( "inches" ); break;
     case EDA_UNITS::MILS:        msg = _( "mils" );   break;
@@ -610,18 +673,20 @@ void EDA_DRAW_FRAME::LoadSettings( APP_SETTINGS_BASE* aCfg )
     SetUserUnits( static_cast<EDA_UNITS>( aCfg->m_System.units ) );
 
     m_undoRedoCountMax = aCfg->m_System.max_undo_items;
-    m_firstRunDialogSetting = aCfg->m_System.first_run_shown;
 
     m_galDisplayOptions.ReadConfig( *cmnCfg, *window, this );
 
-    m_findReplaceData->SetFlags( aCfg->m_FindReplace.flags );
-    m_findReplaceData->SetFindString( aCfg->m_FindReplace.find_string );
-    m_findReplaceData->SetReplaceString( aCfg->m_FindReplace.replace_string );
+    m_findReplaceData->findString = aCfg->m_FindReplace.find_string;
+    m_findReplaceData->replaceString = aCfg->m_FindReplace.replace_string;
+    m_findReplaceData->matchMode =
+            static_cast<EDA_SEARCH_MATCH_MODE>( aCfg->m_FindReplace.match_mode );
+    m_findReplaceData->matchCase = aCfg->m_FindReplace.match_case;
+    m_findReplaceData->searchAndReplace = aCfg->m_FindReplace.search_and_replace;
 
-    for( auto& s : aCfg->m_FindReplace.find_history )
+    for( const wxString& s : aCfg->m_FindReplace.find_history )
         m_findStringHistoryList.Add( s );
 
-    for( auto& s : aCfg->m_FindReplace.replace_history )
+    for( const wxString& s : aCfg->m_FindReplace.replace_history )
         m_replaceStringHistoryList.Add( s );
 }
 
@@ -632,15 +697,15 @@ void EDA_DRAW_FRAME::SaveSettings( APP_SETTINGS_BASE* aCfg )
 
     WINDOW_SETTINGS* window = GetWindowSettings( aCfg );
 
-    aCfg->m_System.units = static_cast<int>( m_userUnits );
-    aCfg->m_System.first_run_shown = m_firstRunDialogSetting;
+    aCfg->m_System.units = static_cast<int>( GetUserUnits() );
     aCfg->m_System.max_undo_items = GetMaxUndoItems();
 
     m_galDisplayOptions.WriteConfig( *window );
 
-    aCfg->m_FindReplace.flags = m_findReplaceData->GetFlags();
-    aCfg->m_FindReplace.find_string = m_findReplaceData->GetFindString();
-    aCfg->m_FindReplace.replace_string = m_findReplaceData->GetReplaceString();
+    aCfg->m_FindReplace.search_and_replace = m_findReplaceData->searchAndReplace;
+
+    aCfg->m_FindReplace.find_string = m_findReplaceData->findString;
+    aCfg->m_FindReplace.replace_string = m_findReplaceData->replaceString;
 
     aCfg->m_FindReplace.find_history.clear();
     aCfg->m_FindReplace.replace_history.clear();
@@ -719,7 +784,6 @@ void EDA_DRAW_FRAME::SetMsgPanel( EDA_ITEM* aItem )
 
 void EDA_DRAW_FRAME::UpdateMsgPanel()
 {
-    GetToolManager()->PostEvent( EVENTS::SelectedItemsModified );
 }
 
 
@@ -811,9 +875,9 @@ bool EDA_DRAW_FRAME::saveCanvasTypeSetting( EDA_DRAW_PANEL_GAL::GAL_TYPE aCanvas
 }
 
 
-wxPoint EDA_DRAW_FRAME::GetNearestGridPosition( const wxPoint& aPosition ) const
+VECTOR2I EDA_DRAW_FRAME::GetNearestGridPosition( const VECTOR2I& aPosition ) const
 {
-    const wxPoint& gridOrigin = GetGridOrigin();
+    const VECTOR2I& gridOrigin = GetGridOrigin();
     VECTOR2D       gridSize = GetCanvas()->GetGAL()->GetGridSize();
 
     double xOffset = fmod( gridOrigin.x, gridSize.x );
@@ -821,13 +885,13 @@ wxPoint EDA_DRAW_FRAME::GetNearestGridPosition( const wxPoint& aPosition ) const
     double yOffset = fmod( gridOrigin.y, gridSize.y );
     int    y = KiROUND( (aPosition.y - yOffset) / gridSize.y );
 
-    return wxPoint( KiROUND( x * gridSize.x + xOffset ), KiROUND( y * gridSize.y + yOffset ) );
+    return VECTOR2I( KiROUND( x * gridSize.x + xOffset ), KiROUND( y * gridSize.y + yOffset ) );
 }
 
 
-wxPoint EDA_DRAW_FRAME::GetNearestHalfGridPosition( const wxPoint& aPosition ) const
+VECTOR2I EDA_DRAW_FRAME::GetNearestHalfGridPosition( const VECTOR2I& aPosition ) const
 {
-    const wxPoint& gridOrigin = GetGridOrigin();
+    const VECTOR2I& gridOrigin = GetGridOrigin();
     VECTOR2D       gridSize = GetCanvas()->GetGAL()->GetGridSize() / 2.0;
 
     double xOffset = fmod( gridOrigin.x, gridSize.x );
@@ -835,7 +899,7 @@ wxPoint EDA_DRAW_FRAME::GetNearestHalfGridPosition( const wxPoint& aPosition ) c
     double yOffset = fmod( gridOrigin.y, gridSize.y );
     int    y = KiROUND( (aPosition.y - yOffset) / gridSize.y );
 
-    return wxPoint( KiROUND( x * gridSize.x + xOffset ), KiROUND( y * gridSize.y + yOffset ) );
+    return VECTOR2I( KiROUND( x * gridSize.x + xOffset ), KiROUND( y * gridSize.y + yOffset ) );
 }
 
 
@@ -872,7 +936,7 @@ std::vector<wxWindow*> EDA_DRAW_FRAME::findDialogs()
 }
 
 
-void EDA_DRAW_FRAME::FocusOnLocation( const wxPoint& aPos )
+void EDA_DRAW_FRAME::FocusOnLocation( const VECTOR2I& aPos )
 {
     bool  centerView = false;
     BOX2D r = GetCanvas()->GetView()->GetViewport();
@@ -922,7 +986,8 @@ static const wxString productName = wxT( "KiCad E.D.A.  " );
 
 void PrintDrawingSheet( const RENDER_SETTINGS* aSettings, const PAGE_INFO& aPageInfo,
                         const wxString& aFullSheetName, const wxString& aFileName,
-                        const TITLE_BLOCK& aTitleBlock, int aSheetCount,
+                        const TITLE_BLOCK& aTitleBlock,
+                        const std::map<wxString, wxString>* aProperties, int aSheetCount,
                         const wxString& aPageNumber, double aMils2Iu, const PROJECT* aProject,
                         const wxString& aSheetLayer, bool aIsFirstPage )
 {
@@ -937,6 +1002,7 @@ void PrintDrawingSheet( const RENDER_SETTINGS* aSettings, const PAGE_INFO& aPage
     drawList.SetSheetLayer( aSheetLayer );
     drawList.SetProject( aProject );
     drawList.SetIsFirstPage( aIsFirstPage );
+    drawList.SetProperties( aProperties );
 
     drawList.BuildDrawItemsList( aPageInfo, aTitleBlock );
 
@@ -946,6 +1012,7 @@ void PrintDrawingSheet( const RENDER_SETTINGS* aSettings, const PAGE_INFO& aPage
 
 
 void EDA_DRAW_FRAME::PrintDrawingSheet( const RENDER_SETTINGS* aSettings, BASE_SCREEN* aScreen,
+                                        const std::map<wxString, wxString>* aProperties,
                                         double aMils2Iu, const wxString &aFilename,
                                         const wxString &aSheetLayer )
 {
@@ -962,8 +1029,8 @@ void EDA_DRAW_FRAME::PrintDrawingSheet( const RENDER_SETTINGS* aSettings, BASE_S
     }
 
     ::PrintDrawingSheet( aSettings, GetPageSettings(), GetScreenDesc(), aFilename, GetTitleBlock(),
-                         aScreen->GetPageCount(), aScreen->GetPageNumber(), aMils2Iu, &Prj(),
-                         aSheetLayer, aScreen->GetVirtualPageNumber() == 1 );
+                         aProperties, aScreen->GetPageCount(), aScreen->GetPageNumber(), aMils2Iu,
+                         &Prj(), aSheetLayer, aScreen->GetVirtualPageNumber() == 1 );
 
     if( origin.y > 0 )
     {
@@ -988,22 +1055,27 @@ bool EDA_DRAW_FRAME::LibraryFileBrowser( bool doOpen, wxFileName& aFilename,
     wxString prompt = doOpen ? _( "Select Library" ) : _( "New Library" );
     aFilename.SetExt( ext );
 
-    wxString dir;
+    wxString projectDir = Prj().IsNullProject() ? aFilename.GetPath() : Prj().GetProjectPath();
+    wxString defaultDir;
 
-    if( GetMruPath().IsEmpty() )
-        dir = aGlobalPath;
+    if( aIsGlobal )
+    {
+        if( !GetMruPath().IsEmpty() && !GetMruPath().StartsWith( projectDir ) )
+            defaultDir = GetMruPath();
+        else
+            defaultDir = aGlobalPath;
+    }
     else
-        dir = GetMruPath();
-
+    {
+        if( !GetMruPath().IsEmpty() && GetMruPath().StartsWith( projectDir ) )
+            defaultDir = GetMruPath();
+        else
+            defaultDir = projectDir;
+    }
 
     if( isDirectory && doOpen )
     {
-        if( !aIsGlobal && GetMruPath().IsEmpty() )
-        {
-            dir = Prj().GetProjectPath();
-        }
-
-        wxDirDialog dlg( this, prompt, dir, wxDD_DEFAULT_STYLE | wxDD_DIR_MUST_EXIST );
+        wxDirDialog dlg( this, prompt, defaultDir, wxDD_DEFAULT_STYLE | wxDD_DIR_MUST_EXIST );
 
         if( dlg.ShowModal() == wxID_CANCEL )
             return false;
@@ -1015,14 +1087,9 @@ bool EDA_DRAW_FRAME::LibraryFileBrowser( bool doOpen, wxFileName& aFilename,
     {
         // Ensure the file has a dummy name, otherwise GTK will display the regex from the filter
         if( aFilename.GetName().empty() )
-            aFilename.SetName( "Library" );
+            aFilename.SetName( wxS( "Library" ) );
 
-        if( !aIsGlobal && GetMruPath().IsEmpty() )
-        {
-            dir = Prj().IsNullProject() ? aFilename.GetFullPath() : Prj().GetProjectPath();
-        }
-
-        wxFileDialog dlg( this, prompt, dir, aFilename.GetFullName(),
+        wxFileDialog dlg( this, prompt, defaultDir, aFilename.GetFullName(),
                           wildcard, doOpen ? wxFD_OPEN | wxFD_FILE_MUST_EXIST
                                            : wxFD_SAVE | wxFD_CHANGE_DIR | wxFD_OVERWRITE_PROMPT );
 
@@ -1056,9 +1123,20 @@ void EDA_DRAW_FRAME::RecreateToolbars()
 }
 
 
-COLOR_SETTINGS* EDA_DRAW_FRAME::GetColorSettings() const
+void EDA_DRAW_FRAME::ShowChangedLanguage()
 {
-    if( !m_colorSettings )
+    EDA_BASE_FRAME::ShowChangedLanguage();
+
+    if( m_searchPane )
+    {
+        m_searchPane->OnLanguageChange();
+    }
+}
+
+
+COLOR_SETTINGS* EDA_DRAW_FRAME::GetColorSettings( bool aForceRefresh ) const
+{
+    if( !m_colorSettings || aForceRefresh )
     {
         COLOR_SETTINGS* colorSettings = Pgm().GetSettingsManager().GetColorSettings();
 
@@ -1119,27 +1197,11 @@ void EDA_DRAW_FRAME::resolveCanvasType()
 {
     m_canvasType = loadCanvasTypeSetting();
 
-    // Nudge user to switch to OpenGL if they are on legacy or Cairo
-    if( m_firstRunDialogSetting < 1 )
-    {
-        if( m_canvasType != EDA_DRAW_PANEL_GAL::GAL_TYPE_OPENGL )
-        {
-            // Save Cairo as default in case OpenGL crashes
-            saveCanvasTypeSetting( EDA_DRAW_PANEL_GAL::GAL_TYPE_CAIRO );
+    // If we had an OpenGL failure this session, use the fallback GAL but don't update the
+    // user preference silently:
 
-            // Switch to OpenGL, which will save the new setting if successful
-            SwitchCanvas( EDA_DRAW_PANEL_GAL::GAL_TYPE_OPENGL );
-
-            // Switch back to Cairo if OpenGL is not supported
-            if( GetCanvas()->GetBackend() == EDA_DRAW_PANEL_GAL::GAL_TYPE_NONE )
-                SwitchCanvas( EDA_DRAW_PANEL_GAL::GAL_TYPE_CAIRO );
-
-            HardRedraw();
-        }
-
-        m_firstRunDialogSetting = 1;
-        SaveSettings( config() );
-    }
+    if( m_openGLFailureOccured && m_canvasType == EDA_DRAW_PANEL_GAL::GAL_TYPE_OPENGL )
+        m_canvasType = EDA_DRAW_PANEL_GAL::GAL_FALLBACK;
 }
 
 
