@@ -55,8 +55,6 @@ SETTINGS_MANAGER::SETTINGS_MANAGER( bool aHeadless ) :
         m_migration_source(),
         m_migrateLibraryTables( true )
 {
-    PATHS::EnsureUserPathsExist();
-
     // Check if the settings directory already exists, and if not, perform a migration if possible
     if( !MigrateIfNeeded() )
     {
@@ -72,9 +70,17 @@ SETTINGS_MANAGER::SETTINGS_MANAGER( bool aHeadless ) :
 
 SETTINGS_MANAGER::~SETTINGS_MANAGER()
 {
-    m_settings.clear();
-    m_color_settings.clear();
+    for( std::unique_ptr<PROJECT>& project : m_projects_list )
+        project.reset();
+
     m_projects.clear();
+
+    for( std::unique_ptr<JSON_SETTINGS>& settings : m_settings )
+        settings.reset();
+
+    m_settings.clear();
+
+    m_color_settings.clear();
 }
 
 
@@ -84,7 +90,7 @@ JSON_SETTINGS* SETTINGS_MANAGER::registerSettings( JSON_SETTINGS* aSettings, boo
 
     ptr->SetManager( this );
 
-    wxLogTrace( traceSettings, "Registered new settings object <%s>", ptr->GetFullFilename() );
+    wxLogTrace( traceSettings, wxT( "Registered new settings object <%s>" ), ptr->GetFullFilename() );
 
     if( aLoadNow )
         ptr->LoadFromFile( GetPathForSettingsFile( ptr.get() ) );
@@ -140,7 +146,7 @@ void SETTINGS_MANAGER::Save( JSON_SETTINGS* aSettings )
 
     if( it != m_settings.end() )
     {
-        wxLogTrace( traceSettings, "Saving %s", ( *it )->GetFullFilename() );
+        wxLogTrace( traceSettings, wxT( "Saving %s" ), ( *it )->GetFullFilename() );
         ( *it )->SaveToFile( GetPathForSettingsFile( it->get() ) );
     }
 }
@@ -156,7 +162,7 @@ void SETTINGS_MANAGER::FlushAndRelease( JSON_SETTINGS* aSettings, bool aSave )
 
     if( it != m_settings.end() )
     {
-        wxLogTrace( traceSettings, "Flush and release %s", ( *it )->GetFullFilename() );
+        wxLogTrace( traceSettings, wxT( "Flush and release %s" ), ( *it )->GetFullFilename() );
 
         if( aSave )
             ( *it )->SaveToFile( GetPathForSettingsFile( it->get() ) );
@@ -173,9 +179,23 @@ void SETTINGS_MANAGER::FlushAndRelease( JSON_SETTINGS* aSettings, bool aSave )
 
 COLOR_SETTINGS* SETTINGS_MANAGER::GetColorSettings( const wxString& aName )
 {
+    // Find settings the fast way
     if( m_color_settings.count( aName ) )
         return m_color_settings.at( aName );
 
+    // Maybe it's the display name (cli is one method of invoke)
+    auto it = std::find_if( m_color_settings.begin(), m_color_settings.end(),
+                            [&aName]( const std::pair<wxString, COLOR_SETTINGS*>& p )
+                            {
+                                return p.second->GetName().Lower() == aName.Lower();
+                            } );
+
+    if( it != m_color_settings.end() )
+    {
+        return it->second;
+    }
+
+    // No match? See if we can load it
     if( !aName.empty() )
     {
         COLOR_SETTINGS* ret = loadColorSettingsByName( aName );
@@ -198,13 +218,13 @@ COLOR_SETTINGS* SETTINGS_MANAGER::GetColorSettings( const wxString& aName )
 
 COLOR_SETTINGS* SETTINGS_MANAGER::loadColorSettingsByName( const wxString& aName )
 {
-    wxLogTrace( traceSettings, "Attempting to load color theme %s", aName );
+    wxLogTrace( traceSettings, wxT( "Attempting to load color theme %s" ), aName );
 
-    wxFileName fn( GetColorSettingsPath(), aName, "json" );
+    wxFileName fn( GetColorSettingsPath(), aName, wxS( "json" ) );
 
     if( !fn.IsOk() || !fn.Exists() )
     {
-        wxLogTrace( traceSettings, "Theme file %s.json not found, falling back to user", aName );
+        wxLogTrace( traceSettings, wxT( "Theme file %s.json not found, falling back to user" ), aName );
         return nullptr;
     }
 
@@ -212,7 +232,7 @@ COLOR_SETTINGS* SETTINGS_MANAGER::loadColorSettingsByName( const wxString& aName
 
     if( settings->GetFilename() != aName.ToStdString() )
     {
-        wxLogTrace( traceSettings, "Warning: stored filename is actually %s, ",
+        wxLogTrace( traceSettings, wxT( "Warning: stored filename is actually %s, " ),
                     settings->GetFilename() );
     }
 
@@ -237,7 +257,7 @@ public:
     {
         wxFileName file( aFilePath );
 
-        if( file.GetExt() == "json" )
+        if( file.GetExt() == wxS( "json" ) )
             m_action( file );
 
         return wxDIR_CONTINUE;
@@ -293,14 +313,14 @@ void SETTINGS_MANAGER::loadAllColorSettings()
 
     wxFileName third_party_path;
     const ENV_VAR_MAP& env = Pgm().GetLocalEnvVariables();
-    auto               it = env.find( "KICAD6_3RD_PARTY" );
+    auto               it = env.find( wxS( "KICAD7_3RD_PARTY" ) );
 
     if( it != env.end() && !it->second.GetValue().IsEmpty() )
         third_party_path.SetPath( it->second.GetValue() );
     else
         third_party_path.SetPath( PATHS::GetDefault3rdPartyPath() );
 
-    third_party_path.AppendDir( "colors" );
+    third_party_path.AppendDir( wxS( "colors" ) );
 
     wxDir third_party_colors_dir( third_party_path.GetFullPath() );
     wxString color_settings_path = GetColorSettingsPath();
@@ -352,18 +372,18 @@ void SETTINGS_MANAGER::SaveColorSettings( COLOR_SETTINGS* aSettings, const std::
 
     if( !aSettings->Store() )
     {
-        wxLogTrace( traceSettings, "Color scheme %s not modified; skipping save",
+        wxLogTrace( traceSettings, wxT( "Color scheme %s not modified; skipping save" ),
                     aNamespace );
         return;
     }
 
     wxASSERT( aSettings->Contains( aNamespace ) );
 
-    wxLogTrace( traceSettings, "Saving color scheme %s, preserving %s",
+    wxLogTrace( traceSettings, wxT( "Saving color scheme %s, preserving %s" ),
                 aSettings->GetFilename(),
                 aNamespace );
 
-    OPT<nlohmann::json> backup = aSettings->GetJson( aNamespace );
+    std::optional<nlohmann::json> backup = aSettings->GetJson( aNamespace );
     wxString path = GetColorSettingsPath();
 
     aSettings->LoadFromFile( path );
@@ -387,6 +407,7 @@ wxString SETTINGS_MANAGER::GetPathForSettingsFile( JSON_SETTINGS* aSettings )
         return GetUserSettingsPath();
 
     case SETTINGS_LOC::PROJECT:
+        // TODO: MDI support
         return Prj().GetProjectPath();
 
     case SETTINGS_LOC::COLORS:
@@ -396,7 +417,7 @@ wxString SETTINGS_MANAGER::GetPathForSettingsFile( JSON_SETTINGS* aSettings )
         return "";
 
     default:
-        wxASSERT_MSG( false, "Unknown settings location!" );
+        wxASSERT_MSG( false, wxT( "Unknown settings location!" ) );
     }
 
     return "";
@@ -445,7 +466,7 @@ public:
         path.Replace( m_src, m_dest, false );
         file.SetPath( path );
 
-        wxLogTrace( traceSettings, "Copying %s to %s", aSrcFilePath, file.GetFullPath() );
+        wxLogTrace( traceSettings, wxT( "Copying %s to %s" ), aSrcFilePath, file.GetFullPath() );
 
         // For now, just copy everything
         KiCopyFile( aSrcFilePath, file.GetFullPath(), m_errors );
@@ -458,8 +479,8 @@ public:
         wxFileName dir( dirPath );
 
         // Whitelist of directories to migrate
-        if( dir.GetName() == "colors" ||
-            dir.GetName() == "3d" )
+        if( dir.GetName() == wxS( "colors" ) ||
+            dir.GetName() == wxS( "3d" ) )
         {
 
             wxString path = dir.GetPath();
@@ -481,24 +502,33 @@ public:
 
 bool SETTINGS_MANAGER::MigrateIfNeeded()
 {
+    wxFileName path( GetUserSettingsPath(), wxS( "" ) );
+    wxLogTrace( traceSettings, wxT( "Using settings path %s" ), path.GetFullPath() );
+
     if( m_headless )
     {
-        wxLogTrace( traceSettings, "Settings migration not checked; running headless" );
-        return false;
-    }
+        // Special case namely for cli
+        // Ensure the settings directory at least exists to prevent additional loading errors from subdirectories
+        // TODO review headless (unit tests) vs cli needs, this should be fine for unit tests though
+        if( !path.DirExists() )
+        {
+            wxLogTrace( traceSettings, wxT( "Path didn't exist; creating it" ) );
+            path.Mkdir( wxS_DIR_DEFAULT, wxPATH_MKDIR_FULL );
+        }
 
-    wxFileName path( GetUserSettingsPath(), "" );
-    wxLogTrace( traceSettings, "Using settings path %s", path.GetFullPath() );
+        wxLogTrace( traceSettings, wxT( "Settings migration not checked; running headless" ) );
+        return true;
+    }
 
     if( path.DirExists() )
     {
         wxFileName common = path;
-        common.SetName( "kicad_common" );
-        common.SetExt( "json" );
+        common.SetName( wxS( "kicad_common" ) );
+        common.SetExt( wxS( "json" ) );
 
         if( common.Exists() )
         {
-            wxLogTrace( traceSettings, "Path exists and has a kicad_common, continuing!" );
+            wxLogTrace( traceSettings, wxT( "Path exists and has a kicad_common, continuing!" ) );
             return true;
         }
     }
@@ -508,23 +538,23 @@ bool SETTINGS_MANAGER::MigrateIfNeeded()
 
     if( dlg.ShowModal() != wxID_OK )
     {
-        wxLogTrace( traceSettings, "Migration dialog canceled; exiting" );
+        wxLogTrace( traceSettings, wxT( "Migration dialog canceled; exiting" ) );
         return false;
     }
 
     if( !path.DirExists() )
     {
-        wxLogTrace( traceSettings, "Path didn't exist; creating it" );
+        wxLogTrace( traceSettings, wxT( "Path didn't exist; creating it" ) );
         path.Mkdir( wxS_DIR_DEFAULT, wxPATH_MKDIR_FULL );
     }
 
     if( m_migration_source.IsEmpty() )
     {
-        wxLogTrace( traceSettings, "No migration source given; starting with defaults" );
+        wxLogTrace( traceSettings, wxT( "No migration source given; starting with defaults" ) );
         return true;
     }
 
-    wxLogTrace( traceSettings, "Migrating from path %s", m_migration_source );
+    wxLogTrace( traceSettings, wxT( "Migrating from path %s" ), m_migration_source );
 
     MIGRATION_TRAVERSER traverser( m_migration_source, path.GetFullPath(), m_migrateLibraryTables );
     wxDir source_dir( m_migration_source );
@@ -546,6 +576,10 @@ bool SETTINGS_MANAGER::MigrateIfNeeded()
             wxT( "KICAD6_3DMODEL_DIR" ),
             wxT( "KICAD6_FOOTPRINT_DIR" ),
             wxT( "KICAD6_TEMPLATE_DIR" ), // Stores the default library table to be copied
+            wxT( "KICAD7_SYMBOL_DIR" ),
+            wxT( "KICAD7_3DMODEL_DIR" ),
+            wxT( "KICAD7_FOOTPRINT_DIR" ),
+            wxT( "KICAD7_TEMPLATE_DIR" ), // Stores the default library table to be copied
 
             // Deprecated keys
             wxT( "KICAD_PTEMPLATES" ),
@@ -573,11 +607,11 @@ bool SETTINGS_MANAGER::GetPreviousVersionPaths( std::vector<wxString>* aPaths )
     wxDir dir;
     std::vector<wxFileName> base_paths;
 
-    base_paths.emplace_back( wxFileName( calculateUserSettingsPath( false ), "" ) );
+    base_paths.emplace_back( wxFileName( calculateUserSettingsPath( false ), wxS( "" ) ) );
 
     // If the env override is set, also check the default paths
     if( wxGetEnv( wxT( "KICAD_CONFIG_HOME" ), nullptr ) )
-        base_paths.emplace_back( wxFileName( calculateUserSettingsPath( false, false ), "" ) );
+        base_paths.emplace_back( wxFileName( calculateUserSettingsPath( false, false ), wxS( "" ) ) );
 
 #ifdef __WXGTK__
     // When running inside FlatPak, KIPLATFORM::ENV::GetUserConfigPath() will return a sandboxed
@@ -587,12 +621,12 @@ bool SETTINGS_MANAGER::GetPreviousVersionPaths( std::vector<wxString>* aPaths )
     // for it.
     {
         wxFileName wxGtkPath;
-        wxGtkPath.AssignDir( "~/.config/kicad" );
+        wxGtkPath.AssignDir( wxS( "~/.config/kicad" ) );
         wxGtkPath.MakeAbsolute();
         base_paths.emplace_back( wxGtkPath.GetPath() );
 
         // We also want to pick up regular flatpak if we are nightly
-        wxGtkPath.AssignDir( "~/.var/app/org.kicad.KiCad/config/kicad" );
+        wxGtkPath.AssignDir( wxS( "~/.var/app/org.kicad.KiCad/config/kicad" ) );
         wxGtkPath.MakeAbsolute();
         base_paths.emplace_back( wxGtkPath.GetPath() );
     }
@@ -611,14 +645,14 @@ bool SETTINGS_MANAGER::GetPreviousVersionPaths( std::vector<wxString>* aPaths )
             if( IsSettingsPathValid( sub_path ) )
             {
                 aPaths->push_back( sub_path );
-                wxLogTrace( traceSettings, "GetPreviousVersionName: %s is valid", sub_path );
+                wxLogTrace( traceSettings, wxT( "GetPreviousVersionName: %s is valid" ), sub_path );
             }
         }
     };
 
     std::set<wxString> checkedPaths;
 
-    for( auto base_path : base_paths )
+    for( const wxFileName& base_path : base_paths )
     {
         if( checkedPaths.count( base_path.GetFullPath() ) )
             continue;
@@ -627,12 +661,12 @@ bool SETTINGS_MANAGER::GetPreviousVersionPaths( std::vector<wxString>* aPaths )
 
         if( !dir.Open( base_path.GetFullPath() ) )
         {
-            wxLogTrace( traceSettings, "GetPreviousVersionName: could not open base path %s",
+            wxLogTrace( traceSettings, wxT( "GetPreviousVersionName: could not open base path %s" ),
                         base_path.GetFullPath() );
             continue;
         }
 
-        wxLogTrace( traceSettings, "GetPreviousVersionName: checking base path %s",
+        wxLogTrace( traceSettings, wxT( "GetPreviousVersionName: checking base path %s" ),
                     base_path.GetFullPath() );
 
         if( dir.GetFirst( &subdir, wxEmptyString, wxDIR_DIRS ) )
@@ -651,18 +685,33 @@ bool SETTINGS_MANAGER::GetPreviousVersionPaths( std::vector<wxString>* aPaths )
         if( IsSettingsPathValid( dir.GetNameWithSep() ) )
         {
             wxLogTrace( traceSettings,
-                        "GetPreviousVersionName: root path %s is valid", dir.GetName() );
+                        wxT( "GetPreviousVersionName: root path %s is valid" ), dir.GetName() );
             aPaths->push_back( dir.GetName() );
         }
     }
 
+    std::sort( aPaths->begin(), aPaths->end(),
+               [&]( const wxString& a, const wxString& b ) -> bool
+               {
+                   wxString verA = wxFileName::DirName( a ).GetDirs().back();
+                   wxString verB = wxFileName::DirName( b ).GetDirs().back();
+
+                   if( !extractVersion( verA.ToStdString() )
+                       || !extractVersion( verB.ToStdString() ) )
+                   {
+                       return false;
+                   }
+
+                   return compareVersions( verA.ToStdString(), verB.ToStdString() ) >= 0;
+               } );
+
     return aPaths->size() > 0;
 }
 
 
 bool SETTINGS_MANAGER::IsSettingsPathValid( const wxString& aPath )
 {
-    wxFileName test( aPath, "kicad_common" );
+    wxFileName test( aPath, wxS( "kicad_common" ) );
 
     if( test.Exists() )
         return true;
@@ -678,14 +727,14 @@ wxString SETTINGS_MANAGER::GetColorSettingsPath()
     wxFileName path;
 
     path.AssignDir( GetUserSettingsPath() );
-    path.AppendDir( "colors" );
+    path.AppendDir( wxS( "colors" ) );
 
     if( !path.DirExists() )
     {
         if( !wxMkdir( path.GetPath() ) )
         {
             wxLogTrace( traceSettings,
-                        "GetColorSettingsPath(): Path %s missing and could not be created!",
+                        wxT( "GetColorSettingsPath(): Path %s missing and could not be created!" ),
                         path.GetPath() );
         }
     }
@@ -747,7 +796,7 @@ int SETTINGS_MANAGER::compareVersions( const std::string& aFirst, const std::str
 
     if( !extractVersion( aFirst, &a_maj, &a_min ) || !extractVersion( aSecond, &b_maj, &b_min ) )
     {
-        wxLogTrace( traceSettings, "compareSettingsVersions: bad input (%s, %s)", aFirst, aSecond );
+        wxLogTrace( traceSettings, wxT( "compareSettingsVersions: bad input (%s, %s)" ), aFirst, aSecond );
         return -1;
     }
 
@@ -786,8 +835,14 @@ bool SETTINGS_MANAGER::extractVersion( const std::string& aVersionString, int* a
     {
         try
         {
-            *aMajor = std::stoi( match[1].str() );
-            *aMinor = std::stoi( match[2].str() );
+            int major = std::stoi( match[1].str() );
+            int minor = std::stoi( match[2].str() );
+
+            if( aMajor )
+                *aMajor = major;
+
+            if( aMinor )
+                *aMinor = minor;
         }
         catch( ... )
         {
@@ -822,7 +877,7 @@ bool SETTINGS_MANAGER::LoadProject( const wxString& aFullPath, bool aSetActive )
 
     if( !lockFile )
     {
-        wxLogTrace( traceSettings, "Project %s is locked; opening read-only", fullPath );
+        wxLogTrace( traceSettings, wxT( "Project %s is locked; opening read-only" ), fullPath );
         readOnly = true;
     }
 
@@ -843,7 +898,7 @@ bool SETTINGS_MANAGER::LoadProject( const wxString& aFullPath, bool aSetActive )
         m_projects_list.erase( it );
     }
 
-    wxLogTrace( traceSettings, "Load project %s", fullPath );
+    wxLogTrace( traceSettings, wxT( "Load project %s" ), fullPath );
 
     std::unique_ptr<PROJECT> project = std::make_unique<PROJECT>();
     project->setProjectFullName( fullPath );
@@ -888,7 +943,7 @@ bool SETTINGS_MANAGER::UnloadProject( PROJECT* aProject, bool aSave )
         return false;
 
     wxString projectPath = aProject->GetProjectFullName();
-    wxLogTrace( traceSettings, "Unload project %s", projectPath );
+    wxLogTrace( traceSettings, wxT( "Unload project %s" ), projectPath );
 
     PROJECT* toRemove = m_projects.at( projectPath );
     auto it = std::find_if( m_projects_list.begin(), m_projects_list.end(),
@@ -908,7 +963,7 @@ bool SETTINGS_MANAGER::UnloadProject( PROJECT* aProject, bool aSave )
         LoadProject( "" );
 
     // Remove the reference in the environment to the previous project
-    wxSetEnv( PROJECT_VAR_NAME, "" );
+    wxSetEnv( PROJECT_VAR_NAME, wxS( "" ) );
 
     // Release lock on the file, in case we had one
     m_project_lock = nullptr;
@@ -923,7 +978,7 @@ bool SETTINGS_MANAGER::UnloadProject( PROJECT* aProject, bool aSave )
 PROJECT& SETTINGS_MANAGER::Prj() const
 {
     // No MDI yet:  First project in the list is the active project
-    wxASSERT_MSG( m_projects_list.size(), "no project in list" );
+    wxASSERT_MSG( m_projects_list.size(), wxT( "no project in list" ) );
     return *m_projects_list.begin()->get();
 }
 
@@ -954,42 +1009,48 @@ std::vector<wxString> SETTINGS_MANAGER::GetOpenProjects() const
 }
 
 
-bool SETTINGS_MANAGER::SaveProject( const wxString& aFullPath )
+bool SETTINGS_MANAGER::SaveProject( const wxString& aFullPath, PROJECT* aProject )
 {
+    if( !aProject )
+        aProject = &Prj();
+
     wxString path = aFullPath;
 
     if( path.empty() )
-        path = Prj().GetProjectFullName();
+        path = aProject->GetProjectFullName();
 
     // TODO: refactor for MDI
-    if( Prj().IsReadOnly() )
+    if( aProject->IsReadOnly() )
         return false;
 
     if( !m_project_files.count( path ) )
         return false;
 
     PROJECT_FILE* project     = m_project_files.at( path );
-    wxString      projectPath = GetPathForSettingsFile( project );
+    wxString      projectPath = aProject->GetProjectPath();
 
     project->SaveToFile( projectPath );
-    Prj().GetLocalSettings().SaveToFile( projectPath );
+    aProject->GetLocalSettings().SaveToFile( projectPath );
 
     return true;
 }
 
 
-void SETTINGS_MANAGER::SaveProjectAs( const wxString& aFullPath )
+void SETTINGS_MANAGER::SaveProjectAs( const wxString& aFullPath, PROJECT* aProject )
 {
-    wxString oldName = Prj().GetProjectFullName();
+    if( !aProject )
+        aProject = &Prj();
+
+    wxString oldName = aProject->GetProjectFullName();
 
     if( aFullPath.IsSameAs( oldName ) )
     {
-        SaveProject( aFullPath );
+        SaveProject( aFullPath, aProject );
         return;
     }
 
     // Changing this will cause UnloadProject to not save over the "old" project when loading below
-    Prj().setProjectFullName( aFullPath );
+    aProject->setProjectFullName( aFullPath );
 
     wxFileName fn( aFullPath );
 
@@ -997,14 +1058,14 @@ void SETTINGS_MANAGER::SaveProjectAs( const wxString& aFullPath )
 
     // Ensure read-only flags are copied; this allows doing a "Save As" on a standalong board/sch
     // without creating project files if the checkbox is turned off
-    project->SetReadOnly( Prj().IsReadOnly() );
-    Prj().GetLocalSettings().SetReadOnly( Prj().IsReadOnly() );
+    project->SetReadOnly( aProject->IsReadOnly() );
+    aProject->GetLocalSettings().SetReadOnly( aProject->IsReadOnly() );
 
     project->SetFilename( fn.GetName() );
     project->SaveToFile( fn.GetPath() );
 
-    Prj().GetLocalSettings().SetFilename( fn.GetName() );
-    Prj().GetLocalSettings().SaveToFile( fn.GetPath() );
+    aProject->GetLocalSettings().SetFilename( fn.GetName() );
+    aProject->GetLocalSettings().SaveToFile( fn.GetPath() );
 
     m_project_files[fn.GetFullPath()] = project;
     m_project_files.erase( oldName );
@@ -1014,9 +1075,12 @@ void SETTINGS_MANAGER::SaveProjectAs( const wxString& aFullPath )
 }
 
 
-void SETTINGS_MANAGER::SaveProjectCopy( const wxString& aFullPath )
+void SETTINGS_MANAGER::SaveProjectCopy( const wxString& aFullPath, PROJECT* aProject )
 {
-    PROJECT_FILE* project = m_project_files.at( Prj().GetProjectFullName() );
+    if( !aProject )
+        aProject = &Prj();
+
+    PROJECT_FILE* project = m_project_files.at( aProject->GetProjectFullName() );
     wxString      oldName = project->GetFilename();
     wxFileName    fn( aFullPath );
 
@@ -1027,9 +1091,11 @@ void SETTINGS_MANAGER::SaveProjectCopy( const wxString& aFullPath )
     project->SaveToFile( fn.GetPath() );
     project->SetFilename( oldName );
 
-    Prj().GetLocalSettings().SetFilename( fn.GetName() );
-    Prj().GetLocalSettings().SaveToFile( fn.GetPath() );
-    Prj().GetLocalSettings().SetFilename( oldName );
+    PROJECT_LOCAL_SETTINGS& localSettings = aProject->GetLocalSettings();
+
+    localSettings.SetFilename( fn.GetName() );
+    localSettings.SaveToFile( fn.GetPath() );
+    localSettings.SetFilename( oldName );
 
     project->SetReadOnly( readOnly );
 }
@@ -1110,21 +1176,19 @@ bool SETTINGS_MANAGER::BackupProject( REPORTER& aReporter ) const
     target.SetName( fileName );
     target.SetExt( ArchiveFileExtension );
 
-    wxDir dir( target.GetPath() );
-
     if( !target.DirExists() && !wxMkdir( target.GetPath() ) )
     {
-        wxLogTrace( traceSettings, "Could not create project backup path %s", target.GetPath() );
+        wxLogTrace( traceSettings, wxT( "Could not create project backup path %s" ), target.GetPath() );
         return false;
     }
 
     if( !target.IsDirWritable() )
     {
-        wxLogTrace( traceSettings, "Backup directory %s is not writable", target.GetPath() );
+        wxLogTrace( traceSettings, wxT( "Backup directory %s is not writable" ), target.GetPath() );
         return false;
     }
 
-    wxLogTrace( traceSettings, "Backing up project to %s", target.GetPath() );
+    wxLogTrace( traceSettings, wxT( "Backing up project to %s" ), target.GetPath() );
 
     PROJECT_ARCHIVER archiver;
 
@@ -1176,7 +1240,7 @@ bool SETTINGS_MANAGER::TriggerBackupIfNeeded( REPORTER& aReporter ) const
             {
                 wxDateTime dt;
                 wxString fn( wxFileName( aFile ).GetName() );
-                fn.Replace( prefix, "" );
+                fn.Replace( prefix, wxS( "" ) );
                 dt.ParseFormat( fn, backupDateTimeFormat );
                 return dt;
             };
@@ -1191,11 +1255,11 @@ bool SETTINGS_MANAGER::TriggerBackupIfNeeded( REPORTER& aReporter ) const
 
     if( !wxDirExists( backupPath ) )
     {
-        wxLogTrace( traceSettings, "Backup path %s doesn't exist, creating it", backupPath );
+        wxLogTrace( traceSettings, wxT( "Backup path %s doesn't exist, creating it" ), backupPath );
 
         if( !wxMkdir( backupPath ) )
         {
-            wxLogTrace( traceSettings, "Could not create backups path!  Skipping backup" );
+            wxLogTrace( traceSettings, wxT( "Could not create backups path!  Skipping backup" ) );
             return false;
         }
     }
@@ -1204,7 +1268,7 @@ bool SETTINGS_MANAGER::TriggerBackupIfNeeded( REPORTER& aReporter ) const
 
     if( !dir.IsOpened() )
     {
-        wxLogTrace( traceSettings, "Could not open project backups path %s", dir.GetName() );
+        wxLogTrace( traceSettings, wxT( "Could not open project backups path %s" ), dir.GetName() );
         return false;
     }
 
