@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2004-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2004-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,6 +28,7 @@
 
 #include <clocale>
 #include <cmath>
+#include <fmt/core.h>
 #include <macros.h>
 #include <richio.h>                        // StrPrintf
 #include <string_utils.h>
@@ -142,7 +143,8 @@ bool ConvertSmartQuotesAndDashes( wxString* aString )
 
 wxString EscapeString( const wxString& aSource, ESCAPE_CONTEXT aContext )
 {
-    wxString converted;
+    wxString          converted;
+    std::vector<bool> braceStack;    // true == formatting construct
 
     converted.reserve( aSource.length() );
 
@@ -159,11 +161,7 @@ wxString EscapeString( const wxString& aSource, ESCAPE_CONTEXT aContext )
         }
         else if( aContext == CTX_LIBID )
         {
-            if( c == '{' )
-                converted += wxT( "{brace}" );
-            else if( c == '/' )
-                converted += wxT( "{slash}" );
-            else if( c == '\\' )
+            if( c == '\\' )
                 converted += wxT( "{backslash}" );
             else if( c == '<' )
                 converted += wxT( "{lt}" );
@@ -178,6 +176,17 @@ wxString EscapeString( const wxString& aSource, ESCAPE_CONTEXT aContext )
             else
                 converted += c;
         }
+        else if( aContext == CTX_IPC )
+        {
+            if( c == '/' )
+                converted += wxT( "{slash}" );
+            else if( c == ',' )
+                converted += wxT( "{comma}" );
+            else if( c == '\"' )
+                converted += wxT( "{dblquote}" );
+            else
+                converted += c;
+        }
         else if( aContext == CTX_QUOTED_STR )
         {
             if( c == '\"' )
@@ -185,6 +194,20 @@ wxString EscapeString( const wxString& aSource, ESCAPE_CONTEXT aContext )
             else
                 converted += c;
         }
+        else if( aContext == CTX_JS_STR )
+        {
+            if( c >= 0x7F || c == '\'' || c == '\\' )
+            {
+                unsigned int code = c;
+                char buffer[16];
+                snprintf( buffer, sizeof(buffer), "\\\\u%4.4X", code );
+                converted += buffer;
+            }
+            else
+            {
+                converted += c;
+            }
+        }
         else if( aContext == CTX_LINE )
         {
             if( c == '\n' || c == '\r' )
@@ -194,9 +217,7 @@ wxString EscapeString( const wxString& aSource, ESCAPE_CONTEXT aContext )
         }
         else if( aContext == CTX_FILENAME )
         {
-            if( c == '{' )
-                converted += wxT( "{brace}" );
-            else if( c == '/' )
+            if( c == '/' )
                 converted += wxT( "{slash}" );
             else if( c == '\\' )
                 converted += wxT( "{backslash}" );
@@ -221,13 +242,22 @@ wxString EscapeString( const wxString& aSource, ESCAPE_CONTEXT aContext )
         {
             if( c == ' ' )
                 converted += wxT( "{space}" );
-            else if( c == '{' )
-                converted += wxT( "{brace}" );
+            else
+                converted += c;
+        }
+        else if( aContext == CTX_CSV )
+        {
+            if( c == ',' )
+                converted += wxT( "{comma}" );
+            else if( c == '\n' || c == '\r' )
+                converted += wxT( "{return}" );
             else
                 converted += c;
         }
         else
+        {
             converted += c;
+        }
     }
 
     return converted;
@@ -250,7 +280,8 @@ wxString UnescapeString( const wxString& aSource )
     for( size_t i = 0; i < sourceLen; ++i )
     {
         wxUniChar ch = aSource[i];
-        if( ( ch == '$' || ch == '^' || ch == '_' )
+
+        if( ( ch == '$' || ch == '~' || ch == '^' || ch == '_' )
                 && i + 1 < sourceLen && aSource[i+1] == '{' )
         {
             for( ; i < sourceLen; ++i )
@@ -281,20 +312,21 @@ wxString UnescapeString( const wxString& aSource )
                     token.append( ch );
             }
 
-            if(      token == wxS( "dblquote" ) )  newbuf.append( wxS( "\"" ) );
-            else if( token == wxS( "quote" ) )     newbuf.append( wxS( "'" ) );
-            else if( token == wxS( "lt" ) )        newbuf.append( wxS( "<" ) );
-            else if( token == wxS( "gt" ) )        newbuf.append( wxS( ">" ) );
-            else if( token == wxS( "backslash" ) ) newbuf.append( wxS( "\\" ) );
-            else if( token == wxS( "slash" ) )     newbuf.append( wxS( "/" ) );
-            else if( token == wxS( "bar" ) )       newbuf.append( wxS( "|" ) );
-            else if( token == wxS( "colon" ) )     newbuf.append( wxS( ":" ) );
-            else if( token == wxS( "space" ) )     newbuf.append( wxS( " " ) );
-            else if( token == wxS( "dollar" ) )    newbuf.append( wxS( "$" ) );
-            else if( token == wxS( "tab" ) )       newbuf.append( wxS( "\t" ) );
-            else if( token == wxS( "return" ) )    newbuf.append( wxS( "\n" ) );
-            else if( token == wxS( "brace" ) )     newbuf.append( wxS( "{" ) );
-            else if( token.IsEmpty() )             newbuf.append( wxS( "{" ) );
+            if(      token == wxT( "dblquote" ) )  newbuf.append( wxT( "\"" ) );
+            else if( token == wxT( "quote" ) )     newbuf.append( wxT( "'" ) );
+            else if( token == wxT( "lt" ) )        newbuf.append( wxT( "<" ) );
+            else if( token == wxT( "gt" ) )        newbuf.append( wxT( ">" ) );
+            else if( token == wxT( "backslash" ) ) newbuf.append( wxT( "\\" ) );
+            else if( token == wxT( "slash" ) )     newbuf.append( wxT( "/" ) );
+            else if( token == wxT( "bar" ) )       newbuf.append( wxT( "|" ) );
+            else if( token == wxT( "comma" ) )     newbuf.append( wxT( "," ) );
+            else if( token == wxT( "colon" ) )     newbuf.append( wxT( ":" ) );
+            else if( token == wxT( "space" ) )     newbuf.append( wxT( " " ) );
+            else if( token == wxT( "dollar" ) )    newbuf.append( wxT( "$" ) );
+            else if( token == wxT( "tab" ) )       newbuf.append( wxT( "\t" ) );
+            else if( token == wxT( "return" ) )    newbuf.append( wxT( "\n" ) );
+            else if( token == wxT( "brace" ) )     newbuf.append( wxT( "{" ) );
+            else if( token.IsEmpty() )             newbuf.append( wxT( "{" ) );
             else
             {
                 newbuf.append( wxT( "{" ) + UnescapeString( token ) + wxT( "}" ) );
@@ -1031,15 +1063,14 @@ void wxStringSplit( const wxString& aText, wxArrayString& aStrings, wxChar aSpli
             aStrings.Add( tmp );
             tmp.Clear();
         }
-
         else
+        {
             tmp << aText[ii];
+        }
     }
 
     if( !tmp.IsEmpty() )
-    {
         aStrings.Add( tmp );
-    }
 }
 
 
@@ -1065,7 +1096,37 @@ void StripTrailingZeros( wxString& aStringValue, unsigned aTrailingZeroAllowed )
 }
 
 
-std::string Double2Str( double aValue )
+std::string FormatDouble2Str( double aValue )
+{
+    std::string buf;
+
+    if( aValue != 0.0 && std::fabs( aValue ) <= 0.0001 )
+    {
+        buf = fmt::format( "{:.16f}", aValue );
+
+        // remove trailing zeros (and the decimal marker if needed)
+        while( !buf.empty() && buf[buf.size() - 1] == '0' )
+        {
+            buf.pop_back();
+        }
+
+        // if the value was really small
+        // we may have just stripped all the zeros after the decimal
+        if( buf[buf.size() - 1] == '.' )
+        {
+            buf.pop_back();
+        }
+    }
+    else
+    {
+        buf = fmt::format( "{:.10g}", aValue );
+    }
+
+    return buf;
+}
+
+
+std::string UIDouble2Str( double aValue )
 {
     char    buf[50];
     int     len;
@@ -1074,12 +1135,12 @@ std::string Double2Str( double aValue )
     {
         // For these small values, %f works fine,
         // and %g gives an exponent
-        len = sprintf( buf,  "%.16f", aValue );
+        len = snprintf( buf, sizeof(buf), "%.16f", aValue );
 
         while( --len > 0 && buf[len] == '0' )
             buf[len] = '\0';
 
-        if( buf[len] == '.' )
+        if( buf[len] == '.' || buf[len] == ',' )
             buf[len] = '\0';
         else
             ++len;
@@ -1088,19 +1149,8 @@ std::string Double2Str( double aValue )
     {
         // For these values, %g works fine, and sometimes %f
         // gives a bad value (try aValue = 1.222222222222, with %.16f format!)
-        len = sprintf( buf, "%.10g", aValue );
+        len = snprintf( buf, sizeof(buf), "%.10g", aValue );
     }
 
     return std::string( buf, len );
 }
-
-
-wxString AngleToStringDegrees( double aAngle )
-{
-    wxString text;
-
-    text.Printf( wxT( "%.3f" ), aAngle / 10.0 );
-    StripTrailingZeros( text, 1 );
-
-    return text;
-}
