@@ -21,13 +21,23 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
+#include <string_utils.h>
+#include <kiplatform/ui.h>
 
+#include <widgets/ui_common.h>
 #include <widgets/net_selector.h>
 
-#include <class_board.h>
+#include <board.h>
 #include <netinfo.h>
 #include <wx/arrstr.h>
 #include <wx/display.h>
+#include <wx/valtext.h>
+#include <wx/listbox.h>
+#include <wx/stattext.h>
+#include <wx/sizer.h>
+#include <wx/textctrl.h>
+#include <wx/panel.h>
+
 
 wxDEFINE_EVENT( NET_SELECTED, wxCommandEvent );
 
@@ -46,6 +56,7 @@ wxDEFINE_EVENT( NET_SELECTED, wxCommandEvent );
 #endif
 
 #define NO_NET _( "<no net>" )
+#define CREATE_NET _( "<create net>" )
 
 
 class NET_SELECTOR_COMBOPOPUP : public wxPanel, public wxComboPopup
@@ -58,6 +69,7 @@ public:
             m_minPopupWidth( -1 ),
             m_maxPopupHeight( 1000 ),
             m_netinfoList( nullptr ),
+            m_board( nullptr ),
             m_selectedNetcode( 0 ),
             m_focusHandler( nullptr )
     { }
@@ -79,22 +91,22 @@ public:
         m_filterCtrl->SetValidator( *m_filterValidator );
         mainSizer->Add( m_filterCtrl, 0, wxEXPAND, 0 );
 
-        m_listBox = new wxListBox( this, wxID_ANY, wxDefaultPosition, wxDefaultSize, 0, 0,
+        m_listBox = new wxListBox( this, wxID_ANY, wxDefaultPosition, wxDefaultSize, 0, nullptr,
                                    wxLB_SINGLE|wxLB_NEEDED_SB );
         mainSizer->Add( m_listBox, 0, wxEXPAND|wxTOP, 2 );
 
         SetSizer( mainSizer );
         Layout();
 
-        Connect( wxEVT_IDLE, wxIdleEventHandler( NET_SELECTOR_COMBOPOPUP::onIdle ), NULL, this );
-        Connect( wxEVT_CHAR_HOOK, wxKeyEventHandler( NET_SELECTOR_COMBOPOPUP::onKeyDown ), NULL, this );
-        Connect( wxEVT_LEFT_DOWN, wxMouseEventHandler( NET_SELECTOR_COMBOPOPUP::onMouseClick ), NULL, this );
-        m_listBox->Connect( wxEVT_LEFT_DOWN, wxMouseEventHandler( NET_SELECTOR_COMBOPOPUP::onMouseClick ), NULL, this );
-        m_filterCtrl->Connect( wxEVT_TEXT, wxCommandEventHandler( NET_SELECTOR_COMBOPOPUP::onFilterEdit ), NULL, this );
-        m_filterCtrl->Connect( wxEVT_TEXT_ENTER, wxCommandEventHandler( NET_SELECTOR_COMBOPOPUP::onEnter ), NULL, this );
+        Connect( wxEVT_IDLE, wxIdleEventHandler( NET_SELECTOR_COMBOPOPUP::onIdle ), nullptr, this );
+        Connect( wxEVT_CHAR_HOOK, wxKeyEventHandler( NET_SELECTOR_COMBOPOPUP::onKeyDown ), nullptr, this );
+        Connect( wxEVT_LEFT_DOWN, wxMouseEventHandler( NET_SELECTOR_COMBOPOPUP::onMouseClick ), nullptr, this );
+        m_listBox->Connect( wxEVT_LEFT_DOWN, wxMouseEventHandler( NET_SELECTOR_COMBOPOPUP::onMouseClick ), nullptr, this );
+        m_filterCtrl->Connect( wxEVT_TEXT, wxCommandEventHandler( NET_SELECTOR_COMBOPOPUP::onFilterEdit ), nullptr, this );
+        m_filterCtrl->Connect( wxEVT_TEXT_ENTER, wxCommandEventHandler( NET_SELECTOR_COMBOPOPUP::onEnter ), nullptr, this );
 
         // <enter> in a ListBox comes in as a double-click on GTK
-        m_listBox->Connect( wxEVT_COMMAND_LISTBOX_DOUBLECLICKED, wxCommandEventHandler( NET_SELECTOR_COMBOPOPUP::onEnter ), NULL, this );
+        m_listBox->Connect( wxEVT_COMMAND_LISTBOX_DOUBLECLICKED, wxCommandEventHandler( NET_SELECTOR_COMBOPOPUP::onEnter ), nullptr, this );
 
         return true;
     }
@@ -108,9 +120,12 @@ public:
 
     wxString GetStringValue() const override
     {
+        if( m_selectedNetcode == -1 )
+            return m_indeterminateLabel;
+
         NETINFO_ITEM* netInfo = m_netinfoList->GetNetItem( m_selectedNetcode );
 
-        if( netInfo && netInfo->GetNet() > 0 )
+        if( netInfo && netInfo->GetNetCode() > 0 )
             return netInfo->GetNetname();
 
         return NO_NET;
@@ -122,6 +137,17 @@ public:
         rebuildList();
     }
 
+    void SetIndeterminateLabel( const wxString& aIndeterminateLabel )
+    {
+        m_indeterminateLabel = aIndeterminateLabel;
+        rebuildList();
+    }
+
+    void SetBoard( BOARD* aBoard )
+    {
+        m_board = aBoard;
+    }
+
     void SetIndeterminate() { m_selectedNetcode = -1; }
     bool IsIndeterminate() { return m_selectedNetcode == -1; }
 
@@ -131,7 +157,7 @@ public:
     void SetSelectedNet( const wxString& aNetname )
     {
         if( m_netinfoList && m_netinfoList->GetNetItem( aNetname ) )
-            m_selectedNetcode = m_netinfoList->GetNetItem( aNetname )->GetNet();
+            m_selectedNetcode = m_netinfoList->GetNetItem( aNetname )->GetNetCode();
     }
 
     wxString GetSelectedNetname()
@@ -154,7 +180,7 @@ public:
 
     void OnPopup() override
     {
-        // While it can sometimes be useful to keep the filter, it's always expected.
+        // While it can sometimes be useful to keep the filter, it's always unexpected.
         // Better to clear it.
         m_filterCtrl->Clear();
 
@@ -171,40 +197,94 @@ public:
 
     void Accept()
     {
-        wxString selectedNetName;
-        int selection = m_listBox->GetSelection();
+        wxString  selectedNetName;
+        wxString  escapedNetName;
+        wxString  remainingName;
+        int       selection     = m_listBox->GetSelection();
 
         if( selection >= 0 )
             selectedNetName = m_listBox->GetString( (unsigned) selection );
 
-        if( selectedNetName.IsEmpty() )
+        auto it = m_unescapedNetNameMap.find( selectedNetName );
+
+        if( it != m_unescapedNetNameMap.end() )
+            escapedNetName = it->second;
+        else    // shouldn't happen....
+            escapedNetName = selectedNetName;
+
+        Dismiss();
+
+        if( escapedNetName.IsEmpty() || escapedNetName == m_indeterminateLabel )
         {
             m_selectedNetcode = -1;
-            GetComboCtrl()->SetValue( INDETERMINATE );
+            GetComboCtrl()->SetValue( m_indeterminateLabel );
         }
-        else if( selectedNetName == NO_NET )
+        else if( escapedNetName == NO_NET )
         {
             m_selectedNetcode = 0;
             GetComboCtrl()->SetValue( NO_NET );
         }
+        else if( escapedNetName.StartsWith( CREATE_NET, &remainingName ) &&
+                !remainingName.IsEmpty() )
+        {
+            // Remove the first character ':' and all whitespace
+            remainingName = remainingName.Mid( 1 ).Trim().Trim( false );
+
+            BOARD*        board  = m_netinfoList->GetParent();
+            NETINFO_ITEM *newnet = new NETINFO_ITEM( m_board, remainingName, 0 );
+
+            // add the new netinfo through the board's function so that
+            // board listeners get notified and things stay in sync.
+            if( board != nullptr )
+                board->Add( newnet );
+            else
+                m_netinfoList->AppendNet( newnet );
+
+            rebuildList();
+
+            if( newnet->GetNetCode() > 0 )
+            {
+                m_selectedNetcode = newnet->GetNetCode();
+                GetComboCtrl()->SetValue( UnescapeString( remainingName ) );
+            }
+            else
+            {
+                // This indicates that the NETINFO_ITEM was not successfully appended
+                // to the list for unknown reasons
+                if( board != nullptr )
+                    board->Remove( newnet );
+                else
+                    m_netinfoList->RemoveNet( newnet );
+
+                delete newnet;
+            }
+        }
         else
         {
-            m_selectedNetcode = m_netinfoList->GetNetItem( selectedNetName )->GetNet();
-            GetComboCtrl()->SetValue( selectedNetName );
+            NETINFO_ITEM* netInfo = m_netinfoList->GetNetItem( escapedNetName );
+
+            if( netInfo == nullptr || netInfo->GetNetCode() == 0 )
+            {
+                m_selectedNetcode = 0;
+                GetComboCtrl()->SetValue( NO_NET );
+            }
+            else
+            {
+                m_selectedNetcode = netInfo->GetNetCode();
+                GetComboCtrl()->SetValue( UnescapeString( escapedNetName ) );
+            }
         }
 
         wxCommandEvent changeEvent( NET_SELECTED );
         wxPostEvent( GetComboCtrl(), changeEvent );
-
-        Dismiss();
     }
 
 protected:
     wxSize updateSize()
     {
-        int    listTop = m_listBox->GetRect().y;
-        int    itemHeight = GetTextSize( wxT( "Xy" ), this ).y + LIST_ITEM_PADDING;
-        int    listHeight = m_listBox->GetCount() * itemHeight + LIST_PADDING;
+        int listTop    = m_listBox->GetRect().y;
+        int itemHeight = KIUI::GetTextSize( wxT( "Xy" ), this ).y + LIST_ITEM_PADDING;
+        int listHeight = m_listBox->GetCount() * itemHeight + LIST_PADDING;
 
         if( listTop + listHeight >= m_maxPopupHeight )
             listHeight = m_maxPopupHeight - listTop - 1;
@@ -213,7 +293,7 @@ protected:
 
         for( size_t i = 0; i < m_listBox->GetCount(); ++i )
         {
-            int itemWidth = GetTextSize( m_listBox->GetString( i ), m_listBox ).x;
+            int itemWidth = KIUI::GetTextSize( m_listBox->GetString( i ), m_listBox ).x;
             listWidth = std::max( listWidth, itemWidth + LIST_PADDING * 3 );
         }
 
@@ -232,25 +312,48 @@ protected:
     void rebuildList()
     {
         wxArrayString netNames;
-        wxString      filter = m_filterCtrl->GetValue().MakeLower();
+        wxString      netstring = m_filterCtrl->GetValue().Trim().Trim( false );
+        wxString      filter = netstring.Lower();
+
+        m_unescapedNetNameMap.clear();
 
         if( !filter.IsEmpty() )
             filter = wxT( "*" ) + filter + wxT( "*" );
 
         for( NETINFO_ITEM* netinfo : *m_netinfoList )
         {
-            if( netinfo->GetNet() > 0 && netinfo->IsCurrent() )
+            if( netinfo->GetNetCode() > 0 && netinfo->IsCurrent() )
             {
-                if( filter.IsEmpty() || wxString( netinfo->GetNetname() ).MakeLower().Matches( filter ) )
-                    netNames.push_back( netinfo->GetNetname() );
+                wxString netname = UnescapeString( netinfo->GetNetname() );
+
+                if( filter.IsEmpty() || wxString( netname ).MakeLower().Matches( filter ) )
+                {
+                    netNames.push_back( netname );
+                    m_unescapedNetNameMap[ netname ] = netinfo->GetNetname();
+                }
             }
         }
-        std::sort( netNames.begin(), netNames.end() );
+
+        std::sort( netNames.begin(), netNames.end(),
+                []( const wxString& lhs, const wxString& rhs )
+                {
+                    return StrNumCmp( lhs, rhs, true /* ignore case */ ) < 0;
+                } );
+
 
         // Special handling for <no net>
         if( filter.IsEmpty() || wxString( NO_NET ).MakeLower().Matches( filter ) )
             netNames.insert( netNames.begin(), NO_NET );
 
+        if( !filter.IsEmpty() && !m_netinfoList->GetNetItem( netstring ) )
+        {
+            wxString newnet = wxString::Format( "%s: %s", CREATE_NET, netstring );
+            netNames.insert( netNames.end(), newnet );
+        }
+
+        if( !m_indeterminateLabel.IsEmpty() )
+            netNames.push_back( m_indeterminateLabel );
+
         m_listBox->Set( netNames );
     }
 
@@ -426,11 +529,7 @@ protected:
 
     void doSetFocus( wxWindow* aWindow )
     {
-#ifdef __WXOSX_MAC__
-        aWindow->OSXForceFocus();
-#else
-        aWindow->SetFocus();
-#endif
+        KIPLATFORM::UI::ForceFocus( aWindow );
     }
 
 protected:
@@ -441,9 +540,13 @@ protected:
     int              m_maxPopupHeight;
 
     NETINFO_LIST*    m_netinfoList;
+    wxString         m_indeterminateLabel;
+    BOARD*           m_board;
 
     int              m_selectedNetcode;
 
+    std::map<wxString, wxString> m_unescapedNetNameMap;
+
     wxEvtHandler*    m_focusHandler;
 };
 
@@ -457,13 +560,13 @@ NET_SELECTOR::NET_SELECTOR( wxWindow *parent, wxWindowID id, const wxPoint &pos,
     m_netSelectorPopup = new NET_SELECTOR_COMBOPOPUP();
     SetPopupControl( m_netSelectorPopup );
 
-    Connect( wxEVT_CHAR_HOOK, wxKeyEventHandler( NET_SELECTOR::onKeyDown ), NULL, this );
+    Connect( wxEVT_CHAR_HOOK, wxKeyEventHandler( NET_SELECTOR::onKeyDown ), nullptr, this );
 }
 
 
 NET_SELECTOR::~NET_SELECTOR()
 {
-    Disconnect( wxEVT_CHAR_HOOK, wxKeyEventHandler( NET_SELECTOR::onKeyDown ), NULL, this );
+    Disconnect( wxEVT_CHAR_HOOK, wxKeyEventHandler( NET_SELECTOR::onKeyDown ), nullptr, this );
 }
 
 
@@ -510,17 +613,30 @@ void NET_SELECTOR::SetNetInfo( NETINFO_LIST* aNetInfoList )
 }
 
 
+void NET_SELECTOR::SetIndeterminateString( const wxString& aString )
+{
+    m_indeterminateString = aString;
+    m_netSelectorPopup->SetIndeterminateLabel( aString );
+}
+
+
+void NET_SELECTOR::SetBoard( BOARD* aBoard )
+{
+    m_netSelectorPopup->SetBoard( aBoard );
+}
+
+
 void NET_SELECTOR::SetSelectedNetcode( int aNetcode )
 {
     m_netSelectorPopup->SetSelectedNetcode( aNetcode );
-    SetValue( m_netSelectorPopup->GetStringValue() );
+    SetValue( UnescapeString( m_netSelectorPopup->GetStringValue() ) );
 }
 
 
 void NET_SELECTOR::SetSelectedNet( const wxString& aNetname )
 {
     m_netSelectorPopup->SetSelectedNet( aNetname );
-    SetValue( m_netSelectorPopup->GetStringValue() );
+    SetValue( UnescapeString( m_netSelectorPopup->GetStringValue() ) );
 }
 
 
@@ -533,7 +649,7 @@ wxString NET_SELECTOR::GetSelectedNetname()
 void NET_SELECTOR::SetIndeterminate()
 {
     m_netSelectorPopup->SetIndeterminate();
-    SetValue( INDETERMINATE );
+    SetValue( m_indeterminateString  );
 }
 
 
