@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2018 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2015 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 2007-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2007-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,9 +26,10 @@
 #include <bitmaps.h>
 #include <board.h>
 #include <footprint.h>
+#include <pad.h>
 #include <common.h>
 #include <confirm.h>
-#include <cvpcb_settings.h>
+#include <settings/cvpcb_settings.h>
 #include <footprint_editor_settings.h>
 #include <fp_lib_table.h>
 #include <id.h>
@@ -53,9 +54,10 @@
 #include <tools/pcb_editor_conditions.h>  // Shared conditions with other Pcbnew frames
 #include <tools/pcb_viewer_tools.h>       // shared tools with other Pcbnew frames
 #include <tools/cvpcb_fpviewer_selection_tool.h>
-#include <widgets/infobar.h>
+#include <widgets/wx_infobar.h>
 #include <wx/choice.h>
 #include <wx/debug.h>
+#include <cvpcb_id.h>
 
 
 BEGIN_EVENT_TABLE( DISPLAY_FOOTPRINTS_FRAME, PCB_BASE_FRAME )
@@ -90,20 +92,12 @@ DISPLAY_FOOTPRINTS_FRAME::DISPLAY_FOOTPRINTS_FRAME( KIWAY* aKiway, wxWindow* aPa
                                                   EDA_DRAW_PANEL_GAL::GAL_FALLBACK );
     SetCanvas( gal_drawPanel );
 
-    // Don't show the default board solder mask clearance.  Only the
-    // footprint or pad clearance setting should be shown if it is not 0.
-    GetBoard()->GetDesignSettings().m_SolderMaskMargin = 0;
+    // Don't show the default board solder mask expansion.  Only the footprint or pad expansion
+    // settings should be shown.
+    GetBoard()->GetDesignSettings().m_SolderMaskExpansion = 0;
 
     LoadSettings( config() );
 
-    // Initialize some display options
-    auto displ_opts = GetDisplayOptions();
-    displ_opts.m_DisplayPadClearance = false;      // Pad clearance has no meaning here
-
-    // Track and via clearance has no meaning here.
-    displ_opts.m_ShowTrackClearanceMode = PCB_DISPLAY_OPTIONS::DO_NOT_SHOW_CLEARANCE;
-    SetDisplayOptions( displ_opts );
-
     // Create the manager and dispatcher & route draw panel events to the dispatcher
     m_toolManager = new TOOL_MANAGER;
     m_toolManager->SetEnvironment( GetBoard(), gal_drawPanel->GetView(),
@@ -133,13 +127,13 @@ DISPLAY_FOOTPRINTS_FRAME::DISPLAY_FOOTPRINTS_FRAME( KIWAY* aKiway, wxWindow* aPa
     m_auimgr.SetManagedWindow( this );
 
     CreateInfoBar();
-    m_auimgr.AddPane( m_mainToolBar, EDA_PANE().HToolbar().Name( wxT( "MainToolbar" ) )
+    m_auimgr.AddPane( m_mainToolBar, EDA_PANE().HToolbar().Name( wxS( "MainToolbar" ) )
                       .Top().Layer( 6 ) );
-    m_auimgr.AddPane( m_optionsToolBar, EDA_PANE().VToolbar().Name( wxT( "OptToolbar" ) )
+    m_auimgr.AddPane( m_optionsToolBar, EDA_PANE().VToolbar().Name( wxS( "OptToolbar" ) )
                       .Left().Layer( 3 ) );
-    m_auimgr.AddPane( GetCanvas(), EDA_PANE().Canvas().Name( wxT( "DrawFrame" ) )
+    m_auimgr.AddPane( GetCanvas(), EDA_PANE().Canvas().Name( wxS( "DrawFrame" ) )
                       .Center() );
-    m_auimgr.AddPane( m_messagePanel, EDA_PANE().Messages().Name( wxT( "MsgPanel" ) )
+    m_auimgr.AddPane( m_messagePanel, EDA_PANE().Messages().Name( wxS( "MsgPanel" ) )
                       .Bottom().Layer( 6 ) );
 
     FinishAUIInitialization();
@@ -155,8 +149,17 @@ DISPLAY_FOOTPRINTS_FRAME::DISPLAY_FOOTPRINTS_FRAME( KIWAY* aKiway, wxWindow* aPa
     CVPCB_SETTINGS* cfg = dynamic_cast<CVPCB_SETTINGS*>( config() );
 
     if( cfg )
+    {
         GetCanvas()->GetView()->SetScale( cfg->m_FootprintViewerZoom );
 
+        wxAuiToolBarItem* toolOpt = m_mainToolBar->FindTool( ID_CVPCB_FPVIEWER_AUTOZOOM_TOOL );
+
+        if( cfg->m_FootprintViewerAutoZoomOnSelect )
+            toolOpt->SetState( wxAUI_BUTTON_STATE_CHECKED );
+        else
+            toolOpt->SetState( 0 );
+    }
+
     updateView();
 
     Show( true );
@@ -213,13 +216,10 @@ void DISPLAY_FOOTPRINTS_FRAME::setupUIConditions()
     mgr->SetConditions( ACTIONS::inchesUnits,       CHECK( cond.Units( EDA_UNITS::INCHES ) ) );
     mgr->SetConditions( ACTIONS::milsUnits,         CHECK( cond.Units( EDA_UNITS::MILS ) ) );
 
-
-    mgr->SetConditions( PCB_ACTIONS::showPadNumbers,
-                        CHECK( cond.PadNumbersDisplay() ) );
-    mgr->SetConditions( PCB_ACTIONS::padDisplayMode,             CHECK( !cond.PadFillDisplay() ) );
-    mgr->SetConditions( PCB_ACTIONS::textOutlines,               CHECK( !cond.TextFillDisplay() ) );
-    mgr->SetConditions( PCB_ACTIONS::graphicsOutlines,
-                        CHECK( !cond.GraphicsFillDisplay() ) );
+    mgr->SetConditions( PCB_ACTIONS::showPadNumbers,   CHECK( cond.PadNumbersDisplay() ) );
+    mgr->SetConditions( PCB_ACTIONS::padDisplayMode,   CHECK( !cond.PadFillDisplay() ) );
+    mgr->SetConditions( PCB_ACTIONS::textOutlines,     CHECK( !cond.TextFillDisplay() ) );
+    mgr->SetConditions( PCB_ACTIONS::graphicsOutlines, CHECK( !cond.GraphicsFillDisplay() ) );
 
 #undef CHECK
 }
@@ -319,6 +319,14 @@ void DISPLAY_FOOTPRINTS_FRAME::ReCreateHToolbar()
     UpdateZoomSelectBox();
     m_mainToolBar->AddControl( m_zoomSelectBox );
 
+    // Option to run Zoom automatique on footprint selection changge
+    m_mainToolBar->AddTool( ID_CVPCB_FPVIEWER_AUTOZOOM_TOOL, wxEmptyString,
+                            KiScaledBitmap( BITMAPS::zoom_auto_fit_in_page, this ),
+                            _( "Automatic Zoom on footprint change" ),
+                            wxITEM_CHECK );
+
+    m_mainToolBar->AddScaledSeparator( this );
+
     m_mainToolBar->UpdateControlWidth( ID_ON_GRID_SELECT );
     m_mainToolBar->UpdateControlWidth( ID_ON_ZOOM_SELECT );
 
@@ -340,7 +348,7 @@ void DISPLAY_FOOTPRINTS_FRAME::UpdateToolbarControlSizes()
 
 void DISPLAY_FOOTPRINTS_FRAME::LoadSettings( APP_SETTINGS_BASE* aCfg )
 {
-    auto cfg = dynamic_cast<CVPCB_SETTINGS*>( aCfg );
+    CVPCB_SETTINGS* cfg = dynamic_cast<CVPCB_SETTINGS*>( aCfg );
     wxCHECK( cfg, /* void */ );
 
     // We don't allow people to change this right now, so make sure it's on
@@ -362,21 +370,28 @@ void DISPLAY_FOOTPRINTS_FRAME::SaveSettings( APP_SETTINGS_BASE* aCfg )
     cfg->m_FootprintViewerDisplayOptions = GetDisplayOptions();
 
     cfg->m_FootprintViewerZoom = GetCanvas()->GetView()->GetScale();
+
+    wxAuiToolBarItem* toolOpt = m_mainToolBar->FindTool( ID_CVPCB_FPVIEWER_AUTOZOOM_TOOL );
+    cfg->m_FootprintViewerAutoZoomOnSelect = ( toolOpt->GetState() & wxAUI_BUTTON_STATE_CHECKED );
 }
 
 
 WINDOW_SETTINGS* DISPLAY_FOOTPRINTS_FRAME::GetWindowSettings( APP_SETTINGS_BASE* aCfg )
 {
-    CVPCB_SETTINGS* cfg = dynamic_cast<CVPCB_SETTINGS*>( aCfg );
-    wxCHECK( cfg, nullptr );
+    CVPCB_SETTINGS* cfg = Pgm().GetSettingsManager().GetAppSettings<CVPCB_SETTINGS>();
     return &cfg->m_FootprintViewer;
 }
 
 
+PCB_VIEWERS_SETTINGS_BASE* DISPLAY_FOOTPRINTS_FRAME::GetViewerSettingsBase() const
+{
+    return Pgm().GetSettingsManager().GetAppSettings<CVPCB_SETTINGS>();
+}
+
+
 MAGNETIC_SETTINGS* DISPLAY_FOOTPRINTS_FRAME::GetMagneticItemsSettings()
 {
-    CVPCB_SETTINGS* cfg = dynamic_cast<CVPCB_SETTINGS*>( Kiface().KifaceSettings() );
-    wxCHECK( cfg, nullptr );
+    CVPCB_SETTINGS* cfg = Pgm().GetSettingsManager().GetAppSettings<CVPCB_SETTINGS>();
     return &cfg->m_FootprintViewerMagneticSettings;
 }
 
@@ -466,6 +481,7 @@ void DISPLAY_FOOTPRINTS_FRAME::InitDisplay()
         return;
 
     GetBoard()->DeleteAllFootprints();
+    GetBoard()->GetNetInfo().RemoveUnusedNets();
     GetCanvas()->GetView()->Clear();
 
     INFOBAR_REPORTER infoReporter( m_infoBar );
@@ -482,6 +498,22 @@ void DISPLAY_FOOTPRINTS_FRAME::InitDisplay()
 
     if( footprint )
     {
+        if( comp )
+        {
+            for( PAD* pad : footprint->Pads() )
+            {
+                const COMPONENT_NET& net = comp->GetNet( pad->GetNumber() );
+
+                if( !net.GetPinFunction().IsEmpty() )
+                {
+                    NETINFO_ITEM* netinfo = new NETINFO_ITEM( GetBoard() );
+                    netinfo->SetNetname( net.GetPinFunction() );
+                    GetBoard()->Add( netinfo );
+                    pad->SetNet( netinfo );
+                }
+            }
+        }
+
         GetBoard()->Add( footprint );
         m_currentFootprint = footprintName;
         m_currentComp = comp;
@@ -511,7 +543,9 @@ void DISPLAY_FOOTPRINTS_FRAME::updateView()
 
     m_toolManager->ResetTools( TOOL_BASE::MODEL_RELOAD );
 
-    if( m_zoomSelectBox->GetSelection() == 0 )
+    wxAuiToolBarItem* toolOpt = m_mainToolBar->FindTool( ID_CVPCB_FPVIEWER_AUTOZOOM_TOOL );
+
+    if( toolOpt->GetState() & wxAUI_BUTTON_STATE_CHECKED )
         m_toolManager->RunAction( ACTIONS::zoomFitScreen, true );
     else
         m_toolManager->RunAction( ACTIONS::centerContents, true );
@@ -532,7 +566,7 @@ void DISPLAY_FOOTPRINTS_FRAME::UpdateMsgPanel()
 }
 
 
-COLOR_SETTINGS* DISPLAY_FOOTPRINTS_FRAME::GetColorSettings() const
+COLOR_SETTINGS* DISPLAY_FOOTPRINTS_FRAME::GetColorSettings( bool aForceRefresh ) const
 {
     auto* settings = Pgm().GetSettingsManager().GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>();
 
