@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2012 Jean-Pierre Charras, jean-pierre.charras@gipsa-lab.inpg.com
  * Copyright (C) 2016 Wayne Stambaugh, stambaughw@gmail.com
- * Copyright (C) 2004-2021 KiCad Developers, see AITHORS.txt for contributors.
+ * Copyright (C) 2004-2022 KiCad Developers, see AITHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,6 +23,10 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
+#include <widgets/bitmap_button.h>
+#include <widgets/font_choice.h>
+#include <widgets/color_swatch.h>
+#include <settings/color_settings.h>
 #include <bitmaps.h>
 #include <kiway.h>
 #include <confirm.h>
@@ -44,16 +48,18 @@
 
 DIALOG_FIELD_PROPERTIES::DIALOG_FIELD_PROPERTIES( SCH_BASE_FRAME* aParent, const wxString& aTitle,
                                                   const EDA_TEXT* aTextItem ) :
-    DIALOG_LIB_TEXT_PROPERTIES_BASE( aParent ),
+    DIALOG_FIELD_PROPERTIES_BASE( aParent, wxID_ANY, aTitle ),
     m_posX( aParent, m_xPosLabel, m_xPosCtrl, m_xPosUnits, true ),
     m_posY( aParent, m_yPosLabel, m_yPosCtrl, m_yPosUnits, true ),
     m_textSize( aParent, m_textSizeLabel, m_textSizeCtrl, m_textSizeUnits, true ),
+    m_font( nullptr ),
     m_firstFocus( true ),
     m_scintillaTricks( nullptr )
 {
-    wxASSERT( aTextItem );
+    COLOR_SETTINGS* colorSettings = aParent->GetColorSettings();
+    COLOR4D         schematicBackground = colorSettings->GetColor( LAYER_SCHEMATIC_BACKGROUND );
 
-    SetTitle( aTitle );
+    wxASSERT( aTextItem );
 
     m_note->SetFont( KIUI::GetInfoFont( this ).Italic() );
     m_note->Show( false );
@@ -69,15 +75,68 @@ DIALOG_FIELD_PROPERTIES::DIALOG_FIELD_PROPERTIES( SCH_BASE_FRAME* aParent, const
             } );
     m_StyledTextCtrl->SetEOLMode( wxSTC_EOL_LF );   // Normalize EOL across platforms
 
+    m_textColorSwatch->SetDefaultColor( COLOR4D::UNSPECIFIED );
+    m_textColorSwatch->SetSwatchBackground( schematicBackground );
+
+    m_separator1->SetIsSeparator();
+
+    m_horizontal->SetIsRadioButton();
+    m_horizontal->SetBitmap( KiBitmap( BITMAPS::text_horizontal ) );
+    m_vertical->SetIsRadioButton();
+    m_vertical->SetBitmap( KiBitmap( BITMAPS::text_vertical ) );
+
+    m_separator2->SetIsSeparator();
+
+    m_bold->SetIsCheckButton();
+    m_bold->SetBitmap( KiBitmap( BITMAPS::text_bold ) );
+    m_italic->SetIsCheckButton();
+    m_italic->SetBitmap( KiBitmap( BITMAPS::text_italic ) );
+
+    m_separator3->SetIsSeparator();
+
+    m_hAlignLeft->SetIsRadioButton();
+    m_hAlignLeft->SetBitmap( KiBitmap( BITMAPS::text_align_left ) );
+    m_hAlignCenter->SetIsRadioButton();
+    m_hAlignCenter->SetBitmap( KiBitmap( BITMAPS::text_align_center ) );
+    m_hAlignRight->SetIsRadioButton();
+    m_hAlignRight->SetBitmap( KiBitmap( BITMAPS::text_align_right ) );
+
+    m_separator4->SetIsSeparator();
+
+    m_vAlignTop->SetIsRadioButton();
+    m_vAlignTop->SetBitmap( KiBitmap( BITMAPS::text_valign_top ) );
+    m_vAlignCenter->SetIsRadioButton();
+    m_vAlignCenter->SetBitmap( KiBitmap( BITMAPS::text_valign_center ) );
+    m_vAlignBottom->SetIsRadioButton();
+    m_vAlignBottom->SetBitmap( KiBitmap( BITMAPS::text_valign_bottom ) );
+
+    m_separator5->SetIsSeparator();
+
+    m_horizontal->Bind( wxEVT_BUTTON, &DIALOG_FIELD_PROPERTIES::onOrientButton, this );
+    m_vertical->Bind( wxEVT_BUTTON, &DIALOG_FIELD_PROPERTIES::onOrientButton, this );
+
+    m_hAlignLeft->Bind( wxEVT_BUTTON, &DIALOG_FIELD_PROPERTIES::onHAlignButton, this );
+    m_hAlignCenter->Bind( wxEVT_BUTTON, &DIALOG_FIELD_PROPERTIES::onHAlignButton, this );
+    m_hAlignRight->Bind( wxEVT_BUTTON, &DIALOG_FIELD_PROPERTIES::onHAlignButton, this );
+
+    m_vAlignTop->Bind( wxEVT_BUTTON, &DIALOG_FIELD_PROPERTIES::onVAlignButton, this );
+    m_vAlignCenter->Bind( wxEVT_BUTTON, &DIALOG_FIELD_PROPERTIES::onVAlignButton, this );
+    m_vAlignBottom->Bind( wxEVT_BUTTON, &DIALOG_FIELD_PROPERTIES::onVAlignButton, this );
+
     m_text = aTextItem->GetText();
     m_isItalic = aTextItem->IsItalic();
     m_isBold = aTextItem->IsBold();
+    m_color = aTextItem->GetTextColor();
     m_position = aTextItem->GetTextPos();
     m_size = aTextItem->GetTextWidth();
-    m_isVertical = ( aTextItem->GetTextAngle() == TEXT_ANGLE_VERT );
-    m_verticalJustification = aTextItem->GetVertJustify() + 1;
-    m_horizontalJustification = aTextItem->GetHorizJustify() + 1;
+    m_isVertical = aTextItem->GetTextAngle().IsVertical();
+    m_verticalJustification = aTextItem->GetVertJustify();
+    m_horizontalJustification = aTextItem->GetHorizJustify();
     m_isVisible = aTextItem->IsVisible();
+
+    // These should be initialized in the child classes implementing dialogs for lib and sch items.
+    m_isNameVisible  = false;
+    m_allowAutoplace = true;
 }
 
 
@@ -99,7 +158,6 @@ void DIALOG_FIELD_PROPERTIES::init()
     // Predefined fields cannot contain some chars, or cannot be empty,
     // and need a SCH_FIELD_VALIDATOR (m_StyledTextCtrl cannot use a SCH_FIELD_VALIDATOR).
     bool use_validator = m_fieldId == REFERENCE_FIELD
-                         || m_fieldId == VALUE_FIELD
                          || m_fieldId == FOOTPRINT_FIELD
                          || m_fieldId == DATASHEET_FIELD
                          || m_fieldId == SHEETNAME_V
@@ -127,8 +185,8 @@ void DIALOG_FIELD_PROPERTIES::init()
     // the text box and display an explanation.
     if( m_fieldId == VALUE_FIELD && m_isPower )
     {
-        m_note->SetLabel( wxString::Format( m_note->GetLabel(),
-                                            _( "Power symbol value field text cannot be changed." ) ) );
+        m_note->SetLabel( wxString::Format( m_note->GetLabel(), _( "Power symbol value field text "
+                                                                   "cannot be changed." ) ) );
         m_note->Show( true );
         m_TextCtrl->Enable( false );
     }
@@ -137,8 +195,6 @@ void DIALOG_FIELD_PROPERTIES::init()
         m_TextCtrl->Enable( true );
     }
 
-    m_sdbSizerButtonsOK->SetDefault();
-
     GetSizer()->SetSizeHints( this );
 
     // Adjust the height of the scintilla text editor after the first layout
@@ -154,6 +210,8 @@ void DIALOG_FIELD_PROPERTIES::init()
         m_StyledTextCtrl->SetUseHorizontalScrollBar( false );
     }
 
+    SetupStandardButtons();
+
     // Now all widgets have the size fixed, call FinishDialogSettings
     finishDialogSettings();
 }
@@ -169,17 +227,18 @@ void DIALOG_FIELD_PROPERTIES::OnTextValueSelectButtonClick( wxCommandEvent& aEve
     else
         fpid = m_TextCtrl->GetValue();
 
-    KIWAY_PLAYER* frame = Kiway().Player( FRAME_FOOTPRINT_VIEWER_MODAL, true );
-
-    if( frame->ShowModal( &fpid, this ) )
+    if( KIWAY_PLAYER* frame = Kiway().Player( FRAME_FOOTPRINT_VIEWER_MODAL, true ) )
     {
-        if( m_StyledTextCtrl->IsShown() )
-            m_StyledTextCtrl->SetValue( fpid );
-        else
-            m_TextCtrl->SetValue( fpid );
-    }
+        if( frame->ShowModal( &fpid, this ) )
+        {
+            if( m_StyledTextCtrl->IsShown() )
+                m_StyledTextCtrl->SetValue( fpid );
+            else
+                m_TextCtrl->SetValue( fpid );
+        }
 
-    frame->Destroy();
+        frame->Destroy();
+    }
 }
 
 
@@ -210,22 +269,79 @@ void DIALOG_FIELD_PROPERTIES::OnSetFocusText( wxFocusEvent& event )
 }
 
 
+void DIALOG_FIELD_PROPERTIES::onOrientButton( wxCommandEvent& aEvent )
+{
+    for( BITMAP_BUTTON* btn : { m_horizontal, m_vertical } )
+    {
+        if( btn->IsChecked() && btn != aEvent.GetEventObject() )
+            btn->Check( false );
+    }
+}
+
+
+void DIALOG_FIELD_PROPERTIES::onHAlignButton( wxCommandEvent& aEvent )
+{
+    for( BITMAP_BUTTON* btn : { m_hAlignLeft, m_hAlignCenter, m_hAlignRight } )
+    {
+        if( btn->IsChecked() && btn != aEvent.GetEventObject() )
+            btn->Check( false );
+    }
+}
+
+
+void DIALOG_FIELD_PROPERTIES::onVAlignButton( wxCommandEvent& aEvent )
+{
+    for( BITMAP_BUTTON* btn : { m_vAlignTop, m_vAlignCenter, m_vAlignBottom } )
+    {
+        if( btn->IsChecked() && btn != aEvent.GetEventObject() )
+            btn->Check( false );
+    }
+}
+
+
 bool DIALOG_FIELD_PROPERTIES::TransferDataToWindow()
 {
     if( m_TextCtrl->IsShown() )
+    {
         m_TextCtrl->SetValue( m_text );
+    }
     else if( m_StyledTextCtrl->IsShown() )
+    {
         m_StyledTextCtrl->SetValue( m_text );
+        m_StyledTextCtrl->EmptyUndoBuffer();
+    }
+
+    m_fontCtrl->SetFontSelection( m_font );
 
     m_posX.SetValue( m_position.x );
     m_posY.SetValue( m_position.y );
     m_textSize.SetValue( m_size );
-    m_orientChoice->SetSelection( m_isVertical ? 1 : 0 );
-    m_hAlignChoice->SetSelection( m_horizontalJustification );
-    m_vAlignChoice->SetSelection( m_verticalJustification );
+
+    m_horizontal->Check( !m_isVertical );
+    m_vertical->Check( m_isVertical );
+
+    m_italic->Check( m_isItalic );
+    m_bold->Check( m_isBold );
+
+    m_textColorSwatch->SetSwatchColor( m_color, false );
+
+    switch ( m_horizontalJustification )
+    {
+    case GR_TEXT_H_ALIGN_LEFT:   m_hAlignLeft->Check( true );   break;
+    case GR_TEXT_H_ALIGN_CENTER: m_hAlignCenter->Check( true ); break;
+    case GR_TEXT_H_ALIGN_RIGHT:  m_hAlignRight->Check( true );  break;
+    }
+
+    switch ( m_verticalJustification )
+    {
+    case GR_TEXT_V_ALIGN_TOP:    m_vAlignTop->Check( true );    break;
+    case GR_TEXT_V_ALIGN_CENTER: m_vAlignCenter->Check( true ); break;
+    case GR_TEXT_V_ALIGN_BOTTOM: m_vAlignBottom->Check( true ); break;
+    }
+
     m_visible->SetValue( m_isVisible );
-    m_italic->SetValue( m_isItalic );
-    m_bold->SetValue( m_isBold );
+    m_nameVisible->SetValue( m_isNameVisible );
+    m_cbAllowAutoPlace->SetValue( m_allowAutoplace );
 
     return true;
 }
@@ -247,14 +363,6 @@ bool DIALOG_FIELD_PROPERTIES::TransferDataFromWindow()
             return false;
         }
     }
-    else if( m_fieldId == VALUE_FIELD )
-    {
-        if( m_text.IsEmpty() )
-        {
-            DisplayError( this, _( "Value may not be empty." ) );
-            return false;
-        }
-    }
     else if( m_fieldId == SHEETFILENAME_V )
     {
         wxFileName fn( m_text );
@@ -268,14 +376,35 @@ bool DIALOG_FIELD_PROPERTIES::TransferDataFromWindow()
         }
     }
 
-    m_isVertical = m_orientChoice->GetSelection() == 1;
     m_position = wxPoint( m_posX.GetValue(), m_posY.GetValue() );
     m_size = m_textSize.GetValue();
-    m_horizontalJustification = m_hAlignChoice->GetSelection();
-    m_verticalJustification = m_vAlignChoice->GetSelection();
-    m_isVisible = m_visible->GetValue();
-    m_isItalic = m_italic->GetValue();
-    m_isBold = m_bold->GetValue();
+
+    if( m_fontCtrl->HaveFontSelection() )
+        m_font = m_fontCtrl->GetFontSelection( m_bold->IsChecked(), m_italic->IsChecked() );
+
+    m_isVertical = m_vertical->IsChecked();
+
+    m_isBold = m_bold->IsChecked();
+    m_isItalic = m_italic->IsChecked();
+    m_color = m_textColorSwatch->GetSwatchColor();
+
+    if( m_hAlignLeft->IsChecked() )
+        m_horizontalJustification = GR_TEXT_H_ALIGN_LEFT;
+    else if( m_hAlignCenter->IsChecked() )
+        m_horizontalJustification = GR_TEXT_H_ALIGN_CENTER;
+    else
+        m_horizontalJustification = GR_TEXT_H_ALIGN_RIGHT;
+
+    if( m_vAlignTop->IsChecked() )
+        m_verticalJustification = GR_TEXT_V_ALIGN_TOP;
+    else if( m_vAlignCenter->IsChecked() )
+        m_verticalJustification = GR_TEXT_V_ALIGN_CENTER;
+    else
+        m_verticalJustification = GR_TEXT_V_ALIGN_BOTTOM;
+
+    m_isVisible      = m_visible->GetValue();
+    m_isNameVisible  = m_nameVisible->GetValue();
+    m_allowAutoplace = m_cbAllowAutoPlace->GetValue();
 
     return true;
 }
@@ -287,9 +416,10 @@ void DIALOG_FIELD_PROPERTIES::updateText( EDA_TEXT* aText )
         aText->SetTextSize( wxSize( m_size, m_size ) );
 
     aText->SetVisible( m_isVisible );
-    aText->SetTextAngle( m_isVertical ? TEXT_ANGLE_VERT : TEXT_ANGLE_HORIZ );
+    aText->SetTextAngle( m_isVertical ? ANGLE_VERTICAL : ANGLE_HORIZONTAL );
     aText->SetItalic( m_isItalic );
     aText->SetBold( m_isBold );
+    aText->SetTextColor( m_color );
 }
 
 
@@ -298,17 +428,37 @@ DIALOG_LIB_FIELD_PROPERTIES::DIALOG_LIB_FIELD_PROPERTIES( SCH_BASE_FRAME* aParen
                                                           const LIB_FIELD* aField ) :
         DIALOG_FIELD_PROPERTIES( aParent, aTitle, aField )
 {
-    m_fieldId = aField->GetId();
+    m_fieldId        = aField->GetId();
+    m_isNameVisible  = aField->IsNameShown();
+    m_allowAutoplace = aField->CanAutoplace();
 
     if( m_fieldId == VALUE_FIELD )
         m_text = UnescapeString( aField->GetText() );
 
+    m_nameVisible->Show();
+    m_cbAllowAutoPlace->Show();
+
     // When in the library editor, power symbols can be renamed.
     m_isPower = false;
     init();
 }
 
 
+void DIALOG_LIB_FIELD_PROPERTIES::UpdateField( LIB_FIELD* aField )
+{
+    aField->SetText( m_text );
+
+    updateText( aField );
+
+    aField->SetNameShown( m_isNameVisible );
+    aField->SetCanAutoplace( m_allowAutoplace );
+
+    aField->SetHorizJustify( EDA_TEXT::MapHorizJustify( m_horizontalJustification ) );
+    aField->SetVertJustify( EDA_TEXT::MapVertJustify( m_verticalJustification  ) );
+    aField->SetTextPos( m_position );
+}
+
+
 DIALOG_SCH_FIELD_PROPERTIES::DIALOG_SCH_FIELD_PROPERTIES( SCH_BASE_FRAME* aParent,
                                                           const wxString& aTitle,
                                                           const SCH_FIELD* aField ) :
@@ -342,18 +492,27 @@ DIALOG_SCH_FIELD_PROPERTIES::DIALOG_SCH_FIELD_PROPERTIES( SCH_BASE_FRAME* aParen
             break;
         }
     }
+    else if( aField->GetParent() && aField->GetParent()->IsType( { SCH_LABEL_LOCATE_ANY_T } ) )
+    {
+        m_fieldId = LABELUSERFIELD_V;
+    }
 
     // show text variable cross-references in a human-readable format
     m_text = aField->Schematic()->ConvertKIIDsToRefs( aField->GetText() );
 
+    m_font = m_field->GetFont();
+
     m_isPower = false;
 
-    m_textLabel->SetLabel( m_field->GetName() + ":" );
+    m_textLabel->SetLabel( aField->GetName() + wxS( ":" ) );
 
     m_position = m_field->GetPosition();
 
-    m_horizontalJustification = m_field->GetEffectiveHorizJustify() + 1;
-    m_verticalJustification = m_field->GetEffectiveVertJustify() + 1;
+    m_isNameVisible = m_field->IsNameShown();
+    m_allowAutoplace = m_field->CanAutoplace();
+
+    m_horizontalJustification = m_field->GetEffectiveHorizJustify();
+    m_verticalJustification = m_field->GetEffectiveVertJustify();
 
     // The library symbol may have been removed so using SCH_SYMBOL::GetLibSymbolRef() here
     // could result in a segfault.  If the library symbol is no longer available, the
@@ -368,8 +527,11 @@ DIALOG_SCH_FIELD_PROPERTIES::DIALOG_SCH_FIELD_PROPERTIES( SCH_BASE_FRAME* aParen
             m_isPower = true;
     }
 
-    m_StyledTextCtrl->Bind( wxEVT_STC_CHARADDED,
-                            &DIALOG_SCH_FIELD_PROPERTIES::onScintillaCharAdded, this );
+    m_StyledTextCtrl->Bind( wxEVT_STC_CHARADDED, &DIALOG_SCH_FIELD_PROPERTIES::onScintillaCharAdded,
+                            this );
+
+    m_nameVisible->Show();
+    m_cbAllowAutoPlace->Show();
 
     init();
 
@@ -399,7 +561,7 @@ void DIALOG_SCH_FIELD_PROPERTIES::onScintillaCharAdded( wxStyledTextEvent &aEven
     {
         wxString text = m_StyledTextCtrl->GetText();
         int currpos = m_StyledTextCtrl->GetCurrentPos();
-        text.Replace( "\n", "" );
+        text.Replace( wxS( "\n" ), wxS( "" ) );
         m_StyledTextCtrl->SetText( text );
         m_StyledTextCtrl->GotoPos( currpos-1 );
         return;
@@ -484,25 +646,35 @@ void DIALOG_SCH_FIELD_PROPERTIES::UpdateField( SCH_FIELD* aField, SCH_SHEET_PATH
         if( fieldType == REFERENCE_FIELD )
             symbol->SetRef( aSheetPath, m_text );
         else if( fieldType == VALUE_FIELD )
-            symbol->SetValue( m_text );
+            symbol->SetValueFieldText( m_text );
         else if( fieldType == FOOTPRINT_FIELD )
-            symbol->SetFootprint( m_text );
+            symbol->SetFootprintFieldText( m_text );
+    }
+    else if( parent && parent->Type() == SCH_GLOBAL_LABEL_T )
+    {
+        if( fieldType == 0 )
+        {
+            if( m_visible->GetValue() != parent->Schematic()->Settings().m_IntersheetRefsShow )
+            {
+                DisplayInfoMessage( this, _( "Intersheet reference visibility is "
+                                             "controlled globally from "
+                                             "Schematic Setup > General > Formatting" ) );
+            }
+        }
     }
 
-    EDA_TEXT_HJUSTIFY_T hJustify = EDA_TEXT::MapHorizJustify( m_horizontalJustification - 1 );
-    EDA_TEXT_VJUSTIFY_T vJustify = EDA_TEXT::MapVertJustify( m_verticalJustification - 1 );
     bool positioningModified = false;
 
     if( aField->GetPosition() != m_position )
         positioningModified = true;
 
-    if( ( aField->GetTextAngle() == TEXT_ANGLE_VERT ) != m_isVertical )
+    if( aField->GetTextAngle().IsVertical() != m_isVertical )
         positioningModified = true;
 
-    if( aField->GetEffectiveHorizJustify() != hJustify )
+    if( aField->GetEffectiveHorizJustify() != m_horizontalJustification )
         positioningModified = true;
 
-    if( aField->GetEffectiveVertJustify() != vJustify )
+    if( aField->GetEffectiveVertJustify() != m_verticalJustification )
         positioningModified = true;
 
     // convert any text variable cross-references to their UUIDs
@@ -512,17 +684,22 @@ void DIALOG_SCH_FIELD_PROPERTIES::UpdateField( SCH_FIELD* aField, SCH_SHEET_PATH
     updateText( aField );
     aField->SetPosition( m_position );
 
+    aField->SetFont( m_font );
+
+    aField->SetNameShown( m_isNameVisible );
+    aField->SetCanAutoplace( m_allowAutoplace );
+
     // Note that we must set justifications before we can ask if they're flipped.  If the old
     // justification is center then it won't know (whereas if the new justification is center
     // the we don't care).
-    aField->SetHorizJustify( hJustify );
-    aField->SetVertJustify( vJustify );
+    aField->SetHorizJustify( m_horizontalJustification );
+    aField->SetVertJustify( m_verticalJustification );
 
     if( aField->IsHorizJustifyFlipped() )
-        aField->SetHorizJustify( EDA_TEXT::MapHorizJustify( -hJustify ) );
+        aField->SetHorizJustify( EDA_TEXT::MapHorizJustify( -m_horizontalJustification ) );
 
     if( aField->IsVertJustifyFlipped() )
-        aField->SetVertJustify( EDA_TEXT::MapVertJustify( -vJustify ) );
+        aField->SetVertJustify( EDA_TEXT::MapVertJustify( -m_verticalJustification ) );
 
     // The value, footprint and datasheet fields should be kept in sync in multi-unit parts.
     // Of course the symbol must be annotated to collect other units.
@@ -552,9 +729,9 @@ void DIALOG_SCH_FIELD_PROPERTIES::UpdateField( SCH_FIELD* aField, SCH_SHEET_PATH
                                                    appendUndo );
 
                     if( fieldType == VALUE_FIELD )
-                        otherUnit->SetValue( m_text );
+                        otherUnit->SetValueFieldText( m_text );
                     else if( fieldType == FOOTPRINT_FIELD )
-                        otherUnit->SetFootprint( m_text );
+                        otherUnit->SetFootprintFieldText( m_text );
                     else
                         otherUnit->GetField( DATASHEET_FIELD )->SetText( m_text );
 
