@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,15 +26,17 @@
 #include <dialogs/dialog_text_entry.h>
 #include <kiway.h>
 #include <symbol_edit_frame.h>
-#include <symbol_library_manager.h>
+#include <lib_symbol_library_manager.h>
 #include <math/util.h> // for KiROUND
 #include <sch_symbol.h>
+#include <kiplatform/ui.h>
 #include <widgets/grid_text_button_helpers.h>
 #include <widgets/wx_grid.h>
+#include <widgets/std_bitmap_button.h>
 #include <string_utils.h>
 
 #ifdef KICAD_SPICE
-#include <dialog_spice_model.h>
+#include <dialog_sim_model.h>
 #endif /* KICAD_SPICE */
 
 #include <dialog_lib_symbol_properties.h>
@@ -57,8 +59,7 @@ DIALOG_LIB_SYMBOL_PROPERTIES::DIALOG_LIB_SYMBOL_PROPERTIES( SYMBOL_EDIT_FRAME* a
     m_delayedFocusGrid( nullptr ),
     m_delayedFocusRow( -1 ),
     m_delayedFocusColumn( -1 ),
-    m_delayedFocusPage( -1 ),
-    m_width( 0 )
+    m_delayedFocusPage( -1 )
 {
     // Give a bit more room for combobox editors
     m_grid->SetDefaultRowSize( m_grid->GetDefaultRowSize() + 4 );
@@ -86,18 +87,17 @@ DIALOG_LIB_SYMBOL_PROPERTIES::DIALOG_LIB_SYMBOL_PROPERTIES( SYMBOL_EDIT_FRAME* a
     m_deleteFilterButton->SetBitmap( KiBitmap( BITMAPS::small_trash ) );
     m_editFilterButton->SetBitmap( KiBitmap( BITMAPS::small_edit ) );
 
+    SetupStandardButtons();
+
     if( aParent->IsSymbolFromLegacyLibrary() && !aParent->IsSymbolFromSchematic() )
     {
         m_stdSizerButtonCancel->SetDefault();
         m_stdSizerButtonOK->SetLabel( _( "Read Only" ) );
         m_stdSizerButtonOK->Enable( false );
     }
-    else
-    {
-        m_stdSizerButtonOK->SetDefault();
-    }
 
 #ifndef KICAD_SPICE
+    m_excludeFromSim->Hide();
     m_spiceFieldsButton->Hide();
 #endif
 
@@ -157,7 +157,7 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::TransferDataToWindow()
     // to bottom: we must change the y coord sign for editing
     for( size_t i = 0; i < m_fields->size(); ++i )
     {
-        wxPoint pos = m_fields->at( i ).GetPosition();
+        VECTOR2I pos = m_fields->at( i ).GetPosition();
         pos.y = -pos.y;
         m_fields->at( i ).SetPosition( pos );
     }
@@ -165,7 +165,7 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::TransferDataToWindow()
     // notify the grid
     wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, m_fields->GetNumberRows() );
     m_grid->ProcessTableMessage( msg );
-    adjustGridColumns( m_grid->GetRect().GetWidth() );
+    adjustGridColumns();
 
     m_SymbolNameCtrl->ChangeValue( UnescapeString( m_libEntry->GetName() ) );
 
@@ -182,6 +182,12 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::TransferDataToWindow()
     m_AsConvertButt->SetValue( m_Parent->GetShowDeMorgan() );
 
     m_OptionPower->SetValue( m_libEntry->IsPower() );
+
+#ifdef KICAD_SPICE
+    LIB_FIELD* simEnableField = m_libEntry->FindField( SIM_ENABLE_FIELD );
+    m_excludeFromSim->SetValue( simEnableField && simEnableField->GetText() == wxT( "0" ) );
+#endif
+
     m_excludeFromBomCheckBox->SetValue( !m_libEntry->GetIncludeInBom() );
     m_excludeFromBoardCheckBox->SetValue( !m_libEntry->GetIncludeOnBoard() );
 
@@ -224,6 +230,47 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::TransferDataToWindow()
 }
 
 
+void DIALOG_LIB_SYMBOL_PROPERTIES::OnExcludeFromSimulation( wxCommandEvent& event )
+{
+#ifdef KICAD_SPICE
+    int simEnableFieldRow = -1;
+
+    for( int ii = MANDATORY_FIELDS; ii < m_grid->GetNumberRows(); ++ii )
+    {
+        if( m_grid->GetCellValue( ii, FDC_NAME ) == SIM_ENABLE_FIELD )
+            simEnableFieldRow = ii;
+    }
+
+    if( event.IsChecked() )
+    {
+        if( simEnableFieldRow == -1 )
+        {
+            simEnableFieldRow = (int) m_fields->size();
+            m_fields->emplace_back( m_libEntry, simEnableFieldRow, SIM_ENABLE_FIELD );
+
+            // notify the grid
+            wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, 1 );
+            m_grid->ProcessTableMessage( msg );
+        }
+
+        m_grid->SetCellValue( simEnableFieldRow, FDC_VALUE, wxT( "0" ) );
+        m_grid->SetCellValue( simEnableFieldRow, FDC_SHOWN, wxT( "0" ) );
+        m_grid->SetCellValue( simEnableFieldRow, FDC_SHOW_NAME, wxT( "0" ) );
+    }
+    else if( simEnableFieldRow >= 0 )
+    {
+        m_fields->erase( m_fields->begin() + simEnableFieldRow );
+
+        // notify the grid
+        wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_DELETED, simEnableFieldRow, 1 );
+        m_grid->ProcessTableMessage( msg );
+    }
+
+    OnModify();
+#endif
+}
+
+
 bool DIALOG_LIB_SYMBOL_PROPERTIES::Validate()
 {
     if( !m_grid->CommitPendingChanges() )
@@ -246,9 +293,9 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::Validate()
     }
 
     // Check for missing field names.
-    for( size_t i = MANDATORY_FIELDS;  i < m_fields->size(); ++i )
+    for( int ii = MANDATORY_FIELDS; ii < (int) m_fields->size(); ++ii )
     {
-        LIB_FIELD& field = m_fields->at( i );
+        LIB_FIELD& field = m_fields->at( ii );
         wxString   fieldName = field.GetName( false );
 
         if( fieldName.IsEmpty() )
@@ -259,7 +306,7 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::Validate()
             m_delayedErrorMessage = _( "Fields must have a name." );
             m_delayedFocusGrid = m_grid;
             m_delayedFocusColumn = FDC_NAME;
-            m_delayedFocusRow = i;
+            m_delayedFocusRow = ii;
             m_delayedFocusPage = 0;
 
             return false;
@@ -273,7 +320,7 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::Validate()
 
         if( parentName.IsEmpty() )
         {
-            m_delayedErrorMessage = _( "Aliased symbol must have a parent selected" );
+            m_delayedErrorMessage = _( "Derived symbol must have a parent selected" );
 
             return false;
         }
@@ -307,8 +354,7 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::TransferDataFromWindow()
     if( !m_grid->CommitPendingChanges() )
         return false;
 
-    // We need to keep the name and the value the same at the moment!
-    wxString   newName = m_fields->at( VALUE_FIELD ).GetText();
+    wxString   newName = EscapeString( m_SymbolNameCtrl->GetValue(), CTX_LIBID );
     wxString   oldName = m_libEntry->GetName();
 
     if( oldName != newName )
@@ -319,7 +365,7 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::TransferDataFromWindow()
         {
             wxString msg;
 
-            msg.Printf( _( "The name '%s' conflicts with an existing entry in the library '%s'." ),
+            msg.Printf( _( "Symbol name '%s' already in use in library '%s'." ),
                         UnescapeString( newName ),
                         libName );
             DisplayErrorMessage( this, msg );
@@ -335,12 +381,12 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::TransferDataFromWindow()
 
     // The Y axis for components in lib is from bottom to top while the screen axis is top
     // to bottom: we must change the y coord sign when writing back to the library
-    for( size_t i = 0; i < m_fields->size(); ++i )
+    for( int ii = 0; ii < (int) m_fields->size(); ++ii )
     {
-        wxPoint pos = m_fields->at( i ).GetPosition();
+        VECTOR2I pos = m_fields->at( ii ).GetPosition();
         pos.y = -pos.y;
-        m_fields->at( i ).SetPosition( pos );
-        m_fields->at( i ).SetId( i );
+        m_fields->at( ii ).SetPosition( pos );
+        m_fields->at( ii ).SetId( ii );
     }
 
     m_libEntry->SetFields( *m_fields );
@@ -365,7 +411,6 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::TransferDataFromWindow()
         m_libEntry->SetParent( newParent );
     }
 
-    // We need to keep the name and the value the same at the moment!
     m_libEntry->SetName( newName );
     m_libEntry->SetDescription( m_DescCtrl->GetValue() );
     m_libEntry->SetKeyWords( m_KeywordCtrl->GetValue() );
@@ -376,9 +421,15 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::TransferDataFromWindow()
     m_Parent->SetShowDeMorgan( m_AsConvertButt->GetValue() );
 
     if( m_OptionPower->GetValue() )
+    {
         m_libEntry->SetPower();
+        // Power symbols must have value matching name for now
+        m_libEntry->GetValueField().SetText( newName );
+    }
     else
+    {
         m_libEntry->SetNormal();
+    }
 
     m_libEntry->SetIncludeInBom( !m_excludeFromBomCheckBox->GetValue() );
     m_libEntry->SetIncludeOnBoard( !m_excludeFromBoardCheckBox->GetValue() );
@@ -444,10 +495,6 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnGridCellChanging( wxGridEvent& event )
             }
         }
     }
-    else if( event.GetRow() == VALUE_FIELD && event.GetCol() == FDC_VALUE )
-    {
-        m_SymbolNameCtrl->ChangeValue( event.GetString() );
-    }
 
     editor->DecRef();
 }
@@ -455,7 +502,7 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnGridCellChanging( wxGridEvent& event )
 
 void DIALOG_LIB_SYMBOL_PROPERTIES::OnSymbolNameText( wxCommandEvent& event )
 {
-    if( !m_Parent->IsSymbolFromSchematic() || m_OptionPower->IsChecked() )
+    if( m_OptionPower->IsChecked() )
         m_grid->SetCellValue( VALUE_FIELD, FDC_VALUE, m_SymbolNameCtrl->GetValue() );
 }
 
@@ -487,11 +534,11 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnAddField( wxCommandEvent& event )
         return;
 
     SYMBOL_EDITOR_SETTINGS* settings = m_Parent->GetSettings();
-    int       fieldID = m_fields->size();
+    int       fieldID = (int) m_fields->size();
     LIB_FIELD newField( m_libEntry, fieldID );
 
-    newField.SetTextSize( wxSize( Mils2iu( settings->m_Defaults.text_size ),
-                                  Mils2iu( settings->m_Defaults.text_size ) ) );
+    newField.SetTextSize( wxSize( schIUScale.MilsToIU( settings->m_Defaults.text_size ),
+                                  schIUScale.MilsToIU( settings->m_Defaults.text_size ) ) );
 
     m_fields->push_back( newField );
 
@@ -608,25 +655,67 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnMoveDown( wxCommandEvent& event )
 void DIALOG_LIB_SYMBOL_PROPERTIES::OnEditSpiceModel( wxCommandEvent& event )
 {
 #ifdef KICAD_SPICE
-    int diff = m_fields->size();
-    auto symbol = SCH_SYMBOL( *m_libEntry, m_libEntry->GetLibId(), nullptr, 0 );
+    if( !m_grid->CommitPendingChanges() )
+        return;
 
-    DIALOG_SPICE_MODEL dialog( this, symbol, m_fields );
+    std::vector<LIB_FIELD> fields;
+
+    for( const LIB_FIELD& field : *m_fields )
+        fields.emplace_back( field );
+
+    DIALOG_SIM_MODEL dialog( this, *m_libEntry, fields );
 
     if( dialog.ShowModal() != wxID_OK )
         return;
 
-    diff = (int) m_fields->size() - diff;
+    // Add in any new fields
+    for( const LIB_FIELD& editedField : fields )
+    {
+        bool found = false;
 
-    if( diff > 0 )
-    {
-        wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, diff );
-        m_grid->ProcessTableMessage( msg );
+        for( LIB_FIELD& existingField : *m_fields )
+        {
+            if( existingField.GetName() == editedField.GetName() )
+            {
+                found = true;
+                existingField.SetText( editedField.GetText() );
+                break;
+            }
+        }
+
+        if( !found )
+        {
+            m_fields->emplace_back( editedField );
+            wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, 1 );
+            m_grid->ProcessTableMessage( msg );
+        }
     }
-    else if( diff < 0 )
+
+    // Remove any deleted fields
+    for( int ii = (int) m_fields->size() - 1; ii >= 0; /* advance in loop */ )
     {
-        wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_DELETED, 0, -diff );
-        m_grid->ProcessTableMessage( msg );
+        LIB_FIELD& existingField = m_fields->at( ii );
+        bool       found = false;
+
+        for( LIB_FIELD& editedField : fields )
+        {
+            if( editedField.GetName() == existingField.GetName() )
+            {
+                found = true;
+                break;
+            }
+        }
+
+        if( found )
+        {
+            ii--;
+        }
+        else
+        {
+            m_fields->erase( m_fields->begin() + ii );
+            wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_DELETED, ii, 1 );
+            m_grid->ProcessTableMessage( msg );
+        }
     }
 
     OnModify();
@@ -712,21 +801,20 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnEditFootprintFilter( wxCommandEvent& event
 }
 
 
-void DIALOG_LIB_SYMBOL_PROPERTIES::adjustGridColumns( int aWidth )
+void DIALOG_LIB_SYMBOL_PROPERTIES::adjustGridColumns()
 {
-    m_width = aWidth;
-
     // Account for scroll bars
-    aWidth -= ( m_grid->GetSize().x - m_grid->GetClientSize().x );
+    int width = KIPLATFORM::UI::GetUnobscuredSize( m_grid ).x;
 
     m_grid->AutoSizeColumn( FDC_NAME );
+    m_grid->SetColSize( FDC_NAME, std::max( 72, m_grid->GetColSize( FDC_NAME ) ) );
 
     int fixedColsWidth = m_grid->GetColSize( FDC_NAME );
 
     for( int i = 2; i < m_grid->GetNumberCols(); i++ )
         fixedColsWidth += m_grid->GetColSize( i );
 
-    m_grid->SetColSize( FDC_VALUE, aWidth - fixedColsWidth );
+    m_grid->SetColSize( FDC_VALUE, std::max( 120, width - fixedColsWidth ) );
 }
 
 
@@ -740,7 +828,7 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnUpdateUI( wxUpdateUIEvent& event )
         int row = m_grid->GetGridCursorRow();
         int col = m_grid->GetGridCursorCol();
 
-        if( row == VALUE_FIELD && col == FDC_VALUE )
+        if( row == VALUE_FIELD && col == FDC_VALUE && m_OptionPower->IsChecked() )
         {
             wxGridCellEditor* editor = m_grid->GetCellEditor( row, col );
             m_SymbolNameCtrl->ChangeValue( editor->GetValue() );
@@ -756,7 +844,7 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnUpdateUI( wxUpdateUIEvent& event )
         m_shownColumns = shownColumns;
 
         if( !m_grid->IsCellEditControlShown() )
-            adjustGridColumns( m_grid->GetRect().GetWidth() );
+            adjustGridColumns();
     }
 
     // Handle a delayed focus.  The delay allows us to:
@@ -803,16 +891,33 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnUpdateUI( wxUpdateUIEvent& event )
         m_delayedFocusRow = -1;
         m_delayedFocusColumn = -1;
     }
+
+#ifdef KICAD_SPICE
+    wxString simEnable;
+
+    for( int ii = MANDATORY_FIELDS; ii < m_fields->GetNumberRows(); ++ii )
+    {
+        if( m_fields->GetValue( ii, FDC_NAME ) == SIM_ENABLE_FIELD )
+        {
+            simEnable = m_fields->GetValue( ii, FDC_VALUE );
+            break;
+        }
+    }
+
+    m_excludeFromSim->SetValue( simEnable == wxS( "0" ) );
+#endif
 }
 
 
 void DIALOG_LIB_SYMBOL_PROPERTIES::OnSizeGrid( wxSizeEvent& event )
 {
-    auto new_size = event.GetSize().GetX();
+    auto new_size = event.GetSize();
 
-    if( new_size != m_width )
+    if( new_size != m_size )
     {
-        adjustGridColumns( event.GetSize().GetX() );
+        m_size = new_size;
+
+        adjustGridColumns();
     }
 
     // Always propagate a wxSizeEvent:
