@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2017 Oliver Walters
- * Copyright (C) 2017-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2017-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,6 +28,7 @@
 #include <symbol_library.h>
 #include <confirm.h>
 #include <eda_doc.h>
+#include <wildcards_and_files_ext.h>
 #include <eeschema_settings.h>
 #include <general.h>
 #include <grid_tricks.h>
@@ -37,15 +38,16 @@
 #include <sch_reference_list.h>
 #include <schematic.h>
 #include <tools/sch_editor_control.h>
+#include <kiplatform/ui.h>
 #include <widgets/grid_text_button_helpers.h>
+#include <widgets/bitmap_button.h>
 #include <widgets/wx_grid.h>
+#include <wx/ffile.h>
 #include <wx/grid.h>
-#include <wx/msgdlg.h>
 #include <wx/textdlg.h>
-
+#include <wx/filedlg.h>
 #include "dialog_symbol_fields_table.h"
 
-
 #define DISPLAY_NAME_COLUMN   0
 #define SHOW_FIELD_COLUMN     1
 #define GROUP_BY_COLUMN       2
@@ -61,7 +63,7 @@
 
 enum
 {
-    MYID_SELECT_FOOTPRINT = GRIDTRICKS_FIRST_SHOWHIDE - 2, // must be within GRID_TRICKS' enum range
+    MYID_SELECT_FOOTPRINT = GRIDTRICKS_FIRST_CLIENT_ID,
     MYID_SHOW_DATASHEET
 };
 
@@ -114,7 +116,7 @@ protected:
         {
             wxString datasheet_uri = m_grid->GetCellValue( m_grid->GetGridCursorRow(),
                                                            DATASHEET_FIELD );
-            GetAssociatedDocument( m_dlg, datasheet_uri, &m_dlg->Prj() );
+            GetAssociatedDocument( m_dlg, datasheet_uri, &m_dlg->Prj(), m_dlg->Prj().SchSearchS() );
         }
         else
         {
@@ -175,6 +177,7 @@ protected:
     std::vector<wxString> m_fieldNames;
     int                   m_sortColumn;
     bool                  m_sortAscending;
+    std::vector<wxString> m_userAddedFields;
 
     // However, the grid view can vary in two ways:
     //   1) the componentRefs can be grouped into fewer rows
@@ -201,27 +204,43 @@ public:
         m_symbolsList.SplitReferences();
     }
 
-    void AddColumn( const wxString& aFieldName )
+    void AddColumn( const wxString& aFieldName, bool aAddedByUser )
     {
         m_fieldNames.push_back( aFieldName );
 
+        if( aAddedByUser )
+            m_userAddedFields.push_back( aFieldName );
+
         for( unsigned i = 0; i < m_symbolsList.GetCount(); ++i )
         {
             SCH_SYMBOL* symbol = m_symbolsList[ i ].GetSymbol();
 
             wxCHECK( symbol && ( symbol->GetInstanceReferences().size() != 0 ), /* void */ );
 
-            wxString val = symbol->GetFieldText( aFieldName, m_frame );
+            wxString val = symbol->GetFieldText( aFieldName );
 
             if( aFieldName == wxT( "Value" ) )
-                val = symbol->GetInstanceReferences()[0].m_Value;
+                val = symbol->GetValueFieldText( true );
             else if( aFieldName == wxT( "Footprint" ) )
-                val = symbol->GetInstanceReferences()[0].m_Footprint;
+                val = symbol->GetFootprintFieldText( true );
 
             m_dataStore[ symbol->m_Uuid ][ aFieldName ] = val;
         }
     }
 
+    void RemoveColumn( int aCol )
+    {
+        wxString fieldName = m_fieldNames[ aCol ];
+
+        m_fieldNames.erase( m_fieldNames.begin() + aCol );
+
+        for( unsigned i = 0; i < m_symbolsList.GetCount(); ++i )
+        {
+            SCH_SYMBOL* symbol = m_symbolsList[ i ].GetSymbol();
+            m_dataStore[ symbol->m_Uuid ].erase( fieldName );
+        }
+    }
+
     int GetNumberRows() override { return m_rows.size(); }
 
     // Columns are fieldNames + quantity column
@@ -232,6 +251,10 @@ public:
         if( aCol == QUANTITY_COLUMN )
             return _( "Qty" );
         else if( aCol < MANDATORY_FIELDS )
+            // FIX ME: the column label should be displayed translated.
+            // but when translated, and the DATASHEET column is shown, a new field
+            // with the translated DATASHEET field name is added when saving fields
+            // return TEMPLATE_FIELDNAME::GetDefaultFieldName( aCol, DO_TRANSLATE );
             return TEMPLATE_FIELDNAME::GetDefaultFieldName( aCol );
         else
             return m_fieldNames[ aCol ];
@@ -270,6 +293,16 @@ public:
         }
     }
 
+    wxString GetRawValue( int aRow, int aCol )
+    {
+        return GetValue( m_rows[ aRow ], aCol );
+    }
+
+    GROUP_TYPE GetRowFlags( int aRow )
+    {
+        return m_rows[ aRow ].m_Flag;
+    }
+
     std::vector<SCH_REFERENCE> GetRowReferences( int aRow ) const
     {
         wxCHECK( aRow < (int)m_rows.size(), std::vector<SCH_REFERENCE>() );
@@ -281,7 +314,7 @@ public:
         std::vector<SCH_REFERENCE> references;
         wxString                   fieldValue;
 
-        for( const auto& ref : group.m_Refs )
+        for( const SCH_REFERENCE& ref : group.m_Refs )
         {
             if( aCol == REFERENCE_FIELD || aCol == QUANTITY_COLUMN )
             {
@@ -347,7 +380,7 @@ public:
         DATA_MODEL_ROW& rowGroup = m_rows[ aRow ];
         wxString fieldName = m_fieldNames[ aCol ];
 
-        for( const auto& ref : rowGroup.m_Refs )
+        for( const SCH_REFERENCE& ref : rowGroup.m_Refs )
             m_dataStore[ ref.GetSymbol()->m_Uuid ][ fieldName ] = aValue;
 
         m_edited = true;
@@ -400,8 +433,21 @@ public:
 
         CollapseForSort();
 
+        // We're going to sort the rows based on their first reference, so the first reference
+        // had better be the lowest one.
+        for( DATA_MODEL_ROW& row : m_rows )
+        {
+            std::sort( row.m_Refs.begin(), row.m_Refs.end(),
+                    []( const SCH_REFERENCE& lhs, const SCH_REFERENCE& rhs )
+                    {
+                        wxString lhs_ref( lhs.GetRef() << lhs.GetRefNumber() );
+                        wxString rhs_ref( rhs.GetRef() << rhs.GetRefNumber() );
+                        return StrNumCmp( lhs_ref, rhs_ref, true ) < 0;
+                    } );
+        }
+
         std::sort( m_rows.begin(), m_rows.end(),
-               [ this ]( const DATA_MODEL_ROW& lhs, const DATA_MODEL_ROW& rhs ) -> bool
+               [this]( const DATA_MODEL_ROW& lhs, const DATA_MODEL_ROW& rhs ) -> bool
                {
                    return cmp( lhs, rhs, this, m_sortColumn, m_sortAscending );
                } );
@@ -455,9 +501,10 @@ public:
         return matchFound;
     }
 
-    void RebuildRows( wxCheckBox* aGroupSymbolsBox, wxDataViewListCtrl* aFieldsCtrl )
+    void RebuildRows( wxSearchCtrl* aFilter, wxCheckBox* aGroupSymbolsBox,
+                      wxDataViewListCtrl* aFieldsCtrl )
     {
-        if ( GetView() )
+        if( GetView() )
         {
             // Commit any pending in-place edits before the row gets moved out from under
             // the editor.
@@ -472,7 +519,14 @@ public:
         for( unsigned i = 0; i < m_symbolsList.GetCount(); ++i )
         {
             SCH_REFERENCE ref = m_symbolsList[ i ];
-            bool          matchFound = false;
+
+            if( !aFilter->GetValue().IsEmpty()
+                    && !WildCompareString( aFilter->GetValue(), ref.GetFullRef(), false ) )
+            {
+                continue;
+            }
+
+            bool matchFound = false;
 
             // See if we already have a row which this symbol fits into
             for( DATA_MODEL_ROW& row : m_rows )
@@ -610,39 +664,52 @@ public:
 
             for( const std::pair<wxString, wxString> srcData : fieldStore )
             {
+                if( srcData.first == _( "Qty" ) )
+                    continue;
+
                 const wxString& srcName = srcData.first;
                 const wxString& srcValue = srcData.second;
                 SCH_FIELD*      destField = symbol.FindField( srcName );
 
-                if( !destField && !srcValue.IsEmpty() )
+                // Add a not existing field if it has a value for this symbol
+                bool createField =
+                        !destField
+                        && ( !srcValue.IsEmpty() || alg::contains( m_userAddedFields, srcName ) );
+
+                if( createField )
                 {
-                    const wxPoint symbolPos = symbol.GetPosition();
+                    const VECTOR2I symbolPos = symbol.GetPosition();
                     destField = symbol.AddField( SCH_FIELD( symbolPos, -1, &symbol, srcName ) );
                 }
 
                 if( !destField )
+                    continue;
+
+                if( destField->GetId() == REFERENCE_FIELD )
                 {
-                    symbol.RemoveField( srcName );
-                }
-                else if( destField->GetId() == REFERENCE_FIELD )
-                {
-                    // Reference is not editable
+                    // Reference is not editable from this dialog
                 }
                 else if( destField->GetId() == VALUE_FIELD )
                 {
                     // Value field cannot be empty
                     if( !srcValue.IsEmpty() )
-                        symbol.SetValue( srcValue );
+                        symbol.SetValueFieldText( srcValue );
                 }
                 else if( destField->GetId() == FOOTPRINT_FIELD )
                 {
-                    symbol.SetFootprint( srcValue );
+                    symbol.SetFootprintFieldText( srcValue );
                 }
                 else
                 {
                     destField->SetText( srcValue );
                 }
             }
+
+            for( int ii = symbol.GetFields().size() - 1; ii >= MANDATORY_FIELDS; ii-- )
+            {
+                if( fieldStore.count( symbol.GetFields()[ii].GetName() ) == 0 )
+                    symbol.GetFields().erase( symbol.GetFields().begin() + ii );
+            }
         }
 
         m_edited = false;
@@ -686,10 +753,13 @@ DIALOG_SYMBOL_FIELDS_TABLE::DIALOG_SYMBOL_FIELDS_TABLE( SCH_EDIT_FRAME* parent )
         m_parent( parent )
 {
     wxSize defaultDlgSize = ConvertDialogToPixels( wxSize( 600, 300 ) );
+    int    nameColWidthMargin = 44;
 
     // Get all symbols from the list of schematic sheets
     m_parent->Schematic().GetSheets().GetSymbols( m_symbolsList, false );
 
+    m_separator1->SetIsSeparator();
+    m_separator2->SetIsSeparator();
     m_bRefresh->SetBitmap( KiBitmap( BITMAPS::small_refresh ) );
 
     m_fieldsCtrl->AppendTextColumn( _( "Field" ), wxDATAVIEW_CELL_INERT, 0, wxALIGN_LEFT, 0 );
@@ -716,6 +786,8 @@ DIALOG_SYMBOL_FIELDS_TABLE::DIALOG_SYMBOL_FIELDS_TABLE( SCH_EDIT_FRAME* parent )
     // expander buttons... but it doesn't.  Fix by forcing the indent to 0.
     m_fieldsCtrl->SetIndent( 0 );
 
+    m_filter->SetDescriptiveText( _( "Filter" ) );
+
     m_dataModel = new FIELDS_EDITOR_GRID_DATA_MODEL( m_parent, m_symbolsList );
 
     LoadFieldNames();   // loads rows into m_fieldsCtrl and columns into m_dataModel
@@ -730,6 +802,8 @@ DIALOG_SYMBOL_FIELDS_TABLE::DIALOG_SYMBOL_FIELDS_TABLE( SCH_EDIT_FRAME* parent )
         nameColWidth = std::max( nameColWidth, KIUI::GetTextSize( fieldName, m_fieldsCtrl ).x );
     }
 
+    nameColWidth += nameColWidthMargin;
+
     int fieldsMinWidth = nameColWidth + m_groupByColWidth + m_showColWidth;
 
     m_fieldsCtrl->GetColumn( DISPLAY_NAME_COLUMN )->SetWidth( nameColWidth );
@@ -740,12 +814,10 @@ DIALOG_SYMBOL_FIELDS_TABLE::DIALOG_SYMBOL_FIELDS_TABLE( SCH_EDIT_FRAME* parent )
     m_splitterMainWindow->SetMinimumPaneSize( fieldsMinWidth );
     m_splitterMainWindow->SetSashPosition( fieldsMinWidth + 40 );
 
-    m_dataModel->RebuildRows( m_groupSymbolsBox, m_fieldsCtrl );
+    m_dataModel->RebuildRows( m_filter, m_groupSymbolsBox, m_fieldsCtrl );
     m_dataModel->Sort( 0, true );
 
-    // wxGrid's column moving is buggy with native headers and this is one dialog where you'd
-    // really like to be able to rearrange columns.
-    m_grid->UseNativeColHeader( false );
+    m_grid->UseNativeColHeader( true );
     m_grid->SetTable( m_dataModel, true );
 
     // must be done after SetTable(), which appears to re-set it
@@ -773,12 +845,12 @@ DIALOG_SYMBOL_FIELDS_TABLE::DIALOG_SYMBOL_FIELDS_TABLE( SCH_EDIT_FRAME* parent )
 
     // set footprint column browse button
     attr = new wxGridCellAttr;
-    attr->SetEditor( new GRID_CELL_FOOTPRINT_ID_EDITOR( this ) );
+    attr->SetEditor( new GRID_CELL_FPID_EDITOR( this, wxEmptyString ) );
     m_grid->SetColAttr( FOOTPRINT_FIELD, attr );
 
     // set datasheet column viewer button
     attr = new wxGridCellAttr;
-    attr->SetEditor( new GRID_CELL_URL_EDITOR( this ) );
+    attr->SetEditor( new GRID_CELL_URL_EDITOR( this, Prj().SchSearchS() ) );
     m_grid->SetColAttr( DATASHEET_FIELD, attr );
 
     // set quantities column attributes
@@ -819,7 +891,7 @@ DIALOG_SYMBOL_FIELDS_TABLE::DIALOG_SYMBOL_FIELDS_TABLE( SCH_EDIT_FRAME* parent )
     m_grid->SetGridCursor( 0, 1 );
     SetInitialFocus( m_grid );
 
-    m_sdbSizerOK->SetDefault();
+    SetupStandardButtons();
 
     finishDialogSettings();
     SetSize( defaultDlgSize );
@@ -918,9 +990,9 @@ bool DIALOG_SYMBOL_FIELDS_TABLE::TransferDataFromWindow()
 
 void DIALOG_SYMBOL_FIELDS_TABLE::AddField( const wxString& aDisplayName,
                                            const wxString& aCanonicalName,
-                                           bool defaultShow, bool defaultSortBy )
+                                           bool defaultShow, bool defaultSortBy, bool addedByUser )
 {
-    m_dataModel->AddColumn( aCanonicalName );
+    m_dataModel->AddColumn( aCanonicalName, addedByUser );
 
     wxVector<wxVariant> fieldsCtrlRow;
 
@@ -959,7 +1031,7 @@ void DIALOG_SYMBOL_FIELDS_TABLE::LoadFieldNames()
     }
 
     // Force References to always be shown
-    auto cfg = dynamic_cast<EESCHEMA_SETTINGS*>( Kiface().KifaceSettings() );
+    EESCHEMA_SETTINGS* cfg = dynamic_cast<EESCHEMA_SETTINGS*>( Kiface().KifaceSettings() );
     wxCHECK( cfg, /*void*/ );
 
     cfg->m_FieldEditorPanel.fields_show["Reference"] = true;
@@ -984,11 +1056,6 @@ void DIALOG_SYMBOL_FIELDS_TABLE::LoadFieldNames()
 
 void DIALOG_SYMBOL_FIELDS_TABLE::OnAddField( wxCommandEvent& event )
 {
-    // quantities column will become new field column, so it needs to be reset
-    auto attr = new wxGridCellAttr;
-    m_grid->SetColAttr( m_dataModel->GetColsCount() - 1, attr );
-    m_grid->SetColFormatCustom( m_dataModel->GetColsCount() - 1, wxGRID_VALUE_STRING );
-
     wxTextEntryDialog dlg( this, _( "New field name:" ), _( "Add Field" ) );
 
     if( dlg.ShowModal() != wxID_OK )
@@ -1012,12 +1079,17 @@ void DIALOG_SYMBOL_FIELDS_TABLE::OnAddField( wxCommandEvent& event )
         }
     }
 
+    // quantities column will become new field column, so it needs to be reset
+    wxGridCellAttr* attr = new wxGridCellAttr;
+    m_grid->SetColAttr( m_dataModel->GetColsCount() - 1, attr );
+    m_grid->SetColFormatCustom( m_dataModel->GetColsCount() - 1, wxGRID_VALUE_STRING );
+
     std::string key( fieldName.ToUTF8() );
 
-    auto cfg = static_cast<EESCHEMA_SETTINGS*>( Kiface().KifaceSettings() );
+    EESCHEMA_SETTINGS* cfg = static_cast<EESCHEMA_SETTINGS*>( Kiface().KifaceSettings() );
     cfg->m_FieldEditorPanel.fields_show[key] = true;
 
-    AddField( fieldName, fieldName, true, false );
+    AddField( fieldName, fieldName, true, false, true );
 
     wxGridTableMessage msg( m_dataModel, wxGRIDTABLE_NOTIFY_COLS_INSERTED,
                             m_fieldsCtrl->GetItemCount(), 1 );
@@ -1032,6 +1104,87 @@ void DIALOG_SYMBOL_FIELDS_TABLE::OnAddField( wxCommandEvent& event )
 }
 
 
+void DIALOG_SYMBOL_FIELDS_TABLE::OnRemoveField( wxCommandEvent& event )
+{
+    int col = -1;
+    int row = m_fieldsCtrl->GetSelectedRow();
+
+   // Should never occur: "Remove Field..." button should be disabled if invalid selection
+   // via OnFieldsCtrlSelectionChanged()
+    wxCHECK_RET( row != -1, wxS( "Some user defined field must be selected first" ) );
+    wxCHECK_RET( row >= MANDATORY_FIELDS, wxS( "Mandatory fields cannot be removed" ) );
+
+    wxString fieldName = m_fieldsCtrl->GetTextValue( row, 0 );
+
+    wxString confirm_msg = wxString::Format( _( "Are you sure you want to remove the field '%s'?" ),
+                                        fieldName );
+
+    if( !IsOK( this, confirm_msg ) )
+        return;
+
+    for( int i = 0; i < m_dataModel->GetNumberCols(); ++i )
+    {
+         if( fieldName == m_dataModel->GetColLabelValue( i ) )
+            col = i;
+    }
+
+    m_fieldsCtrl->DeleteItem( row );
+    m_dataModel->RemoveColumn( col );
+
+    // Make selection and update the state of "Remove field..." button via OnFieldsCtrlSelectionChanged()
+    // Safe to decrement row index because we always have mandatory fields
+    m_fieldsCtrl->SelectRow( --row );
+
+    if( row < MANDATORY_FIELDS )
+        m_removeFieldButton->Enable( false );
+
+    wxGridTableMessage msg( m_dataModel, wxGRIDTABLE_NOTIFY_COLS_DELETED,
+                                m_fieldsCtrl->GetItemCount(), 1 );
+
+    m_grid->ProcessTableMessage( msg );
+
+    // set up attributes on the new quantities column
+    wxGridCellAttr* attr = new wxGridCellAttr;
+    attr->SetReadOnly();
+
+    m_grid->SetColAttr( m_dataModel->GetColsCount() - 1, attr );
+    m_grid->SetColFormatNumber( m_dataModel->GetColsCount() - 1 );
+    m_grid->SetColSize( m_dataModel->GetColsCount() - 1, 50 );
+}
+
+
+void DIALOG_SYMBOL_FIELDS_TABLE::OnFilterText( wxCommandEvent& aEvent )
+{
+    m_dataModel->RebuildRows( m_filter, m_groupSymbolsBox, m_fieldsCtrl );
+    m_dataModel->Sort( m_grid->GetSortingColumn(), m_grid->IsSortOrderAscending() );
+    m_grid->ForceRefresh();
+}
+
+
+void DIALOG_SYMBOL_FIELDS_TABLE::OnFilterMouseMoved( wxMouseEvent& aEvent )
+{
+    wxPoint pos = aEvent.GetPosition();
+    wxRect  ctrlRect = m_filter->GetScreenRect();
+    int     buttonWidth = ctrlRect.GetHeight();         // Presume buttons are square
+
+    if( m_filter->IsSearchButtonVisible() && pos.x < buttonWidth )
+        SetCursor( wxCURSOR_ARROW );
+    else if( m_filter->IsCancelButtonVisible() && pos.x > ctrlRect.GetWidth() - buttonWidth )
+        SetCursor( wxCURSOR_ARROW );
+    else
+        SetCursor( wxCURSOR_IBEAM );
+}
+
+void DIALOG_SYMBOL_FIELDS_TABLE::OnFieldsCtrlSelectionChanged( wxDataViewEvent& event )
+{
+    int row = m_fieldsCtrl->GetSelectedRow();
+
+    if( row >= MANDATORY_FIELDS )
+        m_removeFieldButton->Enable( true );
+    else
+        m_removeFieldButton->Enable( false );
+}
+
 void DIALOG_SYMBOL_FIELDS_TABLE::OnColumnItemToggled( wxDataViewEvent& event )
 {
     EESCHEMA_SETTINGS* cfg = static_cast<EESCHEMA_SETTINGS*>( Kiface().KifaceSettings() );
@@ -1071,7 +1224,7 @@ void DIALOG_SYMBOL_FIELDS_TABLE::OnColumnItemToggled( wxDataViewEvent& event )
         std::string fieldName( m_fieldsCtrl->GetTextValue( row, CANONICAL_NAME_COLUMN ).ToUTF8() );
         cfg->m_FieldEditorPanel.fields_group_by[fieldName] = value;
 
-        m_dataModel->RebuildRows( m_groupSymbolsBox, m_fieldsCtrl );
+        m_dataModel->RebuildRows( m_filter, m_groupSymbolsBox, m_fieldsCtrl );
         m_dataModel->Sort( m_grid->GetSortingColumn(), m_grid->IsSortOrderAscending() );
         m_grid->ForceRefresh();
         break;
@@ -1085,7 +1238,7 @@ void DIALOG_SYMBOL_FIELDS_TABLE::OnColumnItemToggled( wxDataViewEvent& event )
 
 void DIALOG_SYMBOL_FIELDS_TABLE::OnGroupSymbolsToggled( wxCommandEvent& event )
 {
-    m_dataModel->RebuildRows( m_groupSymbolsBox, m_fieldsCtrl );
+    m_dataModel->RebuildRows( m_filter, m_groupSymbolsBox, m_fieldsCtrl );
     m_dataModel->Sort( m_grid->GetSortingColumn(), m_grid->IsSortOrderAscending() );
     m_grid->ForceRefresh();
 }
@@ -1135,7 +1288,7 @@ void DIALOG_SYMBOL_FIELDS_TABLE::OnTableColSize( wxGridSizeEvent& aEvent )
 
 void DIALOG_SYMBOL_FIELDS_TABLE::OnRegroupSymbols( wxCommandEvent& aEvent )
 {
-    m_dataModel->RebuildRows( m_groupSymbolsBox, m_fieldsCtrl );
+    m_dataModel->RebuildRows( m_filter, m_groupSymbolsBox, m_fieldsCtrl );
     m_dataModel->Sort( m_grid->GetSortingColumn(), m_grid->IsSortOrderAscending() );
     m_grid->ForceRefresh();
 }
@@ -1149,18 +1302,6 @@ void DIALOG_SYMBOL_FIELDS_TABLE::OnTableCellClick( wxGridEvent& event )
         m_grid->SetGridCursor( event.GetRow(), event.GetCol() );
 
         m_dataModel->ExpandCollapseRow( event.GetRow() );
-        std::vector<SCH_REFERENCE> refs = m_dataModel->GetRowReferences( event.GetRow() );
-
-        // Focus Eeschema view on the symbol selected in the dialog
-        if( refs.size() == 1 )
-        {
-            SCH_EDITOR_CONTROL* editor = m_parent->GetToolManager()->GetTool<SCH_EDITOR_CONTROL>();
-
-            wxString path = refs[0].GetPath();
-            wxString reference = refs[0].GetRef() + refs[0].GetRefNumber();
-
-            editor->FindSymbolAndItem( &path, &reference, true, HIGHLIGHT_SYMBOL, wxEmptyString );
-        }
     }
     else
     {
@@ -1168,6 +1309,40 @@ void DIALOG_SYMBOL_FIELDS_TABLE::OnTableCellClick( wxGridEvent& event )
     }
 }
 
+void DIALOG_SYMBOL_FIELDS_TABLE::OnTableRangeSelected( wxGridRangeSelectEvent& event )
+{
+    wxGridCellCoordsArray selectedCells = m_grid->GetSelectedCells();
+
+    if( selectedCells.GetCount() == 1 )
+    {
+        int row = selectedCells[0].GetRow();
+        int flag = m_dataModel->GetRowFlags( row );
+        std::vector<SCH_REFERENCE> refs = m_dataModel->GetRowReferences( row );
+
+        // Focus Eeschema view on the symbol selected in the dialog
+        // TODO: Highlight or select more than one unit
+        if( ( flag == GROUP_SINGLETON || flag == CHILD_ITEM ) && refs.size() >= 1 )
+        {
+            SCH_EDITOR_CONTROL* editor = m_parent->GetToolManager()->GetTool<SCH_EDITOR_CONTROL>();
+
+            std::sort( refs.begin(), refs.end(),
+                    []( const SCH_REFERENCE& a, const SCH_REFERENCE& b )
+                    {
+                        return a.GetUnit() < b.GetUnit();
+                    } );
+
+            // search and highlight the symbol found by its full path.
+            // It allows select of not yet annotated or duplicaded symbols
+            wxString symbol_path = refs[0].GetFullPath();
+            // wxString reference = refs[0].GetRef() + refs[0].GetRefNumber();  // Not used
+            editor->FindSymbolAndItem( &symbol_path, nullptr, true, HIGHLIGHT_SYMBOL, wxEmptyString );
+        }
+
+        return;
+    }
+
+    event.Skip();
+}
 
 void DIALOG_SYMBOL_FIELDS_TABLE::OnTableItemContextMenu( wxGridEvent& event )
 {
@@ -1179,8 +1354,14 @@ void DIALOG_SYMBOL_FIELDS_TABLE::OnTableItemContextMenu( wxGridEvent& event )
 
 void DIALOG_SYMBOL_FIELDS_TABLE::OnSizeFieldList( wxSizeEvent& event )
 {
-    int nameColWidth = event.GetSize().GetX() - m_showColWidth - m_groupByColWidth
-                                                - wxSystemSettings::GetMetric( wxSYS_VSCROLL_X );
+    int nameColWidth = KIPLATFORM::UI::GetUnobscuredSize( m_fieldsCtrl ).x;
+    nameColWidth -= m_showColWidth + m_groupByColWidth;
+#ifdef __WXMAC__
+    // TODO: something in wxWidgets 3.1.x pads checkbox columns with extra space.  (It used to
+    // also be that the width of the column would get set too wide (to 30), but that's patched in
+    // our local wxWidgets fork.)
+    nameColWidth -= 30;
+#endif
 
     // GTK loses its head and messes these up when resizing the splitter bar:
     m_fieldsCtrl->GetColumn( SHOW_FIELD_COLUMN )->SetWidth( m_showColWidth );
@@ -1189,6 +1370,8 @@ void DIALOG_SYMBOL_FIELDS_TABLE::OnSizeFieldList( wxSizeEvent& event )
     m_fieldsCtrl->GetColumn( CANONICAL_NAME_COLUMN )->SetHidden( true );
     m_fieldsCtrl->GetColumn( DISPLAY_NAME_COLUMN )->SetWidth( nameColWidth );
 
+    m_fieldsCtrl->Refresh(); // To refresh checkboxes on Windows.
+
     event.Skip();
 }
 
@@ -1200,6 +1383,81 @@ void DIALOG_SYMBOL_FIELDS_TABLE::OnSaveAndContinue( wxCommandEvent& aEvent )
 }
 
 
+void DIALOG_SYMBOL_FIELDS_TABLE::OnExport( wxCommandEvent& aEvent )
+{
+    int last_col = m_grid->GetNumberCols() - 1;
+
+    if( m_dataModel->IsEdited() )
+        if( OKOrCancelDialog( nullptr, _( "Unsaved data" ),
+                              _( "Changes are unsaved. Export unsaved data?" ), "", _( "OK" ),
+                              _( "Cancel" ) )
+            == wxID_CANCEL )
+            return;
+
+
+    // Calculate the netlist filename
+    wxFileName fn = m_parent->Schematic().GetFileName();
+    fn.SetExt( CsvFileExtension );
+
+    wxFileDialog saveDlg( this, _( "Save as CSV" ), wxPathOnly( Prj().GetProjectFullName() ),
+                          fn.GetFullName(), CsvFileWildcard(), wxFD_SAVE | wxFD_OVERWRITE_PROMPT );
+
+    if( saveDlg.ShowModal() == wxID_CANCEL )
+        return;
+
+    wxFFile out( saveDlg.GetPath(), "wb" );
+
+    if( !out.IsOpened() )
+        return;
+
+    // Find the location for the line terminator
+    for( int col = m_grid->GetNumberCols() - 1; col >=0 ; --col )
+    {
+        if( m_grid->IsColShown( col ) )
+        {
+            last_col = col;
+            break;
+        }
+    }
+
+    // Column names
+    for( int col = 0; col < m_grid->GetNumberCols(); col++ )
+    {
+        if( !m_grid->IsColShown( col ) )
+            continue;
+
+        wxString escapedValue = m_grid->GetColLabelValue( col );
+        escapedValue.Replace( wxS( "\"" ), wxS( "\"\"" ) );
+
+        wxString format = col == last_col ? wxS( "\"%s\"\r\n" ) : wxS( "\"%s\"," );
+
+        out.Write( wxString::Format( format, escapedValue ) );
+    }
+
+    // Data rows
+    for( int row = 0; row < m_grid->GetNumberRows(); row++ )
+    {
+        // Don't output child rows
+        if( m_dataModel->GetRowFlags( row ) == CHILD_ITEM )
+            continue;
+
+        for( int col = 0; col < m_grid->GetNumberCols(); col++ )
+        {
+            if( !m_grid->IsColShown( col ) )
+                continue;
+
+            // Get the unanottated version of the field, e.g. no ">   " or "v   " by
+            wxString escapedValue = m_dataModel->GetRawValue( row, col );
+            escapedValue.Replace( wxS( "\"" ), wxS( "\"\"" ) );
+
+            wxString format = col == last_col ? wxS( "\"%s\"\r\n" ) : wxS( "\"%s\"," );
+
+            out.Write( wxString::Format( format, escapedValue ) );
+        }
+    }
+}
+
+
 void DIALOG_SYMBOL_FIELDS_TABLE::OnCancel( wxCommandEvent& event )
 {
     Close();
