@@ -23,6 +23,8 @@
 
 #include "dialog_symbol_properties.h"
 
+#include <memory>
+
 #include <bitmaps.h>
 #include <wx/tooltip.h>
 #include <grid_tricks.h>
@@ -31,9 +33,10 @@
 #include <pin_numbers.h>
 #include <string_utils.h>
 #include <menus_helpers.h>
+#include <kiplatform/ui.h>
 #include <widgets/grid_icon_text_helpers.h>
 #include <widgets/grid_combobox.h>
-#include <widgets/wx_grid.h>
+#include <widgets/std_bitmap_button.h>
 #include <settings/settings_manager.h>
 #include <ee_collectors.h>
 #include <symbol_library.h>
@@ -43,12 +46,16 @@
 #include <schematic.h>
 #include <tool/tool_manager.h>
 #include <tool/actions.h>
+#include <math/vector2d.h>
 
 #ifdef KICAD_SPICE
-#include <dialog_spice_model.h>
+#include <dialog_sim_model.h>
 #endif /* KICAD_SPICE */
 
 
+wxDEFINE_EVENT( SYMBOL_DELAY_FOCUS, wxCommandEvent );
+wxDEFINE_EVENT( SYMBOL_DELAY_SELECTION, wxCommandEvent );
+
 enum PIN_TABLE_COL_ORDER
 {
     COL_NUMBER,
@@ -292,6 +299,9 @@ DIALOG_SYMBOL_PROPERTIES::DIALOG_SYMBOL_PROPERTIES( SCH_EDIT_FRAME* aParent,
         DIALOG_SYMBOL_PROPERTIES_BASE( aParent ),
         m_symbol( nullptr ),
         m_part( nullptr ),
+        m_fieldsSize( 0, 0 ),
+        m_lastRequestedSize( 0, 0 ),
+        m_editorShown( false ),
         m_fields( nullptr ),
         m_dataModel( nullptr )
 {
@@ -303,26 +313,13 @@ DIALOG_SYMBOL_PROPERTIES::DIALOG_SYMBOL_PROPERTIES( SCH_EDIT_FRAME* aParent,
     // so we need to handle m_part == nullptr
     // wxASSERT( m_part );
 
-    m_fields = new FIELDS_GRID_TABLE<SCH_FIELD>( this, aParent, m_fieldsGrid, m_part );
-
-    m_width = 0;
-    m_delayedFocusRow = REFERENCE_FIELD;
-    m_delayedFocusColumn = FDC_VALUE;
-    m_delayedSelection = true;
+    m_fields = new FIELDS_GRID_TABLE<SCH_FIELD>( this, aParent, m_fieldsGrid, m_symbol );
 
 #ifndef KICAD_SPICE
+    m_cbExcludeFromSim->Hide();
     m_spiceFieldsButton->Hide();
 #endif /* not KICAD_SPICE */
 
-    // disable some options inside the edit dialog which can cause problems while dragging
-    if( m_symbol->IsDragging() )
-    {
-        m_orientationLabel->Disable();
-        m_orientationCtrl->Disable();
-        m_mirrorLabel->Disable();
-        m_mirrorCtrl->Disable();
-    }
-
     // Give a bit more room for combobox editors
     m_fieldsGrid->SetDefaultRowSize( m_fieldsGrid->GetDefaultRowSize() + 4 );
     m_pinGrid->SetDefaultRowSize( m_pinGrid->GetDefaultRowSize() + 4 );
@@ -336,7 +333,7 @@ DIALOG_SYMBOL_PROPERTIES::DIALOG_SYMBOL_PROPERTIES( SCH_EDIT_FRAME* aParent,
 
     if( cfg )
     {
-        m_shownColumns = cfg->m_Appearance.edit_component_visible_columns;
+        m_shownColumns = cfg->m_Appearance.edit_symbol_visible_columns;
         m_fieldsGrid->ShowHideColumns( m_shownColumns );
     }
 
@@ -367,7 +364,7 @@ DIALOG_SYMBOL_PROPERTIES::DIALOG_SYMBOL_PROPERTIES( SCH_EDIT_FRAME* aParent,
     m_pinGrid->SetSelectionMode( wxGrid::wxGridSelectRows );
 
     wxToolTip::Enable( true );
-    m_stdDialogButtonSizerOK->SetDefault();
+    SetupStandardButtons();
 
     // Configure button logos
     m_bpAdd->SetBitmap( KiBitmap( BITMAPS::small_plus ) );
@@ -376,13 +373,16 @@ DIALOG_SYMBOL_PROPERTIES::DIALOG_SYMBOL_PROPERTIES( SCH_EDIT_FRAME* aParent,
     m_bpMoveDown->SetBitmap( KiBitmap( BITMAPS::small_down ) );
 
     // wxFormBuilder doesn't include this event...
-    m_fieldsGrid->Connect( wxEVT_GRID_CELL_CHANGING,
-                           wxGridEventHandler( DIALOG_SYMBOL_PROPERTIES::OnGridCellChanging ),
-                           nullptr, this );
+    m_fieldsGrid->Bind( wxEVT_GRID_CELL_CHANGING, &DIALOG_SYMBOL_PROPERTIES::OnGridCellChanging,
+                        this );
+    m_pinGrid->Bind( wxEVT_GRID_COL_SORT, &DIALOG_SYMBOL_PROPERTIES::OnPinTableColSort, this );
+    Bind( SYMBOL_DELAY_FOCUS, &DIALOG_SYMBOL_PROPERTIES::HandleDelayedFocus, this );
+    Bind( SYMBOL_DELAY_SELECTION, &DIALOG_SYMBOL_PROPERTIES::HandleDelayedSelection, this );
 
-    m_pinGrid->Connect( wxEVT_GRID_COL_SORT,
-                        wxGridEventHandler( DIALOG_SYMBOL_PROPERTIES::OnPinTableColSort ),
-                        nullptr, this );
+    QueueEvent( new wxCommandEvent( SYMBOL_DELAY_SELECTION ) );
+    wxCommandEvent *evt = new wxCommandEvent( SYMBOL_DELAY_FOCUS );
+    evt->SetClientData( new VECTOR2I( REFERENCE_FIELD, FDC_VALUE ) );
+    QueueEvent( evt );
 
     finishDialogSettings();
 }
@@ -393,7 +393,11 @@ DIALOG_SYMBOL_PROPERTIES::~DIALOG_SYMBOL_PROPERTIES()
     EESCHEMA_SETTINGS* cfg = dynamic_cast<EESCHEMA_SETTINGS*>( Kiface().KifaceSettings() );
 
     if( cfg )
-        cfg->m_Appearance.edit_component_visible_columns = m_fieldsGrid->GetShownColumns();
+    {
+        cfg->m_Appearance.edit_symbol_visible_columns = m_fieldsGrid->GetShownColumns();
+        cfg->m_Appearance.edit_symbol_width = GetSize().x;
+        cfg->m_Appearance.edit_symbol_height = GetSize().y;
+    }
 
     // Prevents crash bug in wxGrid's d'tor
     m_fieldsGrid->DestroyTable( m_fields );
@@ -401,13 +405,11 @@ DIALOG_SYMBOL_PROPERTIES::~DIALOG_SYMBOL_PROPERTIES()
     if( m_dataModel )
         m_pinGrid->DestroyTable( m_dataModel );
 
-    m_fieldsGrid->Disconnect( wxEVT_GRID_CELL_CHANGING,
-                              wxGridEventHandler( DIALOG_SYMBOL_PROPERTIES::OnGridCellChanging ),
-                              nullptr, this );
-
-    m_pinGrid->Disconnect( wxEVT_GRID_COL_SORT,
-                           wxGridEventHandler( DIALOG_SYMBOL_PROPERTIES::OnPinTableColSort ),
-                           nullptr, this );
+    m_fieldsGrid->Unbind( wxEVT_GRID_CELL_CHANGING, &DIALOG_SYMBOL_PROPERTIES::OnGridCellChanging,
+                          this );
+    m_pinGrid->Unbind( wxEVT_GRID_COL_SORT, &DIALOG_SYMBOL_PROPERTIES::OnPinTableColSort, this );
+    Unbind( SYMBOL_DELAY_FOCUS, &DIALOG_SYMBOL_PROPERTIES::HandleDelayedFocus, this );
+    Unbind( SYMBOL_DELAY_SELECTION, &DIALOG_SYMBOL_PROPERTIES::HandleDelayedSelection, this );
 
     // Delete the GRID_TRICKS.
     m_fieldsGrid->PopEventHandler( true );
@@ -455,13 +457,22 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataToWindow()
     // notify the grid
     wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, m_fields->size() );
     m_fieldsGrid->ProcessTableMessage( msg );
-    AdjustGridColumns( m_fieldsGrid->GetRect().GetWidth() );
+    AdjustFieldsGridColumns();
 
     // If a multi-unit symbol, set up the unit selector and interchangeable checkbox.
     if( m_symbol->GetUnitCount() > 1 )
     {
+        // Ensure symbol unit is the currently selected unit (mandatory in complex hierarchies)
+        // from the current sheet path, because it can be modified by previous calculations
+        m_symbol->UpdateUnit( m_symbol->GetUnitSelection( &GetParent()->GetCurrentSheet() ) );
+
         for( int ii = 1; ii <= m_symbol->GetUnitCount(); ii++ )
-            m_unitChoice->Append( LIB_SYMBOL::SubReference( ii, false ) );
+        {
+            if( m_symbol->HasUnitDisplayName( ii ) )
+                m_unitChoice->Append( m_symbol->GetUnitDisplayName( ii ) );
+            else
+                m_unitChoice->Append( LIB_SYMBOL::SubReference( ii, false ) );
+        }
 
         if( m_symbol->GetUnit() <= ( int )m_unitChoice->GetCount() )
             m_unitChoice->SetSelection( m_symbol->GetUnit() - 1 );
@@ -503,8 +514,12 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataToWindow()
     case SYM_MIRROR_Y: m_mirrorCtrl->SetSelection( 2 ); break;
     }
 
+#ifdef KICAD_SPICE
+    m_cbExcludeFromSim->SetValue( m_symbol->GetFieldText( SIM_ENABLE_FIELD ) == wxS( "0" ) );
+#endif
     m_cbExcludeFromBom->SetValue( !m_symbol->GetIncludeInBom() );
     m_cbExcludeFromBoard->SetValue( !m_symbol->GetIncludeOnBoard() );
+    m_cbDNP->SetValue( m_symbol->GetDNP() );
 
     if( m_part )
     {
@@ -516,32 +531,115 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataToWindow()
     m_tcLibraryID->SetValue( UnescapeString( m_symbol->GetLibId().Format() ) );
 
     Layout();
+    m_fieldsGrid->Layout();
+    wxSafeYield();
 
     return true;
 }
 
 
+void DIALOG_SYMBOL_PROPERTIES::OnExcludeFromSimulation( wxCommandEvent& event )
+{
+#ifdef KICAD_SPICE
+    int simEnableFieldRow = -1;
+
+    for( int ii = MANDATORY_FIELDS; ii < m_fieldsGrid->GetNumberRows(); ++ii )
+    {
+        if( m_fieldsGrid->GetCellValue( ii, FDC_NAME ) == SIM_ENABLE_FIELD )
+            simEnableFieldRow = ii;
+    }
+
+    if( event.IsChecked() )
+    {
+        if( simEnableFieldRow == -1 )
+        {
+            simEnableFieldRow = (int) m_fields->size();
+            m_fields->emplace_back( VECTOR2I( 0, 0 ), simEnableFieldRow, m_symbol, SIM_ENABLE_FIELD );
+
+            // notify the grid
+            wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, 1 );
+            m_fieldsGrid->ProcessTableMessage( msg );
+        }
+
+        m_fieldsGrid->SetCellValue( simEnableFieldRow, FDC_VALUE, wxT( "0" ) );
+        m_fieldsGrid->SetCellValue( simEnableFieldRow, FDC_SHOWN, wxT( "0" ) );
+        m_fieldsGrid->SetCellValue( simEnableFieldRow, FDC_SHOW_NAME, wxT( "0" ) );
+    }
+    else if( simEnableFieldRow >= 0 )
+    {
+        m_fields->erase( m_fields->begin() + simEnableFieldRow );
+
+        // notify the grid
+        wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_DELETED, simEnableFieldRow, 1 );
+        m_fieldsGrid->ProcessTableMessage( msg );
+    }
+
+    OnModify();
+#endif
+}
+
+
 void DIALOG_SYMBOL_PROPERTIES::OnEditSpiceModel( wxCommandEvent& event )
 {
 #ifdef KICAD_SPICE
-    int diff = m_fields->size();
+    if( !m_fieldsGrid->CommitPendingChanges() )
+        return;
 
-    DIALOG_SPICE_MODEL dialog( this, *m_symbol, m_fields );
+    std::vector<SCH_FIELD> fields;
+
+    for( const SCH_FIELD& field : *m_fields )
+        fields.emplace_back( field );
+
+    DIALOG_SIM_MODEL dialog( this, *m_symbol, fields );
 
     if( dialog.ShowModal() != wxID_OK )
         return;
 
-    diff = (int) m_fields->size() - diff;
+    // Add in any new fields
+    for( const SCH_FIELD& editedField : fields )
+    {
+        bool found = false;
 
-    if( diff > 0 )
-    {
-        wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, diff );
-        m_fieldsGrid->ProcessTableMessage( msg );
+        for( SCH_FIELD& existingField : *m_fields )
+        {
+            if( existingField.GetName() == editedField.GetName() )
+            {
+                found = true;
+                existingField.SetText( editedField.GetText() );
+                break;
+            }
+        }
+
+        if( !found )
+        {
+            m_fields->emplace_back( editedField );
+            wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, 1 );
+            m_fieldsGrid->ProcessTableMessage( msg );
+        }
     }
-    else if( diff < 0 )
+
+    // Remove any deleted fields
+    for( int ii = (int) m_fields->size() - 1; ii >= 0; --ii )
     {
-        wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_DELETED, 0, -diff );
-        m_fieldsGrid->ProcessTableMessage( msg );
+        SCH_FIELD& existingField = m_fields->at( ii );
+        bool       found = false;
+
+        for( SCH_FIELD& editedField : fields )
+        {
+            if( editedField.GetName() == existingField.GetName() )
+            {
+                found = true;
+                break;
+            }
+        }
+
+        if( !found )
+        {
+            m_fields->erase( m_fields->begin() + ii );
+            wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_DELETED, ii, 1 );
+            m_fieldsGrid->ClearSelection();
+            m_fieldsGrid->ProcessTableMessage( msg );
+        }
     }
 
     OnModify();
@@ -560,7 +658,6 @@ void DIALOG_SYMBOL_PROPERTIES::OnCancelButtonClick( wxCommandEvent& event )
 
 bool DIALOG_SYMBOL_PROPERTIES::Validate()
 {
-    wxString msg;
     LIB_ID   id;
 
     if( !m_fieldsGrid->CommitPendingChanges() || !m_fieldsGrid->Validate() )
@@ -570,9 +667,9 @@ bool DIALOG_SYMBOL_PROPERTIES::Validate()
     {
         DisplayErrorMessage( this, _( "References must start with a letter." ) );
 
-        m_delayedFocusColumn = FDC_VALUE;
-        m_delayedFocusRow = REFERENCE_FIELD;
-        m_delayedSelection = false;
+        wxCommandEvent *evt = new wxCommandEvent( SYMBOL_DELAY_FOCUS );
+        evt->SetClientData( new VECTOR2I( REFERENCE_FIELD, FDC_VALUE ) );
+        QueueEvent( evt );
 
         return false;
     }
@@ -587,9 +684,9 @@ bool DIALOG_SYMBOL_PROPERTIES::Validate()
         {
             DisplayErrorMessage( this, _( "Fields must have a name." ) );
 
-            m_delayedFocusColumn = FDC_NAME;
-            m_delayedFocusRow = i;
-            m_delayedSelection = false;
+            wxCommandEvent *evt = new wxCommandEvent( SYMBOL_DELAY_FOCUS );
+            evt->SetClientData( new VECTOR2I( i, FDC_VALUE ) );
+            QueueEvent( evt );
 
             return false;
         }
@@ -611,16 +708,12 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataFromWindow()
         return false;
 
     SCH_SCREEN* currentScreen = GetParent()->GetScreen();
-    SCHEMATIC&  schematic = GetParent()->Schematic();
-
     wxCHECK( currentScreen, false );
 
     // This needs to be done before the LIB_ID is changed to prevent stale library symbols in
     // the schematic file.
     currentScreen->Remove( m_symbol );
 
-    wxString msg;
-
     // save old cmp in undo list if not already in edit, or moving ...
     if( m_symbol->GetEditFlags() == 0 )
         GetParent()->SaveCopyInUndoList( currentScreen, m_symbol, UNDO_REDO::CHANGED, false );
@@ -660,7 +753,7 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataFromWindow()
         m_part->SetShowPinNumbers( m_ShowPinNumButt->GetValue() );
     }
 
-    // Restore m_Flag modified by SetUnit() and other change settings
+    // Restore m_Flag modified by SetUnit() and other change settings from the dialog
     m_symbol->ClearFlags();
     m_symbol->SetFlags( flags );
 
@@ -668,38 +761,12 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataFromWindow()
     for( unsigned i = 0;  i < m_fields->size();  ++i )
         m_fields->at( i ).Offset( m_symbol->GetPosition() );
 
-    LIB_SYMBOL* entry = GetParent()->GetLibSymbol( m_symbol->GetLibId() );
-
-    if( entry && entry->IsPower() )
-        m_fields->at( VALUE_FIELD ).SetText( m_symbol->GetLibId().GetLibItemName() );
-
-    // Push all fields to the symbol -except- for those which are TEMPLATE_FIELDNAMES
-    // with empty values.
     SCH_FIELDS& fields = m_symbol->GetFields();
 
     fields.clear();
 
     for( size_t i = 0; i < m_fields->size(); ++i )
-    {
-        SCH_FIELD& field = m_fields->at( i );
-        bool       emptyTemplateField = false;
-
-        if( i >= MANDATORY_FIELDS )
-        {
-            for( const TEMPLATE_FIELDNAME& fieldname :
-                    schematic.Settings().m_TemplateFieldNames.GetTemplateFieldNames() )
-            {
-                if( field.GetName() == fieldname.m_Name && field.GetText().IsEmpty() )
-                {
-                    emptyTemplateField = true;
-                    break;
-                }
-            }
-        }
-
-        if( !emptyTemplateField )
-            fields.push_back( field );
-    }
+        fields.push_back( m_fields->at( i ) );
 
     // Reference has a specific initialization, depending on the current active sheet
     // because for a given symbol, in a complex hierarchy, there are more than one
@@ -708,11 +775,12 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataFromWindow()
 
     // Similar for Value and Footprint, except that the GUI behaviour is that they are kept
     // in sync between multiple instances.
-    m_symbol->SetValue( m_fields->at( VALUE_FIELD ).GetText() );
-    m_symbol->SetFootprint( m_fields->at( FOOTPRINT_FIELD ).GetText() );
+    m_symbol->SetValueFieldText( m_fields->at( VALUE_FIELD ).GetText() );
+    m_symbol->SetFootprintFieldText(  m_fields->at( FOOTPRINT_FIELD ).GetText() );
 
     m_symbol->SetIncludeInBom( !m_cbExcludeFromBom->IsChecked() );
     m_symbol->SetIncludeOnBoard( !m_cbExcludeFromBoard->IsChecked() );
+    m_symbol->SetDNP( m_cbDNP->IsChecked() );
 
     // Update any assignments
     if( m_dataModel )
@@ -720,8 +788,10 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataFromWindow()
         for( const SCH_PIN& model_pin : *m_dataModel )
         {
             // map from the edited copy back to the "real" pin in the symbol.
-            SCH_PIN* src_pin = m_symbol->GetPin( model_pin.GetLibPin() );
-            src_pin->SetAlt( model_pin.GetAlt() );
+            SCH_PIN* src_pin = m_symbol->GetPin( model_pin.GetNumber() );
+
+            if( src_pin )
+                src_pin->SetAlt( model_pin.GetAlt() );
         }
     }
 
@@ -745,8 +815,8 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataFromWindow()
             {
                 GetParent()->SaveCopyInUndoList( screen, otherUnit, UNDO_REDO::CHANGED,
                                                  appendUndo );
-                otherUnit->SetValue( m_fields->at( VALUE_FIELD ).GetText() );
-                otherUnit->SetFootprint( m_fields->at( FOOTPRINT_FIELD ).GetText() );
+                otherUnit->SetValueFieldText( m_fields->at( VALUE_FIELD ).GetText() );
+                otherUnit->SetFootprintFieldText( m_fields->at( FOOTPRINT_FIELD ).GetText() );
 
                 for( size_t ii = DATASHEET_FIELD; ii < m_fields->size(); ++ii )
                 {
@@ -779,6 +849,7 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataFromWindow()
 
                 otherUnit->SetIncludeInBom( !m_cbExcludeFromBom->IsChecked() );
                 otherUnit->SetIncludeOnBoard( !m_cbExcludeFromBoard->IsChecked() );
+                otherUnit->SetDNP( m_cbDNP->IsChecked() );
 
                 if( m_dataModel )
                 {
@@ -816,9 +887,9 @@ void DIALOG_SYMBOL_PROPERTIES::OnGridCellChanging( wxGridEvent& event )
     if( control && control->GetValidator() && !control->GetValidator()->Validate( control ) )
     {
         event.Veto();
-        m_delayedFocusRow = event.GetRow();
-        m_delayedFocusColumn = event.GetCol();
-        m_delayedSelection = false;
+        wxCommandEvent *evt = new wxCommandEvent( SYMBOL_DELAY_FOCUS );
+        evt->SetClientData( new VECTOR2I( event.GetRow(), event.GetCol() ) );
+        QueueEvent( evt );
     }
     else if( event.GetCol() == FDC_NAME )
     {
@@ -834,9 +905,9 @@ void DIALOG_SYMBOL_PROPERTIES::OnGridCellChanging( wxGridEvent& event )
                 DisplayError( this, wxString::Format( _( "Field name '%s' already in use." ),
                                                       newName ) );
                 event.Veto();
-                m_delayedFocusRow = event.GetRow();
-                m_delayedFocusColumn = event.GetCol();
-                m_delayedSelection = false;
+                wxCommandEvent *evt = new wxCommandEvent( SYMBOL_DELAY_FOCUS );
+                evt->SetClientData( new VECTOR2I( event.GetRow(), event.GetCol() ) );
+                QueueEvent( evt );
             }
         }
     }
@@ -848,14 +919,15 @@ void DIALOG_SYMBOL_PROPERTIES::OnGridCellChanging( wxGridEvent& event )
 void DIALOG_SYMBOL_PROPERTIES::OnGridEditorShown( wxGridEvent& aEvent )
 {
     if( aEvent.GetRow() == REFERENCE_FIELD && aEvent.GetCol() == FDC_VALUE )
-        m_delayedSelection= true;
+        QueueEvent( new wxCommandEvent( SYMBOL_DELAY_SELECTION ) );
 
-    /// Queue up an event to ensure the widget gets resized if the editor needs it
-    wxSizeEvent *evt = new wxSizeEvent();
-    evt->SetSize( wxSize( m_width, -1 ) );
+    m_editorShown = true;
+}
 
-    wxQueueEvent( m_fieldsGrid, evt );
 
+void DIALOG_SYMBOL_PROPERTIES::OnGridEditorHidden( wxGridEvent& aEvent )
+{
+    m_editorShown = false;
 }
 
 
@@ -865,9 +937,9 @@ void DIALOG_SYMBOL_PROPERTIES::OnAddField( wxCommandEvent& event )
         return;
 
     SCHEMATIC_SETTINGS& settings = m_symbol->Schematic()->Settings();
-    int                 fieldID = m_fields->size();
+    int                 fieldID = (int) m_fields->size();
     SCH_FIELD           newField( wxPoint( 0, 0 ), fieldID, m_symbol,
-                                  TEMPLATE_FIELDNAME::GetDefaultFieldName( fieldID ) );
+                                  TEMPLATE_FIELDNAME::GetDefaultFieldName( fieldID, DO_TRANSLATE ) );
 
     newField.SetTextAngle( m_fields->at( REFERENCE_FIELD ).GetTextAngle() );
     newField.SetTextSize( wxSize( settings.m_DefaultTextSize, settings.m_DefaultTextSize ) );
@@ -909,6 +981,7 @@ void DIALOG_SYMBOL_PROPERTIES::OnDeleteField( wxCommandEvent& event )
     }
 
     m_fieldsGrid->CommitPendingChanges( true /* quiet mode */ );
+    m_fieldsGrid->ClearSelection();
 
     // Reverse sort so deleting a row doesn't change the indexes of the other rows.
     selectedRows.Sort( []( int* first, int* second ) { return *second - *first; } );
@@ -1018,8 +1091,11 @@ void DIALOG_SYMBOL_PROPERTIES::OnPinTableCellEdited( wxGridEvent& aEvent )
 {
     int row = aEvent.GetRow();
 
-    if( m_pinGrid->GetCellValue( row, COL_ALT_NAME ) == m_dataModel->GetValue( row, COL_BASE_NAME ) )
+    if( m_pinGrid->GetCellValue( row, COL_ALT_NAME )
+            == m_dataModel->GetValue( row, COL_BASE_NAME ) )
+    {
         m_dataModel->SetValue( row, COL_ALT_NAME, wxEmptyString );
+    }
 
     // These are just to get the cells refreshed
     m_dataModel->SetValue( row, COL_TYPE, m_dataModel->GetValue( row, COL_TYPE ) );
@@ -1048,27 +1124,31 @@ void DIALOG_SYMBOL_PROPERTIES::OnPinTableColSort( wxGridEvent& aEvent )
 }
 
 
-void DIALOG_SYMBOL_PROPERTIES::AdjustGridColumns( int aWidth )
+void DIALOG_SYMBOL_PROPERTIES::AdjustFieldsGridColumns()
 {
-    wxGridUpdateLocker deferRepaintsTillLeavingScope;
-
-    m_width = aWidth;
+    wxGridUpdateLocker deferRepaintsTillLeavingScope( m_fieldsGrid );
 
     // Account for scroll bars
-    int fieldsWidth = aWidth - ( m_fieldsGrid->GetSize().x - m_fieldsGrid->GetClientSize().x );
-    int pinTblWidth = aWidth - ( m_pinGrid->GetSize().x - m_pinGrid->GetClientSize().x );
+    int fieldsWidth = KIPLATFORM::UI::GetUnobscuredSize( m_fieldsGrid ).x;
 
     m_fieldsGrid->AutoSizeColumn( 0 );
+    m_fieldsGrid->SetColSize( 0, std::max( 72, m_fieldsGrid->GetColSize( 0 ) ) );
 
     int fixedColsWidth = m_fieldsGrid->GetColSize( 0 );
 
     for( int i = 2; i < m_fieldsGrid->GetNumberCols(); i++ )
         fixedColsWidth += m_fieldsGrid->GetColSize( i );
 
-    int colSize = std::max( fieldsWidth - fixedColsWidth, -1 );
-    colSize = ( colSize == 0 ) ? -1 : colSize; // don't hide the column!
+    m_fieldsGrid->SetColSize( 1, std::max( 120, fieldsWidth - fixedColsWidth ) );
+}
 
-    m_fieldsGrid->SetColSize( 1, colSize );
+
+void DIALOG_SYMBOL_PROPERTIES::AdjustPinsGridColumns()
+{
+    wxGridUpdateLocker deferRepaintsTillLeavingScope( m_pinGrid );
+
+    // Account for scroll bars
+    int pinTblWidth = KIPLATFORM::UI::GetUnobscuredSize( m_pinGrid ).x;
 
     // Stretch the Base Name and Alternate Assignment columns to fit.
     for( int i = 0; i < COL_COUNT; ++i )
@@ -1077,9 +1157,6 @@ void DIALOG_SYMBOL_PROPERTIES::AdjustGridColumns( int aWidth )
             pinTblWidth -= m_pinGrid->GetColSize( i );
     }
 
-    // Why?  I haven't a clue....
-    pinTblWidth += 22;
-
     m_pinGrid->SetColSize( COL_BASE_NAME, pinTblWidth / 2 );
     m_pinGrid->SetColSize( COL_ALT_NAME, pinTblWidth / 2 );
 }
@@ -1094,46 +1171,86 @@ void DIALOG_SYMBOL_PROPERTIES::OnUpdateUI( wxUpdateUIEvent& event )
         m_shownColumns = shownColumns;
 
         if( !m_fieldsGrid->IsCellEditControlShown() )
-            AdjustGridColumns( m_fieldsGrid->GetRect().GetWidth() );
+            AdjustFieldsGridColumns();
     }
 
-    // Handle a delayed focus
-    if( m_delayedFocusRow >= 0 )
+#ifdef KICAD_SPICE
+    wxString simEnable;
+
+    for( int ii = MANDATORY_FIELDS; ii < m_fieldsGrid->GetNumberRows(); ++ii )
     {
-        m_fieldsGrid->SetFocus();
-        m_fieldsGrid->MakeCellVisible( m_delayedFocusRow, m_delayedFocusColumn );
-        m_fieldsGrid->SetGridCursor( m_delayedFocusRow, m_delayedFocusColumn );
-
-        m_fieldsGrid->EnableCellEditControl( true );
-        m_fieldsGrid->ShowCellEditControl();
-
-        m_delayedFocusRow = -1;
-        m_delayedFocusColumn = -1;
+        if( m_fieldsGrid->GetCellValue( ii, FDC_NAME ) == SIM_ENABLE_FIELD )
+        {
+            simEnable = m_fieldsGrid->GetCellValue( ii, FDC_VALUE );
+            break;
+        }
     }
 
-    // Handle a delayed selection
-    if( m_delayedSelection )
-    {
-        wxGridCellEditor* cellEditor = m_fieldsGrid->GetCellEditor( REFERENCE_FIELD, FDC_VALUE );
-
-        if( wxTextEntry* txt = dynamic_cast<wxTextEntry*>( cellEditor->GetControl() ) )
-            KIUI::SelectReferenceNumber( txt );
-
-        cellEditor->DecRef();   // we're done; must release
-
-        m_delayedSelection = false;
-    }
+    m_cbExcludeFromSim->SetValue( simEnable == wxS( "0" ) );
+#endif
 }
 
 
-void DIALOG_SYMBOL_PROPERTIES::OnSizeGrid( wxSizeEvent& event )
+void DIALOG_SYMBOL_PROPERTIES::HandleDelayedFocus( wxCommandEvent& event )
 {
-    int new_size = event.GetSize().GetX();
+    VECTOR2I *loc = static_cast<VECTOR2I*>( event.GetClientData() );
 
-    if( m_width != new_size )
+    wxCHECK_RET( loc, wxT( "Missing focus cell location" ) );
+
+    // Handle a delayed focus
+
+    m_fieldsGrid->SetFocus();
+    m_fieldsGrid->MakeCellVisible( loc->x, loc->y );
+    m_fieldsGrid->SetGridCursor( loc->x, loc->y );
+
+    m_fieldsGrid->EnableCellEditControl( true );
+    m_fieldsGrid->ShowCellEditControl();
+
+    delete loc;
+}
+
+
+void DIALOG_SYMBOL_PROPERTIES::HandleDelayedSelection( wxCommandEvent& event )
+{
+    // Handle a delayed selection
+    wxGridCellEditor* cellEditor = m_fieldsGrid->GetCellEditor( REFERENCE_FIELD, FDC_VALUE );
+
+    if( wxTextEntry* txt = dynamic_cast<wxTextEntry*>( cellEditor->GetControl() ) )
+        KIUI::SelectReferenceNumber( txt );
+
+    cellEditor->DecRef();   // we're done; must release
+}
+
+void DIALOG_SYMBOL_PROPERTIES::OnSizeFieldsGrid( wxSizeEvent& event )
+{
+    wxSize new_size = event.GetSize();
+
+    if( ( !m_editorShown || m_lastRequestedSize != new_size ) && m_fieldsSize != new_size )
     {
-        AdjustGridColumns( new_size );
-        Layout();
+        m_fieldsSize = new_size;
+
+        AdjustFieldsGridColumns();
+    }
+
+    // We store this value to check whether the dialog is changing size.  This might indicate
+    // that the user is scaling the dialog with an editor shown.  Some editors do not close
+    // (at least on GTK) when the user drags a dialog corner
+    m_lastRequestedSize = new_size;
+
+    // Always propagate for a grid repaint (needed if the height changes, as well as width)
+    event.Skip();
+}
+
+
+void DIALOG_SYMBOL_PROPERTIES::OnSizePinsGrid( wxSizeEvent& event )
+{
+    wxSize new_size = event.GetSize();
+
+    if( m_pinsSize != new_size )
+    {
+        m_pinsSize = new_size;
+
+        AdjustPinsGridColumns();
     }
 
     // Always propagate for a grid repaint (needed if the height changes, as well as width)
@@ -1147,6 +1264,11 @@ void DIALOG_SYMBOL_PROPERTIES::OnInitDlg( wxInitDialogEvent& event )
 
     // Now all widgets have the size fixed, call FinishDialogSettings
     finishDialogSettings();
+
+    EESCHEMA_SETTINGS* cfg = dynamic_cast<EESCHEMA_SETTINGS*>( Kiface().KifaceSettings() );
+
+    if( cfg && cfg->m_Appearance.edit_symbol_width > 0 && cfg->m_Appearance.edit_symbol_height > 0 )
+        SetSize( cfg->m_Appearance.edit_symbol_width, cfg->m_Appearance.edit_symbol_height );
 }
 
 
@@ -1156,8 +1278,36 @@ void DIALOG_SYMBOL_PROPERTIES::OnCheckBox( wxCommandEvent& event )
 }
 
 
-void DIALOG_SYMBOL_PROPERTIES::OnChoice( wxCommandEvent& event )
+void DIALOG_SYMBOL_PROPERTIES::OnUnitChoice( wxCommandEvent& event )
 {
+    if( m_dataModel )
+    {
+        EDA_ITEM_FLAGS flags = m_symbol->GetFlags();
+
+        int unit_selection = m_unitChoice->GetSelection() + 1;
+
+        // We need to select a new unit to build the new unit pin list
+        // but we should not change the symbol, so the initial unit will be selected
+        // after rebuilding the pin list
+        int old_unit = m_symbol->GetUnit();
+        m_symbol->SetUnit( unit_selection );
+
+        // Rebuild a copy of the pins of the new unit for editing
+        m_dataModel->clear();
+
+        for( const std::unique_ptr<SCH_PIN>& pin : m_symbol->GetRawPins() )
+            m_dataModel->push_back( *pin );
+
+        m_dataModel->SortRows( COL_NUMBER, true );
+        m_dataModel->BuildAttrs();
+
+        m_symbol->SetUnit( old_unit );
+
+        // Restore m_Flag modified by SetUnit()
+        m_symbol->ClearFlags();
+        m_symbol->SetFlags( flags );
+    }
+
     OnModify();
 }
 
