@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2017 Wayne Stambaugh <stambaughw@gmail.com>
  * Copyright (C) 2021 CERN
- * Copyright (C) 2017-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2017-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -28,7 +28,7 @@
 #include <lib_id.h>
 #include <symbol_lib_table.h>
 #include <lib_table_lexer.h>
-#include <grid_tricks.h>
+#include <lib_table_grid_tricks.h>
 #include <widgets/wx_grid.h>
 #include <confirm.h>
 #include <bitmaps.h>
@@ -45,6 +45,7 @@
 #include <settings/settings_manager.h>
 #include <widgets/grid_readonly_text_helpers.h>
 #include <widgets/grid_text_button_helpers.h>
+#include <widgets/std_bitmap_button.h>
 #include <sch_file_versions.h>
 #include <wx/filedlg.h>
 
@@ -82,27 +83,27 @@ class SYMBOL_LIB_TABLE_GRID : public LIB_TABLE_GRID, public SYMBOL_LIB_TABLE
     friend class SYMBOL_GRID_TRICKS;
 
 protected:
-    LIB_TABLE_ROW* at( size_t aIndex ) override { return &rows.at( aIndex ); }
+    LIB_TABLE_ROW* at( size_t aIndex ) override { return &m_rows.at( aIndex ); }
 
-    size_t size() const override { return rows.size(); }
+    size_t size() const override { return m_rows.size(); }
 
     LIB_TABLE_ROW* makeNewRow() override
     {
         return dynamic_cast< LIB_TABLE_ROW* >( new SYMBOL_LIB_TABLE_ROW );
     }
 
-    LIB_TABLE_ROWS_ITER begin() override { return rows.begin(); }
+    LIB_TABLE_ROWS_ITER begin() override { return m_rows.begin(); }
 
     LIB_TABLE_ROWS_ITER insert( LIB_TABLE_ROWS_ITER aIterator, LIB_TABLE_ROW* aRow ) override
     {
-        return rows.insert( aIterator, aRow );
+        return m_rows.insert( aIterator, aRow );
     }
 
-    void push_back( LIB_TABLE_ROW* aRow ) override { rows.push_back( aRow ); }
+    void push_back( LIB_TABLE_ROW* aRow ) override { m_rows.push_back( aRow ); }
 
     LIB_TABLE_ROWS_ITER erase( LIB_TABLE_ROWS_ITER aFirst, LIB_TABLE_ROWS_ITER aLast ) override
     {
-        return rows.erase( aFirst, aLast );
+        return m_rows.erase( aFirst, aLast );
     }
 
 public:
@@ -129,16 +130,15 @@ public:
 
     SYMBOL_LIB_TABLE_GRID( const SYMBOL_LIB_TABLE& aTableToEdit )
     {
-        rows = aTableToEdit.rows;
+        m_rows = aTableToEdit.m_rows;
     }
 };
 
-
-class SYMBOL_GRID_TRICKS : public GRID_TRICKS
+class SYMBOL_GRID_TRICKS : public LIB_TABLE_GRID_TRICKS
 {
 public:
     SYMBOL_GRID_TRICKS( DIALOG_EDIT_LIBRARY_TABLES* aParent, WX_GRID* aGrid ) :
-        GRID_TRICKS( aGrid ),
+        LIB_TABLE_GRID_TRICKS( aGrid ),
         m_dialog( aParent )
     {
     }
@@ -158,7 +158,7 @@ protected:
             // paste the SYMBOL_LIB_TABLE_ROWs of s-expression (sym_lib_table), starting
             // at column 0 regardless of current cursor column.
 
-            STRING_LINE_READER  slr( TO_UTF8( cb_text ), "Clipboard" );
+            STRING_LINE_READER  slr( TO_UTF8( cb_text ), wxS( "Clipboard" ) );
             LIB_TABLE_LEXER     lexer( &slr );
             SYMBOL_LIB_TABLE    tmp_tbl;
             bool                parsed = true;
@@ -180,7 +180,7 @@ protected:
                     tbl->AppendRows( tmp_tbl.GetCount() - tbl->GetNumberRows() );
 
                 for( unsigned i = 0;  i < tmp_tbl.GetCount();  ++i )
-                    tbl->rows.replace( i, tmp_tbl.At( i ).clone() );
+                    tbl->m_rows.replace( i, tmp_tbl.At( i ).clone() );
             }
 
             m_grid->AutoSizeColumns( false );
@@ -215,6 +215,7 @@ PANEL_SYM_LIB_TABLE::PANEL_SYM_LIB_TABLE( DIALOG_EDIT_LIBRARY_TABLES* aParent, P
 
     pluginChoices.Add( SCH_IO_MGR::ShowType( SCH_IO_MGR::SCH_KICAD ) );
     pluginChoices.Add( SCH_IO_MGR::ShowType( SCH_IO_MGR::SCH_LEGACY ) );
+    pluginChoices.Add( SCH_IO_MGR::ShowType( SCH_IO_MGR::SCH_DATABASE ) );
 
     EESCHEMA_SETTINGS* cfg = Pgm().GetSettingsManager().GetAppSettings<EESCHEMA_SETTINGS>();
 
@@ -223,6 +224,15 @@ PANEL_SYM_LIB_TABLE::PANEL_SYM_LIB_TABLE( DIALOG_EDIT_LIBRARY_TABLES* aParent, P
 
     m_lastProjectLibDir = m_project->GetProjectPath();
 
+    auto autoSizeCol =
+            [&]( WX_GRID* aGrid, int aCol )
+            {
+                int prevWidth = aGrid->GetColSize( aCol );
+
+                aGrid->AutoSizeColumn( aCol, false );
+                aGrid->SetColSize( aCol, std::max( prevWidth, aGrid->GetColSize( aCol ) ) );
+            };
+
     auto setupGrid =
             [&]( WX_GRID* aGrid )
             {
@@ -233,7 +243,6 @@ PANEL_SYM_LIB_TABLE::PANEL_SYM_LIB_TABLE( DIALOG_EDIT_LIBRARY_TABLES* aParent, P
                 aGrid->PushEventHandler( new SYMBOL_GRID_TRICKS( m_parent, aGrid ) );
 
                 aGrid->SetSelectionMode( wxGrid::wxGridSelectRows );
-                aGrid->AutoSizeColumns( false );
 
                 // Set special attributes
                 wxGridCellAttr* attr;
@@ -257,15 +266,17 @@ PANEL_SYM_LIB_TABLE::PANEL_SYM_LIB_TABLE( DIALOG_EDIT_LIBRARY_TABLES* aParent, P
                 attr->SetReadOnly();    // not really; we delegate interactivity to GRID_TRICKS
                 aGrid->SetColAttr( COL_ENABLED, attr );
 
-                // all but COL_OPTIONS, which is edited with Option Editor anyways.
-                aGrid->AutoSizeColumn( COL_NICKNAME, false );
-                aGrid->AutoSizeColumn( COL_TYPE, false );
-                aGrid->AutoSizeColumn( COL_URI, false );
-                aGrid->AutoSizeColumn( COL_DESCR, false );
-                aGrid->AutoSizeColumn( COL_ENABLED, false );
+                attr = new wxGridCellAttr;
+                attr->SetRenderer( new wxGridCellBoolRenderer() );
+                attr->SetReadOnly();    // not really; we delegate interactivity to GRID_TRICKS
+                aGrid->SetColAttr( COL_VISIBLE, attr );
 
-                // would set this to width of title, if it was easily known.
-                aGrid->SetColSize( COL_OPTIONS, 80 );
+                // all but COL_OPTIONS, which is edited with Option Editor anyways.
+                autoSizeCol( aGrid, COL_NICKNAME );
+                autoSizeCol( aGrid, COL_TYPE );
+                autoSizeCol( aGrid, COL_URI );
+                autoSizeCol( aGrid, COL_DESCR );
+                autoSizeCol( aGrid, COL_ENABLED );
 
                 // Gives a selection to each grid, mainly for delete button.  wxGrid's wake up with
                 // a currentCell which is sometimes not highlighted.
@@ -313,11 +324,6 @@ PANEL_SYM_LIB_TABLE::PANEL_SYM_LIB_TABLE( DIALOG_EDIT_LIBRARY_TABLES* aParent, P
 
 PANEL_SYM_LIB_TABLE::~PANEL_SYM_LIB_TABLE()
 {
-    // When the dialog is closed it will hide the current notebook page first, which will
-    // in turn select the other one.  We then end up saving its index as the "current page".
-    // So flip them back again:
-    m_pageNdx = m_pageNdx == 1 ? 0 : 1;
-
     // Delete the GRID_TRICKS.
     // Any additional event handlers should be popped before the window is deleted.
     m_global_grid->PopEventHandler( true );
@@ -357,8 +363,8 @@ bool PANEL_SYM_LIB_TABLE::verifyTables()
                                             wxYES_NO | wxCENTER | wxICON_QUESTION | wxYES_DEFAULT );
                 badCellDlg.SetExtendedMessage( _( "Empty cells will result in all rows that are "
                                                   "invalid to be removed from the table." ) );
-                badCellDlg.SetYesNoLabels( wxMessageDialog::ButtonLabel( "Remove Invalid Cells" ),
-                                           wxMessageDialog::ButtonLabel( "Cancel Table Update" ) );
+                badCellDlg.SetYesNoLabels( wxMessageDialog::ButtonLabel( _( "Remove Invalid Cells" ) ),
+                                           wxMessageDialog::ButtonLabel( _( "Cancel Table Update" ) ) );
 
                 if( badCellDlg.ShowModal() == wxID_NO )
                     return false;
@@ -441,6 +447,9 @@ bool PANEL_SYM_LIB_TABLE::verifyTables()
         {
             SYMBOL_LIB_TABLE_ROW& row = dynamic_cast<SYMBOL_LIB_TABLE_ROW&>( table->At( r ) );
 
+            if( !row.GetParent() )
+                row.SetParent( table );
+
             if( !row.GetIsEnabled() )
                 continue;
 
@@ -462,7 +471,7 @@ bool PANEL_SYM_LIB_TABLE::verifyTables()
                                         _( "Error Loading Library" ) );
                 errdlg.ShowModal();
 
-                return false;
+                return true;
             }
         }
     }
@@ -482,7 +491,8 @@ void PANEL_SYM_LIB_TABLE::browseLibrariesHandler( wxCommandEvent& event )
 {
     wxString wildcards = AllSymbolLibFilesWildcard()
                             + "|" + KiCadSymbolLibFileWildcard()
-                            + "|" + LegacySymbolLibFileWildcard();
+                            + "|" + LegacySymbolLibFileWildcard()
+                            + "|" + DatabaseLibFileWildcard();
 
     EESCHEMA_SETTINGS* cfg = Pgm().GetSettingsManager().GetAppSettings<EESCHEMA_SETTINGS>();
 
@@ -510,12 +520,11 @@ void PANEL_SYM_LIB_TABLE::browseLibrariesHandler( wxCommandEvent& event )
     wxString           detailedMsg   = _( "One of the nicknames will need to be changed after "
                                           "adding this library." );
 
-    wxArrayString files;
-    dlg.GetFilenames( files );
+    wxArrayString filePathsList;
+    dlg.GetPaths( filePathsList );
 
-    for( const wxString& file : files )
+    for( const wxString& filePath : filePathsList )
     {
-        wxString   filePath = dlg.GetDirectory() + wxFileName::GetPathSeparator() + file;
         wxFileName fn( filePath );
         wxString   nickname = LIB_ID::FixIllegalChars( fn.GetName(), true );
         bool       doAdd = true;
@@ -561,7 +570,7 @@ void PANEL_SYM_LIB_TABLE::browseLibrariesHandler( wxCommandEvent& event )
         }
     }
 
-    if( !files.IsEmpty() )
+    if( !filePathsList.IsEmpty() )
     {
         m_cur_grid->MakeCellVisible( m_cur_grid->GetNumberRows() - 1, 0 );
         m_cur_grid->SetGridCursor( m_cur_grid->GetNumberRows() - 1, 1 );
@@ -659,10 +668,10 @@ void PANEL_SYM_LIB_TABLE::moveUpHandler( wxCommandEvent& event )
     if( curRow >= 1 )
     {
         boost::ptr_vector< LIB_TABLE_ROW >::auto_type move_me =
-            tbl->rows.release( tbl->rows.begin() + curRow );
+            tbl->m_rows.release( tbl->m_rows.begin() + curRow );
 
         --curRow;
-        tbl->rows.insert( tbl->rows.begin() + curRow, move_me.release() );
+        tbl->m_rows.insert( tbl->m_rows.begin() + curRow, move_me.release() );
 
         if( tbl->GetView() )
         {
@@ -686,13 +695,13 @@ void PANEL_SYM_LIB_TABLE::moveDownHandler( wxCommandEvent& event )
     int curRow = m_cur_grid->GetGridCursorRow();
 
     // @todo: add multiple selection moves.
-    if( unsigned( curRow + 1 ) < tbl->rows.size() )
+    if( unsigned( curRow + 1 ) < tbl->m_rows.size() )
     {
         boost::ptr_vector< LIB_TABLE_ROW >::auto_type move_me =
-            tbl->rows.release( tbl->rows.begin() + curRow );
+            tbl->m_rows.release( tbl->m_rows.begin() + curRow );
 
         ++curRow;
-        tbl->rows.insert( tbl->rows.begin() + curRow, move_me.release() );
+        tbl->m_rows.insert( tbl->m_rows.begin() + curRow, move_me.release() );
 
         if( tbl->GetView() )
         {
@@ -880,8 +889,8 @@ bool PANEL_SYM_LIB_TABLE::TransferDataFromWindow()
         m_parent->m_GlobalTableChanged = true;
 
         m_globalTable->Clear();
-        m_globalTable->rows.transfer( m_globalTable->rows.end(), global_model()->rows.begin(),
-                                      global_model()->rows.end(), global_model()->rows );
+        m_globalTable->m_rows.transfer( m_globalTable->m_rows.end(), global_model()->m_rows.begin(),
+                                      global_model()->m_rows.end(), global_model()->m_rows );
         m_globalTable->reindex();
     }
 
@@ -890,8 +899,8 @@ bool PANEL_SYM_LIB_TABLE::TransferDataFromWindow()
         m_parent->m_ProjectTableChanged = true;
 
         m_projectTable->Clear();
-        m_projectTable->rows.transfer( m_projectTable->rows.end(), project_model()->rows.begin(),
-                                       project_model()->rows.end(), project_model()->rows );
+        m_projectTable->m_rows.transfer( m_projectTable->m_rows.end(), project_model()->m_rows.begin(),
+                                       project_model()->m_rows.end(), project_model()->m_rows );
         m_projectTable->reindex();
     }
 
@@ -969,13 +978,8 @@ void PANEL_SYM_LIB_TABLE::adjustPathSubsGridColumns( int aWidth )
     aWidth -= ( m_path_subs_grid->GetSize().x - m_path_subs_grid->GetClientSize().x );
 
     m_path_subs_grid->AutoSizeColumn( 0 );
-
-    int remaining_width = aWidth - m_path_subs_grid->GetColSize( 0 );
-
-    if( remaining_width < 0 )
-        m_path_subs_grid->SetColSize( 1, -1 );
-    else
-        m_path_subs_grid->SetColSize( 1, remaining_width );
+    m_path_subs_grid->SetColSize( 0, std::max( 72, m_path_subs_grid->GetColSize( 0 ) ) );
+    m_path_subs_grid->SetColSize( 1, std::max( 120, aWidth - m_path_subs_grid->GetColSize( 0 ) ) );
 }
 
 
