@@ -28,6 +28,8 @@
 #include <confirm.h>
 #include <gestfich.h>
 #include <eda_dde.h>
+#include "eeschema_jobs_handler.h"
+#include "eeschema_helpers.h"
 #include <eeschema_settings.h>
 #include <sch_edit_frame.h>
 #include <symbol_edit_frame.h>
@@ -37,8 +39,8 @@
 #include <dialogs/dialog_global_sym_lib_table_config.h>
 #include <dialogs/panel_sym_lib_table.h>
 #include <kiway.h>
-#include <sim/sim_plot_frame.h>
 #include <settings/settings_manager.h>
+#include <symbol_editor_settings.h>
 #include <sexpr/sexpr.h>
 #include <sexpr/sexpr_parser.h>
 #include <kiface_ids.h>
@@ -48,13 +50,22 @@
 
 #include <schematic.h>
 #include <connection_graph.h>
+#include <panel_template_fieldnames.h>
+#include <panel_eeschema_color_settings.h>
+#include <panel_sym_color_settings.h>
+#include <panel_eeschema_editing_options.h>
+#include <panel_eeschema_annotation_options.h>
+#include <panel_sym_editing_options.h>
+#include <dialogs/panel_gal_display_options.h>
+#include <panel_eeschema_display_options.h>
+#include <panel_sym_display_options.h>
+#include <sim/sim_plot_frame.h>
+
+#include <wx/crt.h>
 
 // The main sheet of the project
 SCH_SHEET*  g_RootSheet = nullptr;
 
-// a transform matrix, to display symbols in lib editor
-TRANSFORM DefaultTransform = TRANSFORM( 1, 0, 0, -1 );
-
 
 namespace SCH {
 
@@ -80,12 +91,14 @@ static std::unique_ptr<SCHEMATIC> readSchematicFromFile( const std::string& aFil
     SCH_SHEET_LIST sheets = schematic->GetSheets();
 
     // Restore all of the loaded symbol instances from the root sheet screen.
-    sheets.UpdateSymbolInstances( schematic->RootScreen()->GetSymbolInstances() );
+    sheets.UpdateSymbolInstanceData( schematic->RootScreen()->GetSymbolInstances() );
+
+    for( SCH_SCREEN* screen = screens.GetFirst(); screen; screen = screens.GetNext() )
+        screen->MigrateSimModels();
 
     sheets.AnnotatePowerSymbols();
 
     // NOTE: This is required for multi-unit symbols to be correct
-    // Normally called from SCH_EDIT_FRAME::FixupJunctions() but could be refactored
     for( SCH_SHEET_PATH& sheet : sheets )
         sheet.UpdateAllScreenReferences();
 
@@ -111,19 +124,20 @@ bool generateSchematicNetlist( const wxString& aFilename, wxString& aNetlist )
 }
 
 
-static struct IFACE : public KIFACE_BASE
+static struct IFACE : public KIFACE_BASE, public UNITS_PROVIDER
 {
     // Of course all are virtual overloads, implementations of the KIFACE.
 
     IFACE( const char* aName, KIWAY::FACE_T aType ) :
-            KIFACE_BASE( aName, aType )
+            KIFACE_BASE( aName, aType ),
+            UNITS_PROVIDER( schIUScale, EDA_UNITS::MILLIMETRES )
     {}
 
     bool OnKifaceStart( PGM_BASE* aProgram, int aCtlBits ) override;
 
     void OnKifaceEnd() override;
 
-    wxWindow* CreateWindow( wxWindow* aParent, int aClassId, KIWAY* aKiway,
+    wxWindow* CreateKiWindow( wxWindow* aParent, int aClassId, KIWAY* aKiway,
                             int aCtlBits = 0 ) override
     {
         switch( aClassId )
@@ -132,6 +146,8 @@ static struct IFACE : public KIFACE_BASE
         {
             SCH_EDIT_FRAME* frame = new SCH_EDIT_FRAME( aKiway, aParent );
 
+            EESCHEMA_HELPERS::SetSchEditFrame( frame );
+
             if( Kiface().IsSingle() )
             {
                 // only run this under single_top, not under a project manager.
@@ -167,6 +183,70 @@ static struct IFACE : public KIFACE_BASE
             // Dialog has completed; nothing to return.
             return nullptr;
 
+        case PANEL_SYM_DISP_OPTIONS:
+        {
+            SETTINGS_MANAGER&  mgr = Pgm().GetSettingsManager();
+            APP_SETTINGS_BASE* cfg = mgr.GetAppSettings<SYMBOL_EDITOR_SETTINGS>();
+
+            return new PANEL_SYM_DISPLAY_OPTIONS( aParent, cfg );
+        }
+
+        case PANEL_SYM_EDIT_OPTIONS:
+        {
+            EDA_BASE_FRAME* frame = aKiway->Player( FRAME_SCH_SYMBOL_EDITOR, false );
+
+            if( !frame )
+                frame = aKiway->Player( FRAME_SCH_VIEWER, false );
+
+            if( !frame )
+                frame = aKiway->Player( FRAME_SCH, false );
+
+            if( frame )
+                return new PANEL_SYM_EDITING_OPTIONS( aParent, frame, frame );
+            else
+                return new PANEL_SYM_EDITING_OPTIONS( aParent, this, nullptr );
+        }
+
+        case PANEL_SYM_COLORS:
+            return new PANEL_SYM_COLOR_SETTINGS( aParent );
+
+        case PANEL_SCH_DISP_OPTIONS:
+        {
+            SETTINGS_MANAGER&  mgr = Pgm().GetSettingsManager();
+            APP_SETTINGS_BASE* cfg = mgr.GetAppSettings<EESCHEMA_SETTINGS>();
+
+            return new PANEL_EESCHEMA_DISPLAY_OPTIONS( aParent, cfg );
+        }
+
+        case PANEL_SCH_EDIT_OPTIONS:
+        {
+            EDA_BASE_FRAME* frame = aKiway->Player( FRAME_SCH, false );
+
+            if( !frame )
+                frame = aKiway->Player( FRAME_SCH_SYMBOL_EDITOR, false );
+
+            if( !frame )
+                frame = aKiway->Player( FRAME_SCH_VIEWER, false );
+
+            if( frame )
+                return new PANEL_EESCHEMA_EDITING_OPTIONS( aParent, frame, frame );
+            else
+                return new PANEL_EESCHEMA_EDITING_OPTIONS( aParent, this, nullptr );
+        }
+
+        case PANEL_SCH_ANNO_OPTIONS:
+        {
+            EDA_BASE_FRAME* schSettingsProvider = aKiway->Player( FRAME_SCH, false );
+
+            return new PANEL_EESCHEMA_ANNOTATION_OPTIONS( aParent, schSettingsProvider );
+        }
+
+        case PANEL_SCH_COLORS:
+            return new PANEL_EESCHEMA_COLOR_SETTINGS( aParent );
+
+        case PANEL_SCH_FIELD_NAME_TEMPLATES:
+            return new PANEL_TEMPLATE_FIELDNAMES( aParent, nullptr );
+
         default:
             return nullptr;
         }
@@ -202,6 +282,12 @@ static struct IFACE : public KIFACE_BASE
                      const wxString& aNewProjectBasePath, const wxString& aNewProjectName,
                      const wxString& aSrcFilePath, wxString& aErrors ) override;
 
+
+    int HandleJob( JOB* aJob ) override;
+
+private:
+    std::unique_ptr<EESCHEMA_JOBS_HANDLER> m_jobHandler;
+
 } kiface( "eeschema", KIWAY::FACE_SCH );
 
 } // namespace
@@ -250,9 +336,12 @@ bool IFACE::OnKifaceStart( PGM_BASE* aProgram, int aCtlBits )
 
     if( !fn.FileExists() )
     {
-        DIALOG_GLOBAL_SYM_LIB_TABLE_CONFIG fpDialog( nullptr );
+        if( !( aCtlBits & KFCTL_CLI ) )
+        {
+            DIALOG_GLOBAL_SYM_LIB_TABLE_CONFIG fpDialog( nullptr );
 
-        fpDialog.ShowModal();
+            fpDialog.ShowModal();
+        }
     }
     else
     {
@@ -269,15 +358,16 @@ bool IFACE::OnKifaceStart( PGM_BASE* aProgram, int aCtlBits )
             // if we are here, a incorrect global symbol library table was found.
             // Incorrect global symbol library table is not a fatal error:
             // the user just has to edit the (partially) loaded table.
-            wxString msg = _(
-                "An error occurred attempting to load the global symbol library table.\n"
-                "Please edit this global symbol library table in Preferences menu."
-                );
+            wxString msg =
+                    _( "An error occurred attempting to load the global symbol library table.\n"
+                       "Please edit this global symbol library table in Preferences menu." );
 
             DisplayErrorMessage( nullptr, msg, ioe.What() );
         }
     }
 
+    m_jobHandler = std::make_unique<EESCHEMA_JOBS_HANDLER>();
+
     return true;
 }
 
@@ -329,9 +419,10 @@ void IFACE::SaveFileAs( const wxString& aProjectBasePath, const wxString& aProje
             wxString msg;
 
             if( !aErrors.empty() )
-                aErrors += "\n";
+                aErrors += wxS( "\n" );
 
-            msg.Printf( _( "Cannot copy file '%s'." ), destFile.GetFullPath() );
+            msg.Printf( _( "Cannot copy file '%s' as it will be overwritten by the new root "
+                           "sheet file." ), destFile.GetFullPath() );
             aErrors += msg;
             return;
         }
@@ -355,8 +446,11 @@ void IFACE::SaveFileAs( const wxString& aProjectBasePath, const wxString& aProje
     else if( ext == LegacySymbolLibFileExtension || ext == LegacySymbolDocumentFileExtension ||
              ext == KiCadSymbolLibFileExtension )
     {
-        if( destFile.GetName() == aProjectName + wxT( "-cache" ) )
-            destFile.SetName( aNewProjectName + wxT( "-cache" )  );
+        if( destFile.GetName() == aProjectName + wxS( "-cache" ) )
+            destFile.SetName( aNewProjectName + wxS( "-cache" ) );
+
+        if( destFile.GetName() == aProjectName + wxS( "-rescue" ) )
+            destFile.SetName( aNewProjectName + wxS( "-rescue" ) );
 
         KiCopyFile( aSrcFilePath, destFile.GetFullPath(), aErrors );
     }
@@ -376,7 +470,7 @@ void IFACE::SaveFileAs( const wxString& aProjectBasePath, const wxString& aProje
                 {
                     if( node->IsList() && node->GetNumberOfChildren() > 1
                             && node->GetChild( 0 )->IsSymbol()
-                            && node->GetChild( 0 )->GetSymbol() == wxT( "source" ) )
+                            && node->GetChild( 0 )->GetSymbol() == "source" )
                     {
                         auto pathNode = dynamic_cast<SEXPR::SEXPR_STRING*>( node->GetChild( 1 ) );
                         auto symNode = dynamic_cast<SEXPR::SEXPR_SYMBOL*>( node->GetChild( 1 ) );
@@ -387,10 +481,10 @@ void IFACE::SaveFileAs( const wxString& aProjectBasePath, const wxString& aProje
                         else if( symNode )
                             path = symNode->m_value;
 
-                        if( path == aProjectName + wxT( ".sch" ) )
-                            path = aNewProjectName + wxT( ".sch" );
-                        else if( path == aProjectBasePath + wxT( "/" ) + aProjectName + wxT( ".sch" ) )
-                            path = aNewProjectBasePath + wxT( "/" ) + aNewProjectName + wxT( ".sch" );
+                        if( path == aProjectName + wxS( ".sch" ) )
+                            path = aNewProjectName + wxS( ".sch" );
+                        else if( path == aProjectBasePath + "/" + aProjectName + wxS( ".sch" ) )
+                            path = aNewProjectBasePath + "/" + aNewProjectName + wxS( ".sch" );
                         else if( path.StartsWith( aProjectBasePath ) )
                             path.Replace( aProjectBasePath, aNewProjectBasePath, false );
 
@@ -401,7 +495,7 @@ void IFACE::SaveFileAs( const wxString& aProjectBasePath, const wxString& aProje
                     }
                 } );
 
-            wxFFile destNetList( destFile.GetFullPath(), wxT( "wb" ) );
+            wxFFile destNetList( destFile.GetFullPath(), "wb" );
 
             if( destNetList.IsOpened() )
                 success = destNetList.Write( sexpr->AsString( 0 ) );
@@ -418,13 +512,13 @@ void IFACE::SaveFileAs( const wxString& aProjectBasePath, const wxString& aProje
             wxString msg;
 
             if( !aErrors.empty() )
-                aErrors += wxT( "\n" );
+                aErrors += wxS( "\n" );
 
             msg.Printf( _( "Cannot copy file '%s'." ), destFile.GetFullPath() );
             aErrors += msg;
         }
     }
-    else if( destFile.GetName() == wxT( "sym-lib-table" ) )
+    else if( destFile.GetName() == wxS( "sym-lib-table" ) )
     {
         SYMBOL_LIB_TABLE symbolLibTable;
         symbolLibTable.Load( aSrcFilePath );
@@ -434,10 +528,12 @@ void IFACE::SaveFileAs( const wxString& aProjectBasePath, const wxString& aProje
             LIB_TABLE_ROW& row = symbolLibTable.At( i );
             wxString       uri = row.GetFullURI();
 
-            uri.Replace( wxT( "/" ) + aProjectName + wxT( "-cache.lib" ), wxT( "/" ) + aNewProjectName + wxT( "-cache.lib" ) );
-            uri.Replace( wxT( "/" ) + aProjectName + wxT( "-rescue.lib" ), wxT( "/" ) + aNewProjectName +
-                         wxT( "-rescue.lib" ) );
-            uri.Replace( wxT( "/" ) + aProjectName + wxT( ".lib" ), wxT( "/" ) + aNewProjectName + wxT( ".lib" ) );
+            uri.Replace( wxS( "/" ) + aProjectName + wxS( "-cache.lib" ),
+                         wxS( "/" ) + aNewProjectName + wxS( "-cache.lib" ) );
+            uri.Replace( wxS( "/" ) + aProjectName + wxS( "-rescue.lib" ),
+                         wxS( "/" ) + aNewProjectName +  wxS( "-rescue.lib" ) );
+            uri.Replace( wxS( "/" ) + aProjectName + wxS( ".lib" ),
+                         wxS( "/" ) + aNewProjectName + wxS( ".lib" ) );
 
             row.SetFullURI( uri );
         }
@@ -451,7 +547,7 @@ void IFACE::SaveFileAs( const wxString& aProjectBasePath, const wxString& aProje
             wxString msg;
 
             if( !aErrors.empty() )
-                aErrors += wxT( "\n" );
+                aErrors += "\n";
 
             msg.Printf( _( "Cannot copy file '%s'." ), destFile.GetFullPath() );
             aErrors += msg;
@@ -459,7 +555,11 @@ void IFACE::SaveFileAs( const wxString& aProjectBasePath, const wxString& aProje
     }
     else
     {
-        wxFAIL_MSG( wxT( "Unexpected filetype for Eeschema::SaveFileAs()" ) );
+        wxFAIL_MSG( wxS( "Unexpected filetype for Eeschema::SaveFileAs()" ) );
     }
 }
 
+int IFACE::HandleJob( JOB* aJob )
+{
+    return m_jobHandler->RunJob( aJob );
+}
\ No newline at end of file
