@@ -3,7 +3,8 @@
  *
  * Copyright (C) 2004-2015 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2008 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 2004-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2022 CERN
+ * Copyright (C) 2004-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -59,10 +60,29 @@ wxString LIB_SYMBOL::GetSearchText()
         text += discount + footprint;
     }
 
+    // TODO(JE) rework this later so we can highlight matches in their column
+    std::map<wxString, wxString> fields;
+    GetChooserFields( fields );
+
+    for( const auto& it : fields )
+        text += discount + it.second;
+
     return text;
 }
 
 
+void LIB_SYMBOL::GetChooserFields( std::map<wxString , wxString>& aColumnMap )
+{
+    for( LIB_ITEM& item : m_drawings[ LIB_FIELD_T ] )
+    {
+        LIB_FIELD* field = static_cast<LIB_FIELD*>( &item );
+
+        if( field->ShowInChooser() )
+            aColumnMap[field->GetName()] = field->EDA_TEXT::GetShownText();
+    }
+}
+
+
 bool operator<( const LIB_SYMBOL& aItem1, const LIB_SYMBOL& aItem2 )
 {
     return aItem1.GetName() < aItem2.GetName();
@@ -86,7 +106,7 @@ LIB_SYMBOL::LIB_SYMBOL( const wxString& aName, LIB_SYMBOL* aParent, SYMBOL_LIB*
 {
     m_lastModDate    = 0;
     m_unitCount      = 1;
-    m_pinNameOffset  = Mils2iu( DEFAULT_PIN_NAME_OFFSET );
+    m_pinNameOffset  = schIUScale.MilsToIU( DEFAULT_PIN_NAME_OFFSET );
     m_options        = ENTRY_NORMAL;
     m_unitsLocked    = false;
     m_showPinNumbers = true;
@@ -131,6 +151,8 @@ LIB_SYMBOL::LIB_SYMBOL( const LIB_SYMBOL& aSymbol, SYMBOL_LIB* aLibrary ) :
     m_description    = aSymbol.m_description;
     m_keyWords       = aSymbol.m_keyWords;
 
+    aSymbol.CopyUnitDisplayNames( m_unitDisplayNames );
+
     ClearSelected();
 
     for( const LIB_ITEM& oldItem : aSymbol.m_drawings )
@@ -147,7 +169,8 @@ LIB_SYMBOL::LIB_SYMBOL( const LIB_SYMBOL& aSymbol, SYMBOL_LIB* aLibrary ) :
         }
         catch( ... )
         {
-            wxFAIL_MSG( wxT( "Failed to clone LIB_ITEM." ) );
+            wxFAIL_MSG( "Failed to clone LIB_ITEM." );
+            return;
         }
     }
 
@@ -158,11 +181,6 @@ LIB_SYMBOL::LIB_SYMBOL( const LIB_SYMBOL& aSymbol, SYMBOL_LIB* aLibrary ) :
 }
 
 
-LIB_SYMBOL::~LIB_SYMBOL()
-{
-}
-
-
 const LIB_SYMBOL& LIB_SYMBOL::operator=( const LIB_SYMBOL& aSymbol )
 {
     if( &aSymbol == this )
@@ -186,6 +204,9 @@ const LIB_SYMBOL& LIB_SYMBOL::operator=( const LIB_SYMBOL& aSymbol )
     m_description    = aSymbol.m_description;
     m_keyWords       = aSymbol.m_keyWords;
 
+    m_unitDisplayNames.clear();
+    aSymbol.CopyUnitDisplayNames( m_unitDisplayNames );
+
     m_drawings.clear();
 
     for( const LIB_ITEM& oldItem : aSymbol.m_drawings )
@@ -349,6 +370,16 @@ int LIB_SYMBOL::Compare( const LIB_SYMBOL& aRhs, int aCompareFlags ) const
             return ( m_includeOnBoard ) ? 1 : -1;
     }
 
+    // Compare unit display names
+    if( m_unitDisplayNames < aRhs.m_unitDisplayNames )
+    {
+        return -1;
+    }
+    else if( m_unitDisplayNames > aRhs.m_unitDisplayNames )
+    {
+        return 1;
+    }
+
     return 0;
 }
 
@@ -359,12 +390,57 @@ wxString LIB_SYMBOL::GetUnitReference( int aUnit )
 }
 
 
+bool LIB_SYMBOL::HasUnitDisplayName( int aUnit )
+{
+    return ( m_unitDisplayNames.count( aUnit ) == 1 );
+}
+
+
+wxString LIB_SYMBOL::GetUnitDisplayName( int aUnit )
+{
+    if( HasUnitDisplayName( aUnit ) )
+    {
+        return m_unitDisplayNames[aUnit];
+    }
+    else
+    {
+        return wxString::Format( _( "Unit %s" ), GetUnitReference( aUnit ) );
+    }
+}
+
+
+void LIB_SYMBOL::CopyUnitDisplayNames( std::map<int, wxString>& aTarget ) const
+{
+    for( const auto& it : m_unitDisplayNames )
+    {
+        aTarget[it.first] = it.second;
+    }
+}
+
+
+void LIB_SYMBOL::SetUnitDisplayName( int aUnit, const wxString& aName )
+{
+    if( aUnit <= GetUnitCount() )
+    {
+        if( aName.Length() > 0 )
+        {
+            m_unitDisplayNames[aUnit] = aName;
+        }
+        else
+        {
+            m_unitDisplayNames.erase( aUnit );
+        }
+    }
+}
+
+
 void LIB_SYMBOL::SetName( const wxString& aName )
 {
     m_name = aName;
     m_libId.SetLibItemName( aName );
 
-    GetValueField().SetText( aName );
+    if( IsPower() )
+        GetValueField().SetText( aName );
 }
 
 
@@ -386,7 +462,7 @@ std::unique_ptr< LIB_SYMBOL > LIB_SYMBOL::Flatten() const
         LIB_SYMBOL_SPTR parent = m_parent.lock();
 
         wxCHECK_MSG( parent, retv,
-                     wxString::Format( wxT( "Parent of derived symbol '%s' undefined" ), m_name ) );
+                     wxString::Format( "Parent of derived symbol '%s' undefined", m_name ) );
 
         // Copy the parent.
         retv.reset( new LIB_SYMBOL( *parent.get() ) );
@@ -447,6 +523,19 @@ std::unique_ptr< LIB_SYMBOL > LIB_SYMBOL::Flatten() const
 }
 
 
+void LIB_SYMBOL::ClearCaches()
+{
+    for( LIB_ITEM& item : m_drawings )
+    {
+        if( EDA_TEXT* eda_text = dynamic_cast<EDA_TEXT*>( &item ) )
+        {
+            eda_text->ClearBoundingBoxCache();
+            eda_text->ClearRenderCache();
+        }
+    }
+}
+
+
 const wxString LIB_SYMBOL::GetLibraryName() const
 {
     if( m_library )
@@ -496,6 +585,9 @@ wxString LIB_SYMBOL::SubReference( int aUnit, bool aAddSeparator )
 {
     wxString subRef;
 
+    if( aUnit < 1 )
+        return subRef;
+
     if( m_subpartIdSeparator != 0 && aAddSeparator )
         subRef << wxChar( m_subpartIdSeparator );
 
@@ -509,25 +601,24 @@ wxString LIB_SYMBOL::SubReference( int aUnit, bool aAddSeparator )
         // use one letter if letter = A .. Z or a ... z, and 2 letters otherwise
         // first letter is expected to be 'A' or 'a' (i.e. 26 letters are available)
         int u;
-        aUnit -= 1;     // Unit number starts to 1. now to 0.
+        wxString suffix;
 
-        while( aUnit >= 26 )    // more than one letter are needed
+        do
         {
-            u = aUnit / 26;
-            subRef << wxChar( m_subpartFirstId + u -1 );
-            aUnit %= 26;
-        }
+            u = ( aUnit - 1 ) % 26;
+            suffix = wxChar( m_subpartFirstId + u ) + suffix;
+            aUnit = ( aUnit - u ) / 26;
+        } while( aUnit > 0 );
 
-        u = m_subpartFirstId + aUnit;
-        subRef << wxChar( u );
+        subRef << suffix;
     }
 
     return subRef;
 }
 
 
-void LIB_SYMBOL::PrintBackground( const RENDER_SETTINGS* aSettings, const wxPoint& aOffset,
-                        int aUnit, int aConvert, const LIB_SYMBOL_OPTIONS& aOpts )
+void LIB_SYMBOL::PrintBackground( const RENDER_SETTINGS* aSettings, const VECTOR2I& aOffset,
+                        int aUnit, int aConvert, const LIB_SYMBOL_OPTIONS& aOpts, bool aDimmed )
 {
     /* draw background for filled items using background option
      * Solid lines will be drawn after the background
@@ -537,6 +628,10 @@ void LIB_SYMBOL::PrintBackground( const RENDER_SETTINGS* aSettings, const wxPoin
     {
         for( LIB_ITEM& item : m_drawings )
         {
+            // Do not print private items
+            if( item.IsPrivate() )
+                continue;
+
             if( item.Type() == LIB_SHAPE_T )
             {
                 LIB_SHAPE& shape = static_cast<LIB_SHAPE&>( item );
@@ -548,20 +643,24 @@ void LIB_SYMBOL::PrintBackground( const RENDER_SETTINGS* aSettings, const wxPoin
                 if( aConvert && shape.m_convert && ( shape.m_convert != aConvert ) )
                     continue;
 
-                if( shape.GetFillType() == FILL_T::FILLED_WITH_BG_BODYCOLOR )
-                    shape.Print( aSettings, aOffset, (void*) false, aOpts.transform );
+                if( shape.GetFillMode() == FILL_T::FILLED_WITH_BG_BODYCOLOR )
+                    shape.Print( aSettings, aOffset, (void*) false, aOpts.transform, aDimmed );
             }
         }
     }
 }
 
 
-void LIB_SYMBOL::Print( const RENDER_SETTINGS* aSettings, const wxPoint& aOffset, int aUnit,
-                        int aConvert, const LIB_SYMBOL_OPTIONS& aOpts )
+void LIB_SYMBOL::Print( const RENDER_SETTINGS* aSettings, const VECTOR2I& aOffset, int aUnit,
+                        int aConvert, const LIB_SYMBOL_OPTIONS& aOpts, bool aDimmed )
 {
 
     for( LIB_ITEM& item : m_drawings )
     {
+        // Do not print private items
+        if( item.IsPrivate() )
+            continue;
+
         // Do not draw items not attached to the current part
         if( aUnit && item.m_unit && ( item.m_unit != aUnit ) )
             continue;
@@ -582,60 +681,53 @@ void LIB_SYMBOL::Print( const RENDER_SETTINGS* aSettings, const wxPoint& aOffset
 
         if( item.Type() == LIB_PIN_T )
         {
-            item.Print( aSettings, aOffset, (void*) &aOpts, aOpts.transform );
+            item.Print( aSettings, aOffset, (void*) &aOpts, aOpts.transform, aDimmed );
         }
         else if( item.Type() == LIB_FIELD_T )
         {
-            item.Print( aSettings, aOffset, (void*) NULL, aOpts.transform );
+            item.Print( aSettings, aOffset, nullptr, aOpts.transform, aDimmed );
         }
         else if( item.Type() == LIB_SHAPE_T )
         {
             LIB_SHAPE& shape = static_cast<LIB_SHAPE&>( item );
-            bool       forceNoFill = shape.GetFillType() == FILL_T::FILLED_WITH_BG_BODYCOLOR;
+            bool       forceNoFill = shape.GetFillMode() == FILL_T::FILLED_WITH_BG_BODYCOLOR;
 
-            shape.Print( aSettings, aOffset, (void*) forceNoFill, aOpts.transform );
+            shape.Print( aSettings, aOffset, (void*) forceNoFill, aOpts.transform, aDimmed );
         }
         else
         {
-            item.Print( aSettings, aOffset, (void*) false, aOpts.transform );
+            item.Print( aSettings, aOffset, (void*) false, aOpts.transform, aDimmed );
         }
     }
 }
 
 
-void LIB_SYMBOL::Plot( PLOTTER* aPlotter, int aUnit, int aConvert, const wxPoint& aOffset,
-                       const TRANSFORM& aTransform ) const
+void LIB_SYMBOL::Plot( PLOTTER *aPlotter, int aUnit, int aConvert, bool aBackground,
+                       const VECTOR2I &aOffset, const TRANSFORM &aTransform, bool aDimmed ) const
 {
     wxASSERT( aPlotter != nullptr );
 
-    aPlotter->SetColor( aPlotter->RenderSettings()->GetLayerColor( LAYER_DEVICE ) );
+    COLOR4D color = aPlotter->RenderSettings()->GetLayerColor( LAYER_DEVICE );
+    COLOR4D bg = aPlotter->RenderSettings()->GetBackgroundColor();
 
-    // draw background for filled items using background option
-    // Solid lines will be drawn after the background
-    for( const LIB_ITEM& item : m_drawings )
+    if( bg == COLOR4D::UNSPECIFIED || !aPlotter->GetColorMode() )
+        bg = COLOR4D::WHITE;
+
+    if( aDimmed )
     {
-        if( item.Type() != LIB_SHAPE_T )
-            continue;
-
-        const LIB_SHAPE& shape = static_cast<const LIB_SHAPE&>( item );
-
-        // Do not draw items not attached to the current part
-        if( aUnit && shape.m_unit && ( shape.m_unit != aUnit ) )
-            continue;
-
-        if( aConvert && shape.m_convert && ( shape.m_convert != aConvert ) )
-            continue;
-
-        if( shape.GetFillType() == FILL_T::FILLED_WITH_BG_BODYCOLOR && aPlotter->GetColorMode() )
-            shape.Plot( aPlotter, aOffset, true, aTransform );
+        color.Desaturate( );
+        color = color.Mix( bg, 0.5f );
     }
+    aPlotter->SetColor( color );
 
-    // Not filled items and filled shapes are now plotted
-    // Items that have BG fills only get re-stroked to ensure the edges are in the foreground
     for( const LIB_ITEM& item : m_drawings )
     {
-        // Lib Fields are not plotted here, because this plot function
-        // is used to plot schematic items, which have they own fields
+        // Do not plot private items
+        if( item.IsPrivate() )
+            continue;
+
+        // LIB_FIELDs are not plotted here, because this plot function is used to plot schematic
+        // items which have their own SCH_FIELDs
         if( item.Type() == LIB_FIELD_T )
             continue;
 
@@ -645,32 +737,39 @@ void LIB_SYMBOL::Plot( PLOTTER* aPlotter, int aUnit, int aConvert, const wxPoint
         if( aConvert && item.m_convert && ( item.m_convert != aConvert ) )
             continue;
 
-        bool forceNoFill = false;
-
-        if( item.Type() == LIB_SHAPE_T )
-        {
-            const LIB_SHAPE& shape = static_cast<const LIB_SHAPE&>( item );
-            forceNoFill = shape.GetFillType() == FILL_T::FILLED_WITH_BG_BODYCOLOR;
-        }
-
-        item.Plot( aPlotter, aOffset, !forceNoFill, aTransform );
+        item.Plot( aPlotter, aBackground, aOffset, aTransform, aDimmed );
     }
 }
 
 
-void LIB_SYMBOL::PlotLibFields( PLOTTER* aPlotter, int aUnit, int aConvert,
-                                const wxPoint& aOffset, const TRANSFORM& aTransform )
+void LIB_SYMBOL::PlotLibFields( PLOTTER* aPlotter, int aUnit, int aConvert, bool aBackground,
+                                const VECTOR2I& aOffset, const TRANSFORM& aTransform, bool aDimmed,
+                                bool aPlotHidden )
 {
     wxASSERT( aPlotter != nullptr );
 
-    aPlotter->SetColor( aPlotter->RenderSettings()->GetLayerColor( LAYER_FIELDS ) );
-    bool fill = aPlotter->GetColorMode();
+    COLOR4D color = aPlotter->RenderSettings()->GetLayerColor( LAYER_FIELDS );
+    COLOR4D bg = aPlotter->RenderSettings()->GetBackgroundColor();
+
+    if( bg == COLOR4D::UNSPECIFIED || !aPlotter->GetColorMode() )
+        bg = COLOR4D::WHITE;
+
+    if( aDimmed )
+    {
+        color.Desaturate( );
+        color = color.Mix( bg, 0.5f );
+    }
+
+    aPlotter->SetColor( color );
 
     for( LIB_ITEM& item : m_drawings )
     {
         if( item.Type() != LIB_FIELD_T )
             continue;
 
+        if( !aPlotHidden && !( (LIB_FIELD&) item ).IsVisible() )
+            continue;
+
         if( aUnit && item.m_unit && ( item.m_unit != aUnit ) )
             continue;
 
@@ -689,7 +788,7 @@ void LIB_SYMBOL::PlotLibFields( PLOTTER* aPlotter, int aUnit, int aConvert,
             field.SetText( text );
         }
 
-        item.Plot( aPlotter, aOffset, fill, aTransform );
+        item.Plot( aPlotter, aBackground, aOffset, aTransform, aDimmed );
         field.SetText( tmp );
     }
 }
@@ -732,35 +831,6 @@ void LIB_SYMBOL::AddDrawItem( LIB_ITEM* aItem, bool aSort )
 }
 
 
-LIB_ITEM* LIB_SYMBOL::GetNextDrawItem( const LIB_ITEM* aItem, KICAD_T aType )
-{
-    if( aItem == nullptr )
-    {
-        LIB_ITEMS_CONTAINER::ITERATOR it1 = m_drawings.begin( aType );
-
-        return (it1 != m_drawings.end( aType ) ) ? &( *( m_drawings.begin( aType ) ) ) : nullptr;
-    }
-
-    // Search for the last item, assume aItem is of type aType
-    wxASSERT( ( aType == TYPE_NOT_INIT ) || ( aType == aItem->Type() ) );
-    LIB_ITEMS_CONTAINER::ITERATOR it = m_drawings.begin( aType );
-
-    while( ( it != m_drawings.end( aType ) ) && ( aItem != &( *it ) ) )
-        ++it;
-
-    // Search the next item
-    if( it != m_drawings.end( aType ) )
-    {
-        ++it;
-
-        if( it != m_drawings.end( aType ) )
-            return &( *it );
-    }
-
-    return nullptr;
-}
-
-
 void LIB_SYMBOL::GetPins( LIB_PINS& aList, int aUnit, int aConvert ) const
 {
     /* Notes:
@@ -788,18 +858,28 @@ void LIB_SYMBOL::GetPins( LIB_PINS& aList, int aUnit, int aConvert ) const
 }
 
 
+std::vector<LIB_PIN*> LIB_SYMBOL::GetAllLibPins() const
+{
+    std::vector<LIB_PIN*> pinList;
+
+    GetPins( pinList, 0, 0 );
+    return pinList;
+}
+
+
+
 LIB_PIN* LIB_SYMBOL::GetPin( const wxString& aNumber, int aUnit, int aConvert ) const
 {
     LIB_PINS pinList;
 
     GetPins( pinList, aUnit, aConvert );
 
-    for( size_t i = 0; i < pinList.size(); i++ )
+    for( LIB_PIN* pin : pinList )
     {
-        wxASSERT( pinList[i]->Type() == LIB_PIN_T );
+        wxASSERT( pin->Type() == LIB_PIN_T );
 
-        if( aNumber == pinList[i]->GetNumber() )
-            return pinList[i];
+        if( aNumber == pin->GetNumber() )
+            return pin;
     }
 
     return nullptr;
@@ -873,10 +953,10 @@ bool LIB_SYMBOL::PinsConflictWith( const LIB_SYMBOL& aOtherPart, bool aTestNums,
 }
 
 
-const EDA_RECT LIB_SYMBOL::GetUnitBoundingBox( int aUnit, int aConvert ) const
+const BOX2I LIB_SYMBOL::GetUnitBoundingBox( int aUnit, int aConvert,
+                                            bool aIgnoreHiddenFields ) const
 {
-    EDA_RECT bBox;
-    bool initialized = false;
+    BOX2I bBox;     // Start with a fresh BOX2I so the Merge algorithm works
 
     for( const LIB_ITEM& item : m_drawings )
     {
@@ -891,18 +971,11 @@ const EDA_RECT LIB_SYMBOL::GetUnitBoundingBox( int aUnit, int aConvert ) const
         if( item.m_convert > 0 && aConvert > 0 && aConvert != item.m_convert )
             continue;
 
-        if ( ( item.Type() == LIB_FIELD_T ) && !( ( LIB_FIELD& ) item ).IsVisible() )
+        if( aIgnoreHiddenFields && ( item.Type() == LIB_FIELD_T )
+            && !( (LIB_FIELD&) item ).IsVisible() )
             continue;
 
-        if( initialized )
-        {
-            bBox.Merge( item.GetBoundingBox() );
-        }
-        else
-        {
-            bBox = item.GetBoundingBox();
-            initialized = true;
-        }
+        bBox.Merge( item.GetBoundingBox() );
     }
 
     return bBox;
@@ -921,9 +994,10 @@ void LIB_SYMBOL::ViewGetLayers( int aLayers[], int& aCount ) const
 }
 
 
-const EDA_RECT LIB_SYMBOL::GetBodyBoundingBox( int aUnit, int aConvert, bool aIncludePins ) const
+const BOX2I LIB_SYMBOL::GetBodyBoundingBox( int aUnit, int aConvert, bool aIncludePins,
+                                            bool aIncludePrivateItems ) const
 {
-    EDA_RECT bbox;
+    BOX2I bbox;
 
     for( const LIB_ITEM& item : m_drawings )
     {
@@ -933,6 +1007,9 @@ const EDA_RECT LIB_SYMBOL::GetBodyBoundingBox( int aUnit, int aConvert, bool aIn
         if( item.m_convert > 0 && aConvert > 0 && aConvert != item.m_convert )
             continue;
 
+        if( item.IsPrivate() && !aIncludePrivateItems )
+            continue;
+
         if( item.Type() == LIB_FIELD_T )
             continue;
 
@@ -946,7 +1023,7 @@ const EDA_RECT LIB_SYMBOL::GetBodyBoundingBox( int aUnit, int aConvert, bool aIn
                 // a well-defined body.
 
                 if( aIncludePins )
-                    bbox.Merge( pin.GetBoundingBox( false, true ) );
+                    bbox.Merge( pin.GetBoundingBox( false, false, false ) );
                 else
                     bbox.Merge( pin.GetPinRoot() );
             }
@@ -973,14 +1050,14 @@ void LIB_SYMBOL::AddField( LIB_FIELD* aField )
 }
 
 
-void LIB_SYMBOL::SetFields( const std::vector <LIB_FIELD>& aFields )
+void LIB_SYMBOL::SetFields( const std::vector<LIB_FIELD>& aFields )
 {
     deleteAllFields();
 
-    for( unsigned i=0;  i<aFields.size();  ++i )
+    for( size_t ii = 0; ii < aFields.size(); ++ii )
     {
         // drawings is a ptr_vector, new and copy an object on the heap.
-        LIB_FIELD* field = new LIB_FIELD( aFields[i] );
+        LIB_FIELD* field = new LIB_FIELD( aFields[ ii ] );
 
         field->SetParent( this );
         m_drawings.push_back( field );
@@ -1094,6 +1171,32 @@ LIB_FIELD& LIB_SYMBOL::GetDatasheetField()
 }
 
 
+wxString LIB_SYMBOL::GetPrefix()
+{
+    wxString refDesignator = GetFieldById( REFERENCE_FIELD )->GetText();
+
+    refDesignator.Replace( wxS( "~" ), wxS( " " ) );
+
+    wxString prefix = refDesignator;
+
+    while( prefix.Length() )
+    {
+        wxUniCharRef last = prefix.Last();
+
+        if( ( last >= '0' && last <= '9' ) || last == '?' || last == '*' )
+            prefix.RemoveLast();
+        else
+            break;
+    }
+
+    // Avoid a prefix containing trailing/leading spaces
+    prefix.Trim( true );
+    prefix.Trim( false );
+
+    return prefix;
+}
+
+
 int LIB_SYMBOL::UpdateFieldOrdinals()
 {
     int retv = 0;
@@ -1133,7 +1236,7 @@ int LIB_SYMBOL::GetNextAvailableFieldId() const
 }
 
 
-void LIB_SYMBOL::SetOffset( const wxPoint& aOffset )
+void LIB_SYMBOL::SetOffset( const VECTOR2I& aOffset )
 {
     for( LIB_ITEM& item : m_drawings )
         item.Offset( aOffset );
@@ -1166,6 +1269,27 @@ bool LIB_SYMBOL::HasConversion() const
     return false;
 }
 
+int LIB_SYMBOL::GetMaxPinNumber() const
+{
+    int                        maxPinNumber = 0;
+    LIB_SYMBOL_SPTR            parent = m_parent.lock();
+    const LIB_ITEMS_CONTAINER& drawItems = parent ? parent->m_drawings : m_drawings;
+
+    for( const LIB_ITEM& item : drawItems[LIB_PIN_T] )
+    {
+        const LIB_PIN* pin = static_cast<const LIB_PIN*>( &item );
+        long           currentPinNumber = 0;
+        bool           isNum = pin->GetNumber().ToLong( &currentPinNumber );
+
+        if( isNum && currentPinNumber > maxPinNumber )
+        {
+            maxPinNumber = currentPinNumber;
+        }
+    }
+
+    return maxPinNumber;
+}
+
 
 void LIB_SYMBOL::ClearTempFlags()
 {
@@ -1181,8 +1305,8 @@ void LIB_SYMBOL::ClearEditFlags()
 }
 
 
-LIB_ITEM* LIB_SYMBOL::LocateDrawItem( int aUnit, int aConvert,
-                                      KICAD_T aType, const wxPoint& aPoint )
+LIB_ITEM* LIB_SYMBOL::LocateDrawItem( int aUnit, int aConvert, KICAD_T aType,
+                                      const VECTOR2I& aPoint )
 {
     for( LIB_ITEM& item : m_drawings )
     {
@@ -1202,10 +1326,10 @@ LIB_ITEM* LIB_SYMBOL::LocateDrawItem( int aUnit, int aConvert,
 
 
 LIB_ITEM* LIB_SYMBOL::LocateDrawItem( int aUnit, int aConvert, KICAD_T aType,
-                                      const wxPoint& aPoint, const TRANSFORM& aTransform )
+                                      const VECTOR2I& aPoint, const TRANSFORM& aTransform )
 {
     /* we use LocateDrawItem( int aUnit, int convert, KICAD_T type, const
-     * wxPoint& pt ) to search items.
+     * VECTOR2I& pt ) to search items.
      * because this function uses DefaultTransform as orient/mirror matrix
      * we temporary copy aTransform in DefaultTransform
      */
@@ -1222,20 +1346,20 @@ LIB_ITEM* LIB_SYMBOL::LocateDrawItem( int aUnit, int aConvert, KICAD_T aType,
 }
 
 
-SEARCH_RESULT LIB_SYMBOL::Visit( INSPECTOR aInspector, void* aTestData,
-                                 const KICAD_T aFilterTypes[] )
+INSPECT_RESULT LIB_SYMBOL::Visit( INSPECTOR aInspector, void* aTestData,
+                                  const std::vector<KICAD_T>& aScanTypes )
 {
     // The part itself is never inspected, only its children
     for( LIB_ITEM& item : m_drawings )
     {
-        if( item.IsType( aFilterTypes ) )
+        if( item.IsType( aScanTypes ) )
         {
-            if( aInspector( &item, aTestData ) == SEARCH_RESULT::QUIT )
-                return SEARCH_RESULT::QUIT;
+            if( aInspector( &item, aTestData ) == INSPECT_RESULT::QUIT )
+                return INSPECT_RESULT::QUIT;
         }
     }
 
-    return SEARCH_RESULT::CONTINUE;
+    return INSPECT_RESULT::CONTINUE;
 }
 
 
@@ -1446,7 +1570,7 @@ std::vector<struct LIB_SYMBOL_UNIT> LIB_SYMBOL::GetUniqueUnits()
         compareDrawItems = GetUnitDrawItems( unitNum, 1 );
 
         wxCHECK2_MSG( compareDrawItems.size() != 0, continue,
-                      wxT( "Multiple unit symbol defined with empty units." ) );
+                      "Multiple unit symbol defined with empty units." );
 
         if( currentDrawItems.size() != compareDrawItems.size() )
         {
@@ -1490,7 +1614,7 @@ std::vector<struct LIB_SYMBOL_UNIT> LIB_SYMBOL::GetUniqueUnits()
             compareDrawItems = GetUnitDrawItems( unitNum, 2 );
 
             wxCHECK2_MSG( compareDrawItems.size() != 0, continue,
-                          wxT( "Multiple unit symbol defined with empty units." ) );
+                          "Multiple unit symbol defined with empty units." );
 
             if( currentDrawItems.size() != compareDrawItems.size() )
             {
