@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2018 CERN
- * Copyright (C) 2019-2021 KiCad Developers, see change_log.txt for contributors.
+ * Copyright (C) 2019-2022 KiCad Developers, see AUTHORS.txt for contributors.
  * @author Jon Evans <jon@craftyjon.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -19,6 +19,7 @@
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <base_units.h>
 #include <lib_pin.h>
 #include <sch_symbol.h>
 #include <sch_pin.h>
@@ -26,11 +27,12 @@
 #include <schematic_settings.h>
 #include <sch_sheet_path.h>
 #include <sch_edit_frame.h>
-
+#include "string_utils.h"
 
 SCH_PIN::SCH_PIN( LIB_PIN* aLibPin, SCH_SYMBOL* aParentSymbol ) :
     SCH_ITEM( aParentSymbol, SCH_PIN_T )
 {
+    m_layer = LAYER_PIN;
     m_alt = wxEmptyString;
     m_number = aLibPin->GetNumber();
     m_libPin = aLibPin;
@@ -46,6 +48,7 @@ SCH_PIN::SCH_PIN( LIB_PIN* aLibPin, SCH_SYMBOL* aParentSymbol ) :
 SCH_PIN::SCH_PIN( SCH_SYMBOL* aParentSymbol, const wxString& aNumber, const wxString& aAlt ) :
     SCH_ITEM( aParentSymbol, SCH_PIN_T )
 {
+    m_layer = LAYER_PIN;
     m_alt = aAlt;
     m_number = aNumber;
     m_libPin = nullptr;
@@ -56,6 +59,7 @@ SCH_PIN::SCH_PIN( SCH_SYMBOL* aParentSymbol, const wxString& aNumber, const wxSt
 SCH_PIN::SCH_PIN( const SCH_PIN& aPin ) :
         SCH_ITEM( aPin )
 {
+    m_layer = aPin.m_layer;
     m_alt = aPin.m_alt;
     m_number = aPin.m_number;
     m_libPin = aPin.m_libPin;
@@ -94,7 +98,7 @@ wxString SCH_PIN::GetShownName() const
     if( !m_alt.IsEmpty() )
         name = m_alt;
 
-    if( name == wxT( "~" ) )
+    if( name == wxS( "~" ) )
         return wxEmptyString;
     else
         return name;
@@ -103,7 +107,7 @@ wxString SCH_PIN::GetShownName() const
 
 wxString SCH_PIN::GetShownNumber() const
 {
-    if( m_number == wxT( "~" ) )
+    if( m_number == wxS( "~" ) )
         return wxEmptyString;
     else
         return m_number;
@@ -140,6 +144,12 @@ int SCH_PIN::GetLength() const
 }
 
 
+const BOX2I SCH_PIN::ViewBBox() const
+{
+    return GetBoundingBox( false, true, true );
+}
+
+
 void SCH_PIN::ViewGetLayers( int aLayers[], int& aCount ) const
 {
     aCount     = 3;
@@ -149,9 +159,12 @@ void SCH_PIN::ViewGetLayers( int aLayers[], int& aCount ) const
 }
 
 
-bool SCH_PIN::Matches( const wxFindReplaceData& aSearchData, void* aAuxDat ) const
+bool SCH_PIN::Matches( const EDA_SEARCH_DATA& aSearchData, void* aAuxDat ) const
 {
-    if( !( aSearchData.GetFlags() & FR_SEARCH_ALL_PINS ) )
+    const SCH_SEARCH_DATA& schSearchData =
+            dynamic_cast<const SCH_SEARCH_DATA&>( aSearchData );
+
+    if( !schSearchData.searchAllPins )
         return false;
 
     return EDA_ITEM::Matches( GetName(), aSearchData )
@@ -159,7 +172,7 @@ bool SCH_PIN::Matches( const wxFindReplaceData& aSearchData, void* aAuxDat ) con
 }
 
 
-bool SCH_PIN::Replace( const wxFindReplaceData& aSearchData, void* aAuxData )
+bool SCH_PIN::Replace( const EDA_SEARCH_DATA& aSearchData, void* aAuxData )
 {
     bool isReplaced = false;
 
@@ -178,17 +191,16 @@ SCH_SYMBOL* SCH_PIN::GetParentSymbol() const
 }
 
 
-wxString SCH_PIN::GetSelectMenuText( EDA_UNITS aUnits ) const
+wxString SCH_PIN::GetItemDescription( UNITS_PROVIDER* aUnitsProvider ) const
 {
-    return wxString::Format( wxT( "%s %s" ),
-                             GetParentSymbol()->GetSelectMenuText( aUnits ),
-                             m_libPin->GetSelectMenuText( aUnits ) );
+    return wxString::Format( "Symbol %s %s",
+                             GetParentSymbol()->GetField( REFERENCE_FIELD )->GetShownText(),
+                             m_libPin->GetItemDescription( aUnitsProvider ) );
 }
 
 
 void SCH_PIN::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITEM>& aList )
 {
-    EDA_UNITS units = aFrame->GetUserUnits();
     wxString  msg;
 
     aList.emplace_back( _( "Type" ), _( "Pin" ) );
@@ -209,7 +221,7 @@ void SCH_PIN::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITE
 
     aList.emplace_back( _( "Visible" ), IsVisible() ? _( "Yes" ) : _( "No" ) );
 
-    aList.emplace_back( _( "Length" ), MessageTextFromValue( units, GetLength() ), true );
+    aList.emplace_back( _( "Length" ), aFrame->MessageTextFromValue( GetLength() ), true );
 
     int i = PinOrientationIndex( GetOrientation() );
     aList.emplace_back( _( "Orientation" ), PinOrientationName( (unsigned) i ) );
@@ -218,7 +230,7 @@ void SCH_PIN::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITE
     SCH_SHEET_PATH* currentSheet = schframe ? &schframe->GetCurrentSheet() : nullptr;
     SCH_SYMBOL*     symbol = GetParentSymbol();
 
-    aList.emplace_back( symbol->GetRef( currentSheet ), symbol->GetValue( currentSheet, true ) );
+    aList.emplace_back( symbol->GetRef( currentSheet ), symbol->GetValueFieldText( true ) );
 
 #if defined(DEBUG)
     if( !IsConnectivityDirty() && dynamic_cast<SCH_EDIT_FRAME*>( aFrame ) )
@@ -233,6 +245,16 @@ void SCH_PIN::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITE
 }
 
 
+bool SCH_PIN::IsStacked( const SCH_PIN* aPin ) const
+{
+    return m_parent == aPin->GetParent()
+            && GetTransformedPosition() == aPin->GetTransformedPosition()
+            && ( ( GetType() == aPin->GetType() )
+                    || ( GetType() == ELECTRICAL_PINTYPE::PT_PASSIVE )
+                    || ( aPin->GetType() == ELECTRICAL_PINTYPE::PT_PASSIVE ) );
+}
+
+
 void SCH_PIN::ClearDefaultNetName( const SCH_SHEET_PATH* aPath )
 {
     std::lock_guard<std::recursive_mutex> lock( m_netmap_mutex );
@@ -247,7 +269,7 @@ void SCH_PIN::ClearDefaultNetName( const SCH_SHEET_PATH* aPath )
 wxString SCH_PIN::GetDefaultNetName( const SCH_SHEET_PATH& aPath, bool aForceNoConnect )
 {
     if( m_libPin->IsPowerConnection() )
-        return m_libPin->GetName();
+        return EscapeString( m_libPin->GetName(), CTX_NETNAME );
 
     std::lock_guard<std::recursive_mutex> lock( m_netmap_mutex );
 
@@ -259,23 +281,57 @@ wxString SCH_PIN::GetDefaultNetName( const SCH_SHEET_PATH& aPath, bool aForceNoC
             return it->second.first;
     }
 
-    wxString name = wxT( "Net-(" );
+    wxString name = "Net-(";
+    bool unconnected = false;
 
     if( aForceNoConnect || GetType() == ELECTRICAL_PINTYPE::PT_NC )
-        name = ( wxT( "unconnected-(" ) );
-
-    name << GetParentSymbol()->GetRef( &aPath );
+    {
+        unconnected = true;
+        name = ( "unconnected-(" );
+    }
 
     bool annotated = true;
 
-    // Add timestamp for uninitialized symbols
-    if( name.Last() == '?' )
+    std::vector<SCH_PIN*> pins = GetParentSymbol()->GetPins( &aPath );
+    bool has_multiple = false;
+
+    for( SCH_PIN* pin : pins )
     {
-        name << GetParentSymbol()->m_Uuid.AsString();
-        annotated = false;
+        if( pin->GetShownName() == GetShownName()
+                && pin->GetShownNumber() != GetShownNumber()
+                && unconnected == ( pin->GetType() == ELECTRICAL_PINTYPE::PT_NC ) )
+        {
+            has_multiple = true;
+            break;
+        }
     }
 
-    name << wxT( "-Pad" ) << m_libPin->GetNumber() << wxT( ")" );
+    // Use timestamp for unannotated symbols
+    if( GetParentSymbol()->GetRef( &aPath, false ).Last() == '?' )
+    {
+        name << GetParentSymbol()->m_Uuid.AsString();
+        name << "-Pad" << m_libPin->GetNumber() << ")";
+        annotated = false;
+    }
+    else if( !m_libPin->GetShownName().IsEmpty()
+            && m_libPin->GetShownName() != m_libPin->GetShownNumber() )
+    {
+        // Pin names might not be unique between different units so we must have the
+        // unit token in the reference designator
+        name << GetParentSymbol()->GetRef( &aPath, true );
+        name << "-" << EscapeString( m_libPin->GetShownName(), CTX_NETNAME );
+
+        if( unconnected || has_multiple )
+            name << "-Pad" << EscapeString( m_libPin->GetShownNumber(), CTX_NETNAME );
+
+        name << ")";
+    }
+    else
+    {
+        // Pin numbers are unique, so we skip the unit token
+        name << GetParentSymbol()->GetRef( &aPath, false );
+        name << "-Pad" << EscapeString( m_libPin->GetShownNumber(), CTX_NETNAME ) << ")";
+    }
 
     if( annotated )
         m_net_name_map[ aPath ] = std::make_pair( name, aForceNoConnect );
@@ -284,17 +340,19 @@ wxString SCH_PIN::GetDefaultNetName( const SCH_SHEET_PATH& aPath, bool aForceNoC
 }
 
 
-wxPoint SCH_PIN::GetTransformedPosition() const
+VECTOR2I SCH_PIN::GetTransformedPosition() const
 {
     TRANSFORM t = GetParentSymbol()->GetTransform();
     return t.TransformCoordinate( GetLocalPosition() ) + GetParentSymbol()->GetPosition();
 }
 
 
-const EDA_RECT SCH_PIN::GetBoundingBox() const
+const BOX2I SCH_PIN::GetBoundingBox( bool aIncludeInvisiblePins, bool aIncludeNameAndNumber,
+                                     bool aIncludeElectricalType ) const
 {
     TRANSFORM t = GetParentSymbol()->GetTransform();
-    EDA_RECT  r = m_libPin->GetBoundingBox();
+    BOX2I     r = m_libPin->GetBoundingBox( aIncludeInvisiblePins, aIncludeNameAndNumber,
+                                            aIncludeElectricalType );
 
     r.RevertYAxis();
 
@@ -305,18 +363,32 @@ const EDA_RECT SCH_PIN::GetBoundingBox() const
 }
 
 
-bool SCH_PIN::HitTest( const wxPoint& aPosition, int aAccuracy ) const
+bool SCH_PIN::HitTest( const VECTOR2I& aPosition, int aAccuracy ) const
 {
     // When looking for an "exact" hit aAccuracy will be 0 which works poorly if the pin has
     // no pin number or name.  Give it a floor.
     if( Schematic() )
         aAccuracy = std::max( aAccuracy, Schematic()->Settings().m_PinSymbolSize / 4 );
 
-    EDA_RECT rect = GetBoundingBox();
+    BOX2I rect = GetBoundingBox( false, true, m_flags & SHOW_ELEC_TYPE );
     return rect.Inflate( aAccuracy ).Contains( aPosition );
 }
 
 
+bool SCH_PIN::HitTest( const BOX2I& aRect, bool aContained, int aAccuracy ) const
+{
+    BOX2I sel = aRect;
+
+    if( aAccuracy )
+        sel.Inflate( aAccuracy );
+
+    if( aContained )
+        return sel.Contains( GetBoundingBox( false, false, false ) );
+
+    return sel.Intersects( GetBoundingBox( false, true, m_flags & SHOW_ELEC_TYPE ) );
+}
+
+
 EDA_ITEM* SCH_PIN::Clone() const
 {
     return new SCH_PIN( *this );
