@@ -4,7 +4,8 @@
  * Copyright (C) 2015 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2014 Dick Hollenbeck, dick@softplc.com
  * Copyright (C) 2015 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2022 CERN
+ * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -46,11 +47,12 @@
 #include <sch_field.h>
 #include <sch_item.h>
 #include <sch_pin.h>
-#include <sch_sheet_path.h>    // SYMBOL_INSTANCE_REFERENCE
+#include <sch_sheet_path.h>    // SCH_SYMBOL_INSTANCE
 #include <symbol_lib_table.h>
 #include <transform.h>
 
 struct PICKED_SYMBOL;
+class KIID_PATH;
 class SCH_SCREEN;
 class LIB_ITEM;
 class LIB_PIN;
@@ -78,7 +80,7 @@ extern std::string toUTFTildaText( const wxString& txt );
 class SCH_SYMBOL : public SCH_ITEM
 {
 public:
-    SCH_SYMBOL( const wxPoint& aPosition = wxPoint( 0, 0 ), EDA_ITEM* aParent = nullptr );
+    SCH_SYMBOL();
 
     /**
      * Create schematic symbol from library symbol object.
@@ -88,14 +90,14 @@ public:
      * @param aSheet is the schematic sheet the symbol is place into.
      * @param aUnit is unit for symbols that have multiple parts per package.
      * @param aConvert is the alternate body style for the schematic symbols.
-     * @param aPositions is the position of the symbol.
+     * @param aPosition is the position of the symbol.
      */
     SCH_SYMBOL( const LIB_SYMBOL& aSymbol, const LIB_ID& aLibId, const SCH_SHEET_PATH* aSheet,
-                int aUnit, int aConvert = 0, const wxPoint& aPosition = wxPoint( 0, 0 ),
+                int aUnit, int aConvert = 0, const VECTOR2I& aPosition = VECTOR2I( 0, 0 ),
                 EDA_ITEM* aParent = nullptr );
 
     SCH_SYMBOL( const LIB_SYMBOL& aSymbol, const SCH_SHEET_PATH* aSheet, const PICKED_SYMBOL& aSel,
-                const wxPoint& aPosition = wxPoint( 0, 0 ), EDA_ITEM* aParent = nullptr );
+                const VECTOR2I& aPosition = VECTOR2I( 0, 0 ), EDA_ITEM* aParent = nullptr );
 
     /**
      * Clone \a aSymbol into a new schematic symbol object.
@@ -135,11 +137,21 @@ public:
      */
     bool IsMissingLibSymbol() const;
 
-    const std::vector<SYMBOL_INSTANCE_REFERENCE>& GetInstanceReferences()
+    const std::vector<SCH_SYMBOL_INSTANCE>& GetInstanceReferences()
     {
         return m_instanceReferences;
     }
 
+    bool GetInstance( SCH_SYMBOL_INSTANCE& aInstance,
+                      const KIID_PATH& aSheetPath, bool aTestFromEnd = false ) const;
+
+    void RemoveInstance( const SCH_SHEET_PATH& aInstancePath );
+
+    void RemoveAllInstances() { m_instanceReferences.clear(); }
+
+    void SortInstances( bool ( *aSortFunction )( const SCH_SYMBOL_INSTANCE& aLhs,
+                                                 const SCH_SYMBOL_INSTANCE& aRhs ) );
+
     void ViewGetLayers( int aLayers[], int& aCount ) const override;
 
     /**
@@ -199,10 +211,15 @@ public:
     void SetLibSymbol( LIB_SYMBOL* aLibSymbol );
 
     /**
-     * Return information about the aliased parts
+     * @return the associated LIB_SYMBOL's description field (or wxEmptyString).
      */
     wxString GetDescription() const;
 
+    /**
+     * @return the associated LIB_SYMBOL's keywords field (or wxEmptyString).
+     */
+    wxString GetKeyWords() const;
+
     /**
      * Return the documentation text for the given part alias
      */
@@ -226,6 +243,20 @@ public:
      */
     void SetUnit( int aUnit );
 
+    /**
+     * Return true if the given unit \a aUnit has a display name set.
+     *
+     * @return true if the display name of a unit is set, otherwise false.
+     */
+    bool HasUnitDisplayName( int aUnit );
+
+    /**
+     * Return the display name for a given unit \a aUnit.
+     *
+     * @return the display name of a unit if set, or the ordinal name of the unit otherwise.
+     */
+    wxString GetUnitDisplayName( int aUnit );
+
     /**
      * Change the unit number to \a aUnit without setting any internal flags.
      * This has meaning only for symbols made up of multiple units per package.
@@ -244,6 +275,11 @@ public:
 
     void SetPrefix( const wxString& aPrefix ) { m_prefix = aPrefix; }
 
+    /**
+     * Set the prefix based on the current reference designator.
+     */
+    void UpdatePrefix();
+
     TRANSFORM& GetTransform() { return m_transform; }
     const TRANSFORM& GetTransform() const { return m_transform; }
 
@@ -279,7 +315,7 @@ public:
      *
      * @return the orientation and mirror of the symbol.
      */
-    int GetOrientation();
+    int GetOrientation() const;
 
     /**
      * Return the list of system text vars & fields for this symbol.
@@ -302,8 +338,10 @@ public:
      *
      * @param aSheetPath is the hierarchical path of the symbol to clear or remove all
      *                   annotations for this symbol if NULL.
+     * @param[in] aResetPrefix The annotation prefix ('R', 'U', etc.) should be reset to the
+     *                         symbol library prefix.
      */
-    void ClearAnnotation( const SCH_SHEET_PATH* aSheetPath );
+    void ClearAnnotation( const SCH_SHEET_PATH* aSheetPath, bool aResetPrefix );
 
     /**
      * Add an instance to the alternate references list (m_instanceReferences), if this entry
@@ -329,17 +367,17 @@ public:
      */
     bool ReplaceInstanceSheetPath( const KIID_PATH& aOldSheetPath, const KIID_PATH& aNewSheetPath );
 
-    const EDA_RECT GetBoundingBox() const override;
+    const BOX2I GetBoundingBox() const override;
 
     /**
      * Return a bounding box for the symbol body but not the pins or fields.
      */
-    EDA_RECT GetBodyBoundingBox() const;
+    BOX2I GetBodyBoundingBox() const;
 
     /**
      * Return a bounding box for the symbol body and pins but not the fields.
      */
-    EDA_RECT GetBodyAndPinsBoundingBox() const;
+    BOX2I GetBodyAndPinsBoundingBox() const;
 
 
     //-----<Fields>-----------------------------------------------------------
@@ -369,7 +407,7 @@ public:
      *
      * @param aFieldName is the name of the field
      */
-    wxString GetFieldText( const wxString& aFieldName, SCH_EDIT_FRAME* aFrame ) const;
+    wxString GetFieldText( const wxString& aFieldName ) const;
 
     /**
      * Populate a std::vector with SCH_FIELDs.
@@ -401,6 +439,8 @@ public:
      */
     void RemoveField( const wxString& aFieldName );
 
+    void RemoveField( SCH_FIELD* aField ) { RemoveField( aField->GetName() ); }
+
     /**
      * Search for a #SCH_FIELD with \a aFieldName
      *
@@ -416,11 +456,17 @@ public:
      *
      * @param aFields are the fields to set in this symbol.
      */
-    void SetFields( const SCH_FIELDS& aFields )
+    void SetFields( std::vector<SCH_FIELD>& aFields )
     {
         m_fields = aFields;     // vector copying, length is changed possibly
     }
 
+    const wxString GetValueFieldText( bool aResolve ) const;
+    void SetValueFieldText( const wxString& aValue );
+
+    const wxString GetFootprintFieldText( bool aResolve ) const;
+    void SetFootprintFieldText( const wxString& aFootprint );
+
     /**
      * Restore fields to the original library values.
      *
@@ -471,6 +517,20 @@ public:
      */
     void GetLibPins( std::vector<LIB_PIN*>& aPinsList ) const;
 
+    /**
+     * @return a list of pin pointers for all units / converts.  Used primarily for SPICE where
+     * we want to treat all units together as a single SPICE element.
+     */
+    std::vector<LIB_PIN*> GetAllLibPins() const;
+
+    /**
+     * @return a count of pins for all units / converts.
+     */
+    size_t GetFullPinCount() { return GetAllLibPins().size(); }
+
+    /**
+     * @return the SCH_PIN associated with a particular LIB_PIN.
+     */
     SCH_PIN* GetPin( LIB_PIN* aLibPin ) const;
 
     /**
@@ -483,16 +543,26 @@ public:
      */
     std::vector<SCH_PIN*> GetPins( const SCH_SHEET_PATH* aSheet = nullptr ) const;
 
+
     std::vector<std::unique_ptr<SCH_PIN>>& GetRawPins() { return m_pins; }
 
     /**
      * Print a symbol.
      *
-     * @param aDC is the device context (can be null).
-     * @param aOffset is the drawing offset (usually wxPoint(0,0), but can be different when
+     * @param aSettings Render settings controlling output
+     * @param aOffset is the drawing offset (usually VECTOR2I(0,0), but can be different when
      *                moving an object)
      */
-    void Print( const RENDER_SETTINGS* aSettings, const wxPoint& aOffset ) override;
+    void Print( const RENDER_SETTINGS* aSettings, const VECTOR2I& aOffset ) override;
+
+    /**
+     * Print only the background parts of a symbol (if any)
+     *
+     * @param aSettings Render settings controlling output
+     * @param aOffset is the drawing offset (usually VECTOR2I(0,0), but can be different when
+     *                moving an object)
+     */
+    void PrintBackground( const RENDER_SETTINGS* aSettings, const VECTOR2I& aOffset ) override;
 
     void SwapData( SCH_ITEM* aItem ) override;
 
@@ -542,9 +612,9 @@ public:
      */
     void AddHierarchicalReference( const KIID_PATH& aPath,
                                    const wxString&  aRef,
-                                   int              aUnit,
-                                   const wxString&  aValue = wxEmptyString,
-                                   const wxString&  aFootprint = wxEmptyString );
+                                   int              aUnit );
+
+    void AddHierarchicalReference( const SCH_SYMBOL_INSTANCE& aInstance );
 
     /// Return the instance-specific unit selection for the given sheet path.
     int GetUnitSelection( const SCH_SHEET_PATH* aSheet ) const;
@@ -555,31 +625,11 @@ public:
     /// Set the selected unit of this symbol for all sheets.
     void SetUnitSelection( int aUnitSelection );
 
-    /// Return the instance-specific value for the given sheet path.
-    const wxString GetValue( const SCH_SHEET_PATH* sheet, bool aResolve ) const;
-    void SetValue( const SCH_SHEET_PATH* sheet, const wxString& aValue );
-
-    /// Set the value for all instances (the default GUI behavior).
-    void SetValue( const wxString& aValue )
-    {
-        SetValue( nullptr, aValue );
-    }
-
-    /// Return the instance-specific footprint assignment for the given sheet path.
-    const wxString GetFootprint( const SCH_SHEET_PATH* sheet, bool aResolve ) const;
-    void SetFootprint( const SCH_SHEET_PATH* sheet, const wxString& aFootprint );
-
-    /// Set the value for all instances (the default GUI behavior).
-    void SetFootprint( const wxString& aFootprint )
-    {
-        SetFootprint( nullptr, aFootprint );
-    }
-
     // Geometric transforms (used in block operations):
 
-    void Move( const wxPoint& aMoveVector ) override
+    void Move( const VECTOR2I& aMoveVector ) override
     {
-        if( aMoveVector == wxPoint( 0, 0 ) )
+        if( aMoveVector == VECTOR2I( 0, 0 ) )
             return;
 
         m_pos += aMoveVector;
@@ -592,9 +642,9 @@ public:
 
     void MirrorHorizontally( int aCenter ) override;
     void MirrorVertically( int aCenter ) override;
-    void Rotate( const wxPoint& aCenter ) override;
+    void Rotate( const VECTOR2I& aCenter ) override;
 
-    bool Matches( const wxFindReplaceData& aSearchData, void* aAuxData ) const override;
+    bool Matches( const EDA_SEARCH_DATA& aSearchData, void* aAuxData ) const override;
 
     void GetEndPoints( std::vector<DANGLING_END_ITEM>& aItemList ) override;
 
@@ -611,19 +661,20 @@ public:
     bool UpdateDanglingState( std::vector<DANGLING_END_ITEM>& aItemList,
                               const SCH_SHEET_PATH* aPath = nullptr ) override;
 
-    wxPoint GetPinPhysicalPosition( const LIB_PIN* Pin ) const;
+    VECTOR2I GetPinPhysicalPosition( const LIB_PIN* Pin ) const;
 
     bool IsConnectable() const override { return true; }
 
     bool CanConnect( const SCH_ITEM* aItem ) const override
     {
         return ( aItem->Type() == SCH_LINE_T && aItem->GetLayer() == LAYER_WIRE ) ||
-                ( aItem->Type() == SCH_NO_CONNECT_T ) ||
-                ( aItem->Type() == SCH_JUNCTION_T ) ||
-                ( aItem->Type() == SCH_SYMBOL_T ) ||
-                ( aItem->Type() == SCH_LABEL_T ) ||
-                ( aItem->Type() == SCH_HIER_LABEL_T ) ||
-                ( aItem->Type() == SCH_GLOBAL_LABEL_T );
+               ( aItem->Type() == SCH_NO_CONNECT_T ) ||
+               ( aItem->Type() == SCH_JUNCTION_T ) ||
+               ( aItem->Type() == SCH_SYMBOL_T ) ||
+               ( aItem->Type() == SCH_DIRECTIVE_LABEL_T ) ||
+               ( aItem->Type() == SCH_LABEL_T ) ||
+               ( aItem->Type() == SCH_HIER_LABEL_T ) ||
+               ( aItem->Type() == SCH_GLOBAL_LABEL_T );
     }
 
     /**
@@ -631,9 +682,10 @@ public:
      */
     bool IsInNetlist() const;
 
-    std::vector<wxPoint> GetConnectionPoints() const override;
+    std::vector<VECTOR2I> GetConnectionPoints() const override;
 
-    SEARCH_RESULT Visit( INSPECTOR inspector, void* testData, const KICAD_T scanTypes[] ) override;
+    INSPECT_RESULT Visit( INSPECTOR inspector, void* testData,
+                          const std::vector<KICAD_T>& aScanTypes ) override;
 
     /**
      * Return the symbol library item at \a aPosition that is part of this symbol.
@@ -642,9 +694,9 @@ public:
      * @param aType is the type of symbol library object to find or any if set to TYPE_NOT_INIT.
      * @return is the symbol library object if found otherwise NULL.
      */
-    LIB_ITEM* GetDrawItem( const wxPoint& aPosition, KICAD_T aType = TYPE_NOT_INIT );
+    LIB_ITEM* GetDrawItem( const VECTOR2I& aPosition, KICAD_T aType = TYPE_NOT_INIT );
 
-    wxString GetSelectMenuText( EDA_UNITS aUnits ) const override;
+    wxString GetItemDescription( UNITS_PROVIDER* aUnitsProvider ) const override;
 
     BITMAPS GetMenuImage() const override;
 
@@ -657,22 +709,31 @@ public:
 
     bool IsReplaceable() const override { return true; }
 
-    wxPoint GetPosition() const override { return m_pos; }
-    void SetPosition( const wxPoint& aPosition ) override { Move( aPosition - m_pos ); }
+    VECTOR2I GetPosition() const override { return m_pos; }
+    void    SetPosition( const VECTOR2I& aPosition ) override { Move( aPosition - m_pos ); }
 
-    bool HitTest( const wxPoint& aPosition, int aAccuracy = 0 ) const override;
-    bool HitTest( const EDA_RECT& aRect, bool aContained, int aAccuracy = 0 ) const override;
+    bool HitTest( const VECTOR2I& aPosition, int aAccuracy = 0 ) const override;
+    bool HitTest( const BOX2I& aRect, bool aContained, int aAccuracy = 0 ) const override;
 
-    void Plot( PLOTTER* aPlotter ) const override;
+    void Plot( PLOTTER* aPlotter, bool aBackground ) const override;
 
     /**
-     * Plot just the symbol pins.  This is separated to match the GAL display order.  The pins are
-     * ALSO plotted with the symbol group.  This replotting allows us to ensure that they are shown above
-     * other elements in the schematic
-     * @param aPlotter
+     * Plot just the symbol pins.  This is separated to match the GAL display order.  The pins
+     * are ALSO plotted with the symbol group.  This replotting allows us to ensure that they
+     * are shown above other elements in the schematic.
+     *
+     * @param aPlotter is the #PLOTTER object used to plot pins.
      */
     void PlotPins( PLOTTER* aPlotter ) const;
 
+    /**
+     * Plot the red 'X' over the symbol.  This is separated to allow it being used from the
+     * screen plot function, overlapping the pins
+     *
+     * @param aPlotter the #PLOTTER object used to draw the X
+     */
+    void PlotDNP( PLOTTER* aPlotter ) const;
+
     EDA_ITEM* Clone() const override;
 
 #if defined(DEBUG)
@@ -689,16 +750,19 @@ public:
     bool GetIncludeOnBoard() const { return m_onBoard; }
     void SetIncludeOnBoard( bool aIncludeOnBoard ) { m_onBoard = aIncludeOnBoard; }
 
-    bool IsPointClickableAnchor( const wxPoint& aPos ) const override;
+    bool GetDNP() const { return m_DNP; }
+    void SetDNP( bool aDNP ) { m_DNP = aDNP; }
+
+    bool IsPointClickableAnchor( const VECTOR2I& aPos ) const override;
 
 private:
-    EDA_RECT doGetBoundingBox( bool aIncludePins, bool aIncludeFields ) const;
+    BOX2I doGetBoundingBox( bool aIncludePins, bool aIncludeFields ) const;
 
-    bool doIsConnected( const wxPoint& aPosition ) const override;
+    bool doIsConnected( const VECTOR2I& aPosition ) const override;
 
-    void Init( const wxPoint& pos = wxPoint( 0, 0 ) );
+    void Init( const VECTOR2I& pos = VECTOR2I( 0, 0 ) );
 
-    wxPoint     m_pos;
+    VECTOR2I    m_pos;
     LIB_ID      m_lib_id;       ///< Name and library the symbol was loaded from, i.e. 74xx:74LS00.
     int         m_unit;         ///< The unit for multiple part per package symbols.
     int         m_convert;      ///< The alternate body style for symbols that have more than
@@ -729,10 +793,11 @@ private:
     bool        m_isInNetlist;  ///< True if the symbol should appear in the netlist
     bool        m_inBom;        ///< True to include in bill of materials export.
     bool        m_onBoard;      ///< True to include in netlist when updating board.
+    bool        m_DNP;          ///< True if symbol is set to 'Do Not Populate'.
 
     // Defines the hierarchical path and reference of the symbol.  This allows support
     // for multiple references to a single sub-sheet.
-    std::vector<SYMBOL_INSTANCE_REFERENCE> m_instanceReferences;
+    std::vector<SCH_SYMBOL_INSTANCE> m_instanceReferences;
 };
 
 #endif /* __SYMBOL_H__ */
