@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2013 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2008 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 2004-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2004-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -36,7 +36,7 @@
 #include <kiplatform/app.h>
 #include <kiway_express.h>
 #include <symbol_edit_frame.h>
-#include <symbol_library_manager.h>
+#include <lib_symbol_library_manager.h>
 #include <lib_text.h>
 #include <symbol_editor_settings.h>
 #include <paths.h>
@@ -65,20 +65,19 @@
 #include <tools/symbol_editor_move_tool.h>
 #include <tools/symbol_editor_pin_tool.h>
 #include <widgets/app_progress_dialog.h>
-#include <widgets/infobar.h>
+#include <widgets/wx_infobar.h>
 #include <widgets/lib_tree.h>
 #include <widgets/wx_progress_reporters.h>
 #include <widgets/symbol_tree_pane.h>
 #include <wildcards_and_files_ext.h>
 #include <panel_sym_lib_table.h>
-#include <wx/choicdlg.h>
 #include <string_utils.h>
 
 
 bool SYMBOL_EDIT_FRAME::m_showDeMorgan = false;
 
 
-BEGIN_EVENT_TABLE( SYMBOL_EDIT_FRAME, EDA_DRAW_FRAME )
+BEGIN_EVENT_TABLE( SYMBOL_EDIT_FRAME, SCH_BASE_FRAME )
     EVT_SIZE( SYMBOL_EDIT_FRAME::OnSize )
 
     EVT_COMBOBOX( ID_LIBEDIT_SELECT_UNIT_NUMBER, SYMBOL_EDIT_FRAME::OnSelectUnit )
@@ -91,6 +90,9 @@ BEGIN_EVENT_TABLE( SYMBOL_EDIT_FRAME, EDA_DRAW_FRAME )
     // Update user interface elements.
     EVT_UPDATE_UI( ID_LIBEDIT_SELECT_UNIT_NUMBER, SYMBOL_EDIT_FRAME::OnUpdateUnitNumber )
 
+    // Drop files event
+    EVT_DROP_FILES( SYMBOL_EDIT_FRAME::OnDropFiles )
+
 END_EVENT_TABLE()
 
 
@@ -109,7 +111,7 @@ SYMBOL_EDIT_FRAME::SYMBOL_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
     m_libMgr = nullptr;
     m_unit = 1;
     m_convert = 1;
-    m_aboutTitle = _( "KiCad Symbol Editor" );
+    m_aboutTitle = _HKI( "KiCad Symbol Editor" );
 
     wxIcon icon;
     wxIconBundle icon_bundle;
@@ -126,7 +128,7 @@ SYMBOL_EDIT_FRAME::SYMBOL_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
     m_settings = Pgm().GetSettingsManager().GetAppSettings<SYMBOL_EDITOR_SETTINGS>();
     LoadSettings( m_settings );
 
-    m_libMgr = new SYMBOL_LIBRARY_MANAGER( *this );
+    m_libMgr = new LIB_SYMBOL_LIBRARY_MANAGER( *this );
     bool loadingCancelled = false;
 
     {
@@ -175,28 +177,34 @@ SYMBOL_EDIT_FRAME::SYMBOL_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
     m_auimgr.SetManagedWindow( this );
 
     CreateInfoBar();
-    m_auimgr.AddPane( m_mainToolBar, EDA_PANE().HToolbar().Name( wxT( "MainToolbar" ) )
+
+    // Rows; layers 4 - 6
+    m_auimgr.AddPane( m_mainToolBar, EDA_PANE().HToolbar().Name( "MainToolbar" )
                       .Top().Layer( 6 ) );
-    m_auimgr.AddPane( m_messagePanel, EDA_PANE().Messages().Name( wxT( "MsgPanel" ) )
+
+    m_auimgr.AddPane( m_messagePanel, EDA_PANE().Messages().Name( "MsgPanel" )
                       .Bottom().Layer( 6 ) );
 
-    m_auimgr.AddPane( m_optionsToolBar, EDA_PANE().VToolbar().Name( wxT( "OptToolbar" ) )
-                      .Left().Layer( 3 ) );
-    m_auimgr.AddPane( m_treePane, EDA_PANE().Palette().Name( wxT( "SymbolTree" ) )
-                      .Left().Layer( 2 )
+    // Columns; layers 1 - 3
+    m_auimgr.AddPane( m_treePane, EDA_PANE().Palette().Name( "SymbolTree" )
+                      .Left().Layer( 3 )
                       .Caption( _( "Libraries" ) )
                       .MinSize( 250, -1 ).BestSize( 250, -1 ) );
-    m_auimgr.AddPane( m_drawToolBar, EDA_PANE().VToolbar().Name( wxT( "ToolsToolbar" ) )
+    m_auimgr.AddPane( m_optionsToolBar, EDA_PANE().VToolbar().Name( "OptToolbar" )
+                      .Left().Layer( 2 ) );
+
+    m_auimgr.AddPane( m_drawToolBar, EDA_PANE().VToolbar().Name( "ToolsToolbar" )
                       .Right().Layer( 2 ) );
 
-    m_auimgr.AddPane( GetCanvas(), wxAuiPaneInfo().Name( wxT( "DrawFrame" ) )
+    // Center
+    m_auimgr.AddPane( GetCanvas(), wxAuiPaneInfo().Name( "DrawFrame" )
                       .CentrePane() );
 
     FinishAUIInitialization();
 
     if( m_settings->m_LibWidth > 0 )
     {
-        wxAuiPaneInfo& treePane = m_auimgr.GetPane( wxT( "SymbolTree" ) );
+        wxAuiPaneInfo& treePane = m_auimgr.GetPane( "SymbolTree" );
 
         // wxAUI hack: force width by setting MinSize() and then Fixed()
         // thanks to ZenJu http://trac.wxwidgets.org/ticket/13180
@@ -224,8 +232,8 @@ SYMBOL_EDIT_FRAME::SYMBOL_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
 
     // Set the working/draw area size to display a symbol to a reasonable value:
     // A 600mm x 600mm with a origin at the area center looks like a large working area
-    double max_size_x = Millimeter2iu( 600 );
-    double max_size_y = Millimeter2iu( 600 );
+    double max_size_x = schIUScale.mmToIU( 600 );
+    double max_size_y = schIUScale.mmToIU( 600 );
     BOX2D bbox;
     bbox.SetOrigin( -max_size_x /2, -max_size_y/2 );
     bbox.SetSize( max_size_x, max_size_y );
@@ -233,6 +241,9 @@ SYMBOL_EDIT_FRAME::SYMBOL_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
 
     m_toolManager->RunAction( ACTIONS::zoomFitScreen, true );
 
+    m_acceptedExts.emplace( KiCadSymbolLibFileExtension, &ACTIONS::ddAddLibrary );
+    DragAcceptFiles( true );
+
     KIPLATFORM::APP::SetShutdownBlockReason( this, _( "Library changes are unsaved" ) );
 
     // Catch unhandled accelerator command characters that were no handled by the library tree
@@ -276,22 +287,18 @@ SYMBOL_EDIT_FRAME::~SYMBOL_EDIT_FRAME()
 
 void SYMBOL_EDIT_FRAME::LoadSettings( APP_SETTINGS_BASE* aCfg )
 {
-    wxCHECK_RET( m_settings, wxT( "Call to SYMBOL_EDIT_FRAME::LoadSettings with null m_boardAdapter" ) );
+    wxCHECK_RET( m_settings, "Call to SYMBOL_EDIT_FRAME::LoadSettings with null m_boardAdapter" );
 
     SCH_BASE_FRAME::LoadSettings( GetSettings() );
 
     GetRenderSettings()->m_ShowPinsElectricalType = m_settings->m_ShowPinElectricalType;
-
-    // Hidden elements must be editable
-    GetRenderSettings()->m_ShowHiddenText = true;
-    GetRenderSettings()->m_ShowHiddenPins = true;
-    GetRenderSettings()->m_ShowUmbilicals = false;
+    GetRenderSettings()->SetDefaultFont( wxEmptyString );
 }
 
 
 void SYMBOL_EDIT_FRAME::SaveSettings( APP_SETTINGS_BASE* aCfg )
 {
-    wxCHECK_RET( m_settings, wxT( "Call to SYMBOL_EDIT_FRAME::LoadSettings with null m_boardAdapter" ) );
+    wxCHECK_RET( m_settings, "Call to SYMBOL_EDIT_FRAME::LoadSettings with null m_boardAdapter" );
 
     GetGalDisplayOptions().m_axesEnabled = true;
 
@@ -308,7 +315,7 @@ APP_SETTINGS_BASE* SYMBOL_EDIT_FRAME::config() const
 }
 
 
-COLOR_SETTINGS* SYMBOL_EDIT_FRAME::GetColorSettings() const
+COLOR_SETTINGS* SYMBOL_EDIT_FRAME::GetColorSettings( bool aForceRefresh ) const
 {
     SETTINGS_MANAGER& mgr = Pgm().GetSettingsManager();
 
@@ -446,7 +453,8 @@ void SYMBOL_EDIT_FRAME::setupUIConditions()
             };
 
     mgr->SetConditions( EE_ACTIONS::showElectricalTypes, CHECK( pinTypeCond ) );
-    mgr->SetConditions( EE_ACTIONS::showSymbolTree, CHECK( showCompTreeCond ) );
+    mgr->SetConditions( ACTIONS::toggleBoundingBoxes,    CHECK( cond.BoundingBoxes() ) );
+    mgr->SetConditions( EE_ACTIONS::showSymbolTree,      CHECK( showCompTreeCond ) );
 
     auto demorganCond =
             [this]( const SELECTION& )
@@ -472,6 +480,12 @@ void SYMBOL_EDIT_FRAME::setupUIConditions()
                 return m_symbol && m_symbol->IsMulti() && !m_symbol->UnitsLocked();
             };
 
+    auto hasMultipleUnitsCond =
+            [this]( const SELECTION& )
+            {
+                return m_symbol && m_symbol->IsMulti();
+            };
+
     auto syncedPinsModeCond =
             [this]( const SELECTION& )
             {
@@ -495,6 +509,8 @@ void SYMBOL_EDIT_FRAME::setupUIConditions()
                         ACTION_CONDITIONS().Enable( demorganCond ).Check( demorganAlternateCond ) );
     mgr->SetConditions( EE_ACTIONS::toggleSyncedPinsMode,
                         ACTION_CONDITIONS().Enable( multiUnitModeCond ).Check( syncedPinsModeCond ) );
+    mgr->SetConditions( EE_ACTIONS::setUnitDisplayName,
+                        ACTION_CONDITIONS().Enable( isEditableCond && hasMultipleUnitsCond ) );
 
 // Only enable a tool if the symbol is edtable
 #define EDIT_TOOL( tool ) ACTION_CONDITIONS().Enable( isEditableCond ).Check( cond.CurrentTool( tool ) )
@@ -502,9 +518,10 @@ void SYMBOL_EDIT_FRAME::setupUIConditions()
     mgr->SetConditions( ACTIONS::deleteTool,             EDIT_TOOL( ACTIONS::deleteTool ) );
     mgr->SetConditions( EE_ACTIONS::placeSymbolPin,      EDIT_TOOL( EE_ACTIONS::placeSymbolPin ) );
     mgr->SetConditions( EE_ACTIONS::placeSymbolText,     EDIT_TOOL( EE_ACTIONS::placeSymbolText ) );
-    mgr->SetConditions( EE_ACTIONS::drawSymbolRectangle, EDIT_TOOL( EE_ACTIONS::drawSymbolRectangle ) );
-    mgr->SetConditions( EE_ACTIONS::drawSymbolCircle,    EDIT_TOOL( EE_ACTIONS::drawSymbolCircle ) );
-    mgr->SetConditions( EE_ACTIONS::drawSymbolArc,       EDIT_TOOL( EE_ACTIONS::drawSymbolArc ) );
+    mgr->SetConditions( EE_ACTIONS::drawSymbolTextBox,   EDIT_TOOL( EE_ACTIONS::drawSymbolTextBox ) );
+    mgr->SetConditions( EE_ACTIONS::drawRectangle,       EDIT_TOOL( EE_ACTIONS::drawRectangle ) );
+    mgr->SetConditions( EE_ACTIONS::drawCircle,          EDIT_TOOL( EE_ACTIONS::drawCircle ) );
+    mgr->SetConditions( EE_ACTIONS::drawArc,             EDIT_TOOL( EE_ACTIONS::drawArc ) );
     mgr->SetConditions( EE_ACTIONS::drawSymbolLines,     EDIT_TOOL( EE_ACTIONS::drawSymbolLines ) );
     mgr->SetConditions( EE_ACTIONS::placeSymbolAnchor,   EDIT_TOOL( EE_ACTIONS::placeSymbolAnchor ) );
 
@@ -540,7 +557,6 @@ bool SYMBOL_EDIT_FRAME::CanCloseSymbolFromSchematic( bool doClose )
 
     if( doClose )
     {
-        GetInfoBar()->ShowMessageFor( wxEmptyString, 1 );
         SetCurSymbol( nullptr, false );
         updateTitle();
     }
@@ -565,6 +581,9 @@ bool SYMBOL_EDIT_FRAME::canCloseWindow( wxCloseEvent& aEvent )
     if( !saveAllLibraries( true ) )
         return false;
 
+    // Save symbol tree column widths
+    m_libMgr->GetAdapter()->SaveSettings();
+
     return true;
 }
 
@@ -592,9 +611,8 @@ void SYMBOL_EDIT_FRAME::RebuildSymbolUnitsList()
     {
         for( int i = 0; i < m_symbol->GetUnitCount(); i++ )
         {
-            wxString sub  = LIB_SYMBOL::SubReference( i+1, false );
-            wxString unit = wxString::Format( _( "Unit %s" ), sub );
-            m_unitSelectBox->Append( unit );
+            wxString unitDisplayName = m_symbol->GetUnitDisplayName( i + 1 );
+            m_unitSelectBox->Append( unitDisplayName );
         }
     }
 
@@ -608,7 +626,7 @@ void SYMBOL_EDIT_FRAME::RebuildSymbolUnitsList()
 
 void SYMBOL_EDIT_FRAME::OnToggleSymbolTree( wxCommandEvent& event )
 {
-    auto& treePane = m_auimgr.GetPane( m_treePane );
+    wxAuiPaneInfo& treePane = m_auimgr.GetPane( m_treePane );
     treePane.Show( !IsSymbolTreeShown() );
     m_auimgr.Update();
 }
@@ -805,7 +823,7 @@ void SYMBOL_EDIT_FRAME::SetCurSymbol( LIB_SYMBOL* aSymbol, bool aUpdateZoom )
 }
 
 
-SYMBOL_LIBRARY_MANAGER& SYMBOL_EDIT_FRAME::GetLibManager()
+LIB_SYMBOL_LIBRARY_MANAGER& SYMBOL_EDIT_FRAME::GetLibManager()
 {
     wxASSERT( m_libMgr );
     return *m_libMgr;
@@ -814,14 +832,17 @@ SYMBOL_LIBRARY_MANAGER& SYMBOL_EDIT_FRAME::GetLibManager()
 
 void SYMBOL_EDIT_FRAME::OnModify()
 {
+    EDA_BASE_FRAME::OnModify();
+
     GetScreen()->SetContentModified();
+    m_autoSaveRequired = true;
 
     if( !IsSymbolFromSchematic() )
         storeCurrentSymbol();
 
     m_treePane->GetLibTree()->RefreshLibTree();
 
-    if( !GetTitle().StartsWith( wxT( "*" ) ) )
+    if( !GetTitle().StartsWith( "*" ) )
         updateTitle();
 }
 
@@ -832,13 +853,13 @@ bool SYMBOL_EDIT_FRAME::SynchronizePins()
 }
 
 
-bool SYMBOL_EDIT_FRAME::AddLibraryFile( bool aCreateNew )
+wxString SYMBOL_EDIT_FRAME::AddLibraryFile( bool aCreateNew )
 {
     // Select the target library table (global/project)
-    SYMBOL_LIB_TABLE* libTable = selectSymLibTable();
+    SYMBOL_LIB_TABLE* libTable = SelectSymLibTable();
 
     if( !libTable )
-        return false;
+        return wxEmptyString;
 
     wxFileName fn = m_libMgr->GetUniqueLibraryName();
 
@@ -847,18 +868,18 @@ bool SYMBOL_EDIT_FRAME::AddLibraryFile( bool aCreateNew )
                              ( libTable == &SYMBOL_LIB_TABLE::GetGlobalLibTable() ),
                              PATHS::GetDefaultUserSymbolsPath() ) )
     {
-        return false;
+        return wxEmptyString;
     }
 
     wxString libName = fn.GetName();
 
     if( libName.IsEmpty() )
-        return false;
+        return wxEmptyString;
 
     if( m_libMgr->LibraryExists( libName ) )
     {
         DisplayError( this, wxString::Format( _( "Library '%s' already exists." ), libName ) );
-        return false;
+        return wxEmptyString;
     }
 
     if( aCreateNew )
@@ -866,9 +887,10 @@ bool SYMBOL_EDIT_FRAME::AddLibraryFile( bool aCreateNew )
         if( !m_libMgr->CreateLibrary( fn.GetFullPath(), libTable ) )
         {
             DisplayError( this, wxString::Format( _( "Could not create the library file '%s'.\n"
-                                                     "Make sure you have write permissions and try again." ),
+                                                     "Make sure you have write permissions and "
+                                                     "try again." ),
                                                   fn.GetFullPath() ) );
-            return false;
+            return wxEmptyString;
         }
     }
     else
@@ -876,7 +898,7 @@ bool SYMBOL_EDIT_FRAME::AddLibraryFile( bool aCreateNew )
         if( !m_libMgr->AddLibrary( fn.GetFullPath(), libTable ) )
         {
             DisplayError( this, _( "Could not open the library file." ) );
-            return false;
+            return wxEmptyString;
         }
     }
 
@@ -886,7 +908,42 @@ bool SYMBOL_EDIT_FRAME::AddLibraryFile( bool aCreateNew )
     std::string packet = fn.GetFullPath().ToStdString();
     this->Kiway().ExpressMail( FRAME_SCH_SYMBOL_EDITOR, MAIL_LIB_EDIT, packet );
 
-    return true;
+    return fn.GetFullPath();
+}
+
+
+void SYMBOL_EDIT_FRAME::DdAddLibrary( wxString aLibFile )
+{
+        // Select the target library table (global/project)
+    SYMBOL_LIB_TABLE* libTable = SelectSymLibTable();
+
+    if( !libTable )
+        return;
+
+    wxFileName fn = wxFileName( aLibFile );
+
+    wxString libName = fn.GetName();
+
+    if( libName.IsEmpty() )
+        return;
+
+    if( m_libMgr->LibraryExists( libName ) )
+    {
+        DisplayError( this, wxString::Format( _( "Library '%s' already exists." ), libName ) );
+        return;
+    }
+
+    if( !m_libMgr->AddLibrary( fn.GetFullPath(), libTable ) )
+    {
+        DisplayError( this, _( "Could not open the library file." ) );
+        return;
+    }
+
+    bool globalTable = ( libTable == &SYMBOL_LIB_TABLE::GetGlobalLibTable() );
+    saveSymbolLibTables( globalTable, !globalTable );
+
+    std::string packet = fn.GetFullPath().ToStdString();
+    this->Kiway().ExpressMail( FRAME_SCH_SYMBOL_EDITOR, MAIL_LIB_EDIT, packet );
 }
 
 
@@ -896,14 +953,25 @@ LIB_ID SYMBOL_EDIT_FRAME::GetTreeLIBID( int* aUnit ) const
 }
 
 
+int SYMBOL_EDIT_FRAME::GetTreeSelectionCount() const
+{
+    return m_treePane->GetLibTree()->GetSelectionCount();
+}
+
+int SYMBOL_EDIT_FRAME::GetTreeLIBIDs( std::vector<LIB_ID>& aSelection ) const
+{
+    return m_treePane->GetLibTree()->GetSelectedLibIds( aSelection );
+}
+
+
 LIB_SYMBOL* SYMBOL_EDIT_FRAME::getTargetSymbol() const
 {
-    LIB_ID libId = GetTreeLIBID();
-
-    if( libId.IsValid() )
+    if( IsSymbolTreeShown() )
     {
-        LIB_SYMBOL* alias = m_libMgr->GetAlias( libId.GetLibItemName(), libId.GetLibNickname() );
-        return alias;
+        LIB_ID libId = GetTreeLIBID();
+
+        if( libId.IsValid() )
+            return m_libMgr->GetAlias( libId.GetLibItemName(), libId.GetLibNickname() );
     }
 
     return m_symbol;
@@ -924,6 +992,14 @@ LIB_ID SYMBOL_EDIT_FRAME::GetTargetLibId() const
 }
 
 
+std::vector<LIB_ID> SYMBOL_EDIT_FRAME::GetSelectedLibIds() const
+{
+    std::vector<LIB_ID> ids;
+    GetTreeLIBIDs( ids );
+    return ids;
+}
+
+
 LIB_TREE_NODE* SYMBOL_EDIT_FRAME::GetCurrentTreeNode() const
 {
     return m_treePane->GetLibTree()->GetCurrentTreeNode();
@@ -983,7 +1059,7 @@ void SYMBOL_EDIT_FRAME::SyncLibraries( bool aShowProgress, bool aPreloadCancelle
         // Try to select the parent library, in case the symbol is not found
         if( !found && selected.IsValid() )
         {
-            selected.SetLibItemName( wxEmptyString );
+            selected.SetLibItemName( "" );
             found = m_libMgr->GetAdapter()->FindItem( selected );
 
             if( found )
@@ -1017,46 +1093,19 @@ void SYMBOL_EDIT_FRAME::RefreshLibraryTree()
 }
 
 
-SYMBOL_LIB_TABLE* SYMBOL_EDIT_FRAME::selectSymLibTable( bool aOptional )
+void SYMBOL_EDIT_FRAME::FocusOnLibId( const LIB_ID& aLibID )
 {
-    // If no project is loaded, always work with the global table
-    if( Prj().IsNullProject() )
+    m_treePane->GetLibTree()->SelectLibId( aLibID );
+}
+
+
+void SYMBOL_EDIT_FRAME::UpdateLibraryTree( const wxDataViewItem& aTreeItem, LIB_SYMBOL* aSymbol )
+{
+    if( aTreeItem.IsOk() )   // Can be not found in tree if the current footprint is imported
+                             // from file therefore not yet in tree.
     {
-        SYMBOL_LIB_TABLE* ret = &SYMBOL_LIB_TABLE::GetGlobalLibTable();
-
-        if( aOptional )
-        {
-            wxMessageDialog dlg( this, _( "Add the library to the global library table?" ),
-                                 _( "Add To Global Library Table" ), wxYES_NO );
-
-            if( dlg.ShowModal() != wxID_OK )
-                ret = nullptr;
-        }
-
-        return ret;
-    }
-
-    wxArrayString libTableNames;
-    libTableNames.Add( _( "Global" ) );
-    libTableNames.Add( _( "Project" ) );
-
-    wxSingleChoiceDialog dlg( this, _( "Choose the Library Table to add the library to:" ),
-                              _( "Add To Library Table" ), libTableNames );
-
-    if( aOptional )
-    {
-        dlg.FindWindow( wxID_CANCEL )->SetLabel( _( "Skip" ) );
-        dlg.FindWindow( wxID_OK )->SetLabel( _( "Add" ) );
-    }
-
-    if( dlg.ShowModal() != wxID_OK )
-        return nullptr;
-
-    switch( dlg.GetSelection() )
-    {
-    case 0:  return &SYMBOL_LIB_TABLE::GetGlobalLibTable();
-    case 1:  return Prj().SchSymbolLibTable();
-    default: return nullptr;
+        static_cast<LIB_TREE_NODE_LIB_ID*>( aTreeItem.GetID() )->Update( aSymbol );
+        m_treePane->GetLibTree()->RefreshLibTree();
     }
 }
 
@@ -1090,11 +1139,11 @@ void SYMBOL_EDIT_FRAME::storeCurrentSymbol()
 }
 
 
-bool SYMBOL_EDIT_FRAME::isCurrentSymbol( const LIB_ID& aLibId ) const
+bool SYMBOL_EDIT_FRAME::IsCurrentSymbol( const LIB_ID& aLibId ) const
 {
     // This will return the root symbol of any alias
     LIB_SYMBOL* symbol = m_libMgr->GetBufferedSymbol( aLibId.GetLibItemName(),
-                                                  aLibId.GetLibNickname() );
+                                                      aLibId.GetLibNickname() );
 
     // Now we can compare the libId of the current symbol and the root symbol
     return ( symbol && m_symbol && symbol->GetLibId() == m_symbol->GetLibId() );
@@ -1117,8 +1166,17 @@ void SYMBOL_EDIT_FRAME::CommonSettingsChanged( bool aEnvVarsChanged, bool aTextV
 {
     SCH_BASE_FRAME::CommonSettingsChanged( aEnvVarsChanged, aTextVarsChanged );
 
-    GetCanvas()->GetGAL()->SetAxesColor( m_colorSettings->GetColor( LAYER_SCHEMATIC_GRID_AXES ) );
-    GetCanvas()->GetGAL()->DrawGrid();
+    SETTINGS_MANAGER*       mgr = GetSettingsManager();
+    SYMBOL_EDITOR_SETTINGS* cfg = mgr->GetAppSettings<SYMBOL_EDITOR_SETTINGS>();
+
+    GetRenderSettings()->m_ShowPinsElectricalType = cfg->m_ShowPinElectricalType;
+
+    GetGalDisplayOptions().ReadWindowSettings( cfg->m_Window );
+
+    if( m_symbol )
+        m_symbol->ClearCaches();
+
+    GetCanvas()->ForceRefresh();
 
     GetCanvas()->GetView()->UpdateAllItems( KIGFX::ALL );
     GetCanvas()->Refresh();
@@ -1191,6 +1249,8 @@ void SYMBOL_EDIT_FRAME::HardRedraw()
             else
                 item.SetSelected();
         }
+
+        m_symbol->ClearCaches();
     }
 
     RebuildView();
@@ -1201,14 +1261,16 @@ const BOX2I SYMBOL_EDIT_FRAME::GetDocumentExtents( bool aIncludeAllVisible ) con
 {
     if( !m_symbol )
     {
-        return BOX2I( VECTOR2I( Mils2iu( -100 ), Mils2iu( -100 ) ),
-                      VECTOR2I( Mils2iu( 200 ), Mils2iu( 200 ) ) );
+        // Gives a reasonable drawing area size
+        int width = schIUScale.mmToIU( 50 );
+        int height = schIUScale.mmToIU( 30 );
+
+        return BOX2I( VECTOR2I( -width/2, -height/2 ),
+                      VECTOR2I( width, height ) );
     }
     else
     {
-        EDA_RECT boundingBox = m_symbol->Flatten()->GetUnitBoundingBox( m_unit, m_convert );
-        return BOX2I( boundingBox.GetOrigin(), VECTOR2I( boundingBox.GetWidth(),
-                                                         boundingBox.GetHeight() ) );
+        return m_symbol->Flatten()->GetUnitBoundingBox( m_unit, m_convert );
     }
 }
 
@@ -1315,7 +1377,10 @@ void SYMBOL_EDIT_FRAME::ClearUndoORRedoList( UNDO_REDO_LIST whichList, int aItem
 
     for( PICKED_ITEMS_LIST* command : list.m_CommandsList )
     {
-        command->ClearListAndDeleteItems();
+        command->ClearListAndDeleteItems( []( EDA_ITEM* aItem )
+                                          {
+                                              delete aItem;
+                                          } );
         delete command;
     }
 
@@ -1339,14 +1404,14 @@ void SYMBOL_EDIT_FRAME::LoadSymbolFromSchematic( SCH_SYMBOL* aSymbol )
     for( int i = 0; i < (int) aSymbol->GetFields().size(); ++i )
     {
         const SCH_FIELD& field = aSymbol->GetFields()[i];
-        wxPoint          pos = field.GetPosition() - aSymbol->GetPosition();
+        VECTOR2I         pos = field.GetPosition() - aSymbol->GetPosition();
         LIB_FIELD        libField( symbol.get(), field.GetId() );
 
         if( i >= MANDATORY_FIELDS && !field.GetName( false ).IsEmpty() )
             libField.SetName( field.GetName( false ) );
 
         libField.SetText( field.GetText() );
-        libField.SetEffects( field );
+        libField.SetAttributes( field );
         libField.SetPosition( wxPoint( pos.x, -pos.y ) );
 
         fullSetOfFields.emplace_back( std::move( libField ) );
@@ -1363,6 +1428,12 @@ void SYMBOL_EDIT_FRAME::LoadSymbolFromSchematic( SCH_SYMBOL* aSymbol )
     m_unit = std::max( 1, aSymbol->GetUnit() );
     m_convert = std::max( 1, aSymbol->GetConvert() );
 
+    // Optimize default edit options for this symbol
+    // Usually if units are locked, graphic items are specific to each unit
+    // and if units are interchangeable, graphic items are common to units
+    SYMBOL_EDITOR_DRAWING_TOOLS* tools = GetToolManager()->GetTool<SYMBOL_EDITOR_DRAWING_TOOLS>();
+    tools->SetDrawSpecificUnit( symbol->UnitsLocked() );
+
     // The buffered screen for the symbol
     SCH_SCREEN* tmpScreen = new SCH_SCREEN();
 
@@ -1402,7 +1473,7 @@ bool SYMBOL_EDIT_FRAME::addLibTableEntry( const wxString& aLibFile, TABLE_SCOPE
 
         while( libTable->HasLibrary( libNickname ) )
         {
-            tmp.Printf( wxT( "%s%d" ), fn.GetName(), suffix );
+            tmp.Printf( "%s%d", fn.GetName(), suffix );
             libNickname = tmp;
             suffix += 1;
         }
@@ -1476,7 +1547,7 @@ bool SYMBOL_EDIT_FRAME::replaceLibTableEntry( const wxString& aLibNickname,
     wxString normalizedPath = NormalizePath( aLibFile, &envVars, projectPath );
 
     row->SetFullURI( normalizedPath );
-    row->SetType( wxT( "KiCad" ) );
+    row->SetType( "KiCad" );
 
     try
     {
@@ -1508,3 +1579,15 @@ bool SYMBOL_EDIT_FRAME::IsSymbolEditable() const
 {
     return m_symbol && ( !IsSymbolFromLegacyLibrary() || IsSymbolFromSchematic() );
 }
+
+
+void SYMBOL_EDIT_FRAME::UpdateItem( EDA_ITEM* aItem, bool isAddOrDelete, bool aUpdateRtree )
+{
+    SCH_BASE_FRAME::UpdateItem( aItem, isAddOrDelete, aUpdateRtree );
+
+    if( EDA_TEXT* eda_text = dynamic_cast<EDA_TEXT*>( aItem ) )
+    {
+        eda_text->ClearBoundingBoxCache();
+        eda_text->ClearRenderCache();
+    }
+}
