@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2017 Jon Evans <jon@craftyjon.com>
- * Copyright (C) 2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2017-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -20,9 +20,11 @@
 
 #include <gerbview_painter.h>
 #include <gal/graphics_abstraction_layer.h>
+#include <pgm_base.h>
+#include <settings/settings_manager.h>
 #include <settings/color_settings.h>
+#include <gerbview_settings.h>
 #include <convert_basic_shapes_to_polygon.h>
-#include <convert_to_biu.h>
 #include <gerbview.h>
 #include <trigo.h>
 
@@ -32,17 +34,17 @@
 
 using namespace KIGFX;
 
+
+GERBVIEW_SETTINGS* gvconfig()
+{
+    return Pgm().GetSettingsManager().GetAppSettings<GERBVIEW_SETTINGS>();
+}
+
+
 GERBVIEW_RENDER_SETTINGS::GERBVIEW_RENDER_SETTINGS()
 {
     m_backgroundColor = COLOR4D::BLACK;
 
-    m_spotFill          = true;
-    m_lineFill          = true;
-    m_polygonFill       = true;
-    m_showNegativeItems = false;
-    m_showCodes         = false;
-    m_diffMode          = true;
-
     m_componentHighlightString = "";
     m_netHighlightString       = "";
     m_attributeHighlightString = "";
@@ -54,17 +56,14 @@ GERBVIEW_RENDER_SETTINGS::GERBVIEW_RENDER_SETTINGS()
 
 void GERBVIEW_RENDER_SETTINGS::LoadColors( const COLOR_SETTINGS* aSettings )
 {
-    size_t palette_size = aSettings->m_Palette.size();
-    size_t palette_idx = 0;
-
     // Layers to draw gerber data read from gerber files:
     for( int i = GERBVIEW_LAYER_ID_START;
          i < GERBVIEW_LAYER_ID_START + GERBER_DRAWLAYERS_COUNT; i++ )
     {
         COLOR4D baseColor = aSettings->GetColor( i );
 
-        if( baseColor == COLOR4D::UNSPECIFIED )
-            baseColor = aSettings->m_Palette[ ( palette_idx++ ) % palette_size ];
+        if( gvconfig()->m_Display.m_DiffMode )
+            baseColor.a = 0.75;
 
         m_layerColors[i] = baseColor;
         m_layerColorsHi[i] = baseColor.Brightened( 0.5 );
@@ -73,29 +72,17 @@ void GERBVIEW_RENDER_SETTINGS::LoadColors( const COLOR_SETTINGS* aSettings )
     }
 
     // Draw layers specific to Gerbview:
-    // LAYER_DCODES, LAYER_NEGATIVE_OBJECTS, LAYER_GERBVIEW_GRID,
-    // LAYER_GERBVIEW_AXES, LAYER_GERBVIEW_BACKGROUND, LAYER_GERBVIEW_DRAWINGSHEET,
+    // LAYER_DCODES, LAYER_NEGATIVE_OBJECTS, LAYER_GERBVIEW_GRID, LAYER_GERBVIEW_AXES,
+    // LAYER_GERBVIEW_BACKGROUND, LAYER_GERBVIEW_DRAWINGSHEET, LAYER_GERBVIEW_PAGE_LIMITS
     for( int i = LAYER_DCODES; i < GERBVIEW_LAYER_ID_END; i++ )
         m_layerColors[i] = aSettings->GetColor( i );
 
     for( int i = GAL_LAYER_ID_START; i < GAL_LAYER_ID_END; i++ )
         m_layerColors[i] = aSettings->GetColor( i );
 
-    update();
-}
-
-
-void GERBVIEW_RENDER_SETTINGS::LoadDisplayOptions( const GBR_DISPLAY_OPTIONS& aOptions )
-{
-    m_spotFill          = aOptions.m_DisplayFlashedItemsFill;
-    m_lineFill          = aOptions.m_DisplayLinesFill;
-    m_polygonFill       = aOptions.m_DisplayPolygonsFill;
-    m_showNegativeItems = aOptions.m_DisplayNegativeObjects;
-    m_showCodes         = aOptions.m_DisplayDCodes;
-    m_diffMode          = aOptions.m_DiffMode;
-    m_hiContrastEnabled = aOptions.m_HighContrastMode;
-    m_showPageLimits    = aOptions.m_DisplayPageLimits;
-    m_backgroundColor   = aOptions.m_BgDrawColor;
+    // Ensure the generic LAYER_DRAWINGSHEET has the same color as the specialized
+    // LAYER_GERBVIEW_DRAWINGSHEET
+    m_layerColors[LAYER_DRAWINGSHEET] = m_layerColors[ LAYER_GERBVIEW_DRAWINGSHEET ];
 
     update();
 }
@@ -128,7 +115,7 @@ COLOR4D GERBVIEW_RENDER_SETTINGS::GetColor( const VIEW_ITEM* aItem, int aLayer )
 
     if( gbrItem && gbrItem->GetLayerPolarity() )
     {
-        if( m_showNegativeItems )
+        if( gvconfig()->m_Appearance.show_negative_objects )
             return m_layerColors[LAYER_NEGATIVE_OBJECTS];
         else
             return transparent;
@@ -164,6 +151,12 @@ COLOR4D GERBVIEW_RENDER_SETTINGS::GetColor( const VIEW_ITEM* aItem, int aLayer )
 }
 
 
+bool GERBVIEW_RENDER_SETTINGS::GetShowPageLimits() const
+{
+    return gvconfig()->m_Display.m_DisplayPageLimits;
+}
+
+
 GERBVIEW_PAINTER::GERBVIEW_PAINTER( GAL* aGal ) :
     PAINTER( aGal )
 {
@@ -173,9 +166,8 @@ GERBVIEW_PAINTER::GERBVIEW_PAINTER( GAL* aGal ) :
 // TODO(JE): Pull up to PAINTER?
 int GERBVIEW_PAINTER::getLineThickness( int aActualThickness ) const
 {
-    // if items have 0 thickness, draw them with the outline
-    // width, otherwise respect the set value (which, no matter
-    // how small will produce something)
+    // if items have 0 thickness, draw them with the outline width, otherwise respect the set
+    // value (which, no matter how small will produce something)
     if( aActualThickness == 0 )
         return m_gerbviewSettings.m_outlineWidth;
 
@@ -185,24 +177,15 @@ int GERBVIEW_PAINTER::getLineThickness( int aActualThickness ) const
 
 bool GERBVIEW_PAINTER::Draw( const VIEW_ITEM* aItem, int aLayer )
 {
-    const EDA_ITEM* item = dynamic_cast<const EDA_ITEM*>( aItem );
+    GERBER_DRAW_ITEM* gbrItem = dynamic_cast<GERBER_DRAW_ITEM*>( const_cast<VIEW_ITEM*>( aItem ) );
 
-    if( !item )
-        return false;
-
-    // the "cast" applied in here clarifies which overloaded draw() is called
-    switch( item->Type() )
+    if( gbrItem )
     {
-    case GERBER_DRAW_ITEM_T:
-        draw( static_cast<GERBER_DRAW_ITEM*>( const_cast<EDA_ITEM*>( item ) ), aLayer );
-        break;
-
-    default:
-        // Painter does not know how to draw the object
-        return false;
+        draw( gbrItem, aLayer );
+        return true;
     }
 
-    return true;
+    return false;
 }
 
 
@@ -221,16 +204,16 @@ void GERBVIEW_PAINTER::draw( /*const*/ GERBER_DRAW_ITEM* aItem, int aLayer )
     // Draw DCODE overlay text
     if( IsDCodeLayer( aLayer ) )
     {
-        wxString codeText;
-        VECTOR2D textPosition;
-        double textSize;
-        double orient;
+        wxString  codeText;
+        VECTOR2I  textPosition;
+        int       textSize;
+        EDA_ANGLE orient;
 
         if( !aItem->GetTextD_CodePrms( textSize, textPosition, orient ) )
             return;
 
         color = m_gerbviewSettings.GetColor( aItem, aLayer );
-        codeText.Printf( "D%d", aItem->m_DCode );
+        codeText.Printf( wxT( "D%d" ), aItem->m_DCode );
 
         m_gal->SetIsStroke( true );
         m_gal->SetIsFill( false );
@@ -242,8 +225,8 @@ void GERBVIEW_PAINTER::draw( /*const*/ GERBER_DRAW_ITEM* aItem, int aLayer )
         m_gal->SetFontUnderlined( false );
         m_gal->SetTextMirrored( false );
         m_gal->SetGlyphSize( VECTOR2D( textSize, textSize) );
-        m_gal->SetHorizontalJustify( GR_TEXT_HJUSTIFY_CENTER );
-        m_gal->SetVerticalJustify( GR_TEXT_VJUSTIFY_CENTER );
+        m_gal->SetHorizontalJustify( GR_TEXT_H_ALIGN_CENTER );
+        m_gal->SetVerticalJustify( GR_TEXT_V_ALIGN_CENTER );
         m_gal->BitmapText( codeText, textPosition, orient );
 
         return;
@@ -255,17 +238,17 @@ void GERBVIEW_PAINTER::draw( /*const*/ GERBER_DRAW_ITEM* aItem, int aLayer )
     if( aItem->IsBrightened() )
         color = COLOR4D( 0.0, 1.0, 0.0, 0.75 );
 
-    m_gal->SetNegativeDrawMode( isNegative && ! m_gerbviewSettings.IsShowNegativeItems() );
+    m_gal->SetNegativeDrawMode( isNegative && !gvconfig()->m_Appearance.show_negative_objects );
     m_gal->SetStrokeColor( color );
     m_gal->SetFillColor( color );
     m_gal->SetIsFill( isFilled );
     m_gal->SetIsStroke( !isFilled );
 
-    switch( aItem->m_Shape )
+    switch( aItem->m_ShapeType )
     {
     case GBR_POLYGON:
     {
-        isFilled = m_gerbviewSettings.m_polygonFill;
+        isFilled = gvconfig()->m_Display.m_DisplayPolygonsFill;
         m_gal->SetIsFill( isFilled );
         m_gal->SetIsStroke( !isFilled );
 
@@ -280,7 +263,7 @@ void GERBVIEW_PAINTER::draw( /*const*/ GERBER_DRAW_ITEM* aItem, int aLayer )
 
         if( aItem->m_AbsolutePolygon.OutlineCount() == 0 )
         {
-            std::vector<VECTOR2I> pts = aItem->m_Polygon.COutline( 0 ).CPoints();
+            std::vector<VECTOR2I> pts = aItem->m_ShapeAsPolygon.COutline( 0 ).CPoints();
 
             for( auto& pt : pts )
                 pt = aItem->GetABPosition( pt );
@@ -296,11 +279,14 @@ void GERBVIEW_PAINTER::draw( /*const*/ GERBER_DRAW_ITEM* aItem, int aLayer )
             m_gal->DrawPolyline( aItem->m_AbsolutePolygon.COutline( 0 ) );
         else
         {
-            // On Opengl, a not convex filled polygon is usually drawn by using triangles as primitives.
-            // CacheTriangulation() can create basic triangle primitives to draw the polygon solid shape
-            // on Opengl
+            // On Opengl, a not convex filled polygon is usually drawn by using triangles as
+            // primitives. CacheTriangulation() can create basic triangle primitives to draw the
+            // polygon solid shape on Opengl
+            // We use the fastest CacheTriangulation calculation mode: no partition created because
+            // the partition is useless in Gerbview, and very time consumming (optimized only
+            // for pcbnew that has different internal unit)
             if( m_gal->IsOpenGlEngine() && !aItem->m_AbsolutePolygon.IsTriangulationUpToDate() )
-                aItem->m_AbsolutePolygon.CacheTriangulation();
+                aItem->m_AbsolutePolygon.CacheTriangulation( false /* fastest triangulation calculation mode */ );
 
             m_gal->DrawPolygon( aItem->m_AbsolutePolygon );
         }
@@ -310,7 +296,7 @@ void GERBVIEW_PAINTER::draw( /*const*/ GERBER_DRAW_ITEM* aItem, int aLayer )
 
     case GBR_CIRCLE:
     {
-        isFilled = m_gerbviewSettings.m_lineFill;
+        isFilled = gvconfig()->m_Display.m_DisplayLinesFill;
         double radius = GetLineLength( aItem->m_Start, aItem->m_End );
         m_gal->DrawCircle( start, radius );
         break;
@@ -318,12 +304,12 @@ void GERBVIEW_PAINTER::draw( /*const*/ GERBER_DRAW_ITEM* aItem, int aLayer )
 
     case GBR_ARC:
     {
-        isFilled = m_gerbviewSettings.m_lineFill;
+        isFilled = gvconfig()->m_Display.m_DisplayLinesFill;
 
         // These are swapped because wxDC fills arcs counterclockwise and GAL
         // fills them clockwise.
-        wxPoint arcStart = aItem->m_End;
-        wxPoint arcEnd = aItem->m_Start;
+        VECTOR2I arcStart = aItem->m_End;
+        VECTOR2I arcEnd = aItem->m_Start;
 
         // Gerber arcs are 3-point (start, center, end)
         // GAL needs center, radius, start angle, end angle
@@ -336,22 +322,22 @@ void GERBVIEW_PAINTER::draw( /*const*/ GERBER_DRAW_ITEM* aItem, int aLayer )
         m_gal->SetIsStroke( !isFilled );
         m_gal->SetLineWidth( isFilled ? width : m_gerbviewSettings.m_outlineWidth );
 
-        double startAngle = startVec.Angle();
-        double endAngle = endVec.Angle();
+        EDA_ANGLE startAngle( startVec );
+        EDA_ANGLE endAngle( endVec );
 
         // GAL fills in direction of increasing angle, so we have to convert
         // the angle from the -PI to PI domain of atan2() to ensure that
         // the arc goes in the right direction
         if( startAngle > endAngle )
-            endAngle += (2 * M_PI);
+            endAngle += ANGLE_360;
 
         // In Gerber, 360-degree arcs are stored in the file with start equal to end
         if( arcStart == arcEnd )
-        {
-            endAngle =  startAngle + 2*M_PI;
-        }
+            endAngle = startAngle + ANGLE_360;
 
-        m_gal->DrawArcSegment( center, radius, startAngle, endAngle, width, ARC_HIGH_DEF );
+        // Adjust the allowed approx error to convert arcs to segments:
+        int arc_to_seg_error = gerbIUScale.mmToIU( 0.005 );    // Allow 5 microns
+        m_gal->DrawArcSegment( center, radius, startAngle, endAngle, width, arc_to_seg_error );
 
 #if 0   // Arc Debugging only
         m_gal->SetIsFill( false );
@@ -366,7 +352,7 @@ void GERBVIEW_PAINTER::draw( /*const*/ GERBER_DRAW_ITEM* aItem, int aLayer )
 #if 0   // Bbox arc Debugging only
         m_gal->SetIsFill( false );
         m_gal->SetIsStroke( true );
-        EDA_RECT box = aItem->GetBoundingBox();
+        BOX2I box = aItem->GetBoundingBox();
         m_gal->SetLineWidth( 5 );
         m_gal->SetStrokeColor( COLOR4D(0.9, 0.9, 0, 0.4) );
         // box coordinates are already in AB position.
@@ -380,11 +366,9 @@ void GERBVIEW_PAINTER::draw( /*const*/ GERBER_DRAW_ITEM* aItem, int aLayer )
     case GBR_SPOT_OVAL:
     case GBR_SPOT_POLY:
     case GBR_SPOT_MACRO:
-    {
-        isFilled = m_gerbviewSettings.m_spotFill;
+        isFilled = gvconfig()->m_Display.m_DisplayFlashedItemsFill;
         drawFlashedShape( aItem, isFilled );
         break;
-    }
 
     case GBR_SEGMENT:
     {
@@ -393,7 +377,7 @@ void GERBVIEW_PAINTER::draw( /*const*/ GERBER_DRAW_ITEM* aItem, int aLayer )
          * In fact, any aperture can be used to plot a line.
          * currently: only a square pen is handled (I believe using a polygon gives a strange plot).
          */
-        isFilled = m_gerbviewSettings.m_lineFill;
+        isFilled = gvconfig()->m_Display.m_DisplayLinesFill;
         m_gal->SetIsFill( isFilled );
         m_gal->SetIsStroke( !isFilled );
 
@@ -402,12 +386,12 @@ void GERBVIEW_PAINTER::draw( /*const*/ GERBER_DRAW_ITEM* aItem, int aLayer )
 
         // TODO(JE) Refactor this to allow const aItem
         D_CODE* code = aItem->GetDcodeDescr();
-        if( code && code->m_Shape == APT_RECT )
+        if( code && code->m_ApertType == APT_RECT )
         {
-            if( aItem->m_Polygon.OutlineCount() == 0 )
+            if( aItem->m_ShapeAsPolygon.OutlineCount() == 0 )
                 aItem->ConvertSegmentToPolygon();
 
-            drawPolygon( aItem, aItem->m_Polygon, isFilled );
+            drawPolygon( aItem, aItem->m_ShapeAsPolygon, isFilled );
         }
         else
         {
@@ -420,7 +404,7 @@ void GERBVIEW_PAINTER::draw( /*const*/ GERBER_DRAW_ITEM* aItem, int aLayer )
     }
 
     default:
-        wxASSERT_MSG( false, "GERBER_DRAW_ITEM shape is unknown!" );
+        wxASSERT_MSG( false, wxT( "GERBER_DRAW_ITEM shape is unknown!" ) );
         break;
     }
     m_gal->SetNegativeDrawMode( false );
@@ -438,8 +422,8 @@ void GERBVIEW_PAINTER::draw( /*const*/ GERBER_DRAW_ITEM* aItem, int aLayer )
 }
 
 
-void GERBVIEW_PAINTER::drawPolygon(
-        GERBER_DRAW_ITEM* aParent, const SHAPE_POLY_SET& aPolygon, bool aFilled, bool aShift )
+void GERBVIEW_PAINTER::drawPolygon( GERBER_DRAW_ITEM* aParent, const SHAPE_POLY_SET& aPolygon,
+                                    bool aFilled, bool aShift )
 {
     wxASSERT( aPolygon.OutlineCount() == 1 );
 
@@ -449,18 +433,16 @@ void GERBVIEW_PAINTER::drawPolygon(
     SHAPE_POLY_SET poly;
     poly.NewOutline();
     const std::vector<VECTOR2I> pts = aPolygon.COutline( 0 ).CPoints();
-    VECTOR2I       offset = aShift ? VECTOR2I( aParent->m_Start ) : VECTOR2I( 0, 0 );
+    VECTOR2I offset = aShift ? VECTOR2I( aParent->m_Start ) : VECTOR2I( 0, 0 );
 
-    for( auto& pt : pts )
+    for( const VECTOR2I& pt : pts )
         poly.Append( aParent->GetABPosition( pt + offset ) );
 
-    if( !m_gerbviewSettings.m_polygonFill )
+    if( !gvconfig()->m_Display.m_DisplayPolygonsFill )
         m_gal->SetLineWidth( m_gerbviewSettings.m_outlineWidth );
 
     if( !aFilled )
-    {
         m_gal->DrawPolyline( poly.COutline( 0 ) );
-    }
     else
         m_gal->DrawPolygon( poly );
 }
@@ -470,7 +452,7 @@ void GERBVIEW_PAINTER::drawFlashedShape( GERBER_DRAW_ITEM* aItem, bool aFilled )
 {
     D_CODE* code = aItem->GetDcodeDescr();
 
-    wxASSERT_MSG( code, "drawFlashedShape: Item has no D_CODE!" );
+    wxASSERT_MSG( code, wxT( "drawFlashedShape: Item has no D_CODE!" ) );
 
     if( !code )
         return;
@@ -479,7 +461,7 @@ void GERBVIEW_PAINTER::drawFlashedShape( GERBER_DRAW_ITEM* aItem, bool aFilled )
     m_gal->SetIsStroke( !aFilled );
     m_gal->SetLineWidth( m_gerbviewSettings.m_outlineWidth );
 
-    switch( aItem->m_Shape )
+    switch( aItem->m_ShapeType )
     {
     case GBR_SPOT_CIRCLE:
     {
@@ -493,7 +475,7 @@ void GERBVIEW_PAINTER::drawFlashedShape( GERBER_DRAW_ITEM* aItem, bool aFilled )
         else    // rectangular hole
         {
             if( code->m_Polygon.OutlineCount() == 0 )
-                code->ConvertShapeToPolygon();
+                code->ConvertShapeToPolygon( aItem );
 
             drawPolygon( aItem, code->m_Polygon, aFilled, true );
         }
@@ -503,11 +485,11 @@ void GERBVIEW_PAINTER::drawFlashedShape( GERBER_DRAW_ITEM* aItem, bool aFilled )
 
     case GBR_SPOT_RECT:
     {
-        wxPoint codeStart;
-        wxPoint aShapePos = aItem->m_Start;
+        VECTOR2I codeStart;
+        VECTOR2I aShapePos = aItem->m_Start;
         codeStart.x = aShapePos.x - code->m_Size.x / 2;
         codeStart.y = aShapePos.y - code->m_Size.y / 2;
-        wxPoint codeEnd = codeStart + code->m_Size;
+        VECTOR2I codeEnd = codeStart + code->m_Size;
         codeStart = aItem->GetABPosition( codeStart );
         codeEnd = aItem->GetABPosition( codeEnd );
 
@@ -518,7 +500,7 @@ void GERBVIEW_PAINTER::drawFlashedShape( GERBER_DRAW_ITEM* aItem, bool aFilled )
         else
         {
             if( code->m_Polygon.OutlineCount() == 0 )
-                code->ConvertShapeToPolygon();
+                code->ConvertShapeToPolygon( aItem );
 
             drawPolygon( aItem, code->m_Polygon, aFilled, true );
         }
@@ -529,8 +511,8 @@ void GERBVIEW_PAINTER::drawFlashedShape( GERBER_DRAW_ITEM* aItem, bool aFilled )
     {
         int radius = 0;
 
-        wxPoint codeStart = aItem->m_Start;
-        wxPoint codeEnd = aItem->m_Start;
+        VECTOR2I codeStart = aItem->m_Start;
+        VECTOR2I codeEnd = aItem->m_Start;
 
         if( code->m_Size.x > code->m_Size.y )   // horizontal oval
         {
@@ -557,7 +539,7 @@ void GERBVIEW_PAINTER::drawFlashedShape( GERBER_DRAW_ITEM* aItem, bool aFilled )
         else
         {
             if( code->m_Polygon.OutlineCount() == 0 )
-                code->ConvertShapeToPolygon();
+                code->ConvertShapeToPolygon( aItem );
 
             drawPolygon( aItem, code->m_Polygon, aFilled, true );
         }
@@ -565,13 +547,11 @@ void GERBVIEW_PAINTER::drawFlashedShape( GERBER_DRAW_ITEM* aItem, bool aFilled )
     }
 
     case GBR_SPOT_POLY:
-    {
         if( code->m_Polygon.OutlineCount() == 0 )
-            code->ConvertShapeToPolygon();
+            code->ConvertShapeToPolygon( aItem );
 
         drawPolygon( aItem, code->m_Polygon, aFilled, true );
         break;
-    }
 
     case GBR_SPOT_MACRO:
         drawApertureMacro( aItem, aFilled );
@@ -586,12 +566,11 @@ void GERBVIEW_PAINTER::drawFlashedShape( GERBER_DRAW_ITEM* aItem, bool aFilled )
 
 void GERBVIEW_PAINTER::drawApertureMacro( GERBER_DRAW_ITEM* aParent, bool aFilled )
 {
-    D_CODE* code = aParent->GetDcodeDescr();
+    D_CODE*         code = aParent->GetDcodeDescr();
     APERTURE_MACRO* macro = code->GetMacro();
-
     SHAPE_POLY_SET* macroShape = macro->GetApertureMacroShape( aParent, aParent->m_Start );
 
-    if( !m_gerbviewSettings.m_polygonFill )
+    if( !gvconfig()->m_Display.m_DisplayPolygonsFill )
         m_gal->SetLineWidth( m_gerbviewSettings.m_outlineWidth );
 
     if( !aFilled )
@@ -600,8 +579,10 @@ void GERBVIEW_PAINTER::drawApertureMacro( GERBER_DRAW_ITEM* aParent, bool aFille
             m_gal->DrawPolyline( macroShape->COutline( i ) );
     }
     else
+    {
         m_gal->DrawPolygon( *macroShape );
+    }
 }
 
 
-const double GERBVIEW_RENDER_SETTINGS::MAX_FONT_SIZE = Millimeter2iu( 10.0 );
+const double GERBVIEW_RENDER_SETTINGS::MAX_FONT_SIZE = gerbIUScale.mmToIU( 10.0 );
