@@ -35,6 +35,7 @@
 #include <macros.h>
 #include <X2_gerber_attributes.h>
 #include <gbr_metadata.h>
+#include <wx/log.h>
 
 extern int ReadInt( char*& text, bool aSkipSeparator = true );
 extern double ReadDouble( char*& text, bool aSkipSeparator = true );
@@ -208,7 +209,7 @@ bool GERBER_FILE_IMAGE::ExecuteRS274XCommand( int aCommand, char* aBuff,
     bool     y_fmt_known = false;
 
     // conv_scale = scaling factor from inch to Internal Unit
-    double   conv_scale = IU_PER_MILS * 1000;
+    double   conv_scale = gerbIUScale.IU_PER_MILS * 1000;
 
     if( m_GerbMetric )
         conv_scale /= 25.4;
@@ -380,7 +381,7 @@ bool GERBER_FILE_IMAGE::ExecuteRS274XCommand( int aCommand, char* aBuff,
         else if( code == MILLIMETER )
             m_GerbMetric = true;
 
-        conv_scale = m_GerbMetric ? IU_PER_MILS / 25.4 : IU_PER_MILS;
+        conv_scale = m_GerbMetric ? gerbIUScale.IU_PER_MILS / 25.4 : gerbIUScale.IU_PER_MILS;
         break;
 
     case FILE_ATTRIBUTE:    // Command %TF ...
@@ -413,13 +414,13 @@ bool GERBER_FILE_IMAGE::ExecuteRS274XCommand( int aCommand, char* aBuff,
         X2_ATTRIBUTE dummy;
         dummy.ParseAttribCmd( m_Current_File, aBuff, aBuffSize, aText, m_LineNum );
 
-        if( dummy.GetAttribute() == ".AperFunction" )
+        if( dummy.GetAttribute() == wxT( ".AperFunction" ) )
         {
             m_AperFunction = dummy.GetPrm( 1 );
 
             // A few function values can have other parameters. Add them
             for( int ii = 2; ii < dummy.GetPrmCount(); ii++ )
-                m_AperFunction << "," << dummy.GetPrm( ii );
+                m_AperFunction << wxT( "," ) << dummy.GetPrm( ii );
         }
     }
         break;
@@ -430,17 +431,17 @@ bool GERBER_FILE_IMAGE::ExecuteRS274XCommand( int aCommand, char* aBuff,
 
         dummy.ParseAttribCmd( m_Current_File, aBuff, aBuffSize, aText, m_LineNum );
 
-        if( dummy.GetAttribute() == ".N" )
+        if( dummy.GetAttribute() == wxT( ".N" ) )
         {
             m_NetAttributeDict.m_NetAttribType |= GBR_NETLIST_METADATA::GBR_NETINFO_NET;
             m_NetAttributeDict.m_Netname = FormatStringFromGerber( dummy.GetPrm( 1 ) );
         }
-        else if( dummy.GetAttribute() == ".C" )
+        else if( dummy.GetAttribute() == wxT( ".C" ) )
         {
             m_NetAttributeDict.m_NetAttribType |= GBR_NETLIST_METADATA::GBR_NETINFO_CMP;
             m_NetAttributeDict.m_Cmpref = FormatStringFromGerber( dummy.GetPrm( 1 ) );
         }
-        else if( dummy.GetAttribute() == ".P" )
+        else if( dummy.GetAttribute() == wxT( ".P" ) )
         {
             m_NetAttributeDict.m_NetAttribType |= GBR_NETLIST_METADATA::GBR_NETINFO_PAD;
             m_NetAttributeDict.m_Cmpref = FormatStringFromGerber( dummy.GetPrm( 1 ) );
@@ -582,9 +583,9 @@ bool GERBER_FILE_IMAGE::ExecuteRS274XCommand( int aCommand, char* aBuff,
         break;
 
     case IMAGE_JUSTIFY: // Command IJAnBn*
-        m_ImageJustifyXCenter = false;          // Image Justify Center on X axis (default = false)
-        m_ImageJustifyYCenter = false;          // Image Justify Center on Y axis (default = false)
-        m_ImageJustifyOffset = wxPoint(0,0);    // Image Justify Offset on XY axis (default = 0,0)
+        m_ImageJustifyXCenter = false;           // Image Justify Center on X axis (default = false)
+        m_ImageJustifyYCenter = false;           // Image Justify Center on Y axis (default = false)
+        m_ImageJustifyOffset = VECTOR2I( 0, 0 ); // Image Justify Offset on XY axis (default = 0,0)
         while( *aText && *aText != '*' )
         {
             // IJ command is (for A or B axis) AC or AL or A<coordinate>
@@ -734,7 +735,7 @@ bool GERBER_FILE_IMAGE::ExecuteRS274XCommand( int aCommand, char* aBuff,
             switch( stdAperture )   // Aperture desceiption has optional parameters. Read them
             {
             case 'C':               // Circle
-                dcode->m_Shape = APT_CIRCLE;
+                dcode->m_ApertType = APT_CIRCLE;
                 while( *aText == ' ' )
                     aText++;
 
@@ -762,7 +763,7 @@ bool GERBER_FILE_IMAGE::ExecuteRS274XCommand( int aCommand, char* aBuff,
 
             case 'O':               // oval
             case 'R':               // rect
-                dcode->m_Shape = (stdAperture == 'O') ? APT_OVAL : APT_RECT;
+                dcode->m_ApertType = (stdAperture == 'O') ? APT_OVAL : APT_RECT;
 
                 while( *aText == ' ' )
                     aText++;
@@ -804,7 +805,7 @@ bool GERBER_FILE_IMAGE::ExecuteRS274XCommand( int aCommand, char* aBuff,
                 /* Regular polygon: a command line like %ADD12P,0.040X10X25X0.025X0.025X0.0150*%
                  * params are: <diameter>, X<edge count>, X<Rotation>, X<X hole dim>, X<Y hole dim>
                  */
-                dcode->m_Shape = APT_POLYGON;
+                dcode->m_ApertType = APT_POLYGON;
                 while( *aText == ' ' )
                     aText++;
 
@@ -820,7 +821,7 @@ bool GERBER_FILE_IMAGE::ExecuteRS274XCommand( int aCommand, char* aBuff,
                 if( *aText == 'X' )
                 {
                     aText++;
-                    dcode->m_Rotation = ReadDouble( aText );
+                    dcode->m_Rotation = EDA_ANGLE( ReadDouble( aText ), DEGREES_T );
                 }
 
                 while( *aText == ' ' )
@@ -852,7 +853,7 @@ bool GERBER_FILE_IMAGE::ExecuteRS274XCommand( int aCommand, char* aBuff,
             APERTURE_MACRO am_lookup;
 
             while( *aText && *aText != '*' && *aText != ',' )
-                am_lookup.name.Append( *aText++ );
+                am_lookup.m_AmName.Append( *aText++ );
 
             // When an aperture definition is like %AMLINE2* 22,1,$1,$2,0,0,-45*
             // the ADDxx<MACRO_NAME> command has parameters, like %ADD14LINE2,0.8X0.5*%
@@ -865,6 +866,16 @@ bool GERBER_FILE_IMAGE::ExecuteRS274XCommand( int aCommand, char* aBuff,
                     double param = ReadDouble( aText );
                     dcode->AppendParam( param );
 
+                    if( !( isspace( *aText ) || *aText == 'X' || *aText == 'x' || *aText == '*' ) )
+                    {
+                        msg.Printf( wxT( "RS274X: aperture macro %s has invalid template "
+                                         "parameters\n" ),
+                                    TO_UTF8( am_lookup.m_AmName ) );
+                        AddMessageToList( msg );
+                        ok = false;
+                        break;
+                    }
+
                     while( isspace( *aText ) )
                         aText++;
 
@@ -880,13 +891,13 @@ bool GERBER_FILE_IMAGE::ExecuteRS274XCommand( int aCommand, char* aBuff,
             if( !pam )
             {
                 msg.Printf( wxT( "RS274X: aperture macro %s not found\n" ),
-                           TO_UTF8( am_lookup.name ) );
+                           TO_UTF8( am_lookup.m_AmName ) );
                 AddMessageToList( msg );
                 ok = false;
                 break;
             }
 
-            dcode->m_Shape = APT_MACRO;
+            dcode->m_ApertType = APT_MACRO;
             dcode->SetMacro( pam );
             dcode->m_Defined = true;
         }
@@ -976,7 +987,7 @@ bool GERBER_FILE_IMAGE::ReadApertureMacro( char *aBuff, unsigned int aBuffSize,
             break;
         }
 
-        am.name.Append( *aText++ );
+        am.m_AmName.Append( *aText++ );
     }
 
     // Read aperture macro parameters
@@ -1009,18 +1020,20 @@ bool GERBER_FILE_IMAGE::ReadApertureMacro( char *aBuff, unsigned int aBuffSize,
         // all other symbols are illegal.
         if( *aText == '$' )  // local parameter declaration, inside the aperture macro
         {
-            am.m_localparamStack.push_back( AM_PARAM() );
-            AM_PARAM& param = am.m_localparamStack.back();
+            am.AddLocalParamDefToStack();
+            AM_PARAM& param = am.GetLastLocalParamDefFromStack();
             aText = GetNextLine(  aBuff, aBuffSize, aText, gerber_file );
+
             if( aText == nullptr)   // End of File
                 return false;
-            param.ReadParam( aText );
+
+            param.ReadParamFromAmDef( aText );
             continue;
         }
         else if( !isdigit(*aText)  )     // Ill. symbol
         {
             msg.Printf( wxT( "RS274X: Aperture Macro \"%s\": ill. symbol, line: \"%s\"" ),
-                        am.name, FROM_UTF8( aBuff ) );
+                        am.m_AmName, FROM_UTF8( aBuff ) );
             AddMessageToList( msg );
             primitive_type = AMP_COMMENT;
         }
@@ -1055,10 +1068,6 @@ bool GERBER_FILE_IMAGE::ReadApertureMacro( char *aBuff, unsigned int aBuffSize,
             paramCount = 6;
             break;
 
-        case AMP_EOF:
-            paramCount = 0;
-            break;
-
         case AMP_OUTLINE:
             paramCount = 4; // partial count. other parameters are vertices and rotation
                             // Second parameter is vertice (coordinate pairs) count.
@@ -1078,7 +1087,7 @@ bool GERBER_FILE_IMAGE::ReadApertureMacro( char *aBuff, unsigned int aBuffSize,
 
         default:
             msg.Printf( wxT( "RS274X: Aperture Macro \"%s\": Invalid primitive id code %d, line %d: \"%s\"" ),
-                        am.name, primitive_type, m_LineNum, FROM_UTF8( aBuff ) );
+                        am.m_AmName, primitive_type, m_LineNum, FROM_UTF8( aBuff ) );
             AddMessageToList( msg );
             return false;
         }
@@ -1087,67 +1096,69 @@ bool GERBER_FILE_IMAGE::ReadApertureMacro( char *aBuff, unsigned int aBuffSize,
             continue;
 
         AM_PRIMITIVE prim( m_GerbMetric );
-        prim.primitive_id = (AM_PRIMITIVE_ID) primitive_type;
+        prim.m_Primitive_id = (AM_PRIMITIVE_ID) primitive_type;
         int ii;
 
         for( ii = 0; ii < paramCount && *aText && *aText != '*'; ++ii )
         {
-            prim.params.push_back( AM_PARAM() );
+            prim.m_Params.push_back( AM_PARAM() );
 
-            AM_PARAM& param = prim.params.back();
+            AM_PARAM& param = prim.m_Params.back();
 
             aText = GetNextLine( aBuff, aBuffSize, aText, gerber_file );
 
             if( aText == nullptr)   // End of File
                 return false;
 
-            param.ReadParam( aText );
+            param.ReadParamFromAmDef( aText );
         }
 
         if( ii < paramCount )
         {
             // maybe some day we can throw an exception and track a line number
-            msg.Printf( "RS274X: read macro descr type %d: read %d parameters, insufficient parameters\n",
-                        prim.primitive_id, ii );
+            msg.Printf( wxT( "RS274X: read macro descr type %d: read %d parameters, insufficient "
+                             "parameters\n" ),
+                        prim.m_Primitive_id, ii );
             AddMessageToList( msg );
         }
 
         // there are more parameters to read if this is an AMP_OUTLINE
-        if( prim.primitive_id == AMP_OUTLINE )
+        if( prim.m_Primitive_id == AMP_OUTLINE )
         {
             // so far we have read [0]:exposure, [1]:#points, [2]:X start, [3]: Y start
             // Now read all the points, plus trailing rotation in degrees.
 
-            // params[1] is a count of polygon points, so it must be given
+            // m_Params[1] is a count of polygon points, so it must be given
             // in advance, i.e. be immediate.
-            wxASSERT( prim.params[1].IsImmediate() );
+            wxASSERT( prim.m_Params[1].IsImmediate() );
 
-            paramCount = (int) prim.params[1].GetValue( nullptr ) * 2 + 1;
+            paramCount = (int) prim.m_Params[1].GetValue( nullptr ) * 2 + 1;
 
             for( int jj = 0; jj < paramCount && *aText != '*'; ++jj )
             {
-                prim.params.push_back( AM_PARAM() );
+                prim.m_Params.push_back( AM_PARAM() );
 
-                AM_PARAM& param = prim.params.back();
+                AM_PARAM& param = prim.m_Params.back();
 
                 aText = GetNextLine( aBuff, aBuffSize, aText, gerber_file );
 
                 if( aText == nullptr )  // End of File
                     return false;
 
-                param.ReadParam( aText );
+                param.ReadParamFromAmDef( aText );
             }
         }
 
         // AMP_CIRCLE can have a optional parameter (rotation)
-        if( prim.primitive_id == AMP_CIRCLE && aText && *aText != '*' )
+        if( prim.m_Primitive_id == AMP_CIRCLE && aText && *aText != '*' )
         {
-            prim.params.push_back( AM_PARAM() );
-            AM_PARAM& param = prim.params.back();
-            param.ReadParam( aText );
+            prim.m_Params.push_back( AM_PARAM() );
+            AM_PARAM& param = prim.m_Params.back();
+            param.ReadParamFromAmDef( aText );
         }
 
-        am.primitives.push_back( prim );
+        // The primitive description is now parsed: push it to the current aperture macro
+        am.AddPrimitiveToList( prim );
     }
 
     m_aperture_macros.insert( am );
