@@ -2,7 +2,7 @@
  * This program source code file is part of KICAD, a free EDA CAD application.
  *
  * Copyright (C) 2016-2018 CERN
- * Copyright (C) 2020 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2020-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * @author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  *
@@ -24,18 +24,19 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
+
+#include <algorithm>
+#include <future>
+#include <mutex>
+
 #include <connectivity/connectivity_algo.h>
 #include <progress_reporter.h>
 #include <geometry/geometry_utils.h>
 #include <board_commit.h>
+#include <thread_pool.h>
 
 #include <wx/log.h>
 
-#include <thread>
-#include <mutex>
-#include <algorithm>
-#include <future>
-
 #ifdef PROFILE
 #include <profile.h>
 #endif
@@ -97,7 +98,7 @@ void CN_CONNECTIVITY_ALGO::markItemNetAsDirty( const BOARD_ITEM* aItem )
 {
     if( aItem->IsConnected() )
     {
-        auto citem = static_cast<const BOARD_CONNECTED_ITEM*>( aItem );
+        const BOARD_CONNECTED_ITEM* citem = static_cast<const BOARD_CONNECTED_ITEM*>( aItem );
         MarkNetAsDirty( citem->GetNetCode() );
     }
     else
@@ -206,21 +207,22 @@ bool CN_CONNECTIVITY_ALGO::Add( BOARD_ITEM* aItem )
 void CN_CONNECTIVITY_ALGO::searchConnections()
 {
 #ifdef PROFILE
-    PROF_COUNTER garbage_collection( "garbage-collection" );
+    PROF_TIMER garbage_collection( "garbage-collection" );
 #endif
     std::vector<CN_ITEM*> garbage;
     garbage.reserve( 1024 );
 
     m_itemList.RemoveInvalidItems( garbage );
 
-    for( auto item : garbage )
+    for( CN_ITEM* item : garbage )
         delete item;
 
 #ifdef PROFILE
     garbage_collection.Show();
-    PROF_COUNTER search_basic( "search-basic" );
+    PROF_TIMER search_basic( "search-basic" );
 #endif
 
+    thread_pool& tp = GetKiCadThreadPool();
     std::vector<CN_ITEM*> dirtyItems;
     std::copy_if( m_itemList.begin(), m_itemList.end(), std::back_inserter( dirtyItems ),
                   [] ( CN_ITEM* aItem )
@@ -238,54 +240,39 @@ void CN_CONNECTIVITY_ALGO::searchConnections()
 
     if( m_itemList.IsDirty() )
     {
-        size_t parallelThreadCount = std::min<size_t>( std::thread::hardware_concurrency(),
-                                                       ( dirtyItems.size() + 7 ) / 8 );
 
-        std::atomic<size_t> nextItem( 0 );
-        std::vector<std::future<size_t>> returns( parallelThreadCount );
+        std::vector<std::future<size_t>> returns( dirtyItems.size() );
 
         auto conn_lambda =
-                [&nextItem, &dirtyItems]( CN_LIST* aItemList,
-                                          PROGRESS_REPORTER* aReporter) -> size_t
+                [&dirtyItems]( size_t aItem, CN_LIST* aItemList,
+                               PROGRESS_REPORTER* aReporter) -> size_t
                 {
-                    for( size_t i = nextItem++; i < dirtyItems.size(); i = nextItem++ )
-                    {
-                        CN_VISITOR visitor( dirtyItems[i] );
-                        aItemList->FindNearby( dirtyItems[i], visitor );
+                    if( aReporter && aReporter->IsCancelled() )
+                        return 0;
 
-                        if( aReporter )
-                        {
-                            if( aReporter->IsCancelled() )
-                                break;
-                            else
-                                aReporter->AdvanceProgress();
-                        }
-                    }
+                    CN_VISITOR visitor( dirtyItems[aItem] );
+                    aItemList->FindNearby( dirtyItems[aItem], visitor );
+
+                    if( aReporter )
+                        aReporter->AdvanceProgress();
 
                     return 1;
                 };
 
-        if( parallelThreadCount <= 1 )
-            conn_lambda( &m_itemList, m_progressReporter );
-        else
+        for( size_t ii = 0; ii < dirtyItems.size(); ++ii )
+            returns[ii] = tp.submit( conn_lambda, ii, &m_itemList, m_progressReporter );
+
+        for( const std::future<size_t>& ret : returns )
         {
-            for( size_t ii = 0; ii < parallelThreadCount; ++ii )
-            {
-                returns[ii] = std::async( std::launch::async, conn_lambda, &m_itemList,
-                                          m_progressReporter );
-            }
+            // Here we balance returns with a 250ms timeout to allow UI updating
+            std::future_status status = ret.wait_for( std::chrono::milliseconds( 250 ) );
 
-            for( size_t ii = 0; ii < parallelThreadCount; ++ii )
+            while( status != std::future_status::ready )
             {
-                // Here we balance returns with a 100ms timeout to allow UI updating
-                std::future_status status;
-                do
-                {
-                    if( m_progressReporter )
-                        m_progressReporter->KeepRefreshing();
+                if( m_progressReporter )
+                    m_progressReporter->KeepRefreshing();
 
-                    status = returns[ii].wait_for( std::chrono::milliseconds( 100 ) );
-                } while( status != std::future_status::ready );
+                status = ret.wait_for( std::chrono::milliseconds( 250 ) );
             }
         }
 
@@ -303,21 +290,25 @@ void CN_CONNECTIVITY_ALGO::searchConnections()
 
 const CN_CONNECTIVITY_ALGO::CLUSTERS CN_CONNECTIVITY_ALGO::SearchClusters( CLUSTER_SEARCH_MODE aMode )
 {
-    constexpr KICAD_T types[] = { PCB_TRACE_T, PCB_ARC_T, PCB_PAD_T, PCB_VIA_T, PCB_ZONE_T,
-                                  PCB_FOOTPRINT_T, EOT };
-    constexpr KICAD_T no_zones[] = { PCB_TRACE_T, PCB_ARC_T, PCB_PAD_T, PCB_VIA_T,
-                                     PCB_FOOTPRINT_T, EOT };
-
     if( aMode == CSM_PROPAGATE )
-        return SearchClusters( aMode, no_zones, -1 );
+    {
+        return SearchClusters( aMode,
+                               { PCB_TRACE_T, PCB_ARC_T, PCB_PAD_T, PCB_VIA_T, PCB_FOOTPRINT_T },
+                               -1 );
+    }
     else
-        return SearchClusters( aMode, types, -1 );
+    {
+        return SearchClusters( aMode,
+                               { PCB_TRACE_T, PCB_ARC_T, PCB_PAD_T, PCB_VIA_T, PCB_ZONE_T, PCB_FOOTPRINT_T },
+                               -1 );
+    }
 }
 
 
-const CN_CONNECTIVITY_ALGO::CLUSTERS CN_CONNECTIVITY_ALGO::SearchClusters( CLUSTER_SEARCH_MODE aMode,
-                                                                           const KICAD_T aTypes[],
-                                                                           int aSingleNet )
+const CN_CONNECTIVITY_ALGO::CLUSTERS
+CN_CONNECTIVITY_ALGO::SearchClusters( CLUSTER_SEARCH_MODE aMode,
+                                      const std::initializer_list<KICAD_T>& aTypes,
+                                      int aSingleNet, CN_ITEM* rootItem )
 {
     bool withinAnyNet = ( aMode != CSM_PROPAGATE );
 
@@ -330,7 +321,7 @@ const CN_CONNECTIVITY_ALGO::CLUSTERS CN_CONNECTIVITY_ALGO::SearchClusters( CLUST
         searchConnections();
 
     auto addToSearchList =
-            [&item_set, withinAnyNet, aSingleNet, aTypes]( CN_ITEM *aItem )
+            [&item_set, withinAnyNet, aSingleNet, &aTypes, rootItem ]( CN_ITEM *aItem )
             {
                 if( withinAnyNet && aItem->Net() <= 0 )
                     return;
@@ -343,16 +334,16 @@ const CN_CONNECTIVITY_ALGO::CLUSTERS CN_CONNECTIVITY_ALGO::SearchClusters( CLUST
 
                 bool found = false;
 
-                for( int i = 0; aTypes[i] != EOT; i++ )
+                for( KICAD_T type : aTypes )
                 {
-                    if( aItem->Parent()->Type() == aTypes[i] )
+                    if( aItem->Parent()->Type() == type )
                     {
                         found = true;
                         break;
                     }
                 }
 
-                if( !found )
+                if( !found && aItem != rootItem )
                     return;
 
                 aItem->SetVisited( false );
@@ -367,9 +358,9 @@ const CN_CONNECTIVITY_ALGO::CLUSTERS CN_CONNECTIVITY_ALGO::SearchClusters( CLUST
 
     while( !item_set.empty() )
     {
-        CN_CLUSTER_PTR cluster = std::make_shared<CN_CLUSTER>();
-        CN_ITEM*       root;
-        auto           it = item_set.begin();
+        std::shared_ptr<CN_CLUSTER> cluster = std::make_shared<CN_CLUSTER>();
+        CN_ITEM*                    root;
+        auto                        it = item_set.begin();
 
         while( it != item_set.end() && (*it)->Visited() )
             it = item_set.erase( item_set.begin() );
@@ -390,7 +381,7 @@ const CN_CONNECTIVITY_ALGO::CLUSTERS CN_CONNECTIVITY_ALGO::SearchClusters( CLUST
             Q.pop_front();
             cluster->Add( current );
 
-            for( auto n : current->ConnectedItems() )
+            for( CN_ITEM* n : current->ConnectedItems() )
             {
                 if( withinAnyNet && n->Net() != root->Net() )
                     continue;
@@ -410,7 +401,7 @@ const CN_CONNECTIVITY_ALGO::CLUSTERS CN_CONNECTIVITY_ALGO::SearchClusters( CLUST
         return CLUSTERS();
 
     std::sort( clusters.begin(), clusters.end(),
-               []( CN_CLUSTER_PTR a, CN_CLUSTER_PTR b )
+               []( const std::shared_ptr<CN_CLUSTER>& a, const std::shared_ptr<CN_CLUSTER>& b )
                {
                    return a->OriginNet() < b->OriginNet();
                } );
@@ -419,42 +410,107 @@ const CN_CONNECTIVITY_ALGO::CLUSTERS CN_CONNECTIVITY_ALGO::SearchClusters( CLUST
 }
 
 
-void reportProgress( PROGRESS_REPORTER* aReporter, int aCount, int aSize, int aDelta )
-{
-    if( aReporter && ( ( aCount % aDelta ) == 0 || aCount == aSize -  1 ) )
-    {
-        aReporter->SetCurrentProgress( (double) aCount / (double) aSize );
-        aReporter->KeepRefreshing( false );
-    }
-}
-
-
 void CN_CONNECTIVITY_ALGO::Build( BOARD* aBoard, PROGRESS_REPORTER* aReporter )
 {
-    int delta = 200;  // Number of additions between 2 calls to the progress bar
-    int ii = 0;
-    int size = 0;
+    // Generate CN_ZONE_LAYERs for each island on each layer of each zone
+    //
+    std::vector<CN_ZONE_LAYER*> zitems;
 
-    size += aBoard->Zones().size();
+    for( ZONE* zone : aBoard->Zones() )
+    {
+        if( zone->IsOnCopperLayer() )
+        {
+            m_itemMap[zone] = ITEM_MAP_ENTRY();
+            markItemNetAsDirty( zone );
+
+            for( PCB_LAYER_ID layer : zone->GetLayerSet().Seq() )
+            {
+                if( IsCopperLayer( layer ) )
+                {
+                    for( int j = 0; j < zone->GetFilledPolysList( layer )->OutlineCount(); j++ )
+                        zitems.push_back( new CN_ZONE_LAYER( zone, layer, j ) );
+                }
+            }
+        }
+    }
+
+    // Setup progress metrics
+    //
+    int    progressDelta = 50;
+    double size = 0.0;
+
+    size += zitems.size();        // Once for building RTrees
+    size += zitems.size();        // Once for adding to connectivity
     size += aBoard->Tracks().size();
 
     for( FOOTPRINT* footprint : aBoard->Footprints() )
         size += footprint->Pads().size();
 
-    size *= 2;      // Our caller us gets the other half of the progress bar
+    size *= 1.5;                  // Our caller gets the other third of the progress bar
 
-    delta = std::max( delta, size / 10 );
+    progressDelta = std::max( progressDelta, (int) size / 4 );
 
-    for( ZONE* zone : aBoard->Zones() )
+    auto report =
+            [&]( int progress )
+            {
+                if( aReporter && ( progress % progressDelta ) == 0 )
+                {
+                    aReporter->SetCurrentProgress( progress / size );
+                    aReporter->KeepRefreshing( false );
+                }
+            };
+
+    // Generate RTrees for CN_ZONE_LAYER items (in parallel)
+    //
+    thread_pool& tp = GetKiCadThreadPool();
+    std::vector<std::future<size_t>> returns( zitems.size() );
+
+    auto cache_zones =
+            [aReporter]( CN_ZONE_LAYER* aZoneLayer ) -> size_t
+            {
+                if( aReporter && aReporter->IsCancelled() )
+                    return 0;
+
+                aZoneLayer->BuildRTree();
+
+                if( aReporter )
+                    aReporter->AdvanceProgress();
+
+                return 1;
+            };
+
+    for( size_t ii = 0; ii < zitems.size(); ++ii )
+        returns[ii] = tp.submit( cache_zones, zitems[ii] );
+
+    for( const std::future<size_t>& ret : returns )
     {
-        Add( zone );
-        reportProgress( aReporter, ii++, size, delta );
+        std::future_status status = ret.wait_for( std::chrono::milliseconds( 250 ) );
+
+        while( status != std::future_status::ready )
+        {
+            if( aReporter )
+                aReporter->KeepRefreshing();
+
+            status = ret.wait_for( std::chrono::milliseconds( 250 ) );
+        }
+
+    }
+
+    // Add CN_ZONE_LAYERS, tracks, and pads to connectivity
+    //
+    int ii = zitems.size();
+
+    for( CN_ZONE_LAYER* zitem : zitems )
+    {
+        m_itemList.Add( zitem );
+        m_itemMap[ zitem->Parent() ].Link( zitem );
+        report( ++ii );
     }
 
     for( PCB_TRACK* tv : aBoard->Tracks() )
     {
         Add( tv );
-        reportProgress( aReporter, ii++, size, delta );
+        report( ++ii );
     }
 
     for( FOOTPRINT* footprint : aBoard->Footprints() )
@@ -462,120 +518,105 @@ void CN_CONNECTIVITY_ALGO::Build( BOARD* aBoard, PROGRESS_REPORTER* aReporter )
         for( PAD* pad : footprint->Pads() )
         {
             Add( pad );
-            reportProgress( aReporter, ii++, size, delta );
+            report( ++ii );
         }
     }
+
+    if( aReporter )
+    {
+        aReporter->SetCurrentProgress( (double) ii / (double) size );
+        aReporter->KeepRefreshing( false );
+    }
 }
 
 
-void CN_CONNECTIVITY_ALGO::Build( const std::vector<BOARD_ITEM*>& aItems )
+void CN_CONNECTIVITY_ALGO::LocalBuild( const std::vector<BOARD_ITEM*>& aItems )
 {
-    for( auto item : aItems )
+    for( BOARD_ITEM* item : aItems )
     {
         switch( item->Type() )
         {
-            case PCB_TRACE_T:
-            case PCB_ARC_T:
-            case PCB_VIA_T:
-            case PCB_PAD_T:
-                Add( item );
-                break;
+        case PCB_TRACE_T:
+        case PCB_ARC_T:
+        case PCB_VIA_T:
+        case PCB_PAD_T:
+        case PCB_FOOTPRINT_T:
+            Add( item );
+            break;
 
-            case PCB_FOOTPRINT_T:
-                for( PAD* pad : static_cast<FOOTPRINT*>( item )->Pads() )
-                    Add( pad );
-
-                break;
-
-            default:
-                break;
+        default:
+            break;
         }
     }
 }
 
 
-void CN_CONNECTIVITY_ALGO::propagateConnections( BOARD_COMMIT* aCommit, PROPAGATE_MODE aMode )
+void CN_CONNECTIVITY_ALGO::propagateConnections( BOARD_COMMIT* aCommit )
 {
-    bool skipConflicts = ( aMode == PROPAGATE_MODE::SKIP_CONFLICTS );
-
-    wxLogTrace( wxT( "CN" ), wxT( "propagateConnections: propagate skip conflicts? %d" ),
-                skipConflicts );
-
-    for( const auto& cluster : m_connClusters )
+    for( const std::shared_ptr<CN_CLUSTER>& cluster : m_connClusters )
     {
-        if( skipConflicts && cluster->IsConflicting() )
+        if( cluster->IsConflicting() )
         {
-            wxLogTrace( wxT( "CN" ), wxT( "Conflicting nets in cluster %p; skipping update" ),
+            // Conflicting pads in cluster: we don't know the user's intent so best to do
+            // nothing.
+            wxLogTrace( wxT( "CN" ), wxT( "Conflicting pads in cluster %p; skipping propagation" ),
                         cluster.get() );
         }
-        else if( cluster->IsOrphaned() )
-        {
-            wxLogTrace( wxT( "CN" ), wxT( "Skipping orphaned cluster %p [net: %s]" ),
-                        cluster.get(),
-                        (const char*) cluster->OriginNetName().c_str() );
-        }
         else if( cluster->HasValidNet() )
         {
-            if( cluster->IsConflicting() )
-            {
-                wxLogTrace( wxT( "CN" ), wxT( "Conflicting nets in cluster %p; chose %d (%s)" ),
-                            cluster.get(),
-                            cluster->OriginNet(),
-                            cluster->OriginNetName() );
-            }
-
-            // normal cluster: just propagate from the pads
+            // Propagate from the origin (will be a pad if there are any, or another item if
+            // there are no pads).
             int n_changed = 0;
 
-            for( auto item : *cluster )
+            for( CN_ITEM* item : *cluster )
             {
-                if( item->CanChangeNet() )
+                if( item->Valid() && item->CanChangeNet()
+                        && item->Parent()->GetNetCode() != cluster->OriginNet() )
                 {
-                    if( item->Valid() && item->Parent()->GetNetCode() != cluster->OriginNet() )
-                    {
-                        MarkNetAsDirty( item->Parent()->GetNetCode() );
-                        MarkNetAsDirty( cluster->OriginNet() );
+                    MarkNetAsDirty( item->Parent()->GetNetCode() );
+                    MarkNetAsDirty( cluster->OriginNet() );
 
-                        if( aCommit )
-                            aCommit->Modify( item->Parent() );
+                    if( aCommit )
+                        aCommit->Modify( item->Parent() );
 
-                        item->Parent()->SetNetCode( cluster->OriginNet() );
-                        n_changed++;
-                    }
+                    item->Parent()->SetNetCode( cluster->OriginNet() );
+                    n_changed++;
                 }
             }
 
             if( n_changed )
             {
-                wxLogTrace( wxT( "CN" ), wxT( "Cluster %p : net : %d %s" ),
+                wxLogTrace( wxT( "CN" ), wxT( "Cluster %p: net: %d %s" ),
                             cluster.get(),
                             cluster->OriginNet(),
                             (const char*) cluster->OriginNetName().c_str() );
             }
             else
-                wxLogTrace( wxT( "CN" ), wxT( "Cluster %p : nothing to propagate" ),
+            {
+                wxLogTrace( wxT( "CN" ), wxT( "Cluster %p: no changeable items to propagate to" ),
                             cluster.get() );
+            }
         }
         else
         {
-            wxLogTrace( wxT( "CN" ), wxT( "Cluster %p : connected to unused net" ),
+            wxLogTrace( wxT( "CN" ), wxT( "Cluster %p: connected to unused net" ),
                         cluster.get() );
         }
     }
 }
 
 
-void CN_CONNECTIVITY_ALGO::PropagateNets( BOARD_COMMIT* aCommit, PROPAGATE_MODE aMode )
+void CN_CONNECTIVITY_ALGO::PropagateNets( BOARD_COMMIT* aCommit )
 {
     m_connClusters = SearchClusters( CSM_PROPAGATE );
-    propagateConnections( aCommit, aMode );
+    propagateConnections( aCommit );
 }
 
 
 void CN_CONNECTIVITY_ALGO::FindIsolatedCopperIslands( ZONE* aZone, PCB_LAYER_ID aLayer,
                                                       std::vector<int>& aIslands )
 {
-    if( aZone->GetFilledPolysList( aLayer ).IsEmpty() )
+    if( aZone->GetFilledPolysList( aLayer )->IsEmpty() )
         return;
 
     aIslands.clear();
@@ -585,16 +626,14 @@ void CN_CONNECTIVITY_ALGO::FindIsolatedCopperIslands( ZONE* aZone, PCB_LAYER_ID
 
     m_connClusters = SearchClusters( CSM_CONNECTIVITY_CHECK );
 
-    for( const auto& cluster : m_connClusters )
+    for( const std::shared_ptr<CN_CLUSTER>& cluster : m_connClusters )
     {
         if( cluster->Contains( aZone ) && cluster->IsOrphaned() )
         {
-            for( auto z : *cluster )
+            for( CN_ITEM* z : *cluster )
             {
                 if( z->Parent() == aZone && z->Layer() == aLayer )
-                {
                     aIslands.push_back( static_cast<CN_ZONE_LAYER*>(z)->SubpolyIndex() );
-                }
             }
         }
     }
@@ -602,12 +641,31 @@ void CN_CONNECTIVITY_ALGO::FindIsolatedCopperIslands( ZONE* aZone, PCB_LAYER_ID
     wxLogTrace( wxT( "CN" ), wxT( "Found %u isolated islands\n" ), (unsigned) aIslands.size() );
 }
 
-void CN_CONNECTIVITY_ALGO::FindIsolatedCopperIslands( std::vector<CN_ZONE_ISOLATED_ISLAND_LIST>& aZones )
+void CN_CONNECTIVITY_ALGO::FindIsolatedCopperIslands( std::vector<CN_ZONE_ISOLATED_ISLAND_LIST>& aZones,
+                                                      bool aConnectivityAlreadyRebuilt )
 {
-    for( auto& z : aZones )
+    int progressDelta = 50;
+    int ii = 0;
+
+    progressDelta = std::max( progressDelta, (int) aZones.size() / 4 );
+
+    if( !aConnectivityAlreadyRebuilt )
     {
-        Remove( z.m_zone );
-        Add( z.m_zone );
+        for( CN_ZONE_ISOLATED_ISLAND_LIST& z : aZones )
+        {
+            Remove( z.m_zone );
+            Add( z.m_zone );
+            ii++;
+
+            if( m_progressReporter && ( ii % progressDelta ) == 0 )
+            {
+                m_progressReporter->SetCurrentProgress( (double) ii / (double) aZones.size() );
+                m_progressReporter->KeepRefreshing( false );
+            }
+
+            if( m_progressReporter && m_progressReporter->IsCancelled() )
+                return;
+        }
     }
 
     m_connClusters = SearchClusters( CSM_CONNECTIVITY_CHECK );
@@ -616,10 +674,10 @@ void CN_CONNECTIVITY_ALGO::FindIsolatedCopperIslands( std::vector<CN_ZONE_ISOLAT
     {
         for( PCB_LAYER_ID layer : zone.m_zone->GetLayerSet().Seq() )
         {
-            if( zone.m_zone->GetFilledPolysList( layer ).IsEmpty() )
+            if( zone.m_zone->GetFilledPolysList( layer )->IsEmpty() )
                 continue;
 
-            for( const CN_CLUSTER_PTR& cluster : m_connClusters )
+            for( const std::shared_ptr<CN_CLUSTER>& cluster : m_connClusters )
             {
                 if( cluster->Contains( zone.m_zone ) && cluster->IsOrphaned() )
                 {
@@ -669,30 +727,62 @@ void CN_CONNECTIVITY_ALGO::MarkNetAsDirty( int aNet )
 
 void CN_VISITOR::checkZoneItemConnection( CN_ZONE_LAYER* aZoneLayer, CN_ITEM* aItem )
 {
-    if( aZoneLayer->Net() != aItem->Net() && !aItem->CanChangeNet() )
+    PCB_LAYER_ID          layer = aZoneLayer->GetLayer();
+    BOARD_CONNECTED_ITEM* item = aItem->Parent();
+
+    if( !item->IsOnLayer( layer ) )
         return;
 
-    if( !aZoneLayer->BBox().Intersects( aItem->BBox() ) )
-        return;
+    auto connect =
+            [&]()
+            {
+                aZoneLayer->Connect( aItem );
+                aItem->Connect( aZoneLayer );
+            };
 
-    int accuracy = 0;
-
-    if( aItem->Parent()->Type() == PCB_VIA_T
-            || aItem->Parent()->Type() == PCB_TRACE_T
-            || aItem->Parent()->Type() == PCB_ARC_T )
+    // Try quick checks first...
+    if( item->Type() == PCB_PAD_T )
     {
-        accuracy = ( static_cast<PCB_TRACK*>( aItem->Parent() )->GetWidth() + 1 ) / 2;
+        PAD* pad = static_cast<PAD*>( item );
+
+        if( pad->ConditionallyFlashed( layer )
+            && pad->GetZoneLayerOverride( layer ) == ZLO_FORCE_NO_ZONE_CONNECTION )
+        {
+            return;
+        }
+    }
+    else if( item->Type() == PCB_VIA_T )
+    {
+        PCB_VIA* via = static_cast<PCB_VIA*>( item );
+
+        if( via->ConditionallyFlashed( layer )
+            && via->GetZoneLayerOverride( layer ) == ZLO_FORCE_NO_ZONE_CONNECTION )
+        {
+            return;
+        }
     }
 
     for( int i = 0; i < aItem->AnchorCount(); ++i )
     {
-        if( aZoneLayer->ContainsPoint( aItem->GetAnchor( i ), accuracy ) )
+        if( aZoneLayer->ContainsPoint( aItem->GetAnchor( i ) ) )
         {
-            aZoneLayer->Connect( aItem );
-            aItem->Connect( aZoneLayer );
+            connect();
             return;
         }
     }
+
+    if( item->Type() == PCB_VIA_T || item->Type() == PCB_PAD_T )
+    {
+        // As long as the pad/via crosses the zone layer, check for the full effective shape
+        // We check for the overlapping layers above
+        if( aZoneLayer->Collide( item->GetEffectiveShape( layer, FLASHING::ALWAYS_FLASHED ).get() ) )
+            connect();
+
+        return;
+    }
+
+    if( aZoneLayer->Collide( item->GetEffectiveShape( layer ).get() ) )
+        connect();
 }
 
 void CN_VISITOR::checkZoneZoneConnection( CN_ZONE_LAYER* aZoneLayerA, CN_ZONE_LAYER* aZoneLayerB )
@@ -700,35 +790,26 @@ void CN_VISITOR::checkZoneZoneConnection( CN_ZONE_LAYER* aZoneLayerA, CN_ZONE_LA
     const ZONE* zoneA = static_cast<const ZONE*>( aZoneLayerA->Parent() );
     const ZONE* zoneB = static_cast<const ZONE*>( aZoneLayerB->Parent() );
 
-    if( aZoneLayerA->Layer() != aZoneLayerB->Layer() )
-        return;
-
-    if( aZoneLayerB->Net() != aZoneLayerA->Net() )
-        return; // we only test zones belonging to the same net
-
     const BOX2I& boxA = aZoneLayerA->BBox();
     const BOX2I& boxB = aZoneLayerB->BBox();
 
-    int radiusA = 0;
-    int radiusB = 0;
+    PCB_LAYER_ID layer = aZoneLayerA->GetLayer();
 
-    if( zoneA->GetFilledPolysUseThickness() )
-        radiusA = ( zoneA->GetMinThickness() + 1 ) / 2;
+    if( aZoneLayerB->GetLayer() != layer )
+        return;
 
-    if( zoneB->GetFilledPolysUseThickness() )
-        radiusB = ( zoneB->GetMinThickness() + 1 ) / 2;
-
-    PCB_LAYER_ID layer = static_cast<PCB_LAYER_ID>( aZoneLayerA->Layer() );
+    if( !boxA.Intersects( boxB ) )
+        return;
 
     const SHAPE_LINE_CHAIN& outline =
-            zoneA->GetFilledPolysList( layer ).COutline( aZoneLayerA->SubpolyIndex() );
+            zoneA->GetFilledPolysList( layer )->COutline( aZoneLayerA->SubpolyIndex() );
 
     for( int i = 0; i < outline.PointCount(); i++ )
     {
         if( !boxB.Contains( outline.CPoint( i ) ) )
             continue;
 
-        if( aZoneLayerB->ContainsPoint( outline.CPoint( i ), radiusA ) )
+        if( aZoneLayerB->ContainsPoint( outline.CPoint( i ) ) )
         {
             aZoneLayerA->Connect( aZoneLayerB );
             aZoneLayerB->Connect( aZoneLayerA );
@@ -737,14 +818,14 @@ void CN_VISITOR::checkZoneZoneConnection( CN_ZONE_LAYER* aZoneLayerA, CN_ZONE_LA
     }
 
     const SHAPE_LINE_CHAIN& outline2 =
-            zoneB->GetFilledPolysList( layer ).COutline( aZoneLayerB->SubpolyIndex() );
+            zoneB->GetFilledPolysList( layer )->COutline( aZoneLayerB->SubpolyIndex() );
 
     for( int i = 0; i < outline2.PointCount(); i++ )
     {
         if( !boxA.Contains( outline2.CPoint( i ) ) )
             continue;
 
-        if( aZoneLayerA->ContainsPoint( outline2.CPoint( i ), radiusB ) )
+        if( aZoneLayerA->ContainsPoint( outline2.CPoint( i ) ) )
         {
             aZoneLayerA->Connect( aZoneLayerB );
             aZoneLayerB->Connect( aZoneLayerA );
@@ -765,9 +846,8 @@ bool CN_VISITOR::operator()( CN_ITEM* aCandidate )
     if( parentA == parentB )
         return true;
 
-    LSET commonLayers = parentA->GetLayerSet() & parentB->GetLayerSet();
-
-    if( !commonLayers.any() )
+    // Don't connect items in different nets that can't be changed
+    if( !aCandidate->CanChangeNet() && !m_item->CanChangeNet() && aCandidate->Net() != m_item->Net() )
         return true;
 
     // If both m_item and aCandidate are marked dirty, they will both be searched
@@ -796,9 +876,37 @@ bool CN_VISITOR::operator()( CN_ITEM* aCandidate )
         return true;
     }
 
+    LSET commonLayers = parentA->GetLayerSet() & parentB->GetLayerSet();
+
     for( PCB_LAYER_ID layer : commonLayers.Seq() )
     {
-        if( parentA->GetEffectiveShape( layer )->Collide( parentB->GetEffectiveShape( layer ).get() ) )
+        FLASHING flashingA = FLASHING::NEVER_FLASHED;
+        FLASHING flashingB = FLASHING::NEVER_FLASHED;
+
+        if( const PAD* pad = dyn_cast<const PAD*>( parentA ) )
+        {
+            if( !pad->ConditionallyFlashed( layer ) )
+                flashingA = FLASHING::ALWAYS_FLASHED;
+        }
+        else if( const PCB_VIA* via = dyn_cast<const PCB_VIA*>( parentA ) )
+        {
+            if( !via->ConditionallyFlashed( layer ) )
+                flashingA = FLASHING::ALWAYS_FLASHED;
+        }
+
+        if( const PAD* pad = dyn_cast<const PAD*>( parentB ) )
+        {
+            if( !pad->ConditionallyFlashed( layer ) )
+                flashingB = FLASHING::ALWAYS_FLASHED;
+        }
+        else if( const PCB_VIA* via = dyn_cast<const PCB_VIA*>( parentB ) )
+        {
+            if( !via->ConditionallyFlashed( layer ) )
+                flashingB = FLASHING::ALWAYS_FLASHED;
+        }
+
+        if( parentA->GetEffectiveShape( layer, flashingA )->Collide(
+                parentB->GetEffectiveShape( layer, flashingB ).get() ) )
         {
             m_item->Connect( aCandidate );
             aCandidate->Connect( m_item );
