@@ -4,7 +4,7 @@
  * Copyright (C) 2018 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2015 Dick Hollenbeck, dick@softplc.com
  * Copyright (C) 2008 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 2004-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2004-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,8 +31,10 @@
 #include <board_design_settings.h>
 #include <board_commit.h>
 #include <bitmaps.h>
+#include <kiplatform/ui.h>
 #include <widgets/grid_text_button_helpers.h>
 #include <widgets/wx_grid.h>
+#include <widgets/std_bitmap_button.h>
 #include <widgets/text_ctrl_eval.h>
 #include <footprint.h>
 #include <footprint_edit_frame.h>
@@ -42,14 +44,78 @@
 #include "3d_rendering/opengl/3d_model.h"
 #include "filename_resolver.h"
 #include <pgm_base.h>
+#include <pcbnew.h>
 #include "dialogs/panel_preview_3d_model.h"
 #include "dialogs/3d_cache_dialogs.h"
 #include <settings/settings_manager.h>
 #include <tool/tool_manager.h>
 #include <tools/pcb_selection_tool.h>
+#include <grid_layer_box_helpers.h>
 
 #include <fp_lib_table.h>
 
+PRIVATE_LAYERS_GRID_TABLE::PRIVATE_LAYERS_GRID_TABLE( PCB_BASE_FRAME* aFrame ) :
+        m_frame( aFrame )
+{
+    m_layerColAttr = new wxGridCellAttr;
+    m_layerColAttr->SetRenderer( new GRID_CELL_LAYER_RENDERER( m_frame ) );
+
+    LSET forbiddenLayers = LSET::AllCuMask() | LSET::AllTechMask();
+    forbiddenLayers.set( Edge_Cuts );
+    forbiddenLayers.set( Margin );
+    m_layerColAttr->SetEditor( new GRID_CELL_LAYER_SELECTOR( m_frame, forbiddenLayers ) );
+}
+
+
+PRIVATE_LAYERS_GRID_TABLE::~PRIVATE_LAYERS_GRID_TABLE()
+{
+    m_layerColAttr->DecRef();
+}
+
+
+bool PRIVATE_LAYERS_GRID_TABLE::CanGetValueAs( int aRow, int aCol, const wxString& aTypeName )
+{
+    return aTypeName == wxGRID_VALUE_NUMBER;
+}
+
+
+bool PRIVATE_LAYERS_GRID_TABLE::CanSetValueAs( int aRow, int aCol, const wxString& aTypeName )
+{
+    return aTypeName == wxGRID_VALUE_NUMBER;
+}
+
+
+wxGridCellAttr* PRIVATE_LAYERS_GRID_TABLE::GetAttr( int aRow, int aCol,
+                                                    wxGridCellAttr::wxAttrKind  )
+{
+    m_layerColAttr->IncRef();
+    return m_layerColAttr;
+}
+
+
+wxString PRIVATE_LAYERS_GRID_TABLE::GetValue( int aRow, int aCol )
+{
+    return m_frame->GetBoard()->GetLayerName( this->at( (size_t) aRow ) );
+}
+
+
+long PRIVATE_LAYERS_GRID_TABLE::GetValueAsLong( int aRow, int aCol )
+{
+    return this->at( (size_t) aRow );
+}
+
+
+void PRIVATE_LAYERS_GRID_TABLE::SetValue( int aRow, int aCol, const wxString &aValue )
+{
+    wxFAIL_MSG( wxString::Format( wxT( "column %d doesn't hold a string value" ), aCol ) );
+}
+
+
+void PRIVATE_LAYERS_GRID_TABLE::SetValueAsLong( int aRow, int aCol, long aValue )
+{
+    this->at( (size_t) aRow ) = ToLAYER_ID( (int) aValue );
+}
+
 
 // Remember the last open page during session.
 
@@ -68,7 +134,9 @@ DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR(
     m_solderPaste( aParent, m_SolderPasteMarginLabel, m_SolderPasteMarginCtrl,
                    m_SolderPasteMarginUnits ),
     m_solderPasteRatio( aParent, m_PasteMarginRatioLabel, m_PasteMarginRatioCtrl,
-                        m_PasteMarginRatioUnits )
+                        m_PasteMarginRatioUnits ),
+    m_gridSize( 0, 0 ),
+    m_lastRequestedSize( 0, 0 )
 {
     SetEvtHandlerEnabled( false );
     // Create the 3D models page
@@ -76,6 +144,7 @@ DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR(
     m_NoteBook->AddPage( m_3dPanel, _("3D Models"), false );
 
     m_texts = new FP_TEXT_GRID_TABLE( m_frame );
+    m_privateLayers = new PRIVATE_LAYERS_GRID_TABLE( m_frame );
 
     m_delayedErrorMessage = wxEmptyString;
     m_delayedFocusCtrl = nullptr;
@@ -91,10 +160,26 @@ DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR(
 
     // Give a bit more room for combobox editors
     m_itemsGrid->SetDefaultRowSize( m_itemsGrid->GetDefaultRowSize() + 4 );
+    m_privateLayersGrid->SetDefaultRowSize( m_privateLayersGrid->GetDefaultRowSize() + 4 );
 
     m_itemsGrid->SetTable( m_texts );
+    m_privateLayersGrid->SetTable( m_privateLayers );
+
     m_itemsGrid->PushEventHandler( new GRID_TRICKS( m_itemsGrid ) );
+    m_privateLayersGrid->PushEventHandler( new GRID_TRICKS( m_privateLayersGrid,
+                                                            [this]( wxCommandEvent& aEvent )
+                                                            {
+                                                                OnAddLayer( aEvent );
+                                                            } ) );
+    m_padGroupsGrid->PushEventHandler( new GRID_TRICKS( m_padGroupsGrid,
+                                                        [this]( wxCommandEvent& aEvent )
+                                                        {
+                                                            OnAddPadGroup( aEvent );
+                                                        } ) );
+
     m_itemsGrid->SetSelectionMode( wxGrid::wxGridSelectRows );
+    m_privateLayersGrid->SetSelectionMode( wxGrid::wxGridSelectRows );
+    m_padGroupsGrid->SetSelectionMode( wxGrid::wxGridSelectRows );
 
     // Show/hide columns according to the user's preference
     m_itemsGrid->ShowHideColumns( m_frame->GetSettings()->m_FootprintTextShownColumns );
@@ -103,14 +188,7 @@ DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR(
 
     // Set font sizes
     wxFont infoFont = KIUI::GetInfoFont( this );
-#if __WXMAC__
-    m_allow90Label->SetFont( infoFont );
-    m_allow180Label->SetFont( infoFont );
-#endif
-
     infoFont.SetStyle( wxFONTSTYLE_ITALIC );
-    m_staticTextInfoValNeg->SetFont( infoFont );
-    m_staticTextInfoValPos->SetFont( infoFont );
     m_staticTextInfoCopper->SetFont( infoFont );
     m_staticTextInfoPaste->SetFont( infoFont );
 
@@ -134,16 +212,15 @@ DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR(
     m_solderPasteRatio.SetUnits( EDA_UNITS::PERCENT );
     m_solderPasteRatio.SetNegativeZero();
 
-    m_sdbSizerStdButtonsOK->SetDefault();
-
     // Configure button logos
     m_bpAdd->SetBitmap( KiBitmap( BITMAPS::small_plus ) );
     m_bpDelete->SetBitmap( KiBitmap( BITMAPS::small_trash ) );
+    m_bpAddLayer->SetBitmap( KiBitmap( BITMAPS::small_plus ) );
+    m_bpDeleteLayer->SetBitmap( KiBitmap( BITMAPS::small_trash ) );
+    m_bpAddPadGroup->SetBitmap( KiBitmap( BITMAPS::small_plus ) );
+    m_bpRemovePadGroup->SetBitmap( KiBitmap( BITMAPS::small_trash ) );
 
-    // wxFormBuilder doesn't include this event...
-    m_itemsGrid->Connect( wxEVT_GRID_CELL_CHANGING,
-                          wxGridEventHandler( DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnGridCellChanging ),
-                          nullptr, this );
+    SetupStandardButtons();
 
     finishDialogSettings();
     SetEvtHandlerEnabled( true );
@@ -157,13 +234,12 @@ DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::~DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR()
 
     // Prevents crash bug in wxGrid's d'tor
     m_itemsGrid->DestroyTable( m_texts );
-
-    m_itemsGrid->Disconnect( wxEVT_GRID_CELL_CHANGING,
-                             wxGridEventHandler( DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnGridCellChanging ),
-                             nullptr, this );
+    m_privateLayersGrid->DestroyTable( m_privateLayers );
 
     // Delete the GRID_TRICKS.
     m_itemsGrid->PopEventHandler( true );
+    m_privateLayersGrid->PopEventHandler( true );
+    m_padGroupsGrid->PopEventHandler( true );
 
     m_page = static_cast<NOTEBOOK_PAGES>( m_NoteBook->GetSelection() );
 
@@ -196,9 +272,9 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::TransferDataToWindow()
     m_texts->push_back( m_footprint->Reference() );
     m_texts->push_back( m_footprint->Value() );
 
-    for( auto item : m_footprint->GraphicalItems() )
+    for( BOARD_ITEM* item : m_footprint->GraphicalItems() )
     {
-        auto textItem = dyn_cast<FP_TEXT*>( item );
+        FP_TEXT* textItem = dyn_cast<FP_TEXT*>( item );
 
         if( textItem )
             m_texts->push_back( *textItem );
@@ -208,10 +284,6 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::TransferDataToWindow()
     wxGridTableMessage tmsg( m_texts, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, m_texts->GetNumberRows() );
     m_itemsGrid->ProcessTableMessage( tmsg );
 
-    // Footprint Properties
-    m_CostRot90Ctrl->SetValue( m_footprint->GetPlacementCost90() );
-    m_CostRot180Ctrl->SetValue( m_footprint->GetPlacementCost180() );
-
     if( m_footprint->GetAttributes() & FP_THROUGH_HOLE )
         m_componentType->SetSelection( 0 );
     else if( m_footprint->GetAttributes() & FP_SMD )
@@ -219,9 +291,19 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::TransferDataToWindow()
     else
         m_componentType->SetSelection( 2 );
 
+    // Private layers
+    for( PCB_LAYER_ID privateLayer : m_footprint->GetPrivateLayers().UIOrder() )
+        m_privateLayers->push_back( privateLayer );
+
+    // Notify the grid
+    wxGridTableMessage gridTableMessagesg( m_privateLayers, wxGRIDTABLE_NOTIFY_ROWS_APPENDED,
+                                           m_privateLayers->GetNumberRows() );
+    m_privateLayersGrid->ProcessTableMessage( gridTableMessagesg );
+
     m_boardOnly->SetValue( m_footprint->GetAttributes() & FP_BOARD_ONLY );
     m_excludeFromPosFiles->SetValue( m_footprint->GetAttributes() & FP_EXCLUDE_FROM_POS_FILES );
     m_excludeFromBOM->SetValue( m_footprint->GetAttributes() & FP_EXCLUDE_FROM_BOM );
+    m_noCourtyards->SetValue( m_footprint->GetAttributes() & FP_ALLOW_MISSING_COURTYARD );
 
     // Local Clearances
 
@@ -229,6 +311,7 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::TransferDataToWindow()
     m_solderMask.SetValue( m_footprint->GetLocalSolderMaskMargin() );
     m_solderPaste.SetValue( m_footprint->GetLocalSolderPasteMargin() );
     m_solderPasteRatio.SetDoubleValue( m_footprint->GetLocalSolderPasteMarginRatio() * 100.0 );
+    m_allowBridges->SetValue( m_footprint->GetAttributes() & FP_ALLOW_SOLDERMASK_BRIDGES );
 
     switch( m_footprint->GetZoneConnection() )
     {
@@ -239,14 +322,22 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::TransferDataToWindow()
     case ZONE_CONNECTION::NONE:      m_ZoneConnectionChoice->SetSelection( 3 ); break;
     }
 
+    for( const wxString& group : m_footprint->GetNetTiePadGroups() )
+    {
+        if( !group.IsEmpty() )
+        {
+            m_padGroupsGrid->AppendRows( 1 );
+            m_padGroupsGrid->SetCellValue( m_padGroupsGrid->GetNumberRows() - 1, 0, group );
+        }
+    }
+
     // Items grid
     for( int col = 0; col < m_itemsGrid->GetNumberCols(); col++ )
     {
         // Adjust min size to the column label size
-        m_itemsGrid->SetColMinimalWidth( col, m_itemsGrid->GetVisibleWidth( col, true, false,
-                                                                            false ) );
+        m_itemsGrid->SetColMinimalWidth( col, m_itemsGrid->GetVisibleWidth( col, true, false ) );
         // Adjust the column size.
-        int col_size = m_itemsGrid->GetVisibleWidth( col, true, true, false );
+        int col_size = m_itemsGrid->GetVisibleWidth( col );
 
         if( col == FPT_LAYER )  // This one's a drop-down.  Check all possible values.
         {
@@ -266,7 +357,7 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::TransferDataToWindow()
     m_itemsGrid->SetRowLabelSize( m_itemsGrid->GetVisibleWidth( -1, true, true, true ) );
 
     Layout();
-    adjustGridColumns( m_itemsGrid->GetRect().GetWidth() );
+    adjustGridColumns();
 
     return true;
 }
@@ -329,6 +420,47 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::Validate()
 
             return false;
         }
+
+        if( text.GetTextWidth() < TEXTS_MIN_SIZE || text.GetTextWidth() > TEXTS_MAX_SIZE )
+        {
+            m_delayedFocusGrid = m_itemsGrid;
+            m_delayedErrorMessage = wxString::Format( _( "The text width must be between %s and %s." ),
+                                                      m_frame->StringFromValue( TEXTS_MIN_SIZE, true ),
+                                                      m_frame->StringFromValue( TEXTS_MAX_SIZE, true ) );
+            m_delayedFocusColumn = FPT_WIDTH;
+            m_delayedFocusRow = i;
+
+            return false;
+        }
+
+        if( text.GetTextHeight() < TEXTS_MIN_SIZE || text.GetTextHeight() > TEXTS_MAX_SIZE )
+        {
+            m_delayedFocusGrid = m_itemsGrid;
+            m_delayedErrorMessage = wxString::Format( _( "The text height must be between %s and %s." ),
+                                                      m_frame->StringFromValue( TEXTS_MIN_SIZE, true ),
+                                                      m_frame->StringFromValue( TEXTS_MAX_SIZE, true ) );
+            m_delayedFocusColumn = FPT_HEIGHT;
+            m_delayedFocusRow = i;
+
+            return false;
+        }
+
+        // Test for acceptable values for thickness and size and clamp if fails
+        int maxPenWidth = Clamp_Text_PenSize( text.GetTextThickness(), text.GetTextSize() );
+
+        if( text.GetTextThickness() > maxPenWidth )
+        {
+            m_itemsGrid->SetCellValue( i, FPT_THICKNESS,
+                                       m_frame->StringFromValue( maxPenWidth, true ) );
+
+            m_delayedFocusGrid = m_itemsGrid;
+            m_delayedErrorMessage = _( "The text thickness is too large for the text size.\n"
+                                       "It will be clamped." );
+            m_delayedFocusColumn = FPT_THICKNESS;
+            m_delayedFocusRow = i;
+
+            return false;
+        }
     }
 
     if( !m_netClearance.Validate( 0, INT_MAX ) )
@@ -346,16 +478,20 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::TransferDataFromWindow()
     if( !DIALOG_SHIM::TransferDataFromWindow() )
         return false;
 
-    if( !m_itemsGrid->CommitPendingChanges() )
+    if( !m_itemsGrid->CommitPendingChanges()
+            || !m_privateLayersGrid->CommitPendingChanges()
+            || !m_padGroupsGrid->CommitPendingChanges() )
+    {
         return false;
+    }
 
     // This only commits the editor, model updating is done below so it is inside
     // the commit
     if( !m_3dPanel->TransferDataFromWindow() )
         return false;
 
-    auto view = m_frame->GetCanvas()->GetView();
-    BOARD_COMMIT commit( m_frame );
+    KIGFX::PCB_VIEW* view = m_frame->GetCanvas()->GetView();
+    BOARD_COMMIT     commit( m_frame );
     commit.Modify( m_footprint );
 
     LIB_ID fpID = m_footprint->GetFPID();
@@ -405,6 +541,13 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::TransferDataFromWindow()
         view->Add( newText );
     }
 
+    LSET privateLayers;
+
+    for( PCB_LAYER_ID layer : *m_privateLayers )
+        privateLayers.set( layer );
+
+    m_footprint->SetPrivateLayers( privateLayers );
+
     int attributes = 0;
 
     switch( m_componentType->GetSelection() )
@@ -423,10 +566,13 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::TransferDataFromWindow()
     if( m_excludeFromBOM->GetValue() )
         attributes |= FP_EXCLUDE_FROM_BOM;
 
-    m_footprint->SetAttributes( attributes );
+    if( m_noCourtyards->GetValue() )
+        attributes |= FP_ALLOW_MISSING_COURTYARD;
 
-    m_footprint->SetPlacementCost90( m_CostRot90Ctrl->GetValue() );
-    m_footprint->SetPlacementCost180( m_CostRot180Ctrl->GetValue() );
+    if( m_allowBridges->GetValue() )
+        attributes |= FP_ALLOW_SOLDERMASK_BRIDGES;
+
+    m_footprint->SetAttributes( attributes );
 
     // Initialize masks clearances
     m_footprint->SetLocalClearance( m_netClearance.GetValue() );
@@ -443,9 +589,19 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::TransferDataFromWindow()
     case 3:  m_footprint->SetZoneConnection( ZONE_CONNECTION::NONE );      break;
     }
 
+    m_footprint->ClearNetTiePadGroups();
+
+    for( int ii = 0; ii < m_padGroupsGrid->GetNumberRows(); ++ii )
+    {
+        wxString group = m_padGroupsGrid->GetCellValue( ii, 0 );
+
+        if( !group.IsEmpty() )
+            m_footprint->AddNetTiePadGroup( group );
+    }
+
     // Copy the models from the panel to the footprint
     std::vector<FP_3DMODEL>& panelList = m_3dPanel->GetModelList();
-    std::list<FP_3DMODEL>*   fpList    = &m_footprint->Models();
+    std::vector<FP_3DMODEL>* fpList    = &m_footprint->Models();
     fpList->clear();
     fpList->insert( fpList->end(), panelList.begin(), panelList.end() );
 
@@ -461,12 +617,6 @@ static bool footprintIsFromBoard( FOOTPRINT* aFootprint )
 }
 
 
-void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnGridCellChanging( wxGridEvent& event )
-{
-    // Currently: nothing to do
-}
-
-
 void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnFootprintNameText( wxCommandEvent& event )
 {
     if( !footprintIsFromBoard( m_footprint ) )
@@ -551,32 +701,122 @@ void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnDeleteField( wxCommandEvent& event
 }
 
 
-void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::adjustGridColumns( int aWidth )
+void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnAddLayer( wxCommandEvent& event )
+{
+    if( !m_privateLayersGrid->CommitPendingChanges() )
+        return;
+
+    PCB_LAYER_ID nextLayer = User_1;
+
+    while( alg::contains( *m_privateLayers, nextLayer ) && nextLayer < User_9 )
+        nextLayer = ToLAYER_ID( nextLayer + 1 );
+
+    m_privateLayers->push_back( nextLayer );
+
+    // notify the grid
+    wxGridTableMessage msg( m_privateLayers, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, 1 );
+    m_privateLayersGrid->ProcessTableMessage( msg );
+
+    m_privateLayersGrid->SetFocus();
+    m_privateLayersGrid->MakeCellVisible( m_privateLayers->size() - 1, 0 );
+    m_privateLayersGrid->SetGridCursor( m_privateLayers->size() - 1, 0 );
+}
+
+
+void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnDeleteLayer( wxCommandEvent& event )
+{
+    if( !m_privateLayersGrid->CommitPendingChanges() )
+        return;
+
+    int curRow = m_privateLayersGrid->GetGridCursorRow();
+
+    if( curRow < 0 )
+        return;
+
+    m_privateLayers->erase( m_privateLayers->begin() + curRow );
+
+    // notify the grid
+    wxGridTableMessage msg( m_privateLayers, wxGRIDTABLE_NOTIFY_ROWS_DELETED, curRow, 1 );
+    m_privateLayersGrid->ProcessTableMessage( msg );
+
+    if( m_privateLayersGrid->GetNumberRows() > 0 )
+    {
+        m_privateLayersGrid->MakeCellVisible( std::max( 0, curRow-1 ),
+                                               m_privateLayersGrid->GetGridCursorCol() );
+        m_privateLayersGrid->SetGridCursor( std::max( 0, curRow-1 ),
+                                             m_privateLayersGrid->GetGridCursorCol() );
+    }
+}
+
+
+void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnAddPadGroup( wxCommandEvent& event )
+{
+    if( !m_padGroupsGrid->CommitPendingChanges() )
+        return;
+
+    m_padGroupsGrid->AppendRows( 1 );
+
+    m_padGroupsGrid->SetFocus();
+    m_padGroupsGrid->MakeCellVisible( m_padGroupsGrid->GetNumberRows() - 1, 0 );
+    m_padGroupsGrid->SetGridCursor( m_padGroupsGrid->GetNumberRows() - 1, 0 );
+
+    m_padGroupsGrid->EnableCellEditControl( true );
+    m_padGroupsGrid->ShowCellEditControl();
+}
+
+
+void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnRemovePadGroup( wxCommandEvent& event )
+{
+    if( !m_padGroupsGrid->CommitPendingChanges() )
+        return;
+
+    wxArrayInt selectedRows = m_padGroupsGrid->GetSelectedRows();
+    int        curRow = m_padGroupsGrid->GetGridCursorRow();
+
+    if( selectedRows.empty() && curRow >= 0 && curRow < m_padGroupsGrid->GetNumberRows() )
+        selectedRows.Add( curRow );
+
+    for( int ii = selectedRows.Count() - 1; ii >= 0; --ii )
+    {
+        int row = selectedRows.Item( ii );
+        m_padGroupsGrid->DeleteRows( row, 1 );
+        curRow = std::min( curRow, row );
+    }
+
+    curRow = std::max( 0, curRow - 1 );
+    m_padGroupsGrid->MakeCellVisible( curRow, m_padGroupsGrid->GetGridCursorCol() );
+    m_padGroupsGrid->SetGridCursor( curRow, m_padGroupsGrid->GetGridCursorCol() );
+}
+
+
+void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::adjustGridColumns()
 {
     // Account for scroll bars
-    int itemsWidth = aWidth - ( m_itemsGrid->GetSize().x - m_itemsGrid->GetClientSize().x );
+    int itemsWidth = KIPLATFORM::UI::GetUnobscuredSize( m_itemsGrid ).x;
 
     itemsWidth -= m_itemsGrid->GetRowLabelSize();
 
     for( int i = 1; i < m_itemsGrid->GetNumberCols(); i++ )
         itemsWidth -= m_itemsGrid->GetColSize( i );
 
-    if( itemsWidth > 0 )
-    {
-        m_itemsGrid->SetColSize( 0, std::max( itemsWidth,
-                                 m_itemsGrid->GetVisibleWidth( 0, true, false, false ) ) );
-    }
+    m_itemsGrid->SetColSize( 0, std::max( itemsWidth,
+                                          m_itemsGrid->GetVisibleWidth( 0, true, false ) ) );
+
+    // Update the width private layers grid
+    m_privateLayersGrid->SetColSize( 0, std::max( m_privateLayersGrid->GetClientSize().x,
+                                                  m_privateLayersGrid->GetVisibleWidth( 0 ) ) );
+
+    // Update the width net tie pad groups grid
+    m_padGroupsGrid->SetColSize( 0, std::max( m_padGroupsGrid->GetClientSize().x,
+                                              m_padGroupsGrid->GetVisibleWidth( 0 ) ) );
 
     // Update the width of the 3D panel
-    m_3dPanel->AdjustGridColumnWidths( aWidth );
+    m_3dPanel->AdjustGridColumnWidths();
 }
 
 
 void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnUpdateUI( wxUpdateUIEvent& event )
 {
-    if( !m_itemsGrid->IsCellEditControlShown() )
-        adjustGridColumns( m_itemsGrid->GetRect().GetWidth() );
-
     // Handle a delayed focus.  The delay allows us to:
     // a) change focus when the error was triggered from within a killFocus handler
     // b) show the correct notebook page in the background before the error dialog comes up
@@ -605,7 +845,7 @@ void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnUpdateUI( wxUpdateUIEvent& event )
     {
         m_delayedFocusCtrl->SetFocus();
 
-        if( auto textEntry = dynamic_cast<wxTextEntry*>( m_delayedFocusCtrl ) )
+        if( wxTextEntry* textEntry = dynamic_cast<wxTextEntry*>( m_delayedFocusCtrl ) )
             textEntry->SelectAll();
 
         m_delayedFocusCtrl = nullptr;
@@ -626,9 +866,34 @@ void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnUpdateUI( wxUpdateUIEvent& event )
 }
 
 
-void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnGridSize( wxSizeEvent& event )
+void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnGridSize( wxSizeEvent& aEvent )
 {
-    adjustGridColumns( event.GetSize().GetX() );
+    wxSize new_size = aEvent.GetSize();
 
-    event.Skip();
+    if( ( !m_itemsGrid->IsCellEditControlShown() || m_lastRequestedSize != new_size )
+            && m_gridSize != new_size )
+    {
+        m_gridSize = new_size;
+
+        // A trick to fix a cosmetic issue: when, in m_itemsGrid, a layer selector widget has
+        // the focus (is activated in column 6) when resizing the grid, the widget is not moved.
+        // So just change the widget having the focus in this case
+        if( m_NoteBook->GetSelection() == 0 && !m_itemsGrid->HasFocus() )
+        {
+            int col = m_itemsGrid->GetGridCursorCol();
+
+            if( col == 6 )  // a layer selector widget can be activated
+                 m_itemsGrid->SetFocus();
+        }
+
+        adjustGridColumns();
+    }
+
+    // We store this value to check whether the dialog is changing size.  This might indicate
+    // that the user is scaling the dialog with an editor shown.  Some editors do not close
+    // (at least on GTK) when the user drags a dialog corner
+    m_lastRequestedSize = new_size;
+
+    // Always propagate for a grid repaint (needed if the height changes, as well as width)
+    aEvent.Skip();
 }
