@@ -41,6 +41,7 @@
 #include <widgets/unit_binder.h>
 
 #include <dialog_graphic_item_properties_base.h>
+#include <tools/drawing_tool.h>
 
 class DIALOG_GRAPHIC_ITEM_PROPERTIES : public DIALOG_GRAPHIC_ITEM_PROPERTIES_BASE
 {
@@ -58,9 +59,6 @@ private:
 
     bool                  m_flipStartEnd;
 
-    wxFloatingPointValidator<double>    m_AngleValidator;
-    double                m_AngleValue;
-
 public:
     DIALOG_GRAPHIC_ITEM_PROPERTIES( PCB_BASE_EDIT_FRAME* aParent, BOARD_ITEM* aItem );
     ~DIALOG_GRAPHIC_ITEM_PROPERTIES() {};
@@ -78,6 +76,8 @@ private:
         finishDialogSettings();
     }
 
+    void onFilledCheckbox( wxCommandEvent& event ) override;
+
     bool Validate() override;
 };
 
@@ -94,9 +94,7 @@ DIALOG_GRAPHIC_ITEM_PROPERTIES::DIALOG_GRAPHIC_ITEM_PROPERTIES( PCB_BASE_EDIT_FR
     m_bezierCtrl1Y( aParent, m_BezierPointC1YLabel, m_BezierC1Y_Ctrl, m_BezierPointC1YUnit ),
     m_bezierCtrl2X( aParent, m_BezierPointC2XLabel, m_BezierC2X_Ctrl, m_BezierPointC2XUnit ),
     m_bezierCtrl2Y( aParent, m_BezierPointC2YLabel, m_BezierC2Y_Ctrl, m_BezierPointC2YUnit ),
-    m_flipStartEnd( false ),
-    m_AngleValidator( 1, &m_AngleValue ),
-    m_AngleValue( 0.0 )
+    m_flipStartEnd( false )
 {
     m_parent = aParent;
     m_item = dynamic_cast<PCB_SHAPE*>( aItem );
@@ -113,9 +111,6 @@ DIALOG_GRAPHIC_ITEM_PROPERTIES::DIALOG_GRAPHIC_ITEM_PROPERTIES( PCB_BASE_EDIT_FR
     m_bezierCtrl2Y.SetCoordType( ORIGIN_TRANSFORMS::ABS_Y_COORD );
 
     m_angle.SetUnits( EDA_UNITS::DEGREES );
-    m_AngleValidator.SetRange( -359.9, 359.9 );
-    m_angleCtrl->SetValidator( m_AngleValidator );
-    m_AngleValidator.SetWindow( m_angleCtrl );
 
     // Do not allow locking items in the footprint editor
     m_locked->Show( dynamic_cast<PCB_EDIT_FRAME*>( aParent ) != nullptr );
@@ -132,13 +127,18 @@ DIALOG_GRAPHIC_ITEM_PROPERTIES::DIALOG_GRAPHIC_ITEM_PROPERTIES( PCB_BASE_EDIT_FR
         m_LayerSelectionCtrl->SetNotAllowedLayerSet( forbiddenLayers );
     }
 
+    for( const std::pair<const PLOT_DASH_TYPE, lineTypeStruct>& typeEntry : lineTypeNames )
+        m_lineStyleCombo->Append( typeEntry.second.name, KiBitmap( typeEntry.second.bitmap ) );
+
+    m_lineStyleCombo->Append( DEFAULT_STYLE );
+
     m_LayerSelectionCtrl->SetLayersHotkeys( false );
     m_LayerSelectionCtrl->SetBoardFrame( m_parent );
     m_LayerSelectionCtrl->Resync();
 
     SetInitialFocus( m_startXCtrl );
 
-    m_StandardButtonsSizerOK->SetDefault();
+    SetupStandardButtons();
 }
 
 
@@ -147,7 +147,39 @@ void PCB_BASE_EDIT_FRAME::ShowGraphicItemPropertiesDialog( BOARD_ITEM* aItem )
     wxCHECK_RET( aItem != NULL, wxT( "ShowGraphicItemPropertiesDialog() error: NULL item" ) );
 
     DIALOG_GRAPHIC_ITEM_PROPERTIES dlg( this, aItem );
-    dlg.ShowQuasiModal();
+
+    if( dlg.ShowQuasiModal() == wxID_OK )
+    {
+        if( aItem->IsOnLayer( GetActiveLayer() ) )
+        {
+            DRAWING_TOOL* drawingTool = m_toolManager->GetTool<DRAWING_TOOL>();
+            drawingTool->SetStroke( aItem->GetStroke(), GetActiveLayer() );
+        }
+    }
+}
+
+
+void DIALOG_GRAPHIC_ITEM_PROPERTIES::onFilledCheckbox( wxCommandEvent& event )
+{
+    if( m_filledCtrl->GetValue() )
+    {
+        m_lineStyleCombo->SetSelection( 0 );
+        m_lineStyleLabel->Enable( false );
+        m_lineStyleCombo->Enable( false );
+    }
+    else
+    {
+        PLOT_DASH_TYPE style = m_item->GetStroke().GetPlotStyle();
+
+        if( style == PLOT_DASH_TYPE::DEFAULT )
+            style = PLOT_DASH_TYPE::SOLID;
+
+        if( (int) style < (int) lineTypeNames.size() )
+            m_lineStyleCombo->SetSelection( (int) style );
+
+        m_lineStyleLabel->Enable( true );
+        m_lineStyleCombo->Enable( true );
+    }
 }
 
 
@@ -176,7 +208,7 @@ bool DIALOG_GRAPHIC_ITEM_PROPERTIES::TransferDataToWindow()
     {
     case SHAPE_T::CIRCLE:
         SetTitle( _( "Circle Properties" ) );
-        m_startPointLabel->SetLabel( _( "Center" ) );
+        m_startPointLabel->SetLabel( _( "Center Point" ) );
 
         m_endPointLabel->SetLabel( _( "Radius" ) );
         m_endXLabel->Show( false );
@@ -189,7 +221,7 @@ bool DIALOG_GRAPHIC_ITEM_PROPERTIES::TransferDataToWindow()
 
     case SHAPE_T::ARC:
         SetTitle( _( "Arc Properties" ) );
-        m_AngleValue = m_item->GetArcAngle() / 10.0;
+        m_angle.SetAngleValue( m_item->GetArcAngle() );
         m_filledCtrl->Show( false );
         break;
 
@@ -216,6 +248,11 @@ bool DIALOG_GRAPHIC_ITEM_PROPERTIES::TransferDataToWindow()
         m_filledCtrl->Show( false );
         break;
 
+    case SHAPE_T::BEZIER:
+        SetTitle( _( "Curve Properties" ) );
+        m_filledCtrl->Show( true );
+        break;
+
     default:
         break;
     }
@@ -254,7 +291,17 @@ bool DIALOG_GRAPHIC_ITEM_PROPERTIES::TransferDataToWindow()
 
     m_filledCtrl->SetValue( m_item->IsFilled() );
     m_locked->SetValue( m_item->IsLocked() );
-    m_thickness.SetValue( m_item->GetWidth() );
+
+    m_thickness.SetValue( m_item->GetStroke().GetWidth() );
+
+    int style = static_cast<int>( m_item->GetStroke().GetPlotStyle() );
+
+    if( style == -1 )
+        m_lineStyleCombo->SetStringSelection( DEFAULT_STYLE );
+    else if( style < (int) lineTypeNames.size() )
+        m_lineStyleCombo->SetSelection( style );
+    else
+        wxFAIL_MSG( "Line type not found in the type lookup map" );
 
     m_LayerSelectionCtrl->SetLayerSelection( m_item->GetLayer() );
 
@@ -288,7 +335,7 @@ bool DIALOG_GRAPHIC_ITEM_PROPERTIES::TransferDataFromWindow()
 
     if( m_item->GetShape() == SHAPE_T::CIRCLE )
     {
-        m_item->SetEnd( m_item->GetStart() + wxPoint( m_endX.GetValue(), 0 ) );
+        m_item->SetEnd( m_item->GetStart() + VECTOR2I( m_endX.GetValue(), 0 ) );
     }
     else if( m_flipStartEnd && m_item->GetShape() != SHAPE_T::ARC )
     {
@@ -304,15 +351,15 @@ bool DIALOG_GRAPHIC_ITEM_PROPERTIES::TransferDataFromWindow()
     // For Bezier curve: Set the two control points
     if( m_item->GetShape() == SHAPE_T::BEZIER )
     {
-        m_item->SetBezierC1( wxPoint( m_bezierCtrl1X.GetValue(), m_bezierCtrl1Y.GetValue() ) );
-        m_item->SetBezierC2( wxPoint( m_bezierCtrl2X.GetValue(), m_bezierCtrl2Y.GetValue() ) );
+        m_item->SetBezierC1( VECTOR2I( m_bezierCtrl1X.GetValue(), m_bezierCtrl1Y.GetValue() ) );
+        m_item->SetBezierC2( VECTOR2I( m_bezierCtrl2X.GetValue(), m_bezierCtrl2Y.GetValue() ) );
     }
 
     if( m_item->GetShape() == SHAPE_T::ARC )
     {
-        VECTOR2D center = CalcArcCenter( m_item->GetStart(), m_item->GetEnd(), m_AngleValue );
+        VECTOR2D c = CalcArcCenter( m_item->GetStart(), m_item->GetEnd(), m_angle.GetAngleValue() );
 
-        m_item->SetCenter( wxPoint( KiROUND( center.x ), KiROUND( center.y ) ) );
+        m_item->SetCenter( c );
     }
 
     if( m_fp_item )
@@ -335,7 +382,21 @@ bool DIALOG_GRAPHIC_ITEM_PROPERTIES::TransferDataFromWindow()
 
     m_item->SetFilled( m_filledCtrl->GetValue() );
     m_item->SetLocked( m_locked->GetValue() );
-    m_item->SetWidth( m_thickness.GetValue() );
+
+    STROKE_PARAMS stroke = m_item->GetStroke();
+
+    stroke.SetWidth( m_thickness.GetValue() );
+
+    auto it = lineTypeNames.begin();
+    std::advance( it, m_lineStyleCombo->GetSelection() );
+
+    if( it == lineTypeNames.end() )
+        stroke.SetPlotStyle( PLOT_DASH_TYPE::DEFAULT );
+    else
+        stroke.SetPlotStyle( it->first );
+
+    m_item->SetStroke( stroke );
+
     m_item->SetLayer( ToLAYER_ID( layer ) );
 
     m_item->RebuildBezierToSegmentsPointsList( m_item->GetWidth() );
@@ -346,8 +407,6 @@ bool DIALOG_GRAPHIC_ITEM_PROPERTIES::TransferDataFromWindow()
     if( wasLocked != m_item->IsLocked() )
         m_parent->GetToolManager()->PostEvent( EVENTS::SelectedEvent );
 
-    m_parent->UpdateMsgPanel();
-
     return true;
 }
 
@@ -364,21 +423,20 @@ bool DIALOG_GRAPHIC_ITEM_PROPERTIES::Validate()
     {
     case SHAPE_T::ARC:
         // Check angle of arc.
-        if( m_angle.GetValue() == 0 )
-            errors.Add( _( "The arc angle cannot be zero." ) );
+        if( m_angle.GetAngleValue() == ANGLE_0 )
+            errors.Add( _( "Arc angle cannot be zero." ) );
 
         if( m_startX.GetValue() == m_endX.GetValue() && m_startY.GetValue() == m_endY.GetValue() )
         {
-            double arcAngleDeg = m_angle.GetDoubleValue()/10.0;
-            errors.Add( wxString::Format( _( "Invalid Arc with radius %f and angle %f" ),
-                                          0.0, arcAngleDeg ) );
+            errors.Add( wxString::Format( _( "Invalid Arc with radius %f and angle %f." ),
+                                          0.0, m_angle.GetDoubleValue() ) );
         }
         else
         {
             VECTOR2D start( m_startX.GetValue(), m_startY.GetValue() );
             VECTOR2D end( m_endX.GetValue(), m_endY.GetValue() );
-            double arcAngleDeg = m_angle.GetDoubleValue()/10.0;
-            VECTOR2D center = CalcArcCenter( start, end, arcAngleDeg );
+            VECTOR2D center = CalcArcCenter( start, end, m_angle.GetAngleValue() );
+
             double radius = ( center - start ).EuclideanNorm();
             double max_offset = std::max( std::abs( center.x ) + radius,
                                           std::abs( center.y ) + radius );
@@ -386,51 +444,51 @@ bool DIALOG_GRAPHIC_ITEM_PROPERTIES::Validate()
             if( max_offset >= ( std::numeric_limits<VECTOR2I::coord_type>::max() / 2 )
                     || center == start || center == end )
             {
-                errors.Add( wxString::Format( _( "Invalid Arc with radius %f and angle %f" ),
-                                              radius, arcAngleDeg ) );
+                errors.Add( wxString::Format( _( "Invalid Arc with radius %f and angle %f." ),
+                                              radius, m_angle.GetDoubleValue() ) );
             }
         }
 
         if( m_thickness.GetValue() <= 0 )
-            errors.Add( _( "The item thickness must be greater than zero." ) );
+            errors.Add( _( "Line width must be greater than zero." ) );
 
         break;
 
     case SHAPE_T::CIRCLE:
         // Check radius.
         if( m_endX.GetValue() <= 0 )
-            errors.Add( _( "The radius must be greater than zero." ) );
+            errors.Add( _( "Radius must be greater than zero." ) );
 
         if( !m_filledCtrl->GetValue() && m_thickness.GetValue() <= 0 )
-            errors.Add( _( "The item thickness must be greater than zero." ) );
+            errors.Add( _( "Line width must be greater than zero for an unfilled circle." ) );
 
         break;
 
     case SHAPE_T::RECT:
         // Check for null rect.
         if( m_startX.GetValue() == m_endX.GetValue() && m_startY.GetValue() == m_endY.GetValue() )
-            errors.Add( _( "The rectangle cannot be empty." ) );
+            errors.Add( _( "Rectangle cannot be empty." ) );
 
         if( !m_filledCtrl->GetValue() && m_thickness.GetValue() <= 0 )
-            errors.Add( _( "The item thickness must be greater than zero." ) );
+            errors.Add( _( "Line width must be greater than zero for an unfilled rectangle." ) );
 
         break;
 
     case SHAPE_T::POLY:
         if( !m_filledCtrl->GetValue() && m_thickness.GetValue() <= 0 )
-            errors.Add( _( "The polygon outline thickness must be >= 0." ) );
+            errors.Add( _( "Line width must be greater than zero for an unfilled polygon." ) );
 
         break;
 
     case SHAPE_T::SEGMENT:
         if( m_thickness.GetValue() <= 0 )
-            errors.Add( _( "The item thickness must be greater than zero." ) );
+            errors.Add( _( "Line width must be greater than zero." ) );
 
         break;
 
     case SHAPE_T::BEZIER:
         if( !m_filledCtrl->GetValue() && m_thickness.GetValue() <= 0 )
-            errors.Add( _( "The item thickness must be greater than zero." ) );
+            errors.Add( _( "Line width must be greater than zero for an unfilled curve." ) );
 
         break;
 
