@@ -18,15 +18,14 @@
  */
 
 
-#include <common.h>
 #include <board.h>
+#include <board_design_settings.h>
 #include <pcb_track.h>
 
 #include <drc/drc_engine.h>
 #include <drc/drc_item.h>
 #include <drc/drc_rule.h>
 #include <drc/drc_test_provider.h>
-#include <drc/drc_length_report.h>
 #include <drc/drc_rtree.h>
 
 #include <geometry/shape_segment.h>
@@ -74,15 +73,7 @@ public:
         return wxT( "Tests differential pair coupling" );
     }
 
-    virtual int GetNumPhases() const override
-    {
-        return 1;
-    }
-
-    virtual std::set<DRC_CONSTRAINT_T> GetConstraintTypes() const override;
-
 private:
-
     BOARD* m_board;
 };
 
@@ -149,13 +140,20 @@ struct DIFF_PAIR_KEY
                 return true;
             else if( netN > b.netN )
                 return false;
+            else if( gapRuleName.IsEmpty() )
+                return gapRuleName < b.gapRuleName;
             else
-                return parentRule < b.parentRule;
+                return uncoupledRuleName < b.uncoupledRuleName;
         }
     }
 
     int       netP, netN;
-    DRC_RULE* parentRule;
+    wxString  gapRuleName;
+    wxString  uncoupledRuleName;
+    std::optional<MINOPTMAX<int>> gapConstraint;
+    DRC_RULE* gapRule;
+    std::optional<MINOPTMAX<int>> uncoupledConstraint;
+    DRC_RULE* uncoupledRule;
 };
 
 struct DIFF_PAIR_COUPLED_SEGMENTS
@@ -166,17 +164,20 @@ struct DIFF_PAIR_COUPLED_SEGMENTS
     PCB_TRACK*   parentP;
     int          computedGap;
     PCB_LAYER_ID layer;
-    bool         couplingOK;
+    bool         couplingFailMin;
+    bool         couplingFailMax;
 
     DIFF_PAIR_COUPLED_SEGMENTS() :
         parentN( nullptr ),
         parentP( nullptr ),
         computedGap( 0 ),
         layer( UNDEFINED_LAYER ),
-        couplingOK( false )
+        couplingFailMin( false ),
+        couplingFailMax( false )
     {}
 };
 
+
 struct DIFF_PAIR_ITEMS
 {
     std::set<BOARD_CONNECTED_ITEM*> itemsP, itemsN;
@@ -186,12 +187,13 @@ struct DIFF_PAIR_ITEMS
     int totalLengthP;
 };
 
-static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp, DRC_RTREE& aTree )
+
+static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp )
 {
     for( BOARD_CONNECTED_ITEM* itemP : aDp.itemsP )
     {
         PCB_TRACK* sp = dyn_cast<PCB_TRACK*>( itemP );
-        OPT<DIFF_PAIR_COUPLED_SEGMENTS> bestCoupled;
+        std::optional<DIFF_PAIR_COUPLED_SEGMENTS> bestCoupled;
         int bestGap = std::numeric_limits<int>::max();
 
         if( !sp )
@@ -258,12 +260,13 @@ static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp, DRC_RTREE& aTree
 
             SHAPE_SEGMENT checkSegStart( bestCoupled->coupledP.A, bestCoupled->coupledN.A );
             SHAPE_SEGMENT checkSegEnd( bestCoupled->coupledP.B, bestCoupled->coupledN.B );
+            DRC_RTREE*    tree = bestCoupled->parentP->GetBoard()->m_CopperItemRTreeCache.get();
 
             // check if there's anything in between the segments suspected to be coupled. If
             // there's nothing, assume they are really coupled.
 
-            if( !aTree.CheckColliding( &checkSegStart, sp->GetLayer(), 0, excludeSelf )
-                  && !aTree.CheckColliding( &checkSegEnd, sp->GetLayer(), 0, excludeSelf ) )
+            if( !tree->CheckColliding( &checkSegStart, sp->GetLayer(), 0, excludeSelf )
+                  && !tree->CheckColliding( &checkSegEnd, sp->GetLayer(), 0, excludeSelf ) )
             {
                 aDp.coupled.push_back( *bestCoupled );
             }
@@ -277,16 +280,16 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
 {
     m_board = m_drcEngine->GetBoard();
 
-
+    int epsilon = m_board->GetDesignSettings().GetDRCEpsilon();
 
     std::map<DIFF_PAIR_KEY, DIFF_PAIR_ITEMS> dpRuleMatches;
 
     auto evaluateDpConstraints =
             [&]( BOARD_ITEM *item ) -> bool
             {
-                DIFF_PAIR_KEY key;
+                DIFF_PAIR_KEY         key;
                 BOARD_CONNECTED_ITEM* citem = static_cast<BOARD_CONNECTED_ITEM*>( item );
-                NETINFO_ITEM* refNet = citem->GetNet();
+                NETINFO_ITEM*         refNet = citem->GetNet();
 
                 if( refNet && DRC_ENGINE::IsNetADiffPair( m_board, refNet, key.netP, key.netN ) )
                 {
@@ -299,15 +302,35 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
 
                     for( int i = 0; i < 2; i++ )
                     {
-                        auto constraint = m_drcEngine->EvalRules( constraintsToCheck[ i ], item,
-                                                                  nullptr, item->GetLayer() );
+                        DRC_CONSTRAINT constraint = m_drcEngine->EvalRules( constraintsToCheck[ i ],
+                                                                            item, nullptr,
+                                                                            item->GetLayer() );
 
-                        if( constraint.IsNull() )
+                        if( constraint.IsNull() || constraint.GetSeverity() == RPT_SEVERITY_IGNORE )
                             continue;
 
                         drc_dbg( 10, wxT( "cns %d item %p\n" ), constraintsToCheck[i], item );
 
-                        key.parentRule = constraint.GetParentRule();
+                        DRC_RULE* parentRule = constraint.GetParentRule();
+                        wxString ruleName = parentRule ? parentRule->m_Name : constraint.GetName();
+
+                        switch( constraintsToCheck[i] )
+                        {
+                        case DIFF_PAIR_GAP_CONSTRAINT:
+                            key.gapConstraint = constraint.GetValue();
+                            key.gapRule = parentRule;
+                            key.gapRuleName = ruleName;
+                            break;
+
+                        case DIFF_PAIR_MAX_UNCOUPLED_CONSTRAINT:
+                            key.uncoupledConstraint = constraint.GetValue();
+                            key.uncoupledRule = parentRule;
+                            key.uncoupledRuleName = ruleName;
+                            break;
+
+                        default:
+                            break;
+                        }
 
                         if( refNet->GetNetCode() == key.netN )
                             dpRuleMatches[key].itemsN.insert( citem );
@@ -321,36 +344,17 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
 
     m_board->GetConnectivity()->GetFromToCache()->Rebuild( m_board );
 
-    forEachGeometryItem( { PCB_TRACE_T, PCB_VIA_T, PCB_ARC_T },
-                    LSET::AllCuMask(), evaluateDpConstraints );
+    forEachGeometryItem( { PCB_TRACE_T, PCB_VIA_T, PCB_ARC_T }, LSET::AllCuMask(),
+                         evaluateDpConstraints );
 
     drc_dbg( 10, wxT( "dp rule matches %d\n" ), (int) dpRuleMatches.size() );
 
+    reportAux( wxT( "DPs evaluated:" ) );
 
-    DRC_RTREE copperTree;
-
-    auto addToTree =
-            [&copperTree]( BOARD_ITEM *item ) -> bool
-            {
-                for( PCB_LAYER_ID layer : item->GetLayerSet().Seq() )
-                {
-                    if( IsCopperLayer( layer ) )
-                        copperTree.Insert( item, layer );
-                }
-
-                return true;
-            };
-
-    forEachGeometryItem( { PCB_TRACE_T, PCB_VIA_T, PCB_PAD_T, PCB_ZONE_T, PCB_ARC_T },
-                         LSET::AllCuMask(), addToTree );
-
-
-    reportAux( wxString::Format( _("DPs evaluated:") ) );
-
-    for( auto& it : dpRuleMatches )
+    for( auto& [ key, itemSet ] : dpRuleMatches )
     {
-        NETINFO_ITEM *niP = m_board->GetNetInfo().GetNetItem( it.first.netP );
-        NETINFO_ITEM *niN = m_board->GetNetInfo().GetNetItem( it.first.netN );
+        NETINFO_ITEM *niP = m_board->GetNetInfo().GetNetItem( key.netP );
+        NETINFO_ITEM *niN = m_board->GetNetInfo().GetNetItem( key.netN );
 
         assert( niP );
         assert( niN );
@@ -359,48 +363,41 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
         wxString nameN = niN->GetNetname();
 
         reportAux( wxString::Format( wxT( "Rule '%s', DP: (+) %s - (-) %s" ),
-                                     it.first.parentRule->m_Name,
-                                     nameP,
-                                     nameN ) );
+                                     key.gapRuleName, nameP, nameN ) );
 
-        extractDiffPairCoupledItems( it.second, copperTree );
+        extractDiffPairCoupledItems( itemSet );
 
-        it.second.totalCoupled = 0;
-        it.second.totalLengthN = 0;
-        it.second.totalLengthP = 0;
+        itemSet.totalCoupled = 0;
+        itemSet.totalLengthN = 0;
+        itemSet.totalLengthP = 0;
 
-        drc_dbg(10, wxT( "       coupled prims : %d\n" ), (int) it.second.coupled.size() );
+        drc_dbg(10, wxT( "       coupled prims : %d\n" ), (int) itemSet.coupled.size() );
 
-        OPT<DRC_CONSTRAINT> gapConstraint =
-                it.first.parentRule->FindConstraint( DIFF_PAIR_GAP_CONSTRAINT );
-        OPT<DRC_CONSTRAINT> maxUncoupledConstraint =
-                it.first.parentRule->FindConstraint( DIFF_PAIR_MAX_UNCOUPLED_CONSTRAINT );
-
-        for( BOARD_CONNECTED_ITEM* item : it.second.itemsN )
+        for( BOARD_CONNECTED_ITEM* item : itemSet.itemsN )
         {
             // fixme: include vias
             if( PCB_TRACK* track = dyn_cast<PCB_TRACK*>( item ) )
-                it.second.totalLengthN += track->GetLength();
+                itemSet.totalLengthN += track->GetLength();
         }
 
-        for( BOARD_CONNECTED_ITEM* item : it.second.itemsP )
+        for( BOARD_CONNECTED_ITEM* item : itemSet.itemsP )
         {
             // fixme: include vias
             if( PCB_TRACK* track = dyn_cast<PCB_TRACK*>( item ) )
-                it.second.totalLengthP += track->GetLength();
+                itemSet.totalLengthP += track->GetLength();
         }
 
-        for( auto& cpair : it.second.coupled )
+        for( DIFF_PAIR_COUPLED_SEGMENTS& dp : itemSet.coupled )
         {
-            int length = cpair.coupledN.Length();
-            int gap = cpair.coupledN.Distance( cpair.coupledP );
+            int length = dp.coupledN.Length();
+            int gap = dp.coupledN.Distance( dp.coupledP );
 
-            gap -= cpair.parentN->GetWidth() / 2;
-            gap -= cpair.parentP->GetWidth() / 2;
+            gap -= dp.parentN->GetWidth() / 2;
+            gap -= dp.parentP->GetWidth() / 2;
 
-            cpair.computedGap = gap;
+            dp.computedGap = gap;
 
-            auto overlay = m_drcEngine->GetDebugOverlay();
+            std::shared_ptr<KIGFX::VIEW_OVERLAY> overlay = m_drcEngine->GetDebugOverlay();
 
             if( overlay )
             {
@@ -408,141 +405,151 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
                 overlay->SetIsStroke(true);
                 overlay->SetStrokeColor( RED );
                 overlay->SetLineWidth( 100000 );
-                overlay->Line( cpair.coupledP );
+                overlay->Line( dp.coupledP );
                 overlay->SetStrokeColor( BLUE );
-                overlay->Line( cpair.coupledN );
+                overlay->Line( dp.coupledN );
             }
 
             drc_dbg( 10, wxT( "               len %d gap %d l %d\n" ),
                      length,
                      gap,
-                     cpair.parentP->GetLayer() );
+                     dp.parentP->GetLayer() );
 
-            if( gapConstraint )
+            if( key.gapConstraint )
             {
-                auto val = gapConstraint->GetValue();
-                bool insideRange = true;
+                if( key.gapConstraint->HasMin()
+                    && key.gapConstraint->Min() >= 0
+                    && ( gap < key.gapConstraint->Min() - epsilon ) )
+                {
+                    dp.couplingFailMin = true;
+                }
 
-                if( val.HasMin() && gap < val.Min() )
-                    insideRange = false;
-
-                if( val.HasMax() && gap > val.Max() )
-                    insideRange = false;
-
-//                if(val.HasMin() && val.HasMax() )
-  //                  drc_dbg(10, "Vmin %d vmax %d\n", val.Min(), val.Max() );
-
-                cpair.couplingOK = insideRange;
-            }
-            else
-            {
-                cpair.couplingOK = true;
+                if( key.gapConstraint->HasMax()
+                    && key.gapConstraint->Max() >= 0
+                    && ( gap > key.gapConstraint->Max() + epsilon ) )
+                {
+                    dp.couplingFailMax = true;
+                }
             }
 
-            if( cpair.couplingOK )
-                it.second.totalCoupled += length;
+            if( !dp.couplingFailMin && !dp.couplingFailMax )
+                itemSet.totalCoupled += length;
         }
 
-        int totalLen = std::max( it.second.totalLengthN, it.second.totalLengthP );
+        int totalLen = std::max( itemSet.totalLengthN, itemSet.totalLengthP );
         reportAux( wxString::Format( wxT( "   - coupled length: %s, total length: %s" ),
-                                     MessageTextFromValue( userUnits(), it.second.totalCoupled ),
-                                     MessageTextFromValue( userUnits(), totalLen ) ) );
+                                     MessageTextFromValue( itemSet.totalCoupled ),
+                                     MessageTextFromValue( totalLen ) ) );
 
-        int totalUncoupled = totalLen - it.second.totalCoupled;
+        int totalUncoupled = totalLen - itemSet.totalCoupled;
 
         bool uncoupledViolation = false;
 
-        if( maxUncoupledConstraint && ( !it.second.itemsP.empty() || ! it.second.itemsN.empty() ) )
+        if( key.uncoupledConstraint && ( !itemSet.itemsP.empty() || !itemSet.itemsN.empty() ) )
         {
-            auto val = maxUncoupledConstraint->GetValue();
+            const MINOPTMAX<int>& val = *key.uncoupledConstraint;
 
-            if ( val.HasMax() && totalUncoupled > val.Max() )
+            if( val.HasMax() && val.Max() >= 0 && totalUncoupled > val.Max() )
             {
-                auto drce = DRC_ITEM::Create( DRCE_DIFF_PAIR_UNCOUPLED_LENGTH_TOO_LONG );
-                wxString msg;
-
-                msg = wxString::Format( _( "(%s maximum uncoupled length: %s; actual: %s)" ),
-                                          maxUncoupledConstraint->GetParentRule()->m_Name,
-                                          MessageTextFromValue( userUnits(), val.Max() ),
-                                          MessageTextFromValue( userUnits(), totalUncoupled ) );
+                auto     drce = DRC_ITEM::Create( DRCE_DIFF_PAIR_UNCOUPLED_LENGTH_TOO_LONG );
+                wxString msg = formatMsg( _( "(%s maximum uncoupled length %s; actual %s)" ),
+                                          key.uncoupledRuleName,
+                                          val.Max(),
+                                          totalUncoupled );
 
                 drce->SetErrorMessage( drce->GetErrorText() + wxS( " " ) + msg );
 
                 BOARD_CONNECTED_ITEM* item = nullptr;
-                auto                  pit = it.second.itemsP.begin();
-                auto                  nit = it.second.itemsN.begin();
+                auto                  p_it = itemSet.itemsP.begin();
+                auto                  n_it = itemSet.itemsN.begin();
 
-                if( pit != it.second.itemsP.end() )
+                if( p_it != itemSet.itemsP.end() )
                 {
-                    item = *pit;
-                    drce->AddItem( *pit );
-                    pit++;
+                    item = *p_it;
+                    drce->AddItem( *p_it );
+                    p_it++;
                 }
 
-                if( nit != it.second.itemsN.end() )
+                if( n_it != itemSet.itemsN.end() )
                 {
-                    item = *nit;
-                    drce->AddItem( *nit );
-                    nit++;
+                    item = *n_it;
+                    drce->AddItem( *n_it );
+                    n_it++;
                 }
 
-                while( pit != it.second.itemsP.end() )
-                    drce->AddItem( *pit++ );
+                while( p_it != itemSet.itemsP.end() )
+                    drce->AddItem( *p_it++ );
 
-                while( nit != it.second.itemsN.end() )
-                    drce->AddItem( *nit++ );
+                while( n_it != itemSet.itemsN.end() )
+                    drce->AddItem( *n_it++ );
 
                 uncoupledViolation = true;
 
-                drce->SetViolatingRule( maxUncoupledConstraint->GetParentRule() );
+                drce->SetViolatingRule( key.uncoupledRule );
 
-                reportViolation( drce, item->GetPosition() );
+                reportViolation( drce, item->GetPosition(), item->GetLayer() );
             }
         }
 
-        if ( gapConstraint && ( uncoupledViolation || !maxUncoupledConstraint ) )
+        if( key.gapConstraint && ( uncoupledViolation || !key.uncoupledConstraint ) )
         {
-            for( auto& cpair : it.second.coupled )
+            for( DIFF_PAIR_COUPLED_SEGMENTS& dp : itemSet.coupled )
             {
-                if( !cpair.couplingOK && ( cpair.parentP || cpair.parentN ) )
+                if( ( dp.couplingFailMin || dp.couplingFailMax ) && ( dp.parentP || dp.parentN ) )
                 {
-                    auto val = gapConstraint->GetValue();
-                    auto drcItem = DRC_ITEM::Create( DRCE_DIFF_PAIR_GAP_OUT_OF_RANGE );
+                    // We have a candidate violation, now we need to re-query for a constraint
+                    // given the actual items, because there may be a location-based rule in play.
+                    DRC_CONSTRAINT constraint = m_drcEngine->EvalRules( DIFF_PAIR_GAP_CONSTRAINT,
+                                                                        dp.parentP, dp.parentN,
+                                                                        dp.parentP->GetLayer() );
+                    MINOPTMAX<int> val = constraint.GetValue();
+
+                    if( !val.HasMin() || val.Min() < 0 || dp.computedGap >= val.Min() )
+                        dp.couplingFailMin = false;
+
+                    if( !val.HasMax() || val.Max() < 0 || dp.computedGap <= val.Max() )
+                        dp.couplingFailMax = false;
+
+                    if( !dp.couplingFailMin && !dp.couplingFailMax )
+                        continue;
+
+                    auto     drcItem = DRC_ITEM::Create( DRCE_DIFF_PAIR_GAP_OUT_OF_RANGE );
                     wxString msg;
 
-                    msg = drcItem->GetErrorText() + wxT( " (" ) +
-                            gapConstraint->GetParentRule()->m_Name + wxS( " " );
+                    if( dp.couplingFailMin )
+                    {
+                        msg = formatMsg( _( "(%s minimum gap %s; actual %s)" ),
+                                         key.gapRuleName,
+                                         val.Min(),
+                                         dp.computedGap );
+                    }
+                    else if( dp.couplingFailMax )
+                    {
+                        msg = formatMsg( _( "(%s maximum gap %s; actual %s)" ),
+                                         key.gapRuleName,
+                                         val.Max(),
+                                         dp.computedGap );
+                    }
 
-                    if( val.HasMin() )
-                        msg += wxString::Format( _( "minimum gap: %s; " ),
-                                                   MessageTextFromValue( userUnits(), val.Min() ) );
-
-                    if( val.HasMax() )
-                        msg += wxString::Format( _( "maximum gap: %s; " ),
-                                                   MessageTextFromValue( userUnits(), val.Max() ) );
-
-                    msg += wxString::Format( _( "actual: %s)" ),
-                        MessageTextFromValue( userUnits(), cpair.computedGap ) );
-
-                    drcItem->SetErrorMessage( msg );
+                    drcItem->SetErrorMessage( drcItem->GetErrorText() + wxS( " " ) + msg );
 
                     BOARD_CONNECTED_ITEM* item = nullptr;
 
-                    if( cpair.parentP )
+                    if( dp.parentP )
                     {
-                        item = cpair.parentP;
-                        drcItem->AddItem( cpair.parentP );
+                        item = dp.parentP;
+                        drcItem->AddItem( dp.parentP );
                     }
 
-                    if( cpair.parentN )
+                    if( dp.parentN )
                     {
-                        item = cpair.parentN;
-                        drcItem->AddItem( cpair.parentN );
+                        item = dp.parentN;
+                        drcItem->AddItem( dp.parentN );
                     }
 
-                    drcItem->SetViolatingRule( gapConstraint->GetParentRule() );
+                    drcItem->SetViolatingRule( key.gapRule );
 
-                    reportViolation( drcItem, item->GetPosition() );
+                    reportViolation( drcItem, item->GetPosition(), item->GetLayer() );
                 }
             }
         }
@@ -554,12 +561,6 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
 }
 
 
-std::set<DRC_CONSTRAINT_T> test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::GetConstraintTypes() const
-{
-    return { DIFF_PAIR_GAP_CONSTRAINT, DIFF_PAIR_MAX_UNCOUPLED_CONSTRAINT };
-}
-
-
 namespace detail
 {
     static DRC_REGISTER_TEST_PROVIDER<test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING> dummy;
