@@ -4,7 +4,7 @@
  * Copyright (C) 2015 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2015 SoftPLC Corporation, Dick Hollenbeck <dick@softplc.com>
  * Copyright (C) 2015-2016 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -37,10 +37,6 @@
 #include <board.h>
 #include <footprint.h>
 #include <confirm.h>
-#include <dialogs/panel_fp_editor_color_settings.h>
-#include <dialogs/panel_fp_editor_defaults.h>
-#include <dialogs/panel_display_options.h>
-#include <dialogs/panel_edit_options.h>
 #include <footprint_edit_frame.h>
 #include <footprint_editor_settings.h>
 #include <footprint_info_impl.h>
@@ -51,7 +47,6 @@
 #include <kiplatform/app.h>
 #include <kiway.h>
 #include <macros.h>
-#include <panel_hotkeys_editor.h>
 #include <pcb_draw_panel_gal.h>
 #include <pcb_edit_frame.h>
 #include <pcbnew.h>
@@ -71,14 +66,11 @@
 #include <tools/group_tool.h>
 #include <tools/position_relative_tool.h>
 #include <widgets/appearance_controls.h>
-#include <widgets/infobar.h>
 #include <widgets/lib_tree.h>
-#include <widgets/paged_dialog.h>
 #include <widgets/panel_selection_filter.h>
 #include <widgets/wx_progress_reporters.h>
 #include <wildcards_and_files_ext.h>
 #include <wx/filedlg.h>
-#include <wx/treebook.h>
 #include <widgets/wx_aui_utils.h>
 
 BEGIN_EVENT_TABLE( FOOTPRINT_EDIT_FRAME, PCB_BASE_FRAME )
@@ -92,8 +84,8 @@ BEGIN_EVENT_TABLE( FOOTPRINT_EDIT_FRAME, PCB_BASE_FRAME )
 
     EVT_TOOL( ID_FPEDIT_SAVE_PNG, FOOTPRINT_EDIT_FRAME::OnSaveFootprintAsPng )
 
-    EVT_TOOL( ID_LOAD_FOOTPRINT_FROM_BOARD, FOOTPRINT_EDIT_FRAME::LoadFootprintFromBoard )
-    EVT_TOOL( ID_ADD_FOOTPRINT_TO_BOARD, FOOTPRINT_EDIT_FRAME::SaveFootprintToBoard )
+    EVT_TOOL( ID_LOAD_FOOTPRINT_FROM_BOARD, FOOTPRINT_EDIT_FRAME::OnLoadFootprintFromBoard )
+    EVT_TOOL( ID_ADD_FOOTPRINT_TO_BOARD, FOOTPRINT_EDIT_FRAME::OnSaveFootprintToBoard )
 
     // Horizontal toolbar
     EVT_MENU( ID_GRID_SETTINGS, FOOTPRINT_EDIT_FRAME::OnGridSettings )
@@ -105,6 +97,10 @@ BEGIN_EVENT_TABLE( FOOTPRINT_EDIT_FRAME, PCB_BASE_FRAME )
     EVT_UPDATE_UI( ID_ADD_FOOTPRINT_TO_BOARD,
                    FOOTPRINT_EDIT_FRAME::OnUpdateSaveFootprintToBoard )
     EVT_UPDATE_UI( ID_TOOLBARH_PCB_SELECT_LAYER, FOOTPRINT_EDIT_FRAME::OnUpdateLayerSelectBox )
+
+    // Drop files event
+    EVT_DROP_FILES( FOOTPRINT_EDIT_FRAME::OnDropFiles )
+
 END_EVENT_TABLE()
 
 
@@ -115,7 +111,7 @@ FOOTPRINT_EDIT_FRAME::FOOTPRINT_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
     m_show_layer_manager_tools( true )
 {
     m_showBorderAndTitleBlock = false;   // true to show the frame references
-    m_aboutTitle = _( "KiCad Footprint Editor" );
+    m_aboutTitle = _HKI( "KiCad Footprint Editor" );
     m_selLayerBox = nullptr;
     m_editorSettings = nullptr;
 
@@ -146,11 +142,11 @@ FOOTPRINT_EDIT_FRAME::FOOTPRINT_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
     // In Footprint Editor, the default net clearance is not known (it depends on the actual
     // board).  So we do not show the default clearance, by setting it to 0.  The footprint or
     // pad specific clearance will be shown.
-    GetBoard()->GetDesignSettings().GetDefault()->SetClearance( 0 );
+    GetBoard()->GetDesignSettings().m_NetSettings->m_DefaultNetClass->SetClearance( 0 );
 
-    // Don't show the default board solder mask clearance in the footprint editor.  Only the
-    // footprint or pad clearance setting should be shown if it is not 0.
-    GetBoard()->GetDesignSettings().m_SolderMaskMargin = 0;
+    // Don't show the default board solder mask expansion in the footprint editor.  Only the
+    // footprint or pad mask expansions settings should be shown.
+    GetBoard()->GetDesignSettings().m_SolderMaskExpansion = 0;
 
     // restore the last footprint from the project, if any
     restoreLastFootprint();
@@ -160,15 +156,11 @@ FOOTPRINT_EDIT_FRAME::FOOTPRINT_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
     // them, at least to be able to edit a bad layer
     GetBoard()->SetVisibleAlls();
 
-    // However the "no net" mark on pads is useless, because there are no nets in footprint
-    // editor: make it non visible.
-    GetBoard()->SetElementVisibility( LAYER_NO_CONNECTS, false );
-
     GetGalDisplayOptions().m_axesEnabled = true;
 
     // In Footprint Editor, set the default paper size to A4 for plot/print
     SetPageSettings( PAGE_INFO( PAGE_INFO::A4 ) );
-    SetScreen( new PCB_SCREEN( GetPageSettings().GetSizeIU() ) );
+    SetScreen( new PCB_SCREEN( GetPageSettings().GetSizeIU( pcbIUScale.IU_PER_MILS ) ) );
 
     // Create the manager and dispatcher & route draw panel events to the dispatcher
     setupTools();
@@ -216,20 +208,20 @@ FOOTPRINT_EDIT_FRAME::FOOTPRINT_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
     // Rows; layers 4 - 6
     m_auimgr.AddPane( m_mainToolBar, EDA_PANE().HToolbar().Name( "MainToolbar" )
                       .Top().Layer( 6 ) );
+
     m_auimgr.AddPane( m_messagePanel, EDA_PANE().Messages().Name( "MsgPanel" )
                       .Bottom().Layer( 6 ) );
 
     // Columns; layers 1 - 3
-    m_auimgr.AddPane( m_optionsToolBar, EDA_PANE().VToolbar().Name( "OptToolbar" )
-                      .Left().Layer( 3 ) );
     m_auimgr.AddPane( m_treePane, EDA_PANE().Palette().Name( "Footprints" )
-                      .Left().Layer(2)
+                      .Left().Layer( 3 )
                       .Caption( _( "Libraries" ) )
-                      .MinSize( 250, 400 ).Resizable() );
+                      .MinSize( 250, -1 ).BestSize( 250, -1 ) );
+    m_auimgr.AddPane( m_optionsToolBar, EDA_PANE().VToolbar().Name( "OptToolbar" )
+                      .Left().Layer( 2 ) );
 
     m_auimgr.AddPane( m_drawToolBar, EDA_PANE().VToolbar().Name( "ToolsToolbar" )
                       .Right().Layer(2) );
-
     m_auimgr.AddPane( m_appearancePanel, EDA_PANE().Name( "LayersManager" )
                       .Right().Layer( 3 )
                       .Caption( _( "Appearance" ) ).PaneBorder( false )
@@ -249,40 +241,52 @@ FOOTPRINT_EDIT_FRAME::FOOTPRINT_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
     // The selection filter doesn't need to grow in the vertical direction when docked
     m_auimgr.GetPane( "SelectionFilter" ).dock_proportion = 0;
 
+    m_acceptedExts.emplace( KiCadFootprintLibPathExtension, &ACTIONS::ddAddLibrary );
+    m_acceptedExts.emplace( KiCadFootprintFileExtension, &PCB_ACTIONS::ddImportFootprint );
+    DragAcceptFiles( true );
+
     ActivateGalCanvas();
 
     FinishAUIInitialization();
 
-    if( m_editorSettings->m_LibWidth > 0 )
-    {
-        wxAuiPaneInfo& treePane = m_auimgr.GetPane( "Footprints" );
-
-        // wxAUI hack: force width by setting MinSize() and then Fixed()
-        // thanks to ZenJu http://trac.wxwidgets.org/ticket/13180
-        treePane.MinSize( m_editorSettings->m_LibWidth, -1 );
-        treePane.Fixed();
-        m_auimgr.Update();
-
-        // now make it resizable again
-        treePane.Resizable();
-        m_auimgr.Update();
-
-        // Note: DO NOT call m_auimgr.Update() anywhere after this; it will nuke the size
-        // back to minimum.
-        treePane.MinSize( 250, -1 );
-    }
-
     // Apply saved visibility stuff at the end
     FOOTPRINT_EDITOR_SETTINGS* cfg = GetSettings();
-    m_appearancePanel->SetUserLayerPresets( cfg->m_LayerPresets );
-    m_appearancePanel->ApplyLayerPreset( cfg->m_ActiveLayerPreset );
+    wxAuiPaneInfo&             treePane = m_auimgr.GetPane( "Footprints" );
+    wxAuiPaneInfo&             layersManager = m_auimgr.GetPane( "LayersManager" );
+
+    // wxAUI hack: force widths by setting MinSize() and then Fixed()
+    // thanks to ZenJu http://trac.wxwidgets.org/ticket/13180
+
+    if( cfg->m_LibWidth > 0 )
+    {
+        SetAuiPaneSize( m_auimgr, treePane, cfg->m_LibWidth, -1 );
+
+        treePane.MinSize( cfg->m_LibWidth, -1 );
+        treePane.Fixed();
+    }
 
     if( cfg->m_AuiPanels.right_panel_width > 0 )
     {
-        wxAuiPaneInfo& layersManager = m_auimgr.GetPane( "LayersManager" );
         SetAuiPaneSize( m_auimgr, layersManager, cfg->m_AuiPanels.right_panel_width, -1 );
+
+        layersManager.MinSize( cfg->m_LibWidth, -1 );
+        layersManager.Fixed();
     }
 
+    // Apply fixed sizes
+    m_auimgr.Update();
+
+    // Now make them resizable again
+    treePane.Resizable();
+    m_auimgr.Update();
+
+    // Note: DO NOT call m_auimgr.Update() anywhere after this; it will nuke the sizes
+    // back to minimum.
+    treePane.MinSize( 250, -1 );
+    layersManager.MinSize( 250, -1 );
+
+    m_appearancePanel->SetUserLayerPresets( cfg->m_LayerPresets );
+    m_appearancePanel->ApplyLayerPreset( cfg->m_ActiveLayerPreset );
     m_appearancePanel->SetTabIndex( cfg->m_AuiPanels.appearance_panel_tab );
 
     GetToolManager()->RunAction( ACTIONS::zoomFitScreen, false );
@@ -305,11 +309,13 @@ FOOTPRINT_EDIT_FRAME::FOOTPRINT_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
 
     // Register a call to update the toolbar sizes. It can't be done immediately because
     // it seems to require some sizes calculated that aren't yet (at least on GTK).
-    CallAfter( [&]()
-               {
-                   // Ensure the controls on the toolbars all are correctly sized
-                    UpdateToolbarControlSizes();
-               } );
+    CallAfter(
+            [&]()
+            {
+                // Ensure the controls on the toolbars all are correctly sized
+                UpdateToolbarControlSizes();
+                m_treePane->FocusSearchFieldIfExists();
+            } );
 }
 
 
@@ -328,9 +334,25 @@ FOOTPRINT_EDIT_FRAME::~FOOTPRINT_EDIT_FRAME()
 }
 
 
+void FOOTPRINT_EDIT_FRAME::UpdateMsgPanel()
+{
+    EDA_DRAW_FRAME::UpdateMsgPanel();
+
+    FOOTPRINT* fp = static_cast<FOOTPRINT*>( GetModel() );
+
+    if( fp )
+    {
+        std::vector<MSG_PANEL_ITEM> msgItems;
+        fp->GetMsgPanelInfo( this, msgItems );
+        SetMsgPanel( msgItems );
+    }
+}
+
+
 bool FOOTPRINT_EDIT_FRAME::IsContentModified() const
 {
-    return GetScreen() && GetScreen()->IsContentModified() && GetBoard() && GetBoard()->GetFirstFootprint();
+    return GetScreen() && GetScreen()->IsContentModified()
+                && GetBoard() && GetBoard()->GetFirstFootprint();
 }
 
 
@@ -365,7 +387,17 @@ void FOOTPRINT_EDIT_FRAME::ToggleSearchTree()
 {
     wxAuiPaneInfo& treePane = m_auimgr.GetPane( m_treePane );
     treePane.Show( !IsSearchTreeShown() );
-    m_auimgr.Update();
+
+    if( IsSearchTreeShown() )
+    {
+        // SetAuiPaneSize also updates m_auimgr
+        SetAuiPaneSize( m_auimgr, treePane, m_editorSettings->m_LibWidth, -1 );
+    }
+    else
+    {
+        m_editorSettings->m_LibWidth = m_treePane->GetSize().x;
+        m_auimgr.Update();
+    }
 }
 
 
@@ -491,12 +523,13 @@ void FOOTPRINT_EDIT_FRAME::restoreLastFootprint()
 
 void FOOTPRINT_EDIT_FRAME::AddFootprintToBoard( FOOTPRINT* aFootprint )
 {
-    m_revertModule.reset( (FOOTPRINT*) aFootprint->Clone() );
+    m_originalFootprintCopy.reset( static_cast<FOOTPRINT*>( aFootprint->Clone() ) );
+    m_originalFootprintCopy->SetParent( nullptr );
 
     m_footprintNameWhenLoaded = aFootprint->GetFPID().GetLibItemName();
 
     PCB_BASE_EDIT_FRAME::AddFootprintToBoard( aFootprint );
-    // Ensure item UUIDs are valide
+    // Ensure item UUIDs are valid
     // ("old" footprints can have null uuids that create issues in fp editor)
     aFootprint->FixUuids();
 
@@ -510,6 +543,10 @@ void FOOTPRINT_EDIT_FRAME::AddFootprintToBoard( FOOTPRINT* aFootprint )
         GetInfoBar()->AddCloseButton();
         GetInfoBar()->ShowMessage( msg, wxICON_INFORMATION );
     }
+    else
+    {
+        GetInfoBar()->Dismiss();
+    }
 
     UpdateMsgPanel();
 }
@@ -593,12 +630,20 @@ void FOOTPRINT_EDIT_FRAME::SaveSettings( APP_SETTINGS_BASE* aCfg )
     cfg->m_AuiPanels.show_layer_manager   = m_show_layer_manager_tools;
     cfg->m_AuiPanels.right_panel_width    = m_appearancePanel->GetSize().x;
     cfg->m_AuiPanels.appearance_panel_tab = m_appearancePanel->GetTabIndex();
-
-    GetSettingsManager()->SaveColorSettings( GetColorSettings(), "board" );
 }
 
 
-COLOR_SETTINGS* FOOTPRINT_EDIT_FRAME::GetColorSettings() const
+
+EDA_ANGLE FOOTPRINT_EDIT_FRAME::GetRotationAngle() const
+{
+    FOOTPRINT_EDITOR_SETTINGS* cfg = const_cast<FOOTPRINT_EDIT_FRAME*>( this )->GetSettings();
+
+    return cfg ? cfg->m_RotationAngle : ANGLE_90;
+}
+
+
+
+COLOR_SETTINGS* FOOTPRINT_EDIT_FRAME::GetColorSettings( bool aForceRefresh ) const
 {
     wxString currentTheme = GetFootprintEditorSettings()->m_ColorTheme;
     return Pgm().GetSettingsManager().GetColorSettings( currentTheme );
@@ -641,7 +686,7 @@ const BOX2I FOOTPRINT_EDIT_FRAME::GetDocumentExtents( bool aIncludeAllVisible )
         else
         {
             BOX2I newFootprintBB( { 0, 0 }, { 0, 0 } );
-            newFootprintBB.Inflate( Millimeter2iu( 12 ) );
+            newFootprintBB.Inflate( pcbIUScale.mmToIU( 12 ) );
             return newFootprintBB;
         }
     }
@@ -708,6 +753,9 @@ bool FOOTPRINT_EDIT_FRAME::canCloseWindow( wxCloseEvent& aEvent )
         }
     }
 
+    // Save footprint tree column widths
+    m_adapter->SaveSettings();
+
     return PCB_BASE_EDIT_FRAME::canCloseWindow( aEvent );
 }
 
@@ -800,6 +848,8 @@ void FOOTPRINT_EDIT_FRAME::ShowChangedLanguage()
     wxAuiPaneInfo& lm_pane_info = m_auimgr.GetPane( m_appearancePanel );
     bool lm_shown = lm_pane_info.IsShown();
     lm_pane_info.Caption( _( "Appearance" ) );
+    wxAuiPaneInfo& sf_pane_info = m_auimgr.GetPane( m_selectionFilterPanel );
+    sf_pane_info.Caption( _( "Selection Filter" ) );
 
     // update the layer manager
     m_appearancePanel->OnLanguageChanged();
@@ -810,6 +860,8 @@ void FOOTPRINT_EDIT_FRAME::ShowChangedLanguage()
     lm_pane_info.Show( lm_shown );
     tree_pane_info.Show( tree_shown );
     m_auimgr.Update();
+
+    UpdateTitle();
 }
 
 
@@ -914,7 +966,7 @@ void FOOTPRINT_EDIT_FRAME::initLibraryTree()
     m_adapter = FP_TREE_SYNCHRONIZING_ADAPTER::Create( this, fpTable );
     auto adapter = static_cast<FP_TREE_SYNCHRONIZING_ADAPTER*>( m_adapter.get() );
 
-    adapter->AddLibraries();
+    adapter->AddLibraries( this );
 }
 
 
@@ -991,21 +1043,6 @@ void FOOTPRINT_EDIT_FRAME::OnDisplayOptionsChanged()
 }
 
 
-void FOOTPRINT_EDIT_FRAME::InstallPreferences( PAGED_DIALOG* aParent,
-                                               PANEL_HOTKEYS_EDITOR* aHotkeysPanel )
-{
-    wxTreebook* book = aParent->GetTreebook();
-
-    book->AddPage( new wxPanel( book ), _( "Footprint Editor" ) );
-    book->AddSubPage( new PANEL_DISPLAY_OPTIONS( this, aParent ), _( "Display Options" ) );
-    book->AddSubPage( new PANEL_EDIT_OPTIONS( this, aParent ), _( "Editing Options" ) );
-    book->AddSubPage( new PANEL_FP_EDITOR_COLOR_SETTINGS( this, book ), _( "Colors" ) );
-    book->AddSubPage( new PANEL_FP_EDITOR_DEFAULTS( this, aParent ), _( "Default Values" ) );
-
-    aHotkeysPanel->AddHotKeys( GetToolManager() );
-}
-
-
 void FOOTPRINT_EDIT_FRAME::setupTools()
 {
     // Create the manager and dispatcher & route draw panel events to the dispatcher
@@ -1035,21 +1072,19 @@ void FOOTPRINT_EDIT_FRAME::setupTools()
     m_toolManager->RegisterTool( new CONVERT_TOOL );
     m_toolManager->RegisterTool( new SCRIPTING_TOOL );
 
-    m_toolManager->GetTool<PCB_SELECTION_TOOL>()->SetIsFootprintEditor( true );
-    m_toolManager->GetTool<EDIT_TOOL>()->SetIsFootprintEditor( true );
-    m_toolManager->GetTool<PAD_TOOL>()->SetIsFootprintEditor( true );
-    m_toolManager->GetTool<DRAWING_TOOL>()->SetIsFootprintEditor( true );
-    m_toolManager->GetTool<PCB_POINT_EDITOR>()->SetIsFootprintEditor( true );
-    m_toolManager->GetTool<PCB_CONTROL>()->SetIsFootprintEditor( true );
-    m_toolManager->GetTool<PCB_PICKER_TOOL>()->SetIsFootprintEditor( true );
-    m_toolManager->GetTool<POSITION_RELATIVE_TOOL>()->SetIsFootprintEditor( true );
-    m_toolManager->GetTool<GROUP_TOOL>()->SetIsFootprintEditor( true );
-    m_toolManager->GetTool<SCRIPTING_TOOL>()->SetIsFootprintEditor( true );
+    for( TOOL_BASE* tool : m_toolManager->Tools() )
+    {
+        if( PCB_TOOL_BASE* pcbTool = dynamic_cast<PCB_TOOL_BASE*>( tool ) )
+            pcbTool->SetIsFootprintEditor( true );
+    }
 
     m_toolManager->GetTool<PCB_VIEWER_TOOLS>()->SetFootprintFrame( true );
     m_toolManager->InitTools();
 
     m_toolManager->InvokeTool( "pcbnew.InteractiveSelection" );
+
+    // Load or reload wizard plugins in case they changed since the last time the frame opened
+    m_toolManager->RunAction( PCB_ACTIONS::pluginsReload, true );
 }
 
 
@@ -1098,23 +1133,12 @@ void FOOTPRINT_EDIT_FRAME::setupUIConditions()
     mgr->SetConditions( ACTIONS::duplicate,              ENABLE( cond.HasItems() ) );
     mgr->SetConditions( ACTIONS::selectAll,              ENABLE( cond.HasItems() ) );
 
-    auto haveAtLeastOneGroupCond =
-            []( const SELECTION& aSel )
-            {
-                for( EDA_ITEM* item : aSel )
-                {
-                    if( item->Type() == PCB_GROUP_T )
-                        return true;
-                }
-
-                return false;
-            };
-
     mgr->SetConditions( PCB_ACTIONS::rotateCw,           ENABLE( cond.HasItems() ) );
     mgr->SetConditions( PCB_ACTIONS::rotateCcw,          ENABLE( cond.HasItems() ) );
-    mgr->SetConditions( PCB_ACTIONS::mirror,             ENABLE( cond.HasItems() ) );
+    mgr->SetConditions( PCB_ACTIONS::mirrorH,            ENABLE( cond.HasItems() ) );
+    mgr->SetConditions( PCB_ACTIONS::mirrorV,            ENABLE( cond.HasItems() ) );
     mgr->SetConditions( PCB_ACTIONS::group,              ENABLE( SELECTION_CONDITIONS::MoreThan( 1 ) ) );
-    mgr->SetConditions( PCB_ACTIONS::ungroup,            ENABLE( haveAtLeastOneGroupCond ) );
+    mgr->SetConditions( PCB_ACTIONS::ungroup,            ENABLE( SELECTION_CONDITIONS::HasType( PCB_GROUP_T ) ) );
 
     mgr->SetConditions( PCB_ACTIONS::padDisplayMode,     CHECK( !cond.PadFillDisplay() ) );
     mgr->SetConditions( PCB_ACTIONS::textOutlines,       CHECK( !cond.TextFillDisplay() ) );
@@ -1123,9 +1147,11 @@ void FOOTPRINT_EDIT_FRAME::setupUIConditions()
     mgr->SetConditions( ACTIONS::zoomTool,               CHECK( cond.CurrentTool( ACTIONS::zoomTool ) ) );
     mgr->SetConditions( ACTIONS::selectionTool,          CHECK( cond.CurrentTool( ACTIONS::selectionTool ) ) );
 
-    mgr->SetConditions( PCB_ACTIONS::checkFootprint,     ENABLE( cond.HasItems() ) );
-    mgr->SetConditions( PCB_ACTIONS::repairFootprint,    ENABLE( cond.HasItems() ) );
-
+    auto constrainedDrawingModeCond =
+            [this]( const SELECTION& )
+            {
+                return GetSettings()->m_Use45Limit;
+            };
 
     auto highContrastCond =
             [this]( const SELECTION& )
@@ -1151,17 +1177,37 @@ void FOOTPRINT_EDIT_FRAME::setupUIConditions()
                 return m_auimgr.GetPane( "LayersManager" ).IsShown();
             };
 
+    mgr->SetConditions( PCB_ACTIONS::toggleHV45Mode,        CHECK( constrainedDrawingModeCond ) );
     mgr->SetConditions( ACTIONS::highContrastMode,          CHECK( highContrastCond ) );
     mgr->SetConditions( PCB_ACTIONS::flipBoard,             CHECK( boardFlippedCond ) );
+    mgr->SetConditions( ACTIONS::toggleBoundingBoxes,       CHECK( cond.BoundingBoxes() ) );
+
     mgr->SetConditions( PCB_ACTIONS::showFootprintTree,     CHECK( footprintTreeCond ) );
     mgr->SetConditions( PCB_ACTIONS::showLayersManager,     CHECK( layerManagerCond ) );
 
     mgr->SetConditions( ACTIONS::print,                     ENABLE( haveFootprintCond ) );
     mgr->SetConditions( PCB_ACTIONS::exportFootprint,       ENABLE( haveFootprintCond ) );
-    mgr->SetConditions( PCB_ACTIONS::footprintProperties,   ENABLE( haveFootprintCond ) );
-    mgr->SetConditions( PCB_ACTIONS::cleanupGraphics,       ENABLE( haveFootprintCond ) );
     mgr->SetConditions( PCB_ACTIONS::placeImportedGraphics, ENABLE( haveFootprintCond ) );
 
+    mgr->SetConditions( PCB_ACTIONS::footprintProperties,   ENABLE( haveFootprintCond ) );
+    mgr->SetConditions( PCB_ACTIONS::editTextAndGraphics,   ENABLE( haveFootprintCond ) );
+    mgr->SetConditions( PCB_ACTIONS::checkFootprint,        ENABLE( haveFootprintCond ) );
+    mgr->SetConditions( PCB_ACTIONS::repairFootprint,       ENABLE( haveFootprintCond ) );
+    mgr->SetConditions( PCB_ACTIONS::cleanupGraphics,       ENABLE( haveFootprintCond ) );
+
+    auto isArcKeepCenterMode = [this]( const SELECTION& )
+    {
+        return GetSettings()->m_ArcEditMode == ARC_EDIT_MODE::KEEP_CENTER_ADJUST_ANGLE_RADIUS;
+    };
+
+    auto isArcKeepEndpointMode = [this]( const SELECTION& )
+    {
+        return GetSettings()->m_ArcEditMode == ARC_EDIT_MODE::KEEP_ENDPOINTS_OR_START_DIRECTION;
+    };
+
+    mgr->SetConditions( PCB_ACTIONS::pointEditorArcKeepCenter,   CHECK( isArcKeepCenterMode ) );
+    mgr->SetConditions( PCB_ACTIONS::pointEditorArcKeepEndpoint, CHECK( isArcKeepEndpointMode ) );
+
 
 // Only enable a tool if the part is edtable
 #define CURRENT_EDIT_TOOL( action ) mgr->SetConditions( action, \
@@ -1176,7 +1222,14 @@ void FOOTPRINT_EDIT_FRAME::setupUIConditions()
     CURRENT_EDIT_TOOL( PCB_ACTIONS::drawArc );
     CURRENT_EDIT_TOOL( PCB_ACTIONS::drawPolygon );
     CURRENT_EDIT_TOOL( PCB_ACTIONS::drawRuleArea );
+    CURRENT_EDIT_TOOL( PCB_ACTIONS::placeImage );
     CURRENT_EDIT_TOOL( PCB_ACTIONS::placeText );
+    CURRENT_EDIT_TOOL( PCB_ACTIONS::drawTextBox );
+    CURRENT_EDIT_TOOL( PCB_ACTIONS::drawAlignedDimension );
+    CURRENT_EDIT_TOOL( PCB_ACTIONS::drawOrthogonalDimension );
+    CURRENT_EDIT_TOOL( PCB_ACTIONS::drawCenterDimension );
+    CURRENT_EDIT_TOOL( PCB_ACTIONS::drawRadialDimension );
+    CURRENT_EDIT_TOOL( PCB_ACTIONS::drawLeader );
     CURRENT_EDIT_TOOL( PCB_ACTIONS::setAnchor );
     CURRENT_EDIT_TOOL( PCB_ACTIONS::gridSetOrigin );
 
@@ -1204,7 +1257,13 @@ void FOOTPRINT_EDIT_FRAME::CommonSettingsChanged( bool aEnvVarsChanged, bool aTe
 {
     PCB_BASE_EDIT_FRAME::CommonSettingsChanged( aEnvVarsChanged, aTextVarsChanged );
 
+    auto cfg = Pgm().GetSettingsManager().GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>();
+    GetGalDisplayOptions().ReadWindowSettings( cfg->m_Window );
+
+    GetBoard()->GetDesignSettings() = cfg->m_DesignSettings;
+
     GetCanvas()->GetView()->UpdateAllLayersColor();
+    GetCanvas()->GetView()->MarkTargetDirty( KIGFX::TARGET_NONCACHED );
     GetCanvas()->ForceRefresh();
 
     UpdateUserInterface();
@@ -1219,13 +1278,11 @@ void FOOTPRINT_EDIT_FRAME::CommonSettingsChanged( bool aEnvVarsChanged, bool aTe
 
 void FOOTPRINT_EDIT_FRAME::OnSaveFootprintAsPng( wxCommandEvent& event )
 {
-    wxString   fullFileName;
-
     LIB_ID id = GetLoadedFPID();
 
     if( id.empty() )
     {
-        wxMessageBox( _( "No footprint selected." ) );
+        DisplayErrorMessage( this, _( "No footprint selected." ) );
         return;
     }
 
