@@ -46,6 +46,7 @@
 #include <env_paths.h>
 #include <paths.h>
 #include <settings/settings_manager.h>
+#include <project/project_file.h>
 #include <footprint_editor_settings.h>
 #include "footprint_viewer_frame.h"
 #include "tools/pad_tool.h"
@@ -363,7 +364,7 @@ void FOOTPRINT_EDIT_FRAME::ExportFootprint( FOOTPRINT* aFootprint )
 
         if( fp == nullptr )
         {
-            wxMessageBox( wxString::Format( _( "Insufficient permissions to write file '%s'." ),
+            DisplayErrorMessage( this, wxString::Format( _( "Insufficient permissions to write file '%s'." ),
                                             dlg.GetPath() ) );
             return;
         }
@@ -570,8 +571,14 @@ bool PCB_BASE_EDIT_FRAME::AddLibrary( const wxString& aFilename, FP_LIB_TABLE* a
 
     if( libName.IsEmpty() )
         return false;
+    IO_MGR::PCB_FILE_T lib_type = IO_MGR::GuessPluginTypeFromLibPath( libPath );
 
-    wxString type = IO_MGR::ShowType( IO_MGR::GuessPluginTypeFromLibPath( libPath ) );
+    wxString type = IO_MGR::ShowType( lib_type );
+
+    // KiCad lib is our default guess.  So it might not have the .pretty extension
+    // In this case, the extension is part of the library name
+    if( lib_type == IO_MGR::KICAD_SEXP && fn.GetExt() != KiCadFootprintLibPathExtension )
+        libName = fn.GetFullName();
 
     // try to use path normalized to an environmental variable or project path
     wxString normalizedPath = NormalizePath( libPath, &Pgm().GetLocalEnvVariables(), &Prj() );
@@ -824,7 +831,17 @@ bool FOOTPRINT_EDIT_FRAME::SaveFootprint( FOOTPRINT* aFootprint )
 
     // Legacy libraries are readable, but modifying legacy format is not allowed
     // So prompt the user if he try to add/replace a footprint in a legacy lib
-    wxString libfullname = tbl->FindRow( libraryName )->GetFullURI();
+    wxString libfullname;
+
+    try
+    {
+        libfullname = tbl->FindRow( libraryName )->GetFullURI();
+    }
+    catch( IO_ERROR& error )
+    {
+        DisplayInfoMessage( this, error.What() );
+        return false;
+    }
 
     if( IO_MGR::GuessPluginTypeFromLibPath( libfullname ) == IO_MGR::LEGACY )
     {
@@ -851,6 +868,39 @@ bool FOOTPRINT_EDIT_FRAME::SaveFootprint( FOOTPRINT* aFootprint )
 }
 
 
+bool FOOTPRINT_EDIT_FRAME::DuplicateFootprint( FOOTPRINT* aFootprint )
+{
+    LIB_ID     fpID = aFootprint->GetFPID();
+    wxString   libraryName = fpID.GetLibNickname();
+    wxString   footprintName = fpID.GetLibItemName();
+
+    // Legacy libraries are readable, but modifying legacy format is not allowed
+    // So prompt the user if he try to add/replace a footprint in a legacy lib
+    wxString libFullName = Prj().PcbFootprintLibs()->FindRow( libraryName )->GetFullURI();
+
+    if( IO_MGR::GuessPluginTypeFromLibPath( libFullName ) == IO_MGR::LEGACY )
+    {
+        DisplayInfoMessage( this, INFO_LEGACY_LIB_WARN_EDIT );
+        return false;
+    }
+
+    FP_LIB_TABLE* tbl = Prj().PcbFootprintLibs();
+    int           i = 1;
+    wxString      newName = footprintName;
+
+    // Append a number to the name until the name is unique in the library.
+    while( tbl->FootprintExists( libraryName, newName ) )
+        newName.Printf( "%s_%d", footprintName, i++ );
+
+    aFootprint->SetFPID( LIB_ID( libraryName, newName ) );
+
+    if( aFootprint->GetValue() == footprintName )
+        aFootprint->SetValue( newName );
+
+    return SaveFootprintInLibrary( aFootprint, libraryName );
+}
+
+
 bool FOOTPRINT_EDIT_FRAME::SaveFootprintInLibrary( FOOTPRINT* aFootprint,
                                                    const wxString& aLibraryName )
 {
@@ -911,10 +961,9 @@ bool FOOTPRINT_EDIT_FRAME::SaveFootprintToBoard( bool aAddNew )
         return false;
     }
 
-    TOOL_MANAGER* pcb_ToolManager = pcbframe ? pcbframe->GetToolManager() : nullptr;
+    TOOL_MANAGER* pcb_ToolManager = pcbframe->GetToolManager();
 
-    if( aAddNew && pcb_ToolManager
-            && pcb_ToolManager->GetTool<BOARD_EDITOR_CONTROL>()->PlacingFootprint() )
+    if( aAddNew && pcb_ToolManager->GetTool<BOARD_EDITOR_CONTROL>()->PlacingFootprint() )
     {
         DisplayError( this, _( "Previous footprint placement still in progress." ) );
         return false;
@@ -976,6 +1025,92 @@ bool FOOTPRINT_EDIT_FRAME::SaveFootprintToBoard( bool aAddNew )
 }
 
 
+static int ID_SAVE_AS_NAME     = 4172;
+static int ID_MAKE_NEW_LIBRARY = 4173;
+
+
+EDA_LIST_DIALOG* FOOTPRINT_EDIT_FRAME::buildSaveAsDialog( const wxString& aFootprintName,
+                                                          const wxString& aLibraryPreselect )
+{
+    COMMON_SETTINGS*           cfg = Pgm().GetCommonSettings();
+    PROJECT_FILE&              project = Kiway().Prj().GetProjectFile();
+    FP_LIB_TABLE*              tbl = Prj().PcbFootprintLibs();
+    std::vector<wxString>      nicknames = tbl->GetLogicalLibs();
+    wxArrayString              headers;
+    std::vector<wxArrayString> itemsToDisplay;
+
+    headers.Add( _( "Nickname" ) );
+    headers.Add( _( "Description" ) );
+
+    for( const wxString& nickname : nicknames )
+    {
+        if( alg::contains( project.m_PinnedFootprintLibs, nickname )
+                || alg::contains( cfg->m_Session.pinned_fp_libs, nickname ) )
+        {
+            wxArrayString item;
+
+            item.Add( LIB_TREE_MODEL_ADAPTER::GetPinningSymbol() + nickname );
+            item.Add( tbl->GetDescription( nickname ) );
+            itemsToDisplay.push_back( item );
+        }
+    }
+
+    for( const wxString& nickname : nicknames )
+    {
+        if( !alg::contains( project.m_PinnedFootprintLibs, nickname )
+                && !alg::contains( cfg->m_Session.pinned_fp_libs, nickname ) )
+        {
+            wxArrayString item;
+
+            item.Add( nickname );
+            item.Add( tbl->GetDescription( nickname ) );
+            itemsToDisplay.push_back( item );
+        }
+    }
+
+    EDA_LIST_DIALOG* dlg = new EDA_LIST_DIALOG( this, _( "Save Footprint As" ), headers,
+                                                itemsToDisplay, aLibraryPreselect, false );
+
+    dlg->SetListLabel( _( "Save in library:" ) );
+    dlg->SetOKLabel( _( "Save" ) );
+
+    wxBoxSizer* bNameSizer = new wxBoxSizer( wxHORIZONTAL );
+
+    wxStaticText* label = new wxStaticText( dlg, wxID_ANY, _( "Name:" ) );
+    bNameSizer->Add( label, 0, wxALIGN_CENTER_VERTICAL|wxTOP|wxBOTTOM|wxLEFT, 5 );
+
+    wxTextCtrl* nameTextCtrl = new wxTextCtrl( dlg, ID_SAVE_AS_NAME, aFootprintName );
+    bNameSizer->Add( nameTextCtrl, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5 );
+
+    wxTextValidator nameValidator( wxFILTER_EXCLUDE_CHAR_LIST );
+    nameValidator.SetCharExcludes( FOOTPRINT::StringLibNameInvalidChars( false ) );
+    nameTextCtrl->SetValidator( nameValidator );
+
+    wxButton* newLibraryButton = new wxButton( dlg, ID_MAKE_NEW_LIBRARY, _( "New Library..." ) );
+    dlg->m_ButtonsSizer->Prepend( 80, 20 );
+    dlg->m_ButtonsSizer->Prepend( newLibraryButton, 0, wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT, 10 );
+
+    dlg->GetSizer()->Prepend( bNameSizer, 0, wxEXPAND|wxTOP|wxLEFT|wxRIGHT, 5 );
+
+    dlg->Bind( wxEVT_BUTTON,
+            [dlg]( wxCommandEvent& )
+            {
+                dlg->EndModal( ID_MAKE_NEW_LIBRARY );
+            }, ID_MAKE_NEW_LIBRARY );
+
+    // Move nameTextCtrl to the head of the tab-order
+    if( dlg->GetChildren().DeleteObject( nameTextCtrl ) )
+        dlg->GetChildren().Insert( nameTextCtrl );
+
+    dlg->SetInitialFocus( nameTextCtrl );
+
+    dlg->Layout();
+    dlg->GetSizer()->Fit( dlg );
+
+    return dlg;
+}
+
+
 bool FOOTPRINT_EDIT_FRAME::SaveFootprintAs( FOOTPRINT* aFootprint )
 {
     if( aFootprint == nullptr )
@@ -988,56 +1123,32 @@ bool FOOTPRINT_EDIT_FRAME::SaveFootprintAs( FOOTPRINT* aFootprint )
     wxString libraryName = aFootprint->GetFPID().GetLibNickname();
     wxString footprintName = aFootprint->GetFPID().GetLibItemName();
     bool     updateValue = aFootprint->GetValue() == footprintName;
+    bool     done = false;
 
-    wxArrayString              headers;
-    std::vector<wxArrayString> itemsToDisplay;
-    std::vector<wxString>      nicknames = tbl->GetLogicalLibs();
+    std::unique_ptr<EDA_LIST_DIALOG> dlg;
 
-    headers.Add( _( "Nickname" ) );
-    headers.Add( _( "Description" ) );
-
-    for( const wxString& nickname : nicknames )
+    while( !done )
     {
-        wxArrayString item;
-        item.Add( nickname );
-        item.Add( tbl->GetDescription( nickname ) );
-        itemsToDisplay.push_back( item );
+        dlg.reset( buildSaveAsDialog( footprintName, libraryName ) );
+
+        int ret = dlg->ShowModal();
+
+        if( ret == wxID_CANCEL )
+        {
+            return false;
+        }
+        else if( ret == wxID_OK )
+        {
+            done = true;
+        }
+        else if( ret == ID_MAKE_NEW_LIBRARY )
+        {
+            wxFileName newLibrary( CreateNewLibrary() );
+            libraryName = newLibrary.GetName();
+        }
     }
 
-    EDA_LIST_DIALOG dlg( this, _( "Save Footprint As" ), headers, itemsToDisplay, libraryName );
-    dlg.SetListLabel( _( "Save in library:" ) );
-    dlg.SetOKLabel( _( "Save" ) );
-
-    wxBoxSizer* bNameSizer = new wxBoxSizer( wxHORIZONTAL );
-
-    wxStaticText* label = new wxStaticText( &dlg, wxID_ANY, _( "Name:" ),
-                                            wxDefaultPosition, wxDefaultSize, 0 );
-    bNameSizer->Add( label, 0, wxALIGN_CENTER_VERTICAL|wxTOP|wxBOTTOM|wxLEFT, 5 );
-
-    wxTextCtrl* nameTextCtrl = new wxTextCtrl( &dlg, wxID_ANY, footprintName,
-                                               wxDefaultPosition, wxDefaultSize, 0 );
-    bNameSizer->Add( nameTextCtrl, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5 );
-
-    wxTextValidator nameValidator( wxFILTER_EXCLUDE_CHAR_LIST );
-    nameValidator.SetCharExcludes( FOOTPRINT::StringLibNameInvalidChars( false ) );
-    nameTextCtrl->SetValidator( nameValidator );
-
-    wxSizer* mainSizer = dlg.GetSizer();
-    mainSizer->Prepend( bNameSizer, 0, wxEXPAND|wxTOP|wxLEFT|wxRIGHT, 5 );
-
-    // Move nameTextCtrl to the head of the tab-order
-    if( dlg.GetChildren().DeleteObject( nameTextCtrl ) )
-        dlg.GetChildren().Insert( nameTextCtrl );
-
-    dlg.SetInitialFocus( nameTextCtrl );
-
-    dlg.Layout();
-    mainSizer->Fit( &dlg );
-
-    if( dlg.ShowModal() != wxID_OK )
-        return false;                   // canceled by user
-
-    libraryName = dlg.GetTextSelection();
+    libraryName = dlg->GetTextSelection();
 
     if( libraryName.IsEmpty() )
     {
@@ -1045,7 +1156,7 @@ bool FOOTPRINT_EDIT_FRAME::SaveFootprintAs( FOOTPRINT* aFootprint )
         return false;
     }
 
-    footprintName = nameTextCtrl->GetValue();
+    footprintName = static_cast<wxTextCtrl*>( dlg->FindWindow( ID_SAVE_AS_NAME ) )->GetValue();
     footprintName.Trim( true );
     footprintName.Trim( false );
 
@@ -1105,7 +1216,7 @@ bool FOOTPRINT_EDIT_FRAME::SaveFootprintAs( FOOTPRINT* aFootprint )
 
 bool FOOTPRINT_EDIT_FRAME::RevertFootprint()
 {
-    if( GetScreen()->IsContentModified() && m_revertModule )
+    if( GetScreen()->IsContentModified() && m_originalFootprintCopy )
     {
         wxString msg = wxString::Format( _( "Revert '%s' to last version saved?" ),
                                          GetLoadedFPID().GetLibItemName().wx_str() );
@@ -1113,7 +1224,7 @@ bool FOOTPRINT_EDIT_FRAME::RevertFootprint()
         if( ConfirmRevertDialog( this, msg ) )
         {
             Clear_Pcb( false );
-            AddFootprintToBoard( static_cast<FOOTPRINT*>( m_revertModule->Clone() ) );
+            AddFootprintToBoard( static_cast<FOOTPRINT*>( m_originalFootprintCopy->Clone() ) );
 
             Zoom_Automatique( false );
 
@@ -1157,14 +1268,9 @@ FOOTPRINT* PCB_BASE_FRAME::CreateNewFootprint( const wxString& aFootprintName, b
 
         switch( footprintType )
         {
-        case 0:
-            footprintTranslated = FP_THROUGH_HOLE;
-            break;
-        case 1:
-            footprintTranslated = FP_SMD;
-            break;
-        default:
-            footprintTranslated = 0;
+        case 0:  footprintTranslated = FP_THROUGH_HOLE; break;
+        case 1:  footprintTranslated = FP_SMD;          break;
+        default: footprintTranslated = 0;               break;
         }
     }
 
@@ -1253,22 +1359,40 @@ wxString PCB_BASE_FRAME::SelectLibrary( const wxString& aNicknameExisting )
     headers.Add( _( "Nickname" ) );
     headers.Add( _( "Description" ) );
 
-    FP_LIB_TABLE*   fptbl = Prj().PcbFootprintLibs();
-
+    COMMON_SETTINGS*             cfg = Pgm().GetCommonSettings();
+    PROJECT_FILE&                project = Kiway().Prj().GetProjectFile();
+    FP_LIB_TABLE*                fptbl = Prj().PcbFootprintLibs();
     std::vector< wxArrayString > itemsToDisplay;
     std::vector< wxString >      nicknames = fptbl->GetLogicalLibs();
 
     for( const wxString& nickname : nicknames )
     {
-        wxArrayString item;
+        if( alg::contains( project.m_PinnedFootprintLibs, nickname )
+                || alg::contains( cfg->m_Session.pinned_fp_libs, nickname ) )
+        {
+            wxArrayString item;
 
-        item.Add( nickname );
-        item.Add( fptbl->GetDescription( nickname ) );
-
-        itemsToDisplay.push_back( item );
+            item.Add( LIB_TREE_MODEL_ADAPTER::GetPinningSymbol() + nickname );
+            item.Add( fptbl->GetDescription( nickname ) );
+            itemsToDisplay.push_back( item );
+        }
     }
 
-    EDA_LIST_DIALOG dlg( this, _( "Select Library" ), headers, itemsToDisplay, aNicknameExisting );
+    for( const wxString& nickname : nicknames )
+    {
+        if( !alg::contains( project.m_PinnedFootprintLibs, nickname )
+                && !alg::contains( cfg->m_Session.pinned_fp_libs, nickname ) )
+        {
+            wxArrayString item;
+
+            item.Add( nickname );
+            item.Add( fptbl->GetDescription( nickname ) );
+            itemsToDisplay.push_back( item );
+        }
+    }
+
+    EDA_LIST_DIALOG dlg( this, _( "Select Library" ), headers, itemsToDisplay, aNicknameExisting,
+                         false );
 
     if( dlg.ShowModal() != wxID_OK )
         return wxEmptyString;
