@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2012-2015 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2008-2016 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 2004-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2004-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -34,12 +34,16 @@
 #include <footprint_viewer_frame.h>
 #include <fp_lib_table.h>
 #include <kiway.h>
+#include <kiway_express.h>
+#include <netlist_reader/pcb_netlist.h>
 #include <widgets/msgpanel.h>
+#include <widgets/wx_listbox.h>
 #include <pcb_draw_panel_gal.h>
 #include <pcb_painter.h>
 #include <pcbnew_id.h>
 #include <footprint_editor_settings.h>
 #include <pgm_base.h>
+#include <project/project_file.h>
 #include <settings/settings_manager.h>
 #include <tool/action_toolbar.h>
 #include <tool/common_control.h>
@@ -56,7 +60,7 @@
 #include <tools/pcb_selection_tool.h>
 #include <tools/board_editor_control.h>
 #include <wildcards_and_files_ext.h>
-#include <wx/listbox.h>
+#include <lib_tree_model_adapter.h>
 #include <wx/srchctrl.h>
 #include <wx/tokenzr.h>
 #include <wx/choice.h>
@@ -69,7 +73,7 @@ using namespace std::placeholders;
 #define PREVIOUS_PART   -1
 
 
-BEGIN_EVENT_TABLE( FOOTPRINT_VIEWER_FRAME, EDA_DRAW_FRAME )
+BEGIN_EVENT_TABLE( FOOTPRINT_VIEWER_FRAME, PCB_BASE_FRAME )
     // Window events
     EVT_SIZE( FOOTPRINT_VIEWER_FRAME::OnSize )
     EVT_ACTIVATE( FOOTPRINT_VIEWER_FRAME::OnActivate )
@@ -83,6 +87,7 @@ BEGIN_EVENT_TABLE( FOOTPRINT_VIEWER_FRAME, EDA_DRAW_FRAME )
     EVT_TOOL( ID_ADD_FOOTPRINT_TO_BOARD, FOOTPRINT_VIEWER_FRAME::AddFootprintToPCB )
     EVT_CHOICE( ID_ON_ZOOM_SELECT, FOOTPRINT_VIEWER_FRAME::OnSelectZoom )
     EVT_CHOICE( ID_ON_GRID_SELECT, FOOTPRINT_VIEWER_FRAME::OnSelectGrid )
+    EVT_MENU( ID_GRID_SETTINGS, FOOTPRINT_VIEWER_FRAME::OnGridSettings )
 
     EVT_UPDATE_UI( ID_ADD_FOOTPRINT_TO_BOARD, FOOTPRINT_VIEWER_FRAME::OnUpdateFootprintButton )
 
@@ -107,18 +112,19 @@ END_EVENT_TABLE()
 FOOTPRINT_VIEWER_FRAME::FOOTPRINT_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent,
                                                 FRAME_T aFrameType ) :
     PCB_BASE_FRAME( aKiway, aParent, aFrameType, _( "Footprint Library Browser" ),
-            wxDefaultPosition, wxDefaultSize,
-            aFrameType == FRAME_FOOTPRINT_VIEWER_MODAL ? ( aParent ? PARENT_STYLE : MODAL_STYLE )
-                                                       : NONMODAL_STYLE,
-            aFrameType == FRAME_FOOTPRINT_VIEWER_MODAL ? FOOTPRINT_VIEWER_FRAME_NAME_MODAL
-                                                       : FOOTPRINT_VIEWER_FRAME_NAME )
+                    wxDefaultPosition, wxDefaultSize,
+                    aFrameType == FRAME_FOOTPRINT_VIEWER_MODAL ? ( aParent ? PARENT_STYLE : MODAL_STYLE )
+                                                               : NONMODAL_STYLE,
+                    aFrameType == FRAME_FOOTPRINT_VIEWER_MODAL ? FOOTPRINT_VIEWER_FRAME_NAME_MODAL
+                                                               : FOOTPRINT_VIEWER_FRAME_NAME ),
+   m_comp( LIB_ID(), wxEmptyString, wxEmptyString, KIID_PATH(), {} )
 {
     wxASSERT( aFrameType == FRAME_FOOTPRINT_VIEWER_MODAL || aFrameType == FRAME_FOOTPRINT_VIEWER );
 
     if( aFrameType == FRAME_FOOTPRINT_VIEWER_MODAL )
         SetModal( true );
 
-    m_aboutTitle = _( "KiCad Footprint Library Viewer" );
+    m_aboutTitle = _HKI( "KiCad Footprint Library Viewer" );
 
     // Force the items to always snap
     m_magneticItems.pads     = MAGNETIC_OPTIONS::CAPTURE_ALWAYS;
@@ -135,16 +141,19 @@ FOOTPRINT_VIEWER_FRAME::FOOTPRINT_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent
     icon.CopyFromBitmap( KiBitmap( BITMAPS::icon_footprint_browser ) );
     SetIcon( icon );
 
+    m_libListWidth = 200;
+    m_fpListWidth = 300;
+
     wxPanel* libPanel = new wxPanel( this );
     wxSizer* libSizer = new wxBoxSizer( wxVERTICAL );
 
     m_libFilter = new wxSearchCtrl( libPanel, ID_MODVIEW_LIB_FILTER, wxEmptyString,
-                                  wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER );
+                                    wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER );
     m_libFilter->SetDescriptiveText( _( "Filter" ) );
     libSizer->Add( m_libFilter, 0, wxEXPAND, 5 );
 
-    m_libList = new wxListBox( libPanel, ID_MODVIEW_LIB_LIST, wxDefaultPosition, wxDefaultSize,
-                               0, nullptr, wxLB_HSCROLL | wxNO_BORDER );
+    m_libList = new WX_LISTBOX( libPanel, ID_MODVIEW_LIB_LIST, wxDefaultPosition, wxDefaultSize,
+                                0, nullptr, wxLB_HSCROLL | wxNO_BORDER );
     libSizer->Add( m_libList, 1, wxEXPAND, 5 );
 
     libPanel->SetSizer( libSizer );
@@ -154,7 +163,7 @@ FOOTPRINT_VIEWER_FRAME::FOOTPRINT_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent
     wxSizer* fpSizer = new wxBoxSizer( wxVERTICAL );
 
     m_fpFilter = new wxSearchCtrl( fpPanel, ID_MODVIEW_FOOTPRINT_FILTER, wxEmptyString,
-                                 wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER );
+                                   wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER );
     m_fpFilter->SetDescriptiveText( _( "Filter" ) );
     m_fpFilter->SetToolTip(
             _( "Filter on footprint name, keywords, description and pad count.\n"
@@ -169,8 +178,8 @@ FOOTPRINT_VIEWER_FRAME::FOOTPRINT_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent
     m_fpFilter->SetMinSize( wxSize( -1, GetTextExtent( wxT( "qb" ) ).y + 10 ) );
 #endif
 
-    m_fpList = new wxListBox( fpPanel, ID_MODVIEW_FOOTPRINT_LIST, wxDefaultPosition, wxDefaultSize,
-                              0, nullptr, wxLB_HSCROLL | wxNO_BORDER );
+    m_fpList = new WX_LISTBOX( fpPanel, ID_MODVIEW_FOOTPRINT_LIST, wxDefaultPosition, wxDefaultSize,
+                               0, nullptr, wxLB_HSCROLL | wxNO_BORDER );
 
     m_fpList->Connect( wxEVT_LEFT_DCLICK, wxMouseEventHandler( FOOTPRINT_VIEWER_FRAME::DClickOnFootprintList ), nullptr, this );
     fpSizer->Add( m_fpList, 1, wxEXPAND, 5 );
@@ -193,11 +202,11 @@ FOOTPRINT_VIEWER_FRAME::FOOTPRINT_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent
     // In viewer, the default net clearance is not known (it depends on the actual board).
     // So we do not show the default clearance, by setting it to 0
     // The footprint or pad specific clearance will be shown
-    GetBoard()->GetDesignSettings().GetDefault()->SetClearance( 0 );
+    GetBoard()->GetDesignSettings().m_NetSettings->m_DefaultNetClass->SetClearance( 0 );
 
     // Don't show the default board solder mask clearance in the footprint viewer.  Only the
     // footprint or pad clearance setting should be shown if it is not 0.
-    GetBoard()->GetDesignSettings().m_SolderMaskMargin = 0;
+    GetBoard()->GetDesignSettings().m_SolderMaskExpansion = 0;
 
     // Ensure all layers and items are visible:
     GetBoard()->SetVisibleAlls();
@@ -207,9 +216,6 @@ FOOTPRINT_VIEWER_FRAME::FOOTPRINT_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent
     LoadSettings( config() );
     GetGalDisplayOptions().m_axesEnabled = true;
 
-    // Call resolveCanvasType after loading settings:
-    resolveCanvasType();
-
     // Create the manager and dispatcher & route draw panel events to the dispatcher
     m_toolManager = new TOOL_MANAGER;
     m_toolManager->SetEnvironment( GetBoard(), drawPanel->GetView(),
@@ -240,6 +246,9 @@ FOOTPRINT_VIEWER_FRAME::FOOTPRINT_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent
     ReCreateLibraryList();
     UpdateTitle();
 
+    // Call resolveCanvasType after loading settings:
+    resolveCanvasType();
+
     // If a footprint was previously loaded, reload it
     if( getCurNickname().size() && getCurFootprintName().size() )
     {
@@ -266,7 +275,7 @@ FOOTPRINT_VIEWER_FRAME::FOOTPRINT_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent
     // Vertical items; layers 1 - 3
     m_auimgr.AddPane( libPanel, EDA_PANE().Palette().Name( "Libraries" ).Left().Layer(2)
                       .CaptionVisible( false ).MinSize( 100, -1 ).BestSize( 200, -1 ) );
-    m_auimgr.AddPane( fpPanel, EDA_PANE().Palette().Name( "Footprints" ).Left().Layer( 1)
+    m_auimgr.AddPane( fpPanel, EDA_PANE().Palette().Name( "Footprints" ).Left().Layer(1)
                       .CaptionVisible( false ).MinSize( 100, -1 ).BestSize( 300, -1 ) );
 
     m_auimgr.AddPane( GetCanvas(), EDA_PANE().Canvas().Name( "DrawFrame" ).Center() );
@@ -274,16 +283,61 @@ FOOTPRINT_VIEWER_FRAME::FOOTPRINT_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent
     // after changing something to the aui manager call Update() to reflect the changes
     m_auimgr.Update();
 
+    if( m_libListWidth > 0 )
+    {
+        wxAuiPaneInfo& treePane = m_auimgr.GetPane( "Libraries" );
+
+        // wxAUI hack: force width by setting MinSize() and then Fixed()
+        // thanks to ZenJu http://trac.wxwidgets.org/ticket/13180
+        treePane.MinSize( m_libListWidth, -1 );
+        treePane.Fixed();
+        m_auimgr.Update();
+
+        // now make it resizable again
+        treePane.Resizable();
+        m_auimgr.Update();
+
+        // Note: DO NOT call m_auimgr.Update() anywhere after this; it will nuke the size
+        // back to minimum.
+        treePane.MinSize( 100, -1 );
+    }
+
+    if( m_fpListWidth > 0 )
+    {
+        wxAuiPaneInfo& treePane = m_auimgr.GetPane( "Footprints" );
+
+        // wxAUI hack: force width by setting MinSize() and then Fixed()
+        // thanks to ZenJu http://trac.wxwidgets.org/ticket/13180
+        treePane.MinSize( m_fpListWidth, -1 );
+        treePane.Fixed();
+        m_auimgr.Update();
+
+        // now make it resizable again
+        treePane.Resizable();
+        m_auimgr.Update();
+
+        // Note: DO NOT call m_auimgr.Update() anywhere after this; it will nuke the size
+        // back to minimum.
+        treePane.MinSize( 100, -1 );
+    }
+
     // The canvas should not steal the focus from the list boxes
     GetCanvas()->SetCanFocus( false );
     GetCanvas()->GetGAL()->SetAxesEnabled( true );
     ActivateGalCanvas();
 
-    // Restore last zoom.  (If auto-zooming we'll adjust when we load the footprint.)
+    // Restore last zoom and auto zoom option.  (If auto-zooming we'll adjust when we load the footprint.)
     PCBNEW_SETTINGS* cfg = GetPcbNewSettings();
     wxASSERT( cfg );
     GetCanvas()->GetView()->SetScale( cfg->m_FootprintViewerZoom );
 
+    wxAuiToolBarItem* toolOpt = m_mainToolBar->FindTool( ID_FPVIEWER_AUTOZOOM_TOOL );
+
+    if( cfg->m_FootprintViewerAutoZoomOnSelect )
+        toolOpt->SetState( wxAUI_BUTTON_STATE_CHECKED );
+    else
+        toolOpt->SetState( 0 );
+
     updateView();
     setupUnits( config() );
 
@@ -316,6 +370,21 @@ SELECTION& FOOTPRINT_VIEWER_FRAME::GetCurrentSelection()
 }
 
 
+void FOOTPRINT_VIEWER_FRAME::UpdateMsgPanel()
+{
+    EDA_DRAW_FRAME::UpdateMsgPanel();
+
+    FOOTPRINT* fp = static_cast<FOOTPRINT*>( GetModel() );
+
+    if( fp )
+    {
+        std::vector<MSG_PANEL_ITEM> msgItems;
+        fp->GetMsgPanelInfo( this, msgItems );
+        SetMsgPanel( msgItems );
+    }
+}
+
+
 void FOOTPRINT_VIEWER_FRAME::setupUIConditions()
 {
     PCB_BASE_FRAME::setupUIConditions();
@@ -328,12 +397,11 @@ void FOOTPRINT_VIEWER_FRAME::setupUIConditions()
 #define ENABLE( x ) ACTION_CONDITIONS().Enable( x )
 #define CHECK( x )  ACTION_CONDITIONS().Check( x )
 
-    mgr->SetConditions( ACTIONS::toggleGrid,             CHECK( cond.GridVisible() ) );
-    mgr->SetConditions( ACTIONS::toggleCursorStyle,      CHECK( cond.FullscreenCursor() ) );
-    mgr->SetConditions( ACTIONS::millimetersUnits,
-                        CHECK( cond.Units( EDA_UNITS::MILLIMETRES ) ) );
-    mgr->SetConditions( ACTIONS::inchesUnits,            CHECK( cond.Units( EDA_UNITS::INCHES ) ) );
-    mgr->SetConditions( ACTIONS::milsUnits,              CHECK( cond.Units( EDA_UNITS::MILS ) ) );
+    mgr->SetConditions( ACTIONS::toggleGrid,        CHECK( cond.GridVisible() ) );
+    mgr->SetConditions( ACTIONS::toggleCursorStyle, CHECK( cond.FullscreenCursor() ) );
+    mgr->SetConditions( ACTIONS::millimetersUnits,  CHECK( cond.Units( EDA_UNITS::MILLIMETRES ) ) );
+    mgr->SetConditions( ACTIONS::inchesUnits,       CHECK( cond.Units( EDA_UNITS::INCHES ) ) );
+    mgr->SetConditions( ACTIONS::milsUnits,         CHECK( cond.Units( EDA_UNITS::MILS ) ) );
 
 
     mgr->SetConditions( ACTIONS::zoomTool,
@@ -347,6 +415,7 @@ void FOOTPRINT_VIEWER_FRAME::setupUIConditions()
     mgr->SetConditions( PCB_ACTIONS::padDisplayMode,     CHECK( !cond.PadFillDisplay() ) );
     mgr->SetConditions( PCB_ACTIONS::textOutlines,       CHECK( !cond.TextFillDisplay() ) );
     mgr->SetConditions( PCB_ACTIONS::graphicsOutlines,   CHECK( !cond.GraphicsFillDisplay() ) );
+    mgr->SetConditions( ACTIONS::toggleBoundingBoxes,    CHECK( cond.BoundingBoxes() ) );
 
 #undef ENABLE
 #undef CHECK
@@ -391,32 +460,54 @@ void FOOTPRINT_VIEWER_FRAME::ReCreateLibraryList()
 {
     m_libList->Clear();
 
+    COMMON_SETTINGS*      cfg = Pgm().GetCommonSettings();
+    PROJECT_FILE&         project = Kiway().Prj().GetProjectFile();
     std::vector<wxString> nicknames = Prj().PcbFootprintLibs()->GetLogicalLibs();
-    std::set<wxString>    excludes;
+    std::vector<wxString> pinnedMatches;
+    std::vector<wxString> otherMatches;
 
-    if( !m_libFilter->GetValue().IsEmpty() )
+    auto process =
+            [&]( const wxString& aNickname )
+            {
+                if( alg::contains( project.m_PinnedFootprintLibs, aNickname )
+                        || alg::contains( cfg->m_Session.pinned_fp_libs, aNickname ) )
+                {
+                    pinnedMatches.push_back( aNickname );
+                }
+                else
+                {
+                    otherMatches.push_back( aNickname );
+                }
+            };
+
+    if( m_libFilter->GetValue().IsEmpty() )
+    {
+        for( const wxString& nickname : nicknames )
+            process( nickname );
+    }
+    else
     {
         wxStringTokenizer tokenizer( m_libFilter->GetValue() );
 
         while( tokenizer.HasMoreTokens() )
         {
             const wxString       term = tokenizer.GetNextToken().Lower();
-            EDA_COMBINED_MATCHER matcher( term );
+            EDA_COMBINED_MATCHER matcher( term, CTX_LIBITEM );
             int                  matches, position;
 
             for( const wxString& nickname : nicknames )
             {
-                if( !matcher.Find( nickname.Lower(), matches, position ) )
-                    excludes.insert( nickname );
+                if( matcher.Find( nickname.Lower(), matches, position ) )
+                    process( nickname );
             }
         }
     }
 
-    for( const wxString& nickname : nicknames )
-    {
-        if( !excludes.count( nickname ) )
-            m_libList->Append( nickname );
-    }
+    for( const wxString& nickname : pinnedMatches )
+        m_libList->Append( LIB_TREE_MODEL_ADAPTER::GetPinningSymbol() + nickname );
+
+    for( const wxString& nickname : otherMatches )
+        m_libList->Append( nickname );
 
     // Search for a previous selection:
     int index =  m_libList->FindString( getCurNickname(), true );
@@ -478,7 +569,7 @@ void FOOTPRINT_VIEWER_FRAME::ReCreateFootprintList()
         while( tokenizer.HasMoreTokens() )
         {
             const wxString       term = tokenizer.GetNextToken().Lower();
-            EDA_COMBINED_MATCHER matcher( term );
+            EDA_COMBINED_MATCHER matcher( term, CTX_LIBITEM );
             int                  matches, position;
 
             for( const std::unique_ptr<FOOTPRINT_INFO>& footprint : fp_info_list->GetList() )
@@ -588,7 +679,7 @@ void FOOTPRINT_VIEWER_FRAME::OnCharHook( wxKeyEvent& aEvent )
 }
 
 
-void FOOTPRINT_VIEWER_FRAME::selectPrev( wxListBox* aListBox )
+void FOOTPRINT_VIEWER_FRAME::selectPrev( WX_LISTBOX* aListBox )
 {
     int prev = aListBox->GetSelection() - 1;
 
@@ -607,7 +698,7 @@ void FOOTPRINT_VIEWER_FRAME::selectPrev( wxListBox* aListBox )
 }
 
 
-void FOOTPRINT_VIEWER_FRAME::selectNext( wxListBox* aListBox )
+void FOOTPRINT_VIEWER_FRAME::selectNext( WX_LISTBOX* aListBox )
 {
     int next = aListBox->GetSelection() + 1;
 
@@ -633,7 +724,7 @@ void FOOTPRINT_VIEWER_FRAME::ClickOnLibList( wxCommandEvent& aEvent )
     if( ii < 0 )
         return;
 
-    wxString name = m_libList->GetString( ii );
+    wxString name = m_libList->GetBaseString( ii );
 
     if( getCurNickname() == name )
         return;
@@ -655,7 +746,7 @@ void FOOTPRINT_VIEWER_FRAME::ClickOnFootprintList( wxCommandEvent& aEvent )
     if( ii < 0 )
         return;
 
-    wxString name = m_fpList->GetString( ii );
+    wxString name = m_fpList->GetBaseString( ii );
 
     if( getCurFootprintName().CmpNoCase( name ) != 0 )
     {
@@ -665,24 +756,31 @@ void FOOTPRINT_VIEWER_FRAME::ClickOnFootprintList( wxCommandEvent& aEvent )
         GetToolManager()->ResetTools( TOOL_BASE::MODEL_RELOAD );
 
         GetBoard()->DeleteAllFootprints();
+        GetBoard()->GetNetInfo().RemoveUnusedNets();
 
         LIB_ID id;
         id.SetLibNickname( getCurNickname() );
         id.SetLibItemName( getCurFootprintName() );
 
+        FOOTPRINT* footprint = nullptr;
+
         try
         {
-            GetBoard()->Add( loadFootprint( id ) );
+            footprint = loadFootprint( id );
         }
         catch( const IO_ERROR& ioe )
         {
-            wxString msg =
-                    wxString::Format( _( "Could not load footprint '%s' from library '%s'."
-                                         "\n\n%s" ),
-                                      getCurFootprintName(), getCurNickname(), ioe.Problem() );
+            wxString msg = wxString::Format( _( "Could not load footprint '%s' from library '%s'."
+                                             "\n\n%s" ),
+                                             getCurFootprintName(),
+                                             getCurNickname(),
+                                             ioe.Problem() );
             DisplayError( this, msg );
         }
 
+        if( footprint )
+            displayFootprint( footprint );
+
         UpdateTitle();
 
         updateView();
@@ -693,6 +791,25 @@ void FOOTPRINT_VIEWER_FRAME::ClickOnFootprintList( wxCommandEvent& aEvent )
 }
 
 
+void FOOTPRINT_VIEWER_FRAME::displayFootprint( FOOTPRINT* aFootprint )
+{
+    for( PAD* pad : aFootprint->Pads() )
+    {
+        const COMPONENT_NET& net = m_comp.GetNet( pad->GetNumber() );
+
+        if( !net.GetPinFunction().IsEmpty() )
+        {
+            NETINFO_ITEM* netinfo = new NETINFO_ITEM( GetBoard() );
+            netinfo->SetNetname( net.GetPinFunction() );
+            GetBoard()->Add( netinfo );
+            pad->SetNet( netinfo );
+        }
+    }
+
+    GetBoard()->Add( aFootprint );
+}
+
+
 void FOOTPRINT_VIEWER_FRAME::DClickOnFootprintList( wxMouseEvent& aEvent )
 {
     wxCommandEvent evt;
@@ -716,7 +833,8 @@ void FOOTPRINT_VIEWER_FRAME::AddFootprintToPCB( wxCommandEvent& aEvent )
     }
     else if( GetBoard()->GetFirstFootprint() )
     {
-        PCB_EDIT_FRAME* pcbframe = (PCB_EDIT_FRAME*) Kiway().Player( FRAME_PCB_EDITOR, false );
+        PCB_EDIT_FRAME*  pcbframe = (PCB_EDIT_FRAME*) Kiway().Player( FRAME_PCB_EDITOR, false );
+        PCBNEW_SETTINGS* cfg = pcbframe->GetPcbNewSettings();
 
         if( pcbframe == nullptr )      // happens when the board editor is not active (or closed)
         {
@@ -732,6 +850,11 @@ void FOOTPRINT_VIEWER_FRAME::AddFootprintToPCB( wxCommandEvent& aEvent )
             return;
         }
 
+        wxWindow* blocking_dialog = pcbframe->Kiway().GetBlockingDialog();
+
+        if( blocking_dialog )
+            blocking_dialog->Close( true );
+
         toolMgr->RunAction( PCB_ACTIONS::selectionClear, true );
         BOARD_COMMIT commit( pcbframe );
 
@@ -744,7 +867,7 @@ void FOOTPRINT_VIEWER_FRAME::AddFootprintToPCB( wxCommandEvent& aEvent )
         for( PAD* pad : newFootprint->Pads() )
         {
             // Set the pads ratsnest settings to the global settings
-            pad->SetLocalRatsnestVisible( pcbframe->GetDisplayOptions().m_ShowGlobalRatsnest );
+            pad->SetLocalRatsnestVisible( cfg->m_Display.m_ShowGlobalRatsnest );
 
             // Pads in the library all have orphaned nets.  Replace with Default.
             pad->SetNetCode( 0 );
@@ -753,8 +876,7 @@ void FOOTPRINT_VIEWER_FRAME::AddFootprintToPCB( wxCommandEvent& aEvent )
         // Put it on FRONT layer,
         // (Can be stored flipped if the lib is an archive built from a board)
         if( newFootprint->IsFlipped() )
-            newFootprint->Flip( newFootprint->GetPosition(),
-                                pcbframe->Settings().m_FlipLeftRight );
+            newFootprint->Flip( newFootprint->GetPosition(), cfg->m_FlipLeftRight );
 
         KIGFX::VIEW_CONTROLS* viewControls = pcbframe->GetCanvas()->GetViewControls();
         VECTOR2D              cursorPos = viewControls->GetCursorPosition();
@@ -768,7 +890,7 @@ void FOOTPRINT_VIEWER_FRAME::AddFootprintToPCB( wxCommandEvent& aEvent )
         commit.Push( wxT( "Insert footprint" ) );
 
         pcbframe->Raise();
-        toolMgr->RunAction( PCB_ACTIONS::placeFootprint, true, newFootprint );
+        toolMgr->RunAction( PCB_ACTIONS::placeFootprint, false, newFootprint );
 
         newFootprint->ClearFlags();
     }
@@ -789,6 +911,18 @@ void FOOTPRINT_VIEWER_FRAME::LoadSettings( APP_SETTINGS_BASE* aCfg )
     auto fpedit = Pgm().GetSettingsManager().GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>();
     m_displayOptions = fpedit->m_Display;
     GetGalDisplayOptions().ReadWindowSettings( fpedit->m_Window );
+
+    m_libListWidth = cfg->m_FootprintViewerLibListWidth;
+    m_fpListWidth = cfg->m_FootprintViewerFPListWidth;
+
+    // Set parameters to a reasonable value.
+    int maxWidth = cfg->m_FootprintViewer.state.size_x - 80;
+
+    if( m_libListWidth + m_fpListWidth > maxWidth )
+    {
+        m_libListWidth = maxWidth * ( m_libListWidth / ( m_libListWidth + m_fpListWidth ) );
+        m_fpListWidth = maxWidth - m_libListWidth;
+    }
 }
 
 
@@ -804,6 +938,18 @@ void FOOTPRINT_VIEWER_FRAME::SaveSettings( APP_SETTINGS_BASE* aCfg )
 
     if( GetCanvas() && GetCanvas()->GetView() )
         cfg->m_FootprintViewerZoom = GetCanvas()->GetView()->GetScale();
+
+    wxAuiToolBarItem* toolOpt = m_mainToolBar->FindTool( ID_FPVIEWER_AUTOZOOM_TOOL );
+    cfg->m_FootprintViewerAutoZoomOnSelect = ( toolOpt->GetState() & wxAUI_BUTTON_STATE_CHECKED );
+
+    if( m_libListWidth && m_libList )
+        m_libListWidth = m_libList->GetSize().x;
+
+    m_fpListWidth = m_fpList->GetSize().x;
+
+    cfg->m_FootprintViewerLibListWidth = m_libListWidth;
+    cfg->m_FootprintViewerFPListWidth = m_fpListWidth;
+
 }
 
 
@@ -816,7 +962,7 @@ WINDOW_SETTINGS* FOOTPRINT_VIEWER_FRAME::GetWindowSettings( APP_SETTINGS_BASE* a
 }
 
 
-COLOR_SETTINGS* FOOTPRINT_VIEWER_FRAME::GetColorSettings() const
+COLOR_SETTINGS* FOOTPRINT_VIEWER_FRAME::GetColorSettings( bool aForceRefresh ) const
 {
     auto* settings = Pgm().GetSettingsManager().GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>();
 
@@ -831,6 +977,8 @@ void FOOTPRINT_VIEWER_FRAME::CommonSettingsChanged( bool aEnvVarsChanged, bool a
 {
     PCB_BASE_FRAME::CommonSettingsChanged( aEnvVarsChanged, aTextVarsChanged );
 
+    GetCanvas()->ForceRefresh();
+
     if( aEnvVarsChanged )
         ReCreateLibraryList();
 }
@@ -874,7 +1022,7 @@ void FOOTPRINT_VIEWER_FRAME::OnActivate( wxActivateEvent& event )
         {
             for( unsigned ii = 0;  ii < libNicknames.size();  ii++ )
             {
-                if( libNicknames[ii] != m_libList->GetString( ii ) )
+                if( libNicknames[ii] != m_libList->GetBaseString( ii ) )
                 {
                     stale = true;
                     break;
@@ -899,6 +1047,52 @@ void FOOTPRINT_VIEWER_FRAME::OnUpdateFootprintButton( wxUpdateUIEvent& aEvent )
 }
 
 
+void FOOTPRINT_VIEWER_FRAME::KiwayMailIn( KIWAY_EXPRESS& mail )
+{
+    const std::string& payload = mail.GetPayload();
+
+    switch( mail.Command() )
+    {
+    case MAIL_SYMBOL_NETLIST:
+    {
+        /*
+         * Symbol netlist format:
+         *   library:footprint
+         *   reference
+         *   value
+         *   pinName,netName,pinFunction,pinType
+         *   pinName,netName,pinFunction,pinType
+         *   ...
+         */
+        std::vector<std::string> strings = split( payload, "\r" );
+        LIB_ID libid;
+
+        if( strings.size() >= 3 )
+        {
+            libid.Parse( strings[0] );
+
+            m_comp.SetFPID( libid );
+            m_comp.SetReference( strings[1] );
+            m_comp.SetValue( strings[2] );
+
+            m_comp.ClearNets();
+
+            for( size_t ii = 3; ii < strings.size(); ++ii )
+            {
+                std::vector<std::string> pinData = split( strings[ii], "," );
+                m_comp.AddNet( pinData[0], pinData[1], pinData[2], pinData[3] );
+            }
+        }
+
+        break;
+    }
+
+    default:
+        ;
+    }
+}
+
+
 bool FOOTPRINT_VIEWER_FRAME::ShowModal( wxString* aFootprint, wxWindow* aParent )
 {
     if( aFootprint && !aFootprint->IsEmpty() )
@@ -984,17 +1178,20 @@ void FOOTPRINT_VIEWER_FRAME::OnIterateFootprintList( wxCommandEvent& event )
 void FOOTPRINT_VIEWER_FRAME::UpdateTitle()
 {
     wxString title;
-    wxString path;
 
     if( !getCurNickname().IsEmpty() )
     {
-        title = getCurNickname();
+        try
+        {
+            FP_LIB_TABLE* libtable = Prj().PcbFootprintLibs();
+            const LIB_TABLE_ROW* row = libtable->FindRow( getCurNickname() );
 
-        FP_LIB_TABLE* libtable = Prj().PcbFootprintLibs();
-        const LIB_TABLE_ROW* row = libtable->FindRow( getCurNickname() );
-
-        if( row )
-            title += wxT( " \u2014 " ) + row->GetFullURI( true );
+            title = getCurNickname() + wxT( " \u2014 " ) + row->GetFullURI( true );
+        }
+        catch( ... )
+        {
+            title = _( "[no library selected]" );
+        }
     }
     else
     {
@@ -1031,19 +1228,19 @@ void FOOTPRINT_VIEWER_FRAME::SelectAndViewFootprint( int aMode )
         m_fpList->SetSelection( selection );
         m_fpList->EnsureVisible( selection );
 
-        setCurFootprintName( m_fpList->GetString((unsigned) selection ) );
+        setCurFootprintName( m_fpList->GetBaseString( selection ) );
 
         // Delete the current footprint
         GetBoard()->DeleteAllFootprints();
+        GetBoard()->GetNetInfo().RemoveUnusedNets();
 
         FOOTPRINT* footprint = Prj().PcbFootprintLibs()->FootprintLoad( getCurNickname(),
                                                                         getCurFootprintName() );
 
         if( footprint )
-            GetBoard()->Add( footprint, ADD_MODE::APPEND );
+            displayFootprint( footprint );
 
         Update3DView( true, true );
-
         updateView();
     }
 
@@ -1060,7 +1257,9 @@ void FOOTPRINT_VIEWER_FRAME::updateView()
 
     m_toolManager->ResetTools( TOOL_BASE::MODEL_RELOAD );
 
-    if( m_zoomSelectBox->GetSelection() == 0 )
+    wxAuiToolBarItem* toolOpt = m_mainToolBar->FindTool( ID_FPVIEWER_AUTOZOOM_TOOL );
+
+    if( toolOpt->GetState() & wxAUI_BUTTON_STATE_CHECKED )
         m_toolManager->RunAction( ACTIONS::zoomFitScreen, true );
     else
         m_toolManager->RunAction( ACTIONS::centerContents, true );
