@@ -4,7 +4,8 @@
  * Copyright (C) 2018 Jean-Pierre Charras, jean-pierre.charras@ujf-grenoble.fr
  * Copyright (C) 2012 SoftPLC Corporation, Dick Hollenbeck <dick@softplc.com>
  * Copyright (C) 2011 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2022 CERN
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -40,6 +41,7 @@
 #include <pgm_base.h>
 #include <3d_viewer/eda_3d_viewer_frame.h>          // To include VIEWER3D_FRAMENAME
 #include <footprint_editor_settings.h>
+#include <pcbnew_settings.h>
 #include <fp_lib_table.h>
 #include <pcbnew_id.h>
 #include <board.h>
@@ -51,7 +53,7 @@
 
 #include <pcb_painter.h>
 #include <settings/settings_manager.h>
-#include <pcbnew_settings.h>
+#include <settings/cvpcb_settings.h>
 #include <tool/tool_manager.h>
 #include <tool/tool_dispatcher.h>
 #include <tools/pcb_actions.h>
@@ -60,26 +62,37 @@
 #include <zoom_defines.h>
 #include <ratsnest/ratsnest_view_item.h>
 
+#if defined( KICAD_USE_3DCONNEXION )
+#include <navlib/nl_pcbnew_plugin.h>
+#endif
+
+using KIGFX::RENDER_SETTINGS;
+using KIGFX::PCB_RENDER_SETTINGS;
 
 wxDEFINE_EVENT( BOARD_CHANGED, wxCommandEvent );
 
 PCB_BASE_FRAME::PCB_BASE_FRAME( KIWAY* aKiway, wxWindow* aParent, FRAME_T aFrameType,
                                 const wxString& aTitle, const wxPoint& aPos, const wxSize& aSize,
-                                long aStyle, const wxString & aFrameName ) :
-        EDA_DRAW_FRAME( aKiway, aParent, aFrameType, aTitle, aPos, aSize, aStyle, aFrameName ),
+                                long aStyle, const wxString& aFrameName ) :
+        EDA_DRAW_FRAME( aKiway, aParent, aFrameType, aTitle, aPos, aSize, aStyle, aFrameName, pcbIUScale ),
         m_pcb( nullptr ),
-        m_originTransforms( *this )
+        m_originTransforms( *this ),
+        m_spaceMouse( nullptr )
 {
-    m_settings = static_cast<PCBNEW_SETTINGS*>( Kiface().KifaceSettings() );
 }
 
 
 PCB_BASE_FRAME::~PCB_BASE_FRAME()
 {
+#if defined( KICAD_USE_3DCONNEXION )
+    delete m_spaceMouse;
+#endif
+
     // Ensure m_canvasType is up to date, to save it in config
     m_canvasType = GetCanvas()->GetBackend();
 
     delete m_pcb;
+    m_pcb = nullptr;
 }
 
 
@@ -92,10 +105,40 @@ bool PCB_BASE_FRAME::canCloseWindow( wxCloseEvent& aEvent )
     if( viewer3D )
         viewer3D->Close( true );
 
+    // Similarly, wxConvBrokenFileNames uses some statically allocated variables that make it
+    // crash when run later from a d'tor.
+    Prj().Cleanup3DCache();
+
     return true;
 }
 
 
+void PCB_BASE_FRAME::handleActivateEvent( wxActivateEvent& aEvent )
+{
+    EDA_DRAW_FRAME::handleActivateEvent( aEvent );
+
+#if defined( KICAD_USE_3DCONNEXION )
+    if( m_spaceMouse != nullptr )
+    {
+        m_spaceMouse->SetFocus( aEvent.GetActive() );
+    }
+#endif
+}
+
+
+void PCB_BASE_FRAME::handleIconizeEvent( wxIconizeEvent& aEvent )
+{
+    EDA_DRAW_FRAME::handleIconizeEvent( aEvent );
+
+#if defined( KICAD_USE_3DCONNEXION )
+    if( m_spaceMouse != nullptr && aEvent.IsIconized() )
+    {
+        m_spaceMouse->SetFocus( false );
+    }
+#endif
+}
+
+
 EDA_3D_VIEWER_FRAME* PCB_BASE_FRAME::Get3DViewerFrame()
 {
     wxWindow* frame = FindWindowByName( QUALIFIED_VIEWER3D_FRAMENAME( this ) );
@@ -168,6 +211,17 @@ void PCB_BASE_FRAME::SetBoard( BOARD* aBoard, PROGRESS_REPORTER* aReporter )
         delete m_pcb;
         m_pcb = aBoard;
 
+        if( GetBoard() && GetCanvas() )
+        {
+            RENDER_SETTINGS* rs = GetCanvas()->GetView()->GetPainter()->GetSettings();
+
+            if( rs )
+            {
+                rs->SetDashLengthRatio( GetBoard()->GetPlotOptions().GetDashedLineDashRatio() );
+                rs->SetGapLengthRatio( GetBoard()->GetPlotOptions().GetDashedLineGapRatio() );
+            }
+        }
+
         wxCommandEvent e( BOARD_CHANGED );
         ProcessEventLocally( e );
     }
@@ -186,15 +240,53 @@ void PCB_BASE_FRAME::AddFootprintToBoard( FOOTPRINT* aFootprint )
         // Put it on FRONT layer (note that it might be stored flipped if the lib is an archive
         // built from a board)
         if( aFootprint->IsFlipped() )
-            aFootprint->Flip( aFootprint->GetPosition(), m_settings->m_FlipLeftRight );
+            aFootprint->Flip( aFootprint->GetPosition(), GetPcbNewSettings()->m_FlipLeftRight );
 
         // Place it in orientation 0 even if it is not saved with orientation 0 in lib (note that
         // it might be stored in another orientation if the lib is an archive built from a board)
-        aFootprint->SetOrientation( 0 );
+        aFootprint->SetOrientation( ANGLE_0 );
+
+        UpdateUserUnits( aFootprint );
     }
 }
 
 
+void PCB_BASE_FRAME::UpdateUserUnits( BOARD_ITEM* aItem, bool* aSelectedItemsModified )
+{
+    EDA_UNITS    units = GetUserUnits();
+    KIGFX::VIEW* view  = GetCanvas()->GetView();
+
+    INSPECTOR_FUNC inspector =
+            [&]( EDA_ITEM* descendant, void* aTestData )
+            {
+                PCB_DIMENSION_BASE* dimension = static_cast<PCB_DIMENSION_BASE*>( descendant );
+
+                if( dimension->GetUnitsMode() == DIM_UNITS_MODE::AUTOMATIC )
+                {
+                    dimension->SetUnits( units );
+                    dimension->Update();
+
+                    if( aSelectedItemsModified && dimension->IsSelected() )
+                        *aSelectedItemsModified = true;
+
+                    view->Update( dimension );
+                }
+
+                return INSPECT_RESULT::CONTINUE;
+            };
+
+    aItem->Visit( inspector, nullptr, { PCB_DIM_ALIGNED_T,
+                                        PCB_DIM_LEADER_T,
+                                        PCB_DIM_ORTHOGONAL_T,
+                                        PCB_DIM_CENTER_T,
+                                        PCB_DIM_RADIAL_T,PCB_FP_DIM_ALIGNED_T,
+                                        PCB_FP_DIM_LEADER_T,
+                                        PCB_FP_DIM_ORTHOGONAL_T,
+                                        PCB_FP_DIM_CENTER_T,
+                                        PCB_FP_DIM_RADIAL_T } );
+}
+
+
 EDA_ITEM* PCB_BASE_FRAME::GetItem( const KIID& aId ) const
 {
     return GetBoard()->GetItem( aId );
@@ -268,7 +360,7 @@ void PCB_BASE_FRAME::FocusOnItems( std::vector<BOARD_ITEM*> aItems, PCB_LAYER_ID
     if( aItems.empty() )
         return;
 
-    wxPoint        focusPt;
+    VECTOR2I       focusPt;
     KIGFX::VIEW*   view = GetCanvas()->GetView();
     SHAPE_POLY_SET viewportPoly( view->GetViewport() );
 
@@ -349,42 +441,49 @@ void PCB_BASE_FRAME::FocusOnItems( std::vector<BOARD_ITEM*> aItems, PCB_LAYER_ID
                 GetCanvas()->Refresh();
                 return;
 
-        case PCB_SHAPE_T:
-        case PCB_TEXT_T:
-        case PCB_FP_TEXT_T:
-        case PCB_FP_SHAPE_T:
-        case PCB_FP_ZONE_T:
-        case PCB_TRACE_T:
-        case PCB_ARC_T:
-        case PCB_DIMENSION_T:
-        case PCB_DIM_ALIGNED_T:
-        case PCB_DIM_LEADER_T:
-        case PCB_DIM_CENTER_T:
-        case PCB_DIM_ORTHOGONAL_T:
-            item->TransformShapeWithClearanceToPolygon( itemPoly, aLayer, 0, Millimeter2iu( 0.1 ),
-                                                        ERROR_INSIDE );
-            break;
+            case PCB_SHAPE_T:
+            case PCB_TEXT_T:
+            case PCB_TEXTBOX_T:
+            case PCB_FP_TEXT_T:
+            case PCB_FP_TEXTBOX_T:
+            case PCB_FP_SHAPE_T:
+            case PCB_FP_ZONE_T:
+            case PCB_TRACE_T:
+            case PCB_ARC_T:
+            case PCB_DIM_ALIGNED_T:
+            case PCB_DIM_LEADER_T:
+            case PCB_DIM_CENTER_T:
+            case PCB_DIM_RADIAL_T:
+            case PCB_DIM_ORTHOGONAL_T:
+            case PCB_FP_DIM_ALIGNED_T:
+            case PCB_FP_DIM_LEADER_T:
+            case PCB_FP_DIM_CENTER_T:
+            case PCB_FP_DIM_RADIAL_T:
+            case PCB_FP_DIM_ORTHOGONAL_T:
+                item->TransformShapeToPolygon( itemPoly, aLayer, 0, pcbIUScale.mmToIU( 0.1 ),
+                                               ERROR_INSIDE );
+                break;
 
-        case PCB_ZONE_T:
-        {
-            ZONE* zone = static_cast<ZONE*>( item );
-            #if 0
-            // Using the filled area shapes to find a Focus point can give good results, but
-            // unfortunately the calculations are highly time consuming, even for not very
-            // large areas (can be easily a few minutes for large areas).
-            // so we used only the zone outline that usually do not have too many vertices.
-            zone->TransformShapeWithClearanceToPolygon( itemPoly, aLayer, 0, Millimeter2iu( 0.1 ),
-                                                        ERROR_INSIDE );
+            case PCB_ZONE_T:
+            {
+                ZONE* zone = static_cast<ZONE*>( item );
+                #if 0
+                // Using the filled area shapes to find a Focus point can give good results, but
+                // unfortunately the calculations are highly time consuming, even for not very
+                // large areas (can be easily a few minutes for large areas).
+                // so we used only the zone outline that usually do not have too many vertices.
+                zone->TransformShapeToPolygon( itemPoly, aLayer, 0, pcbIUScale.mmToIU( 0.1 ),
+                                               ERROR_INSIDE );
 
-            if( itemPoly.IsEmpty() )
+                if( itemPoly.IsEmpty() )
+                    itemPoly = *zone->Outline();
+                #else
+                // much faster calculation time when using only the zone outlines
                 itemPoly = *zone->Outline();
-            #else
-            // much faster calculation time when using only the zone outlines
-            itemPoly = *zone->Outline();
-            #endif
+                #endif
 
-            break;
-        }
+                break;
+            }
 
             default:
             {
@@ -424,7 +523,7 @@ void PCB_BASE_FRAME::FocusOnItems( std::vector<BOARD_ITEM*> aItems, PCB_LAYER_ID
 
     while( !itemPoly.IsEmpty() )
     {
-        focusPt = (wxPoint) itemPoly.BBox().Centre();
+        focusPt = itemPoly.BBox().Centre();
 
         try
         {
@@ -444,12 +543,35 @@ void PCB_BASE_FRAME::FocusOnItems( std::vector<BOARD_ITEM*> aItems, PCB_LAYER_ID
 }
 
 
+void PCB_BASE_FRAME::HideSolderMask()
+{
+    KIGFX::PCB_VIEW* view = GetCanvas()->GetView();
+
+    if( view && GetBoard()->m_SolderMask && view->HasItem( GetBoard()->m_SolderMask ) )
+        view->Remove( GetBoard()->m_SolderMask );
+}
+
+
+void PCB_BASE_FRAME::ShowSolderMask()
+{
+    KIGFX::PCB_VIEW* view = GetCanvas()->GetView();
+
+    if( view && GetBoard()->m_SolderMask )
+    {
+        if( view->HasItem( GetBoard()->m_SolderMask ) )
+            view->Remove( GetBoard()->m_SolderMask );
+
+        view->Add( GetBoard()->m_SolderMask );
+    }
+}
+
+
 void PCB_BASE_FRAME::SetPageSettings( const PAGE_INFO& aPageSettings )
 {
     m_pcb->SetPageSettings( aPageSettings );
 
     if( GetScreen() )
-        GetScreen()->InitDataPoints( aPageSettings.GetSizeIU() );
+        GetScreen()->InitDataPoints( aPageSettings.GetSizeIU( pcbIUScale.IU_PER_MILS ) );
 }
 
 
@@ -464,39 +586,38 @@ const wxSize PCB_BASE_FRAME::GetPageSizeIU() const
     // this function is only needed because EDA_DRAW_FRAME is not compiled
     // with either -DPCBNEW or -DEESCHEMA, so the virtual is used to route
     // into an application specific source file.
-    return m_pcb->GetPageSettings().GetSizeIU();
+    return m_pcb->GetPageSettings().GetSizeIU( pcbIUScale.IU_PER_MILS );
 }
 
 
-const wxPoint& PCB_BASE_FRAME::GetGridOrigin() const
+const VECTOR2I& PCB_BASE_FRAME::GetGridOrigin() const
 {
     return m_pcb->GetDesignSettings().GetGridOrigin();
 }
 
 
-void PCB_BASE_FRAME::SetGridOrigin( const wxPoint& aPoint )
+void PCB_BASE_FRAME::SetGridOrigin( const VECTOR2I& aPoint )
 {
     m_pcb->GetDesignSettings().SetGridOrigin( aPoint );
 }
 
 
-const wxPoint& PCB_BASE_FRAME::GetAuxOrigin() const
+const VECTOR2I& PCB_BASE_FRAME::GetAuxOrigin() const
 {
     return m_pcb->GetDesignSettings().GetAuxOrigin();
 }
 
 
-const wxPoint PCB_BASE_FRAME::GetUserOrigin() const
+const VECTOR2I PCB_BASE_FRAME::GetUserOrigin() const
 {
-    auto& displ_opts = GetDisplayOptions();
-    wxPoint origin( 0, 0 );
+    VECTOR2I origin( 0, 0 );
 
-    switch( displ_opts.m_DisplayOrigin )
+    switch( GetPcbNewSettings()->m_Display.m_DisplayOrigin )
     {
-    case PCB_DISPLAY_OPTIONS::PCB_ORIGIN_PAGE:                           break;
-    case PCB_DISPLAY_OPTIONS::PCB_ORIGIN_AUX:  origin = GetAuxOrigin();  break;
-    case PCB_DISPLAY_OPTIONS::PCB_ORIGIN_GRID: origin = GetGridOrigin(); break;
-    default:                                   wxASSERT( false );        break;
+    case PCB_DISPLAY_ORIGIN::PCB_ORIGIN_PAGE:                           break;
+    case PCB_DISPLAY_ORIGIN::PCB_ORIGIN_AUX:  origin = GetAuxOrigin();  break;
+    case PCB_DISPLAY_ORIGIN::PCB_ORIGIN_GRID: origin = GetGridOrigin(); break;
+    default:                                  wxASSERT( false );        break;
     }
 
     return origin;
@@ -557,9 +678,9 @@ void PCB_BASE_FRAME::SetPlotSettings( const PCB_PLOT_PARAMS& aSettings )
 }
 
 
-EDA_RECT PCB_BASE_FRAME::GetBoardBoundingBox( bool aBoardEdgesOnly ) const
+BOX2I PCB_BASE_FRAME::GetBoardBoundingBox( bool aBoardEdgesOnly ) const
 {
-    EDA_RECT area = aBoardEdgesOnly ? m_pcb->GetBoardEdgesBoundingBox() : m_pcb->GetBoundingBox();
+    BOX2I area = aBoardEdgesOnly ? m_pcb->GetBoardEdgesBoundingBox() : m_pcb->GetBoundingBox();
 
     if( area.GetWidth() == 0 && area.GetHeight() == 0 )
     {
@@ -582,7 +703,7 @@ EDA_RECT PCB_BASE_FRAME::GetBoardBoundingBox( bool aBoardEdgesOnly ) const
 
 
 // Virtual function
-void PCB_BASE_FRAME::ReCreateMenuBar()
+void PCB_BASE_FRAME::doReCreateMenuBar()
 {
 }
 
@@ -594,9 +715,6 @@ void PCB_BASE_FRAME::ShowChangedLanguage()
 
     // tooltips in toolbars
     RecreateToolbars();
-
-    // status bar
-    UpdateMsgPanel();
 }
 
 
@@ -705,8 +823,8 @@ void PCB_BASE_FRAME::DisplayGridMsg()
     wxString line;
 
     line.Printf( wxT( "grid X %s  Y %s" ),
-                 MessageTextFromValue( m_userUnits, gridSize.x, false ),
-                 MessageTextFromValue( m_userUnits, gridSize.y, false ) );
+                 MessageTextFromValue( gridSize.x, false ),
+                 MessageTextFromValue( gridSize.y, false ) );
 
     SetStatusText( line, 4 );
 }
@@ -732,7 +850,8 @@ void PCB_BASE_FRAME::UpdateStatusBar()
         double   ro = hypot( dx, dy );
 
         line.Printf( wxT( "r %s  theta %.3f" ),
-                     MessageTextFromValue( GetUserUnits(), ro, false ), theta );
+                     MessageTextFromValue( ro, false ),
+                     theta );
 
         SetStatusText( line, 3 );
     }
@@ -743,8 +862,8 @@ void PCB_BASE_FRAME::UpdateStatusBar()
 
     // Display absolute coordinates:
     line.Printf( wxT( "X %s  Y %s" ),
-                 MessageTextFromValue( GetUserUnits(), userXpos, false ),
-                 MessageTextFromValue( GetUserUnits(), userYpos, false ) );
+                 MessageTextFromValue( userXpos, false ),
+                 MessageTextFromValue( userYpos, false ) );
     SetStatusText( line, 2 );
 
     if( !GetShowPolarCoords() )  // display relative cartesian coordinates
@@ -758,9 +877,9 @@ void PCB_BASE_FRAME::UpdateStatusBar()
         userYpos = m_originTransforms.ToDisplayRelY( relYpos );
 
         line.Printf( wxT( "dx %s  dy %s  dist %s" ),
-                     MessageTextFromValue( GetUserUnits(), userXpos, false ),
-                     MessageTextFromValue( GetUserUnits(), userYpos, false ),
-                     MessageTextFromValue( GetUserUnits(), hypot( userXpos, userYpos ), false ) );
+                     MessageTextFromValue( userXpos, false ),
+                     MessageTextFromValue( userYpos, false ),
+                     MessageTextFromValue( hypot( userXpos, userYpos ), false ) );
         SetStatusText( line, 3 );
     }
 
@@ -790,14 +909,9 @@ void PCB_BASE_FRAME::LoadSettings( APP_SETTINGS_BASE* aCfg )
         aCfg->m_Window.zoom_factors = { ZOOM_LIST_PCBNEW };
     }
 
-    // Some, but not all derived classes have a PCBNEW_SETTINGS.
-    PCBNEW_SETTINGS* cfg = dynamic_cast<PCBNEW_SETTINGS*>( aCfg );
-
-    if( cfg )
-    {
-        m_displayOptions = cfg->m_Display;
-        m_polarCoords = cfg->m_PolarCoords;
-    }
+    // Some, but not all, derived classes have a PCBNEW_SETTINGS.
+    if( PCBNEW_SETTINGS* pcbnew_cfg = dynamic_cast<PCBNEW_SETTINGS*>( aCfg ) )
+        m_polarCoords = pcbnew_cfg->m_PolarCoords;
 
     wxASSERT( GetCanvas() );
 
@@ -809,7 +923,7 @@ void PCB_BASE_FRAME::LoadSettings( APP_SETTINGS_BASE* aCfg )
         {
             rs->SetHighlightFactor( aCfg->m_Graphics.highlight_factor );
             rs->SetSelectFactor( aCfg->m_Graphics.select_factor );
-            rs->SetHighContrastFactor( aCfg->m_Graphics.high_contrast_factor );
+            rs->SetDefaultFont( wxEmptyString );    // Always the KiCad font for PCBs
         }
     }
 }
@@ -834,10 +948,7 @@ void PCB_BASE_FRAME::SaveSettings( APP_SETTINGS_BASE* aCfg )
     PCBNEW_SETTINGS* cfg = dynamic_cast<PCBNEW_SETTINGS*>( aCfg );
 
     if( cfg )
-    {
-        cfg->m_Display = m_displayOptions;
         cfg->m_PolarCoords = m_polarCoords;
-    }
 }
 
 
@@ -853,10 +964,30 @@ FOOTPRINT_EDITOR_SETTINGS* PCB_BASE_FRAME::GetFootprintEditorSettings() const
 }
 
 
+PCB_VIEWERS_SETTINGS_BASE* PCB_BASE_FRAME::GetViewerSettingsBase() const
+{
+    switch( GetFrameType() )
+    {
+    case FRAME_PCB_EDITOR:
+    case FRAME_FOOTPRINT_EDITOR:
+    case FRAME_FOOTPRINT_WIZARD:
+    case FRAME_PCB_DISPLAY3D:
+    default:
+        return Pgm().GetSettingsManager().GetAppSettings<PCBNEW_SETTINGS>();
+
+    case FRAME_FOOTPRINT_VIEWER:
+    case FRAME_FOOTPRINT_VIEWER_MODAL:
+    case FRAME_FOOTPRINT_PREVIEW:
+    case FRAME_CVPCB:
+    case FRAME_CVPCB_DISPLAY:
+        return Pgm().GetSettingsManager().GetAppSettings<CVPCB_SETTINGS>();
+    }
+}
+
+
 MAGNETIC_SETTINGS* PCB_BASE_FRAME::GetMagneticItemsSettings()
 {
-    wxCHECK( m_settings, nullptr );
-    return &m_settings->m_MagneticItems;
+    return &GetPcbNewSettings()->m_MagneticItems;
 }
 
 
@@ -864,25 +995,31 @@ void PCB_BASE_FRAME::CommonSettingsChanged( bool aEnvVarsChanged, bool aTextVars
 {
     EDA_DRAW_FRAME::CommonSettingsChanged( aEnvVarsChanged, aTextVarsChanged );
 
-    GetCanvas()->GetView()->GetPainter()->GetSettings()->LoadColors( GetColorSettings() );
+    RENDER_SETTINGS*     settings = GetCanvas()->GetView()->GetPainter()->GetSettings();
+    PCB_RENDER_SETTINGS* renderSettings = static_cast<KIGFX::PCB_RENDER_SETTINGS*>( settings );
 
-    GetCanvas()->GetView()->UpdateAllItemsConditionally( KIGFX::REPAINT,
-            [&]( KIGFX::VIEW_ITEM* aItem ) -> bool
+    renderSettings->LoadColors( GetColorSettings( true ) );
+    renderSettings->LoadDisplayOptions( GetDisplayOptions() );
+
+    // Note: KIGFX::REPAINT isn't enough for things that go from invisible to visible as
+    // they won't be found in the view layer's itemset for re-painting.
+    GetCanvas()->GetView()->UpdateAllItemsConditionally(
+            [&]( KIGFX::VIEW_ITEM* aItem ) -> int
             {
                 if( dynamic_cast<RATSNEST_VIEW_ITEM*>( aItem ) )
                 {
-                    return true;    // ratsnest display
+                    return KIGFX::ALL;        // ratsnest display
                 }
                 else if( dynamic_cast<PCB_TRACK*>( aItem ) )
                 {
-                    return true;    // track, arc & via clearance display
+                    return KIGFX::REPAINT;    // track, arc & via clearance display
                 }
                 else if( dynamic_cast<PAD*>( aItem ) )
                 {
-                    return true;    // pad clearance display
+                    return KIGFX::REPAINT;    // pad clearance display
                 }
 
-                return false;
+                return 0;
             } );
 
     GetCanvas()->GetView()->UpdateAllItems( KIGFX::COLOR );
@@ -899,7 +1036,10 @@ void PCB_BASE_FRAME::CommonSettingsChanged( bool aEnvVarsChanged, bool aTextVars
 
 void PCB_BASE_FRAME::OnModify()
 {
+    EDA_BASE_FRAME::OnModify();
+
     GetScreen()->SetContentModified();
+    m_autoSaveRequired = true;
 
     GetBoard()->IncrementTimeStamp();
 
@@ -908,6 +1048,14 @@ void PCB_BASE_FRAME::OnModify()
 }
 
 
+void PCB_BASE_FRAME::rebuildConnectivity()
+{
+    GetBoard()->BuildConnectivity();
+    GetToolManager()->PostEvent( EVENTS::ConnectivityChangedEvent );
+    GetCanvas()->RedrawRatsnest();
+}
+
+
 PCB_DRAW_PANEL_GAL* PCB_BASE_FRAME::GetCanvas() const
 {
     return static_cast<PCB_DRAW_PANEL_GAL*>( EDA_DRAW_FRAME::GetCanvas() );
@@ -932,19 +1080,35 @@ void PCB_BASE_FRAME::ActivateGalCanvas()
     KIGFX::PCB_RENDER_SETTINGS* settings = painter->GetSettings();
     const PCB_DISPLAY_OPTIONS&  displ_opts = GetDisplayOptions();
 
-    settings->LoadDisplayOptions( displ_opts, ShowPageLimits() );
+    settings->LoadDisplayOptions( displ_opts );
     settings->LoadColors( GetColorSettings() );
 
     view->RecacheAllItems();
     canvas->SetEventDispatcher( m_toolDispatcher );
     canvas->StartDrawing();
+
+#if defined( KICAD_USE_3DCONNEXION )
+    try
+    {
+        if( m_spaceMouse == nullptr )
+        {
+            m_spaceMouse = new NL_PCBNEW_PLUGIN( GetCanvas() );
+        }
+    }
+    catch( const std::system_error& e )
+    {
+        wxLogTrace( wxT( "KI_TRACE_NAVLIB" ), e.what() );
+    }
+#endif
 }
 
 
 void PCB_BASE_FRAME::SetDisplayOptions( const PCB_DISPLAY_OPTIONS& aOptions, bool aRefresh )
 {
-    bool hcChanged   = m_displayOptions.m_ContrastModeDisplay != aOptions.m_ContrastModeDisplay;
-    m_displayOptions = aOptions;
+    bool hcChanged    = m_displayOptions.m_ContrastModeDisplay != aOptions.m_ContrastModeDisplay;
+    bool hcVisChanged = m_displayOptions.m_ContrastModeDisplay == HIGH_CONTRAST_MODE::HIDDEN
+                        || aOptions.m_ContrastModeDisplay == HIGH_CONTRAST_MODE::HIDDEN;
+    m_displayOptions  = aOptions;
 
     EDA_DRAW_PANEL_GAL* canvas = GetCanvas();
     KIGFX::PCB_VIEW*    view   = static_cast<KIGFX::PCB_VIEW*>( canvas->GetView() );
@@ -956,16 +1120,27 @@ void PCB_BASE_FRAME::SetDisplayOptions( const PCB_DISPLAY_OPTIONS& aOptions, boo
     // Vias on a restricted layer set must be redrawn when high contrast mode is changed
     if( hcChanged )
     {
-        GetCanvas()->GetView()->UpdateAllItemsConditionally( KIGFX::REPAINT,
-                []( KIGFX::VIEW_ITEM* aItem ) -> bool
+        // Note: KIGFX::REPAINT isn't enough for things that go from invisible to visible as
+        // they won't be found in the view layer's itemset for re-painting.
+        GetCanvas()->GetView()->UpdateAllItemsConditionally(
+                [&]( KIGFX::VIEW_ITEM* aItem ) -> int
                 {
                     if( PCB_VIA* via = dynamic_cast<PCB_VIA*>( aItem ) )
                     {
-                        return via->GetViaType() == VIATYPE::BLIND_BURIED
-                                || via->GetViaType() == VIATYPE::MICROVIA;
+                        if( via->GetViaType() == VIATYPE::BLIND_BURIED
+                                || via->GetViaType() == VIATYPE::MICROVIA
+                                || via->GetRemoveUnconnected() )
+                        {
+                            return hcVisChanged ? KIGFX::ALL : KIGFX::REPAINT;
+                        }
+                    }
+                    else if( PAD* pad = dynamic_cast<PAD*>( aItem ) )
+                    {
+                        if( pad->GetRemoveUnconnected() )
+                            return hcVisChanged ? KIGFX::ALL : KIGFX::REPAINT;
                     }
 
-                    return false;
+                    return 0;
                 } );
     }
 
