@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2004 Jean-Pierre Charras, jaen-pierre.charras@gipsa-lab.inpg.com
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -65,7 +65,7 @@ enum class DIM_UNITS_MODE
 /**
  * Frame to show around dimension text
  */
-enum class DIM_TEXT_FRAME
+enum class DIM_TEXT_BORDER
 {
     NONE,
     RECTANGLE,
@@ -97,14 +97,14 @@ class PCB_DIMENSION_BASE : public BOARD_ITEM
 public:
     PCB_DIMENSION_BASE( BOARD_ITEM* aParent, KICAD_T aType = PCB_DIMENSION_T );
 
-    bool IsType( const KICAD_T aScanTypes[] ) const override
+    bool IsType( const std::vector<KICAD_T>& aScanTypes ) const override
     {
         if( BOARD_ITEM::IsType( aScanTypes ) )
             return true;
 
-        for( const KICAD_T* p = aScanTypes; *p != EOT; ++p )
+        for( KICAD_T scanType : aScanTypes )
         {
-            if( *p == PCB_LOCATE_GRAPHIC_T )
+            if( scanType == PCB_LOCATE_GRAPHIC_T )
                 return true;
         }
 
@@ -118,14 +118,14 @@ public:
      * one or more feature points, so every dimension has at least an origin.
      * @return the origin point of this dimension
      */
-    virtual const wxPoint& GetStart() const { return m_start; }
-    virtual void SetStart( const wxPoint& aPoint ) { m_start = aPoint; }
+    virtual const VECTOR2I& GetStart() const { return m_start; }
+    virtual void            SetStart( const VECTOR2I& aPoint ) { m_start = aPoint; }
 
-    virtual const wxPoint& GetEnd() const { return m_end; }
-    virtual void SetEnd( const wxPoint& aPoint ) { m_end = aPoint; }
+    virtual const VECTOR2I& GetEnd() const { return m_end; }
+    virtual void            SetEnd( const VECTOR2I& aPoint ) { m_end = aPoint; }
 
-    wxPoint GetPosition() const override { return m_start; }
-    void SetPosition( const wxPoint& aPos ) override { m_start = aPos; }
+    VECTOR2I GetPosition() const override { return m_start; }
+    void     SetPosition( const VECTOR2I& aPos ) override { m_start = aPos; }
 
     bool GetOverrideTextEnabled() const { return m_overrideTextEnabled; }
     void SetOverrideTextEnabled( bool aOverride ) { m_overrideTextEnabled = aOverride; }
@@ -158,7 +158,7 @@ public:
     wxString GetSuffix() const { return m_suffix; }
     void SetSuffix( const wxString& aSuffix );
 
-    void GetUnits( EDA_UNITS& aUnits ) const { aUnits = m_units; }
+    EDA_UNITS GetUnits() const { return m_units; }
     void SetUnits( EDA_UNITS aUnits );
 
     DIM_UNITS_MODE GetUnitsMode() const;
@@ -221,9 +221,9 @@ public:
 
     // BOARD_ITEM overrides
 
-    void Move( const wxPoint& offset ) override;
-    void Rotate( const wxPoint& aRotCentre, double aAngle ) override;
-    void Flip( const wxPoint& aCentre, bool aFlipLeftRight ) override;
+    void Move( const VECTOR2I& offset ) override;
+    void Rotate( const VECTOR2I& aRotCentre, const EDA_ANGLE& aAngle ) override;
+    void Flip( const VECTOR2I& aCentre, bool aFlipLeftRight ) override;
 
     /**
      * Mirror the dimension relative to a given horizontal axis.
@@ -233,24 +233,25 @@ public:
      *
      * @param axis_pos is the vertical axis position to mirror around.
      */
-    void Mirror( const wxPoint& axis_pos, bool aMirrorLeftRight = false );
+    virtual void Mirror( const VECTOR2I& axis_pos, bool aMirrorLeftRight = false );
 
     void GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITEM>& aList ) override;
 
-    bool HitTest( const wxPoint& aPosition, int aAccuracy ) const override;
-    bool HitTest( const EDA_RECT& aRect, bool aContained, int aAccuracy = 0 ) const override;
+    bool HitTest( const VECTOR2I& aPosition, int aAccuracy ) const override;
+    bool HitTest( const BOX2I& aRect, bool aContained, int aAccuracy = 0 ) const override;
 
-    const EDA_RECT GetBoundingBox() const override;
+    const BOX2I GetBoundingBox() const override;
 
-    std::shared_ptr<SHAPE> GetEffectiveShape( PCB_LAYER_ID aLayer ) const override;
+    std::shared_ptr<SHAPE> GetEffectiveShape( PCB_LAYER_ID aLayer,
+            FLASHING aFlash = FLASHING::DEFAULT ) const override;
 
-    wxString GetSelectMenuText( EDA_UNITS aUnits ) const override;
+    wxString GetItemDescription( UNITS_PROVIDER* aUnitsProvider ) const override;
 
     const BOX2I ViewBBox() const override;
 
-    void TransformShapeWithClearanceToPolygon( SHAPE_POLY_SET& aCornerBuffer, PCB_LAYER_ID aLayer,
-                                               int aClearance, int aError, ERROR_LOC aErrorLoc,
-                                               bool aIgnoreLineWidth ) const override;
+    void TransformShapeToPolygon( SHAPE_POLY_SET& aBuffer, PCB_LAYER_ID aLayer, int aClearance,
+                                  int aError, ERROR_LOC aErrorLoc,
+                                  bool aIgnoreLineWidth = false ) const override;
 
 #if defined(DEBUG)
     virtual void Show( int nestLevel, std::ostream& os ) const override { ShowDummy( os ); }
@@ -304,13 +305,11 @@ protected:
     // Internal
     PCB_TEXT          m_text;             ///< The actual text object
     int               m_measuredValue;    ///< value of PCB dimensions
-    wxPoint           m_start;
-    wxPoint           m_end;
+    VECTOR2I          m_start;
+    VECTOR2I          m_end;
 
     ///< Internal cache of drawn shapes
     std::vector<std::shared_ptr<SHAPE>> m_shapes;
-
-    static constexpr float s_arrowAngle = 27.5;
 };
 
 
@@ -344,7 +343,7 @@ protected:
 class PCB_DIM_ALIGNED : public PCB_DIMENSION_BASE
 {
 public:
-    PCB_DIM_ALIGNED( BOARD_ITEM* aParent, KICAD_T aType = PCB_DIM_ALIGNED_T );
+    PCB_DIM_ALIGNED( BOARD_ITEM* aParent, KICAD_T aType );
 
     // Do not create a copy constructor & operator=.
     // The ones generated by the compiler are adequate.
@@ -353,18 +352,19 @@ public:
 
     static inline bool ClassOf( const EDA_ITEM* aItem )
     {
-        return aItem && PCB_DIM_ALIGNED_T == aItem->Type();
+        return aItem && ( aItem->Type() == PCB_DIM_ALIGNED_T
+                            || aItem->Type() == PCB_FP_DIM_ALIGNED_T );
     }
 
     EDA_ITEM* Clone() const override;
 
-    virtual void SwapData( BOARD_ITEM* aImage ) override;
+    void Mirror( const VECTOR2I& axis_pos, bool aMirrorLeftRight = false ) override;
 
     BITMAPS GetMenuImage() const override;
 
-    const wxPoint& GetCrossbarStart() const { return m_crossBarStart; }
+    const VECTOR2I& GetCrossbarStart() const { return m_crossBarStart; }
 
-    const wxPoint& GetCrossbarEnd() const { return m_crossBarEnd; }
+    const VECTOR2I& GetCrossbarEnd() const { return m_crossBarEnd; }
 
     /**
      * Set the distance from the feature points to the crossbar line.
@@ -379,7 +379,7 @@ public:
      *
      * @param aCrossbarStart is the start point of the crossbar.
      */
-    void UpdateHeight( const wxPoint& aCrossbarStart, const wxPoint& aCrossbarEnd );
+    void UpdateHeight( const VECTOR2I& aCrossbarStart, const VECTOR2I& aCrossbarEnd );
 
     void SetExtensionHeight( int aHeight ) { m_extensionHeight = aHeight; }
     int GetExtensionHeight() const { return m_extensionHeight; }
@@ -391,7 +391,7 @@ public:
      */
     double GetAngle() const
     {
-        wxPoint delta( m_end - m_start );
+        VECTOR2I delta( m_end - m_start );
 
         return atan2( (double)delta.y, (double)delta.x );
     }
@@ -404,6 +404,8 @@ public:
     void GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITEM>& aList ) override;
 
 protected:
+    virtual void swapData( BOARD_ITEM* aImage ) override;
+
     void updateGeometry() override;
 
     void updateText() override;
@@ -412,8 +414,8 @@ protected:
     int          m_height;           ///< Perpendicular distance from features to crossbar
     int          m_extensionHeight;  ///< Length of extension lines past the crossbar
 
-    wxPoint      m_crossBarStart;    ///< Crossbar start control point
-    wxPoint      m_crossBarEnd;      ///< Crossbar end control point
+    VECTOR2I m_crossBarStart; ///< Crossbar start control point
+    VECTOR2I m_crossBarEnd;   ///< Crossbar end control point
 
 };
 
@@ -431,19 +433,18 @@ public:
         VERTICAL    // Aligned with y-axis
     };
 
-    PCB_DIM_ORTHOGONAL( BOARD_ITEM* aParent );
+    PCB_DIM_ORTHOGONAL( BOARD_ITEM* aParent, bool aInFP = false );
 
     ~PCB_DIM_ORTHOGONAL() = default;
 
     static inline bool ClassOf( const EDA_ITEM* aItem )
     {
-        return aItem && PCB_DIM_ORTHOGONAL_T == aItem->Type();
+        return aItem && ( aItem->Type() == PCB_DIM_ORTHOGONAL_T
+                            || aItem->Type() == PCB_FP_DIM_ORTHOGONAL_T );
     }
 
     EDA_ITEM* Clone() const override;
 
-    void SwapData( BOARD_ITEM* aImage ) override;
-
     BITMAPS GetMenuImage() const override;
 
     /**
@@ -458,9 +459,11 @@ public:
     {
         return wxT( "PCB_DIM_ORTHOGONAL" );
     }
-    void     Rotate( const wxPoint& aRotCentre, double aAngle ) override;
+    void     Rotate( const VECTOR2I& aRotCentre, const EDA_ANGLE& aAngle ) override;
 
 protected:
+    void swapData( BOARD_ITEM* aImage ) override;
+
     void updateGeometry() override;
 
     void updateText() override;
@@ -472,16 +475,73 @@ private:
 };
 
 
+/**
+ * A radial dimension indicates either the radius or diameter of an arc or circle.
+ *
+ * A guide to the geometry of a circle dimension:
+ *
+ *     |
+ *   --a--
+ *     |
+ *
+ *
+ *          b_
+ *          |\
+ *            \
+ *             c---d TEXT
+ *
+ * Point a (the center of the arc or circle) is m_start, point b (a point on the arc or circle)
+ * is m_end, point c is m_leaderLength away from b on the a-b vector, and point d is the end of
+ * the "text line". The c-d line is drawn from c to the text center, and clipped on the text
+ * bounding box.
+ */
+class PCB_DIM_RADIAL : public PCB_DIMENSION_BASE
+{
+public:
+    PCB_DIM_RADIAL( BOARD_ITEM* aParent, bool aInFP = false );
+
+    static inline bool ClassOf( const EDA_ITEM* aItem )
+    {
+        return aItem && ( aItem->Type() == PCB_DIM_RADIAL_T
+                            || aItem->Type() == PCB_FP_DIM_RADIAL_T );
+    }
+
+    EDA_ITEM* Clone() const override;
+
+    void SetLeaderLength( int aLength ) { m_leaderLength = aLength; }
+    int GetLeaderLength() const { return m_leaderLength; }
+
+    // Returns the point (c).
+    VECTOR2I GetKnee() const;
+
+    BITMAPS GetMenuImage() const override;
+
+    wxString GetClass() const override
+    {
+        return wxT( "PCB_DIM_RADIAL" );
+    }
+
+protected:
+    virtual void swapData( BOARD_ITEM* aImage ) override;
+
+    void updateText() override;
+    void updateGeometry() override;
+
+private:
+    bool m_isDiameter;
+    int  m_leaderLength;
+};
+
+
 /**
  * A leader is a dimension-like object pointing to a specific point.
  *
  * A guide to the geometry of a leader:
  *
- *     a
- *        _
- *       |\
- *          \
- *            b---c TEXT
+ *     a_
+ *     |\
+ *       \
+ *        b---c TEXT
  *
  * Point (a) is m_start, point (b) is m_end, point (c) is the end of the "text line"
  * The b-c line is drawn from b to the text center, and clipped on the text bounding box.
@@ -489,17 +549,16 @@ private:
 class PCB_DIM_LEADER : public PCB_DIMENSION_BASE
 {
 public:
-    PCB_DIM_LEADER( BOARD_ITEM* aParent );
+    PCB_DIM_LEADER( BOARD_ITEM* aParent, bool aInFP = false );
 
     static inline bool ClassOf( const EDA_ITEM* aItem )
     {
-        return aItem && PCB_DIM_LEADER_T == aItem->Type();
+        return aItem && ( aItem->Type() == PCB_DIM_LEADER_T
+                            || aItem->Type() == PCB_FP_DIM_LEADER_T );
     }
 
     EDA_ITEM* Clone() const override;
 
-    virtual void SwapData( BOARD_ITEM* aImage ) override;
-
     BITMAPS GetMenuImage() const override;
 
     wxString GetClass() const override
@@ -507,16 +566,18 @@ public:
         return wxT( "PCB_DIM_LEADER" );
     }
 
-    void SetTextFrame( DIM_TEXT_FRAME aFrame ) { m_textFrame = aFrame; }
-    DIM_TEXT_FRAME GetTextFrame() const { return m_textFrame; }
+    void SetTextBorder( DIM_TEXT_BORDER aFrame ) { m_textBorder = aFrame; }
+    DIM_TEXT_BORDER GetTextBorder() const { return m_textBorder; }
 
     void GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITEM>& aList ) override;
 
 protected:
+    virtual void swapData( BOARD_ITEM* aImage ) override;
+
     void updateGeometry() override;
 
 private:
-    DIM_TEXT_FRAME m_textFrame;
+    DIM_TEXT_BORDER m_textBorder;
 };
 
 
@@ -529,17 +590,16 @@ private:
 class PCB_DIM_CENTER : public PCB_DIMENSION_BASE
 {
 public:
-    PCB_DIM_CENTER( BOARD_ITEM* aParent );
+    PCB_DIM_CENTER( BOARD_ITEM* aParent, bool aInFP = false );
 
     static inline bool ClassOf( const EDA_ITEM* aItem )
     {
-        return aItem && PCB_DIM_CENTER_T == aItem->Type();
+        return aItem && ( aItem->Type() == PCB_DIM_CENTER_T
+                            || aItem->Type() == PCB_FP_DIM_CENTER_T );
     }
 
     EDA_ITEM* Clone() const override;
 
-    virtual void SwapData( BOARD_ITEM* aImage ) override;
-
     BITMAPS GetMenuImage() const override;
 
     wxString GetClass() const override
@@ -547,11 +607,13 @@ public:
         return wxT( "PCB_DIM_CENTER" );
     }
 
-    const EDA_RECT GetBoundingBox() const override;
+    const BOX2I GetBoundingBox() const override;
 
     const BOX2I ViewBBox() const override;
 
 protected:
+    virtual void swapData( BOARD_ITEM* aImage ) override;
+
     void updateGeometry() override;
 };
 
