@@ -2,7 +2,7 @@
  * KiRouter - a push-and-(sometimes-)shove PCB router
  *
  * Copyright (C) 2013-2016 CERN
- * Copyright (C) 2016-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2016-2022 KiCad Developers, see AUTHORS.txt for contributors.
  * Author: Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  *
  * This program is free software: you can redistribute it and/or modify it
@@ -33,6 +33,9 @@
 #include <layer_ids.h>
 #include <geometry/convex_hull.h>
 #include <confirm.h>
+#include <tools/pcb_tool_base.h>
+#include <tool/tool_manager.h>
+#include <settings/app_settings.h>
 
 #include <pcb_painter.h>
 
@@ -44,11 +47,15 @@
 #include <drc/drc_rule.h>
 #include <drc/drc_engine.h>
 
+#include <connectivity/connectivity_data.h>
+
 #include <wx/log.h>
 
 #include <memory>
 
 #include <advanced_config.h>
+#include <pcbnew_settings.h>
+#include <macros.h>
 
 #include "pns_kicad_iface.h"
 
@@ -66,6 +73,31 @@
 typedef VECTOR2I::extended_type ecoord;
 
 
+struct CLEARANCE_CACHE_KEY
+{
+    const PNS::ITEM*  A;
+    const PNS::ITEM*  B;
+    bool              Flag;
+
+    bool operator==(const CLEARANCE_CACHE_KEY& other) const
+    {
+        return A == other.A && B == other.B && Flag == other.Flag;
+    }
+};
+
+namespace std
+{
+    template <>
+    struct hash<CLEARANCE_CACHE_KEY>
+    {
+        std::size_t operator()( const CLEARANCE_CACHE_KEY& k ) const
+        {
+            return hash<const void*>()( k.A ) ^ hash<const void*>()( k.B ) ^ hash<int>()( k.Flag );
+        }
+    };
+}
+
+
 class PNS_PCBNEW_RULE_RESOLVER : public PNS::RULE_RESOLVER
 {
 public:
@@ -84,6 +116,10 @@ public:
     virtual bool DpNetPair( const PNS::ITEM* aItem, int& aNetP, int& aNetN ) override;
     virtual bool IsDiffPair( const PNS::ITEM* aA, const PNS::ITEM* aB ) override;
 
+    virtual bool IsInNetTie( const PNS::ITEM* aA ) override;
+    virtual bool IsNetTieExclusion( const PNS::ITEM* aItem, const VECTOR2I& aCollisionPos,
+                                    const PNS::ITEM* aCollidingItem ) override;
+
     virtual bool QueryConstraint( PNS::CONSTRAINT_TYPE aType, const PNS::ITEM* aItemA,
                                   const PNS::ITEM* aItemB, int aLayer,
                                   PNS::CONSTRAINT* aConstraint ) override;
@@ -92,6 +128,7 @@ public:
     int ClearanceEpsilon() const { return m_clearanceEpsilon; }
 
     void ClearCacheForItem( const PNS::ITEM* aItem ) override;
+    void ClearCaches() override;
 
 private:
     int holeRadius( const PNS::ITEM* aItem ) const;
@@ -115,11 +152,9 @@ private:
     PCB_VIA            m_dummyVias[2];
     int                m_clearanceEpsilon;
 
-    typedef std::tuple<const PNS::ITEM*, const PNS::ITEM*, bool> CLEARANCE_CACHE_KEY;
-
-    std::map<CLEARANCE_CACHE_KEY, int> m_clearanceCache;
-    std::map<CLEARANCE_CACHE_KEY, int> m_holeClearanceCache;
-    std::map<CLEARANCE_CACHE_KEY, int> m_holeToHoleClearanceCache;
+    std::unordered_map<CLEARANCE_CACHE_KEY, int> m_clearanceCache;
+    std::unordered_map<CLEARANCE_CACHE_KEY, int> m_holeClearanceCache;
+    std::unordered_map<CLEARANCE_CACHE_KEY, int> m_holeToHoleClearanceCache;
 };
 
 
@@ -181,6 +216,35 @@ bool PNS_PCBNEW_RULE_RESOLVER::IsDiffPair( const PNS::ITEM* aA, const PNS::ITEM*
 }
 
 
+bool PNS_PCBNEW_RULE_RESOLVER::IsInNetTie( const PNS::ITEM* aA )
+{
+    BOARD_ITEM* item = aA->Parent();
+    BOARD_ITEM* parentFootprint = item ? item->GetParentFootprint() : nullptr;
+
+    if( parentFootprint )
+        return static_cast<FOOTPRINT*>( parentFootprint )->IsNetTie();
+
+    return false;
+}
+
+
+bool PNS_PCBNEW_RULE_RESOLVER::IsNetTieExclusion( const PNS::ITEM* aItem,
+                                                  const VECTOR2I& aCollisionPos,
+                                                  const PNS::ITEM* aCollidingItem )
+{
+    std::shared_ptr<DRC_ENGINE> drcEngine = m_board->GetDesignSettings().m_DRCEngine;
+    BOARD_ITEM*                 collidingItem = aCollidingItem->Parent();
+
+    if( drcEngine && collidingItem )
+    {
+        return drcEngine->IsNetTieExclusion( aItem->Net(), ToLAYER_ID( aItem->Layer() ),
+                                             aCollisionPos, collidingItem );
+    }
+
+    return false;
+}
+
+
 bool isCopper( const PNS::ITEM* aItem )
 {
     BOARD_ITEM* parent = aItem->Parent();
@@ -188,7 +252,23 @@ bool isCopper( const PNS::ITEM* aItem )
     if( parent && parent->Type() == PCB_PAD_T )
     {
         PAD* pad = static_cast<PAD*>( parent );
-        return pad->IsOnCopperLayer() && pad->GetAttribute() != PAD_ATTRIB::NPTH;
+
+        if( !pad->IsOnCopperLayer() )
+            return false;
+
+        if( pad->GetAttribute() != PAD_ATTRIB::NPTH )
+            return true;
+
+        // round NPTH with a hole size >= pad size are not on a copper layer
+        // All other NPTH are seen on copper layers
+        // This is a basic criteria, but probably enough for a NPTH
+        if( pad->GetShape() == PAD_SHAPE::CIRCLE )
+        {
+            if( pad->GetSize().x <= pad->GetDrillSize().x )
+                return false;
+        }
+
+        return true;
     }
 
     return true;
@@ -199,12 +279,7 @@ bool isEdge( const PNS::ITEM* aItem )
 {
     const BOARD_ITEM *parent = aItem->Parent();
 
-    if( parent )
-    {
-        return parent->GetLayer() == Edge_Cuts || parent->GetLayer () == Margin;
-    }
-
-    return false;
+    return parent && ( parent->IsOnLayer( Edge_Cuts ) || parent->IsOnLayer( Margin ) );
 }
 
 
@@ -251,7 +326,7 @@ bool PNS_PCBNEW_RULE_RESOLVER::QueryConstraint( PNS::CONSTRAINT_TYPE aType,
 
         if( parentA )
         {
-            parentA->SetLayer( (PCB_LAYER_ID) aLayer );
+            parentA->SetLayer( ToLAYER_ID( aLayer ) );
             static_cast<BOARD_CONNECTED_ITEM*>( parentA )->SetNetCode( aItemA->Net(), true );
         }
     }
@@ -269,17 +344,25 @@ bool PNS_PCBNEW_RULE_RESOLVER::QueryConstraint( PNS::CONSTRAINT_TYPE aType,
 
         if( parentB )
         {
-            parentB->SetLayer( (PCB_LAYER_ID) aLayer );
+            parentB->SetLayer( ToLAYER_ID( aLayer ) );
             static_cast<BOARD_CONNECTED_ITEM*>( parentB )->SetNetCode( aItemB->Net(), true );
         }
     }
 
     if( parentA )
-        hostConstraint = drcEngine->EvalRules( hostType, parentA, parentB, (PCB_LAYER_ID) aLayer );
+        hostConstraint = drcEngine->EvalRules( hostType, parentA, parentB, ToLAYER_ID( aLayer ) );
 
     if( hostConstraint.IsNull() )
         return false;
 
+    if( hostConstraint.GetSeverity() == RPT_SEVERITY_IGNORE )
+    {
+        aConstraint->m_Value.SetMin( -1 );
+        aConstraint->m_RuleName = hostConstraint.GetName();
+        aConstraint->m_Type = aType;
+        return true;
+    }
+
     switch ( aType )
     {
         case PNS::CONSTRAINT_TYPE::CT_CLEARANCE:
@@ -303,46 +386,70 @@ bool PNS_PCBNEW_RULE_RESOLVER::QueryConstraint( PNS::CONSTRAINT_TYPE aType,
 
 void PNS_PCBNEW_RULE_RESOLVER::ClearCacheForItem( const PNS::ITEM* aItem )
 {
-    m_clearanceCache.erase( std::make_tuple( aItem, nullptr, false ) );
-    m_clearanceCache.erase( std::make_tuple( aItem, nullptr, true ) );
+    CLEARANCE_CACHE_KEY key = { aItem, nullptr, false };
+    m_clearanceCache.erase( key );
+
+    key.Flag = true;
+    m_clearanceCache.erase( key );
+}
+
+
+void PNS_PCBNEW_RULE_RESOLVER::ClearCaches()
+{
+    m_clearanceCache.clear();
+    m_holeClearanceCache.clear();
+    m_holeToHoleClearanceCache.clear();
 }
 
 
 int PNS_PCBNEW_RULE_RESOLVER::Clearance( const PNS::ITEM* aA, const PNS::ITEM* aB,
                                          bool aUseClearanceEpsilon )
 {
-    CLEARANCE_CACHE_KEY key( aA, aB, aUseClearanceEpsilon );
+    CLEARANCE_CACHE_KEY key = { aA, aB, aUseClearanceEpsilon };
     auto it = m_clearanceCache.find( key );
 
     if( it != m_clearanceCache.end() )
         return it->second;
 
     PNS::CONSTRAINT constraint;
-    int rv = 0;
-    int layer;
+    int             rv = 0;
+    LAYER_RANGE     layers;
 
-    if( !aA->Layers().IsMultilayer() || !aB || aB->Layers().IsMultilayer() )
-        layer = aA->Layer();
+    if( !aB )
+        layers = aA->Layers();
+    else if( isEdge( aA ) )
+        layers = aB->Layers();
+    else if( isEdge( aB ) )
+        layers = aA->Layers();
     else
-        layer = aB->Layer();
+        layers = aA->Layers().Intersection( aB->Layers() );
 
-    if( isCopper( aA ) && ( !aB || isCopper( aB ) ) )
-    {
-        if( QueryConstraint( PNS::CONSTRAINT_TYPE::CT_CLEARANCE, aA, aB, layer, &constraint ) )
-            rv = constraint.m_Value.Min();
-    }
+    // Normalize layer range (no -1 magic numbers)
+    layers = layers.Intersection( LAYER_RANGE( PCBNEW_LAYER_ID_START, PCB_LAYER_ID_COUNT - 1 ) );
 
-    if( isEdge( aA ) || ( aB && isEdge( aB ) ) )
+    for( int layer = layers.Start(); layer <= layers.End(); ++layer )
     {
-        if( QueryConstraint( PNS::CONSTRAINT_TYPE::CT_EDGE_CLEARANCE, aA, aB, layer, &constraint ) )
+        if( isCopper( aA ) && ( !aB || isCopper( aB ) ) )
         {
-            if( constraint.m_Value.Min() > rv )
-                rv = constraint.m_Value.Min();
+            if( QueryConstraint( PNS::CONSTRAINT_TYPE::CT_CLEARANCE, aA, aB, layer, &constraint ) )
+            {
+                if( constraint.m_Value.Min() > rv )
+                    rv = constraint.m_Value.Min();
+            }
+        }
+
+        if( isEdge( aA ) || ( aB && isEdge( aB ) ) )
+        {
+            if( QueryConstraint( PNS::CONSTRAINT_TYPE::CT_EDGE_CLEARANCE, aA, aB, layer, &constraint ) )
+            {
+                if( constraint.m_Value.Min() > rv )
+                    rv = constraint.m_Value.Min();
+            }
         }
     }
 
-    if( aUseClearanceEpsilon )
-        rv -= m_clearanceEpsilon;
+    if( aUseClearanceEpsilon && rv > 0 )
+        rv = std::max( 0, rv - m_clearanceEpsilon );
 
     m_clearanceCache[ key ] = rv;
     return rv;
@@ -352,7 +459,7 @@ int PNS_PCBNEW_RULE_RESOLVER::Clearance( const PNS::ITEM* aA, const PNS::ITEM* a
 int PNS_PCBNEW_RULE_RESOLVER::HoleClearance( const PNS::ITEM* aA, const PNS::ITEM* aB,
                                              bool aUseClearanceEpsilon )
 {
-    CLEARANCE_CACHE_KEY key( aA, aB, aUseClearanceEpsilon );
+    CLEARANCE_CACHE_KEY key = { aA, aB, aUseClearanceEpsilon };
     auto it = m_holeClearanceCache.find( key );
 
     if( it != m_holeClearanceCache.end() )
@@ -370,8 +477,18 @@ int PNS_PCBNEW_RULE_RESOLVER::HoleClearance( const PNS::ITEM* aA, const PNS::ITE
     if( QueryConstraint( PNS::CONSTRAINT_TYPE::CT_HOLE_CLEARANCE, aA, aB, layer, &constraint ) )
         rv = constraint.m_Value.Min();
 
-    if( aUseClearanceEpsilon )
-        rv -= m_clearanceEpsilon;
+#define HAS_PLATED_HOLE( a ) ( a )->IsRoutable()
+
+    if( IsCopperLayer( layer )
+            && ( HAS_PLATED_HOLE( aA ) || HAS_PLATED_HOLE( aB ) )
+            && QueryConstraint( PNS::CONSTRAINT_TYPE::CT_CLEARANCE, aA, aB, layer, &constraint )
+            && constraint.m_Value.Min() > rv )
+    {
+        rv = constraint.m_Value.Min();
+    }
+
+    if( aUseClearanceEpsilon && rv > 0 )
+        rv = std::max( 0, rv - m_clearanceEpsilon );
 
     m_holeClearanceCache[ key ] = rv;
     return rv;
@@ -381,7 +498,7 @@ int PNS_PCBNEW_RULE_RESOLVER::HoleClearance( const PNS::ITEM* aA, const PNS::ITE
 int PNS_PCBNEW_RULE_RESOLVER::HoleToHoleClearance( const PNS::ITEM* aA, const PNS::ITEM* aB,
                                                    bool aUseClearanceEpsilon )
 {
-    CLEARANCE_CACHE_KEY key( aA, aB, aUseClearanceEpsilon );
+    CLEARANCE_CACHE_KEY key = { aA, aB, aUseClearanceEpsilon };
     auto it = m_holeToHoleClearanceCache.find( key );
 
     if( it != m_holeToHoleClearanceCache.end() )
@@ -399,8 +516,8 @@ int PNS_PCBNEW_RULE_RESOLVER::HoleToHoleClearance( const PNS::ITEM* aA, const PN
     if( QueryConstraint( PNS::CONSTRAINT_TYPE::CT_HOLE_TO_HOLE, aA, aB, layer, &constraint ) )
         rv = constraint.m_Value.Min();
 
-    if( aUseClearanceEpsilon )
-        rv -= m_clearanceEpsilon;
+    if( aUseClearanceEpsilon && rv > 0 )
+        rv = std::max( 0, rv - m_clearanceEpsilon );
 
     m_holeToHoleClearanceCache[ key ] = rv;
     return rv;
@@ -479,10 +596,23 @@ bool PNS_KICAD_IFACE_BASE::ImportSizes( PNS::SIZES_SETTINGS& aSizes, PNS::ITEM*
     if( aStartItem && m_startLayer < 0 )
         m_startLayer = aStartItem->Layer();
 
+    aSizes.SetClearance( bds.m_MinClearance );
     aSizes.SetMinClearance( bds.m_MinClearance );
+    aSizes.SetClearanceSource( _( "board minimum clearance" ) );
+
+    if( m_ruleResolver->QueryConstraint( PNS::CONSTRAINT_TYPE::CT_CLEARANCE, aStartItem, nullptr,
+                                         m_startLayer, &constraint ) )
+    {
+        if( constraint.m_Value.Min() > bds.m_MinClearance )
+        {
+            aSizes.SetClearance( constraint.m_Value.Min() );
+            aSizes.SetClearanceSource( constraint.m_RuleName );
+        }
+    }
 
     int  trackWidth = bds.m_TrackMinWidth;
     bool found = false;
+    aSizes.SetWidthSource( _( "board minimum track width" ) );
 
     if( bds.m_UseConnectedTrackWidth && !bds.m_TempOverrideTrackWidth && aStartItem != nullptr )
     {
@@ -502,8 +632,6 @@ bool PNS_KICAD_IFACE_BASE::ImportSizes( PNS::SIZES_SETTINGS& aSizes, PNS::ITEM*
 
             if( trackWidth == constraint.m_Value.Opt() )
                 aSizes.SetWidthSource( constraint.m_RuleName );
-            else
-                aSizes.SetWidthSource( _( "board minimum width" ) );
         }
     }
 
@@ -515,8 +643,6 @@ bool PNS_KICAD_IFACE_BASE::ImportSizes( PNS::SIZES_SETTINGS& aSizes, PNS::ITEM*
             aSizes.SetWidthSource( _( "netclass 'Default'" ) );
         else if( trackWidth == bds.GetCurrentTrackWidth() )
             aSizes.SetWidthSource( _( "user choice" ) );
-        else
-            aSizes.SetWidthSource( _( "board minimum width" ) );
     }
 
     aSizes.SetTrackWidth( trackWidth );
@@ -533,8 +659,8 @@ bool PNS_KICAD_IFACE_BASE::ImportSizes( PNS::SIZES_SETTINGS& aSizes, PNS::ITEM*
             viaDiameter = std::max( viaDiameter, constraint.m_Value.Opt() );
         }
 
-        if( m_ruleResolver->QueryConstraint( PNS::CONSTRAINT_TYPE::CT_VIA_HOLE, aStartItem, nullptr,
-                                             m_startLayer, &constraint ) )
+        if( m_ruleResolver->QueryConstraint( PNS::CONSTRAINT_TYPE::CT_VIA_HOLE, aStartItem,
+                                             nullptr, m_startLayer, &constraint ) )
         {
             viaDrill = std::max( viaDrill, constraint.m_Value.Opt() );
         }
@@ -552,6 +678,9 @@ bool PNS_KICAD_IFACE_BASE::ImportSizes( PNS::SIZES_SETTINGS& aSizes, PNS::ITEM*
     int diffPairGap = bds.m_MinClearance;
     int diffPairViaGap = bds.m_MinClearance;
 
+    aSizes.SetDiffPairWidthSource( _( "board minimum track width" ) );
+    aSizes.SetDiffPairGapSource( _( "board minimum clearance" ) );
+
     found = false;
 
     // First try to pick up diff pair width from starting track, if enabled
@@ -565,6 +694,9 @@ bool PNS_KICAD_IFACE_BASE::ImportSizes( PNS::SIZES_SETTINGS& aSizes, PNS::ITEM*
                                                        nullptr, m_startLayer, &constraint ) )
         {
             diffPairWidth = std::max( diffPairWidth, constraint.m_Value.Opt() );
+
+            if( diffPairWidth == constraint.m_Value.Opt() )
+                aSizes.SetDiffPairWidthSource( constraint.m_RuleName );
         }
 
         if( m_ruleResolver->QueryConstraint( PNS::CONSTRAINT_TYPE::CT_DIFF_PAIR_GAP, aStartItem,
@@ -572,6 +704,9 @@ bool PNS_KICAD_IFACE_BASE::ImportSizes( PNS::SIZES_SETTINGS& aSizes, PNS::ITEM*
         {
             diffPairGap = std::max( diffPairGap, constraint.m_Value.Opt() );
             diffPairViaGap = std::max( diffPairViaGap, constraint.m_Value.Opt() );
+
+            if( diffPairGap == constraint.m_Value.Opt() )
+                aSizes.SetDiffPairGapSource( constraint.m_RuleName );
         }
     }
     else
@@ -579,6 +714,9 @@ bool PNS_KICAD_IFACE_BASE::ImportSizes( PNS::SIZES_SETTINGS& aSizes, PNS::ITEM*
         diffPairWidth  = bds.GetCurrentDiffPairWidth();
         diffPairGap    = bds.GetCurrentDiffPairGap();
         diffPairViaGap = bds.GetCurrentDiffPairViaGap();
+
+        aSizes.SetDiffPairWidthSource( _( "user choice" ) );
+        aSizes.SetDiffPairGapSource( _( "user choice" ) );
     }
 
     aSizes.SetDiffPairWidth( diffPairWidth );
@@ -744,17 +882,20 @@ public:
     PNS_PCBNEW_DEBUG_DECORATOR( KIGFX::VIEW* aView = nullptr ) :
             PNS::DEBUG_DECORATOR(),
             m_view( nullptr ),
-            m_items( nullptr )
+            m_items( nullptr ),
+            m_depth( 0 )
     {
         SetView( aView );
     }
 
+
     ~PNS_PCBNEW_DEBUG_DECORATOR()
     {
         PNS_PCBNEW_DEBUG_DECORATOR::Clear();
         delete m_items;
     }
 
+
     void SetView( KIGFX::VIEW* aView )
     {
         Clear();
@@ -765,36 +906,61 @@ public:
         if( m_view == nullptr )
             return;
 
+        if( m_view->GetGAL() )
+            m_depth = m_view->GetGAL()->GetMinDepth();
+
         m_items = new KIGFX::VIEW_GROUP( m_view );
         m_items->SetLayer( LAYER_SELECT_OVERLAY ) ;
         m_view->Add( m_items );
     }
 
-    virtual void AddPoint( const VECTOR2I& aP, const COLOR4D& aColor, int aSize,
-                           const std::string& aName,
-                           const SRC_LOCATION_INFO& aSrcLoc = SRC_LOCATION_INFO() ) override
+
+    void AddPoint( const VECTOR2I& aP, const KIGFX::COLOR4D& aColor, int aSize,
+                   const wxString&          aName = wxT( "" ),
+                   const SRC_LOCATION_INFO& aSrcLoc = SRC_LOCATION_INFO() ) override
+
     {
-        SHAPE_LINE_CHAIN l;
+        SHAPE_LINE_CHAIN sh;
 
-        l.Append( aP - VECTOR2I( -aSize, -aSize ) );
-        l.Append( aP + VECTOR2I( -aSize, -aSize ) );
+        sh.SetWidth( 10000 );
 
-        AddLine( l, aColor, 10000, aName );
+        sh.Append( aP.x - aSize, aP.y - aSize );
+        sh.Append( aP.x + aSize, aP.y + aSize );
+        sh.Append( aP.x, aP.y );
+        sh.Append( aP.x - aSize, aP.y + aSize );
+        sh.Append( aP.x + aSize, aP.y - aSize );
 
-        l.Clear();
-        l.Append( aP - VECTOR2I( aSize, -aSize ) );
-        l.Append( aP + VECTOR2I( aSize, -aSize ) );
-
-        AddLine( l, aColor, 10000, aName );
+        AddShape( &sh, aColor, sh.Width(), aName, aSrcLoc );
     }
 
-    virtual void AddBox( const BOX2I& aB, const COLOR4D& aColor, const std::string& aName,
-                         const SRC_LOCATION_INFO& aSrcLoc = SRC_LOCATION_INFO() ) override
+
+    void AddItem( const PNS::ITEM* aItem, const KIGFX::COLOR4D& aColor, int aOverrideWidth = 0,
+                  const wxString&          aName = wxT( "" ),
+                  const SRC_LOCATION_INFO& aSrcLoc = SRC_LOCATION_INFO() )
+    {
+        if( !m_view || !aItem )
+            return;
+
+        ROUTER_PREVIEW_ITEM* pitem = new ROUTER_PREVIEW_ITEM( aItem, m_view );
+
+        pitem->SetColor( aColor.WithAlpha( 0.5 ) );
+        pitem->SetWidth( aOverrideWidth );
+        pitem->SetDepth( nextDepth() );
+
+        m_items->Add( pitem );
+        m_view->Update( m_items );
+    }
+
+
+    void AddShape( const BOX2I& aBox, const KIGFX::COLOR4D& aColor, int aOverrideWidth = 0,
+                   const wxString&          aName = wxT( "" ),
+                   const SRC_LOCATION_INFO& aSrcLoc = SRC_LOCATION_INFO() ) override
     {
         SHAPE_LINE_CHAIN l;
+        l.SetWidth( aOverrideWidth );
 
-        VECTOR2I o = aB.GetOrigin();
-        VECTOR2I s = aB.GetSize();
+        VECTOR2I o = aBox.GetOrigin();
+        VECTOR2I s = aBox.GetSize();
 
         l.Append( o );
         l.Append( o.x + s.x, o.y );
@@ -802,50 +968,57 @@ public:
         l.Append( o.x, o.y + s.y );
         l.Append( o );
 
-        AddLine( l, aColor, 10000, aName, aSrcLoc );
-    }
-
-    virtual void AddSegment( const SEG& aS, const COLOR4D& aColor, const std::string& aName,
-                             const SRC_LOCATION_INFO& aSrcLoc = SRC_LOCATION_INFO() ) override
-    {
-        SHAPE_LINE_CHAIN l;
-
-        l.Append( aS.A );
-        l.Append( aS.B );
-
-        AddLine( l, aColor, 10000, aName, aSrcLoc );
+        AddShape( &l, aColor, aOverrideWidth, aName, aSrcLoc );
     }
 
 
-    virtual void AddLine( const SHAPE_LINE_CHAIN& aLine, const COLOR4D& aColor,
-                          int aWidth, const std::string& aName,
-                          const SRC_LOCATION_INFO& aSrcLoc = SRC_LOCATION_INFO() ) override
+    void AddShape( const SHAPE* aShape, const KIGFX::COLOR4D& aColor, int aOverrideWidth = 0,
+                   const wxString&          aName = wxT( "" ),
+                   const SRC_LOCATION_INFO& aSrcLoc = SRC_LOCATION_INFO() )
     {
-        if( !m_view )
+        if( !m_view || !aShape )
             return;
 
-        ROUTER_PREVIEW_ITEM* pitem = new ROUTER_PREVIEW_ITEM( nullptr, m_view );
+        ROUTER_PREVIEW_ITEM* pitem = new ROUTER_PREVIEW_ITEM( *aShape, m_view );
 
-        pitem->SetColor( aColor );
-        pitem->Line( aLine, aWidth );
+        pitem->SetColor( aColor.WithAlpha( 0.5 ) );
+        pitem->SetWidth( aOverrideWidth );
+        pitem->SetDepth( nextDepth() );
 
-        // Should not be needed, as m_items has been passed as a parent group in alloc;
         m_items->Add( pitem );
         m_view->Update( m_items );
     }
 
+
     void Clear() override
     {
         if( m_view && m_items )
         {
             m_items->FreeItems();
             m_view->Update( m_items );
+
+            if( m_view->GetGAL() )
+                m_depth = m_view->GetGAL()->GetMinDepth();
         }
     }
 
 private:
+    double nextDepth()
+    {
+        // Use different depths so that the transculent shapes won't overwrite each other.
+
+        m_depth++;
+
+        if( m_depth >= 0 && m_view->GetGAL() )
+            m_depth = m_view->GetGAL()->GetMinDepth();
+
+        return m_depth;
+    }
+
     KIGFX::VIEW* m_view;
     KIGFX::VIEW_GROUP* m_items;
+
+    double m_depth;
 };
 
 
@@ -870,7 +1043,7 @@ PNS_KICAD_IFACE::PNS_KICAD_IFACE()
     m_tool = nullptr;
     m_view = nullptr;
     m_previewItems = nullptr;
-    m_dispOptions = nullptr;
+    m_commitFlags = 0;
 }
 
 
@@ -947,34 +1120,30 @@ std::unique_ptr<PNS::SOLID> PNS_KICAD_IFACE_BASE::syncPad( PAD* aPad )
     solid->SetPadToDie( aPad->GetPadToDieLength() );
     solid->SetOrientation( aPad->GetOrientation() );
 
-    wxPoint wx_c = aPad->ShapePos();
-    wxPoint offset = aPad->GetOffset();
+    if( aPad->IsFreePad() )
+        solid->SetIsFreePad();
+
+    VECTOR2I wx_c = aPad->ShapePos();
+    VECTOR2I offset = aPad->GetOffset();
 
     VECTOR2I c( wx_c.x, wx_c.y );
 
-    RotatePoint( &offset, aPad->GetOrientation() );
+    RotatePoint( offset, aPad->GetOrientation() );
 
     solid->SetPos( VECTOR2I( c.x - offset.x, c.y - offset.y ) );
     solid->SetOffset( VECTOR2I( offset.x, offset.y ) );
 
     if( aPad->GetDrillSize().x > 0 )
-    {
-        SHAPE_SEGMENT* slot = (SHAPE_SEGMENT*) aPad->GetEffectiveHoleShape()->Clone();
+        solid->SetHole( aPad->GetEffectiveHoleShape()->Clone() );
 
-        if( aPad->GetAttribute() != PAD_ATTRIB::NPTH )
-        {
-            BOARD_DESIGN_SETTINGS& bds = m_board->GetDesignSettings();
-            slot->SetWidth( slot->GetWidth() + bds.GetHolePlatingThickness() * 2 );
-        }
-
-        solid->SetHole( slot );
-    }
-
-    std::shared_ptr<SHAPE> shape = aPad->GetEffectiveShape();
+    // We generate a single SOLID for a pad, so we have to treat it as ALWAYS_FLASHED and then
+    // perform layer-specific flashing tests internally.
+    std::shared_ptr<SHAPE> shape = aPad->GetEffectiveShape( UNDEFINED_LAYER,
+                                                            FLASHING::ALWAYS_FLASHED );
 
     if( shape->HasIndexableSubshapes() && shape->GetIndexableSubshapeCount() == 1 )
     {
-        std::vector<SHAPE*> subshapes;
+        std::vector<const SHAPE*> subshapes;
         shape->GetIndexableSubshapes( subshapes );
 
         solid->SetShape( subshapes[0]->Clone() );
@@ -1041,7 +1210,7 @@ std::unique_ptr<PNS::VIA> PNS_KICAD_IFACE_BASE::syncVia( PCB_VIA* aVia )
 
     BOARD_DESIGN_SETTINGS& bds = m_board->GetDesignSettings();
     via->SetHole( SHAPE_CIRCLE( aVia->GetPosition(),
-                                aVia->GetDrillValue() / 2 + bds.GetHolePlatingThickness() ) );
+                                aVia->GetDrillValue() / 2 ) );
 
     return via;
 }
@@ -1060,16 +1229,16 @@ bool PNS_KICAD_IFACE_BASE::syncZone( PNS::NODE* aWorld, ZONE* aZone, SHAPE_POLY_
         return false;
 
     LSET      layers = aZone->GetLayerSet();
-    EDA_UNITS units = EDA_UNITS::MILLIMETRES;       // TODO: get real units
 
     poly = aZone->Outline();
     poly->CacheTriangulation( false );
 
     if( !poly->IsTriangulationUpToDate() )
     {
-        KIDIALOG dlg( nullptr, wxString::Format( _( "%s is malformed." ),
-                                                 aZone->GetSelectMenuText( units ) ),
-                      KIDIALOG::KD_WARNING );
+        UNITS_PROVIDER unitsProvider( pcbIUScale, GetUnits() );
+        KIDIALOG       dlg( nullptr, wxString::Format( _( "%s is malformed." ),
+                                              aZone->GetItemDescription( &unitsProvider ) ),
+                            KIDIALOG::KD_WARNING );
         dlg.ShowDetailedText( wxString::Format( _( "This zone cannot be handled by the router.\n"
                                                    "Please verify it is not a self-intersecting "
                                                    "polygon." ) ) );
@@ -1121,36 +1290,46 @@ bool PNS_KICAD_IFACE_BASE::syncTextItem( PNS::NODE* aWorld, EDA_TEXT* aText, PCB
     if( !IsCopperLayer( aLayer ) )
         return false;
 
-    int textWidth = aText->GetEffectiveTextPenWidth();
-    std::vector<wxPoint> textShape = aText->TransformToSegmentList();
+    std::unique_ptr<PNS::SOLID> solid = std::make_unique<PNS::SOLID>();
 
-    if( textShape.size() < 2 )
-        return false;
+    solid->SetLayer( aLayer );
+    solid->SetNet( -1 );
+    solid->SetParent( dynamic_cast<BOARD_ITEM*>( aText ) );
 
-    for( size_t jj = 0; jj < textShape.size(); jj += 2 )
+    PCB_TEXT* pcb_text = dynamic_cast<PCB_TEXT*>( aText );
+
+    if( pcb_text && pcb_text->IsKnockout() )
     {
-        VECTOR2I start( textShape[jj] );
-        VECTOR2I end( textShape[jj+1] );
-        std::unique_ptr<PNS::SOLID> solid = std::make_unique<PNS::SOLID>();
+        TEXT_ATTRIBUTES attrs = pcb_text->GetAttributes();
+        SHAPE_POLY_SET buffer;
+        int margin = attrs.m_StrokeWidth * 1.5
+                     + GetKnockoutTextMargin( attrs.m_Size, attrs.m_StrokeWidth );
+        pcb_text->TransformBoundingBoxToPolygon( &buffer, margin );
+        // buffer should contain a single rectangular polygon
+        SHAPE_SIMPLE* rectShape = new SHAPE_SIMPLE;
 
-        solid->SetLayer( aLayer );
-        solid->SetNet( -1 );
-        solid->SetParent( dynamic_cast<BOARD_ITEM*>( aText ) );
-        solid->SetShape( new SHAPE_SEGMENT( start, end, textWidth ) );
-        solid->SetIsCompoundShapePrimitive();
-        solid->SetRoutable( false );
+        for( int ii = 0; ii < buffer.Outline(0).PointCount(); ii++ )
+        {
+            VECTOR2I point = buffer.Outline(0).CPoint(ii);
+            rectShape->Append( point );
+        }
 
-        aWorld->Add( std::move( solid ) );
+        solid->SetShape( rectShape );
     }
+    else
+    solid->SetShape( aText->GetEffectiveTextShape()->Clone() );
+
+    solid->SetRoutable( false );
+
+    aWorld->Add( std::move( solid ) );
 
     return true;
 
     /* A coarser (but faster) method:
-     *
     SHAPE_POLY_SET outline;
     SHAPE_SIMPLE* shape = new SHAPE_SIMPLE();
 
-    aText->TransformBoundingBoxWithClearanceToPolygon( &outline, 0 );
+    aText->TransformBoundingBoxToPolygon( &outline, 0 );
 
     for( auto iter = outline.CIterate( 0 ); iter; iter++ )
         shape->Append( *iter );
@@ -1173,10 +1352,6 @@ bool PNS_KICAD_IFACE_BASE::syncGraphicalItem( PNS::NODE* aWorld, PCB_SHAPE* aIte
             || aItem->GetLayer() == Margin
             || IsCopperLayer( aItem->GetLayer() ) )
     {
-        // TODO: where do we handle filled polygons on copper layers?
-        if( aItem->GetShape() == SHAPE_T::POLY && aItem->IsFilled() )
-            return false;
-
         std::vector<SHAPE*> shapes = aItem->MakeEffectiveShapes();
 
         for( SHAPE* shape : shapes )
@@ -1240,7 +1415,7 @@ bool PNS_KICAD_IFACE::IsAnyLayerVisible( const LAYER_RANGE& aLayer ) const
 }
 
 
-bool PNS_KICAD_IFACE::IsFlashedOnLayer( const PNS::ITEM* aItem, int aLayer ) const
+bool PNS_KICAD_IFACE_BASE::IsFlashedOnLayer( const PNS::ITEM* aItem, int aLayer ) const
 {
     /// Default is all layers
     if( aLayer < 0 )
@@ -1254,14 +1429,14 @@ bool PNS_KICAD_IFACE::IsFlashedOnLayer( const PNS::ITEM* aItem, int aLayer ) con
         {
             const PCB_VIA* via = static_cast<const PCB_VIA*>( aItem->Parent() );
 
-            return via->FlashLayer( static_cast<PCB_LAYER_ID>( aLayer ) );
+            return via->FlashLayer( ToLAYER_ID( aLayer ) );
         }
 
         case PCB_PAD_T:
         {
             const PAD* pad = static_cast<const PAD*>( aItem->Parent() );
 
-            return pad->FlashLayer( static_cast<PCB_LAYER_ID>( aLayer ) );
+            return pad->FlashLayer( ToLAYER_ID( aLayer ) );
         }
 
         default:
@@ -1287,10 +1462,13 @@ bool PNS_KICAD_IFACE::IsItemVisible( const PNS::ITEM* aItem ) const
     if( settings->GetHighContrast() )
         isOnVisibleLayer = item->IsOnLayer( settings->GetPrimaryHighContrastLayer() );
 
-    if( m_view->IsVisible( item ) && isOnVisibleLayer
-            && item->ViewGetLOD( item->GetLayer(), m_view ) < m_view->GetScale() )
+    if( m_view->IsVisible( item ) && isOnVisibleLayer )
     {
-        return true;
+        for( PCB_LAYER_ID layer : item->GetLayerSet().Seq() )
+        {
+            if( item->ViewGetLOD( layer, m_view ) < m_view->GetScale() )
+                return true;
+        }
     }
 
     // Items hidden in the router are not hidden on the board
@@ -1315,7 +1493,7 @@ void PNS_KICAD_IFACE_BASE::SyncWorld( PNS::NODE *aWorld )
 
     for( BOARD_ITEM* gitem : m_board->Drawings() )
     {
-        if ( gitem->Type() == PCB_SHAPE_T )
+        if ( gitem->Type() == PCB_SHAPE_T || gitem->Type() == PCB_TEXTBOX_T )
         {
             syncGraphicalItem( aWorld, static_cast<PCB_SHAPE*>( gitem ) );
         }
@@ -1348,7 +1526,7 @@ void PNS_KICAD_IFACE_BASE::SyncWorld( PNS::NODE *aWorld )
             if( pad->GetProperty() == PAD_PROP::CASTELLATED )
             {
                 std::unique_ptr<SHAPE> hole;
-                hole.reset( new SHAPE_SEGMENT( *pad->GetEffectiveHoleShape() ) );
+                hole.reset( pad->GetEffectiveHoleShape()->Clone() );
                 aWorld->AddEdgeExclusion( std::move( hole ) );
             }
         }
@@ -1359,12 +1537,9 @@ void PNS_KICAD_IFACE_BASE::SyncWorld( PNS::NODE *aWorld )
         for( FP_ZONE* zone : footprint->Zones() )
             syncZone( aWorld, zone, boardOutline );
 
-        if( footprint->IsNetTie() )
-            continue;
-
         for( BOARD_ITEM* mgitem : footprint->GraphicalItems() )
         {
-            if( mgitem->Type() == PCB_FP_SHAPE_T )
+            if( mgitem->Type() == PCB_FP_SHAPE_T || mgitem->Type() == PCB_FP_TEXTBOX_T )
             {
                 syncGraphicalItem( aWorld, static_cast<PCB_SHAPE*>( mgitem ) );
             }
@@ -1430,7 +1605,7 @@ void PNS_KICAD_IFACE_BASE::SetDebugDecorator( PNS::DEBUG_DECORATOR *aDec )
 }
 
 
-void PNS_KICAD_IFACE::DisplayItem( const PNS::ITEM* aItem, int aClearance, bool aEdit )
+void PNS_KICAD_IFACE::DisplayItem( const PNS::ITEM* aItem, int aClearance, bool aEdit, bool aIsHeadTrace )
 {
     if( aItem->IsVirtual() )
         return;
@@ -1445,18 +1620,20 @@ void PNS_KICAD_IFACE::DisplayItem( const PNS::ITEM* aItem, int aClearance, bool
     {
         pitem->SetClearance( aClearance );
 
-        switch( m_dispOptions->m_ShowTrackClearanceMode )
+        auto* settings = static_cast<PCBNEW_SETTINGS*>( m_tool->GetManager()->GetSettings() );
+
+        switch( settings->m_Display.m_TrackClearance )
         {
-        case PCB_DISPLAY_OPTIONS::SHOW_TRACK_CLEARANCE_WITH_VIA_ALWAYS:
-        case PCB_DISPLAY_OPTIONS::SHOW_WHILE_ROUTING_OR_DRAGGING:
+        case SHOW_WITH_VIA_ALWAYS:
+        case SHOW_WITH_VIA_WHILE_ROUTING_OR_DRAGGING:
             pitem->ShowClearance( aItem->OfKind( tracksOrVias ) );
             break;
 
-        case PCB_DISPLAY_OPTIONS::SHOW_TRACK_CLEARANCE_WITH_VIA_WHILE_ROUTING:
+        case SHOW_WITH_VIA_WHILE_ROUTING:
             pitem->ShowClearance( aItem->OfKind( tracksOrVias ) && !aEdit );
             break;
 
-        case PCB_DISPLAY_OPTIONS::SHOW_TRACK_CLEARANCE_WHILE_ROUTING:
+        case SHOW_WHILE_ROUTING:
             pitem->ShowClearance( aItem->OfKind( tracks ) && !aEdit );
             break;
 
@@ -1466,15 +1643,64 @@ void PNS_KICAD_IFACE::DisplayItem( const PNS::ITEM* aItem, int aClearance, bool
         }
     }
 
+    if( aIsHeadTrace )
+    {
+        pitem->SetIsHeadTrace( true );
+        pitem->Update( aItem );
+    }
+
     m_previewItems->Add( pitem );
     m_view->Update( m_previewItems );
 }
 
 
-void PNS_KICAD_IFACE::DisplayRatline( const SHAPE_LINE_CHAIN& aRatline, int aColor )
+void PNS_KICAD_IFACE::DisplayPathLine( const SHAPE_LINE_CHAIN& aLine, int aImportance )
 {
-    ROUTER_PREVIEW_ITEM* pitem = new ROUTER_PREVIEW_ITEM( nullptr, m_view );
-    pitem->Line( aRatline, 10000, aColor );
+    ROUTER_PREVIEW_ITEM* pitem = new ROUTER_PREVIEW_ITEM( aLine, m_view );
+    pitem->SetDepth( ROUTER_PREVIEW_ITEM::PathOverlayDepth );
+
+    COLOR4D color;
+
+    if( aImportance >= 1 )
+        color = COLOR4D( 1.0, 1.0, 0.0, 0.6 );
+    else if( aImportance == 0 )
+        color = COLOR4D( 0.7, 0.7, 0.7, 0.6 );
+
+    pitem->SetColor( color );
+
+    m_previewItems->Add( pitem );
+    m_view->Update( m_previewItems );
+}
+
+
+void PNS_KICAD_IFACE::DisplayRatline( const SHAPE_LINE_CHAIN& aRatline, int aNetCode )
+{
+    ROUTER_PREVIEW_ITEM* pitem = new ROUTER_PREVIEW_ITEM( aRatline, m_view );
+
+    KIGFX::RENDER_SETTINGS*     renderSettings = m_view->GetPainter()->GetSettings();
+    KIGFX::PCB_RENDER_SETTINGS* rs = static_cast<KIGFX::PCB_RENDER_SETTINGS*>( renderSettings );
+    bool                        colorByNet = rs->GetNetColorMode() != NET_COLOR_MODE::OFF;
+    COLOR4D                     defaultColor = rs->GetColor( nullptr, LAYER_RATSNEST );
+    COLOR4D                     color = defaultColor;
+
+    std::shared_ptr<CONNECTIVITY_DATA>  connectivity = m_board->GetConnectivity();
+    std::set<int>                       highlightedNets = rs->GetHighlightNetCodes();
+    std::map<int, KIGFX::COLOR4D>&      netColors = rs->GetNetColorMap();
+    std::map<wxString, KIGFX::COLOR4D>& ncColors = rs->GetNetclassColorMap();
+    const std::map<int, wxString>&      ncMap = connectivity->GetNetclassMap();
+
+    if( colorByNet && netColors.count( aNetCode ) )
+        color = netColors.at( aNetCode );
+    else if( colorByNet && ncMap.count( aNetCode ) && ncColors.count( ncMap.at( aNetCode ) ) )
+        color = ncColors.at( ncMap.at( aNetCode ) );
+    else
+        color = defaultColor;
+
+    if( color == COLOR4D::UNSPECIFIED )
+        color = defaultColor;
+
+    pitem->SetColor( color.Brightened( 0.5 ).WithAlpha( std::min( 1.0, color.a + 0.4 ) ) );
+
     m_previewItems->Add( pitem );
     m_view->Update( m_previewItems );
 }
@@ -1670,29 +1896,34 @@ void PNS_KICAD_IFACE::Commit()
 
     EraseView();
 
-    for( auto fpOffset : m_fpOffsets )
+    for( const std::pair<const PAD*, OFFSET>& fpOffset : m_fpOffsets )
     {
-        VECTOR2I offset = fpOffset.second.p_new - fpOffset.second.p_old;
+        VECTOR2I   offset = fpOffset.second.p_new - fpOffset.second.p_old;
         FOOTPRINT* footprint = fpOffset.first->GetParent();
-
-        VECTOR2I p_orig = footprint->GetPosition();
-        VECTOR2I p_new = p_orig + offset;
+        VECTOR2I   p_orig = footprint->GetPosition();
+        VECTOR2I   p_new = p_orig + offset;
 
         if( processedFootprints.find( footprint ) != processedFootprints.end() )
             continue;
 
         processedFootprints.insert( footprint );
         m_commit->Modify( footprint );
-        footprint->SetPosition( wxPoint( p_new.x, p_new.y ) );
+        footprint->SetPosition( p_new );
     }
 
     m_fpOffsets.clear();
 
-    m_commit->Push( _( "Interactive Router" ) );
+    m_commit->Push( _( "Interactive Router" ), m_commitFlags );
     m_commit = std::make_unique<BOARD_COMMIT>( m_tool );
 }
 
 
+EDA_UNITS PNS_KICAD_IFACE::GetUnits() const
+{
+    return static_cast<EDA_UNITS>( m_tool->GetManager()->GetSettings()->m_System.units );
+}
+
+
 void PNS_KICAD_IFACE::SetView( KIGFX::VIEW* aView )
 {
     wxLogTrace( wxT( "PNS" ), wxT( "SetView %p" ), aView );
@@ -1725,7 +1956,6 @@ void PNS_KICAD_IFACE::SetView( KIGFX::VIEW* aView )
 void PNS_KICAD_IFACE::UpdateNet( int aNetCode )
 {
     wxLogTrace( wxT( "PNS" ), wxT( "Update-net %d" ), aNetCode );
-
 }
 
 
@@ -1740,9 +1970,3 @@ void PNS_KICAD_IFACE::SetHostTool( PCB_TOOL_BASE* aTool )
     m_tool = aTool;
     m_commit = std::make_unique<BOARD_COMMIT>( m_tool );
 }
-
-
-void PNS_KICAD_IFACE::SetDisplayOptions( const PCB_DISPLAY_OPTIONS* aDispOptions )
-{
-    m_dispOptions = aDispOptions;
-}
