@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2017-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2017-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,6 +23,7 @@
 
 
 #include "pad_tool.h"
+#include "pcb_painter.h"
 #include <macros.h>
 #include <class_draw_panel_gal.h>
 #include <view/view_controls.h>
@@ -41,7 +42,11 @@
 #include <tools/pcb_selection_conditions.h>
 #include <tools/edit_tool.h>
 #include <dialogs/dialog_enum_pads.h>
-#include <widgets/infobar.h>
+#include <widgets/wx_infobar.h>
+
+
+using KIGFX::PCB_RENDER_SETTINGS;
+
 
 PAD_TOOL::PAD_TOOL() :
         PCB_TOOL_BASE( "pcbnew.PadTool" ),
@@ -59,7 +64,18 @@ void PAD_TOOL::Reset( RESET_REASON aReason )
     if( aReason == MODEL_RELOAD )
         m_lastPadNumber = wxT( "1" );
 
-    m_editPad = niluuid;
+    if( board() && board()->GetItem( m_editPad ) == DELETED_BOARD_ITEM::GetInstance() )
+    {
+        PCB_DISPLAY_OPTIONS opts = frame()->GetDisplayOptions();
+        bool highContrast = ( opts.m_ContrastModeDisplay != HIGH_CONTRAST_MODE::NORMAL );
+
+        if( m_wasHighContrast != highContrast )
+            m_toolMgr->RunAction( ACTIONS::highContrastMode, true );
+
+        frame()->GetInfoBar()->Dismiss();
+
+        m_editPad = niluuid;
+    }
 }
 
 
@@ -74,7 +90,7 @@ bool PAD_TOOL::Init()
 
         SELECTION_CONDITION padSel = SELECTION_CONDITIONS::HasType( PCB_PAD_T );
         SELECTION_CONDITION singlePadSel = SELECTION_CONDITIONS::Count( 1 ) &&
-                                           SELECTION_CONDITIONS::OnlyType( PCB_PAD_T );
+                                           SELECTION_CONDITIONS::OnlyTypes( { PCB_PAD_T } );
 
         auto explodeCondition =
                 [&]( const SELECTION& aSel )
@@ -111,7 +127,8 @@ bool PAD_TOOL::Init()
     ctxMenu.AddItem( PCB_ACTIONS::rotateCcw,               SELECTION_CONDITIONS::ShowAlways );
     ctxMenu.AddItem( PCB_ACTIONS::rotateCw,                SELECTION_CONDITIONS::ShowAlways );
     ctxMenu.AddItem( PCB_ACTIONS::flip,                    SELECTION_CONDITIONS::ShowAlways );
-    ctxMenu.AddItem( PCB_ACTIONS::mirror,                  SELECTION_CONDITIONS::ShowAlways );
+    ctxMenu.AddItem( PCB_ACTIONS::mirrorH,                 SELECTION_CONDITIONS::ShowAlways );
+    ctxMenu.AddItem( PCB_ACTIONS::mirrorV,                 SELECTION_CONDITIONS::ShowAlways );
     ctxMenu.AddItem( PCB_ACTIONS::properties,              SELECTION_CONDITIONS::ShowAlways );
 
     // Finally, add the standard zoom/grid items
@@ -178,7 +195,7 @@ static void doPushPadProperties( BOARD& board, const PAD& aSrcPad, BOARD_COMMIT&
 {
     const FOOTPRINT* refFootprint = aSrcPad.GetParent();
 
-    double pad_orient = aSrcPad.GetOrientation() - refFootprint->GetOrientation();
+    EDA_ANGLE srcPadAngle = aSrcPad.GetOrientation() - refFootprint->GetOrientation();
 
     for( FOOTPRINT* footprint : board.Footprints() )
     {
@@ -188,14 +205,14 @@ static void doPushPadProperties( BOARD& board, const PAD& aSrcPad, BOARD_COMMIT&
         if( footprint->GetFPID() != refFootprint->GetFPID() )
             continue;
 
-        for( auto pad : footprint->Pads() )
+        for( PAD* pad : footprint->Pads() )
         {
             if( aPadShapeFilter && ( pad->GetShape() != aSrcPad.GetShape() ) )
                 continue;
 
-            double currpad_orient = pad->GetOrientation() - footprint->GetOrientation();
+            EDA_ANGLE padAngle = pad->GetOrientation() - footprint->GetOrientation();
 
-            if( aPadOrientFilter && ( currpad_orient != pad_orient ) )
+            if( aPadOrientFilter && ( padAngle != srcPadAngle ) )
                 continue;
 
             if( aPadLayerFilter && ( pad->GetLayerSet() != aSrcPad.GetLayerSet() ) )
@@ -268,9 +285,7 @@ int PAD_TOOL::EnumeratePads( const TOOL_EVENT& aEvent )
     if( !board()->GetFirstFootprint() || board()->GetFirstFootprint()->Pads().empty() )
         return 0;
 
-    GENERAL_COLLECTOR collector;
-    const KICAD_T types[] = { PCB_PAD_T, EOT };
-
+    GENERAL_COLLECTOR        collector;
     GENERAL_COLLECTORS_GUIDE guide = frame()->GetCollectorsGuide();
     guide.SetIgnoreMTextsMarkedNoShow( true );
     guide.SetIgnoreMTextsOnBack( true );
@@ -290,8 +305,7 @@ int PAD_TOOL::EnumeratePads( const TOOL_EVENT& aEvent )
 
     m_toolMgr->RunAction( PCB_ACTIONS::selectionClear, true );
 
-    std::string tool = aEvent.GetCommandStr().get();
-    frame()->PushTool( tool );
+    frame()->PushTool( aEvent );
 
     VECTOR2I        oldCursorPos;  // store the previous mouse cursor position, during mouse drag
     std::list<PAD*> selectedPads;
@@ -311,20 +325,22 @@ int PAD_TOOL::EnumeratePads( const TOOL_EVENT& aEvent )
     auto setCursor =
             [&]()
             {
-                frame()->GetCanvas()->SetCurrentCursor( KICURSOR::BULLSEYE );
+                canvas()->SetCurrentCursor( KICURSOR::BULLSEYE );
             };
 
     Activate();
     // Must be done after Activate() so that it gets set into the correct context
     getViewControls()->ShowCursor( true );
+    getViewControls()->ForceCursorPosition( false );
     // Set initial cursor
     setCursor();
 
     STATUS_TEXT_POPUP statusPopup( frame() );
-    wxString msg = _( "Click on pad %s%d\nPress <esc> to cancel or double-click to commit" );
+    wxString msg = _( "Click on pad %s%d\nPress <esc> to cancel all; double-click to finish" );
     statusPopup.SetText( wxString::Format( msg, padPrefix, seqPadNum ) );
     statusPopup.Popup();
     statusPopup.Move( wxGetMousePosition() + wxPoint( 20, 20 ) );
+    canvas()->SetStatusPopup( statusPopup.GetPanel() );
 
     while( TOOL_EVENT* evt = Wait() )
     {
@@ -338,14 +354,14 @@ int PAD_TOOL::EnumeratePads( const TOOL_EVENT& aEvent )
             m_toolMgr->RunAction( PCB_ACTIONS::selectionClear, true );
             commit.Revert();
 
-            frame()->PopTool( tool );
+            frame()->PopTool( aEvent );
             break;
         }
         else if( evt->IsActivate() )
         {
             commit.Push( _( "Renumber pads" ) );
 
-            frame()->PopTool( tool );
+            frame()->PopTool( aEvent );
             break;
         }
         else if( evt->IsDrag( BUT_LEFT ) || evt->IsClick( BUT_LEFT ) )
@@ -364,7 +380,7 @@ int PAD_TOOL::EnumeratePads( const TOOL_EVENT& aEvent )
             // the mouse move and search pads that are on the line.
             int distance = ( cursorPos - oldCursorPos ).EuclideanNorm();
             // Search will be made every 0.1 mm:
-            int segments = distance / int( 0.1*IU_PER_MM ) + 1;
+            int           segments = distance / int( 0.1 * pcbIUScale.IU_PER_MM ) + 1;
             const wxPoint line_step( ( cursorPos - oldCursorPos ) / segments );
 
             collector.Empty();
@@ -372,7 +388,7 @@ int PAD_TOOL::EnumeratePads( const TOOL_EVENT& aEvent )
             for( int j = 0; j < segments; ++j )
             {
                 wxPoint testpoint( cursorPos.x - j * line_step.x, cursorPos.y - j * line_step.y );
-                collector.Collect( board(), types, testpoint, guide );
+                collector.Collect( board(), { PCB_PAD_T }, testpoint, guide );
 
                 for( int i = 0; i < collector.GetCount(); ++i )
                     selectedPads.push_back( static_cast<PAD*>( collector[i] ) );
@@ -437,11 +453,10 @@ int PAD_TOOL::EnumeratePads( const TOOL_EVENT& aEvent )
                 }
             }
         }
-        else if( ( evt->IsKeyPressed() && evt->KeyCode() == WXK_RETURN ) ||
-                 evt->IsDblClick( BUT_LEFT ) )
+        else if( evt->IsDblClick( BUT_LEFT ) )
         {
             commit.Push( _( "Renumber pads" ) );
-            frame()->PopTool( tool );
+            frame()->PopTool( aEvent );
             break;
         }
         else if( evt->IsClick( BUT_RIGHT ) )
@@ -465,8 +480,11 @@ int PAD_TOOL::EnumeratePads( const TOOL_EVENT& aEvent )
         getView()->Update( p );
     }
 
+    canvas()->SetStatusPopup( nullptr );
     statusPopup.Hide();
-    frame()->GetCanvas()->SetCurrentCursor( KICURSOR::ARROW );
+
+    canvas()->SetCurrentCursor( KICURSOR::ARROW );
+    getViewControls()->ForceCursorPosition( false );
     return 0;
 }
 
@@ -544,7 +562,7 @@ int PAD_TOOL::PlacePad( const TOOL_EVENT& aEvent )
 
     PAD_PLACER placer( this );
 
-    doInteractiveItemPlacement( aEvent.GetCommandStr().get(), &placer,  _( "Place pad" ),
+    doInteractiveItemPlacement( aEvent, &placer, _( "Place pad" ),
                                 IPO_REPEAT | IPO_SINGLE_CLICK | IPO_ROTATE | IPO_FLIP );
 
     return 0;
@@ -553,17 +571,22 @@ int PAD_TOOL::PlacePad( const TOOL_EVENT& aEvent )
 
 int PAD_TOOL::EditPad( const TOOL_EVENT& aEvent )
 {
-    PCB_DISPLAY_OPTIONS opts = frame()->GetDisplayOptions();
-    WX_INFOBAR*         infoBar = frame()->GetInfoBar();
-    PCB_SELECTION&      selection = m_toolMgr->GetTool<PCB_SELECTION_TOOL>()->GetSelection();
-    wxString            msg;
+    PCB_DISPLAY_OPTIONS  opts = frame()->GetDisplayOptions();
+    PCB_RENDER_SETTINGS* settings = static_cast<PCB_RENDER_SETTINGS*>( view()->GetPainter()->GetSettings() );
+    WX_INFOBAR*          infoBar = frame()->GetInfoBar();
+    PCB_SELECTION&       selection = m_toolMgr->GetTool<PCB_SELECTION_TOOL>()->GetSelection();
+    wxString             msg;
 
     if( m_editPad != niluuid )
     {
         PAD* pad = dynamic_cast<PAD*>( frame()->GetItem( m_editPad ) );
 
         if( pad )
-            recombinePad( pad );
+        {
+            BOARD_COMMIT commit( frame() );
+            RecombinePad( pad, false, commit );
+            commit.Push( _( "Recombine pad" ) );
+        }
 
         m_editPad = niluuid;
     }
@@ -575,14 +598,22 @@ int PAD_TOOL::EditPad( const TOOL_EVENT& aEvent )
         m_wasHighContrast = ( opts.m_ContrastModeDisplay != HIGH_CONTRAST_MODE::NORMAL );
         frame()->SetActiveLayer( layer );
 
+        settings->m_PadEditModePad = pad;
+
+        canvas()->GetView()->UpdateAllItemsConditionally( KIGFX::REPAINT,
+                [&]( KIGFX::VIEW_ITEM* aItem ) -> bool
+                {
+                    return dynamic_cast<PAD*>( aItem ) != nullptr;
+                } );
+
         if( !m_wasHighContrast )
-            m_toolMgr->RunAction( ACTIONS::highContrastMode, false );
+            m_toolMgr->RunAction( ACTIONS::highContrastMode, true );
 
         if( PCB_ACTIONS::explodePad.GetHotKey() == PCB_ACTIONS::recombinePad.GetHotKey() )
         {
             msg.Printf( _( "Pad Edit Mode.  Press %s again to exit." ),
-                        KeyNameFromKeyCode( PCB_ACTIONS::recombinePad.GetHotKey() ) );}
-
+                        KeyNameFromKeyCode( PCB_ACTIONS::recombinePad.GetHotKey() ) );
+        }
         else
         {
             msg.Printf( _( "Pad Edit Mode.  Press %s to exit." ),
@@ -600,7 +631,21 @@ int PAD_TOOL::EditPad( const TOOL_EVENT& aEvent )
         bool highContrast = ( opts.m_ContrastModeDisplay != HIGH_CONTRAST_MODE::NORMAL );
 
         if( m_wasHighContrast != highContrast )
-            m_toolMgr->RunAction( ACTIONS::highContrastMode, false );
+            m_toolMgr->RunAction( ACTIONS::highContrastMode, true );
+
+        settings->m_PadEditModePad = nullptr;
+
+        // Note: KIGFX::REPAINT isn't enough for things that go from invisible to visible as
+        // they won't be found in the view layer's itemset for re-painting.
+        canvas()->GetView()->UpdateAllItemsConditionally( KIGFX::ALL,
+                [&]( KIGFX::VIEW_ITEM* aItem ) -> bool
+                {
+                    return dynamic_cast<PAD*>( aItem ) != nullptr;
+                } );
+
+        // Refresh now (otherwise there's an uncomfortably long pause while the infoBar
+        // closes before refresh).
+        canvas()->ForceRefresh();
 
         infoBar->Dismiss();
     }
@@ -630,8 +675,9 @@ PCB_LAYER_ID PAD_TOOL::explodePad( PAD* aPad )
             FP_SHAPE* shape = new FP_SHAPE( board()->GetFirstFootprint() );
 
             shape->SetShape( primitive->GetShape() );
+            shape->SetIsAnnotationProxy( primitive->IsAnnotationProxy());
             shape->SetFilled( primitive->IsFilled() );
-            shape->SetWidth( primitive->GetWidth() );
+            shape->SetStroke( primitive->GetStroke() );
 
             switch( shape->GetShape() )
             {
@@ -673,6 +719,7 @@ PCB_LAYER_ID PAD_TOOL::explodePad( PAD* aPad )
 
         aPad->SetShape( aPad->GetAnchorPadShape() );
         aPad->DeletePrimitivesList();
+        aPad->SetFlags( ENTERED );
         m_editPad = aPad->m_Uuid;
     }
 
@@ -682,30 +729,39 @@ PCB_LAYER_ID PAD_TOOL::explodePad( PAD* aPad )
 }
 
 
-void PAD_TOOL::recombinePad( PAD* aPad )
+std::vector<FP_SHAPE*> PAD_TOOL::RecombinePad( PAD* aPad, bool aIsDryRun, BOARD_COMMIT& aCommit )
 {
-    int  maxError = board()->GetDesignSettings().m_MaxError;
+    int        maxError = board()->GetDesignSettings().m_MaxError;
+    FOOTPRINT* footprint = static_cast<FOOTPRINT*>( aPad->GetParentFootprint() );
+
+    // Don't leave an object in the point editor that might no longer exist after
+    // recombining the pad.
+    m_toolMgr->RunAction( PCB_ACTIONS::selectionClear, true );
+
+    for( BOARD_ITEM* item : footprint->GraphicalItems() )
+        item->ClearFlags( SKIP_STRUCT );
 
     auto findNext =
             [&]( PCB_LAYER_ID aLayer ) -> FP_SHAPE*
             {
                 SHAPE_POLY_SET padPoly;
-                aPad->TransformShapeWithClearanceToPolygon( padPoly, aLayer, 0, maxError,
-                                                            ERROR_INSIDE );
+                aPad->TransformShapeToPolygon( padPoly, aLayer, 0, maxError, ERROR_INSIDE );
 
-                for( BOARD_ITEM* item : board()->GetFirstFootprint()->GraphicalItems() )
+                for( BOARD_ITEM* item : footprint->GraphicalItems() )
                 {
-                    PCB_SHAPE* shape = dynamic_cast<PCB_SHAPE*>( item );
+                    FP_SHAPE* shape = dynamic_cast<FP_SHAPE*>( item );
 
-                    if( !shape || ( shape->GetEditFlags() & STRUCT_DELETED ) )
+                    if( !shape || ( shape->GetFlags() & SKIP_STRUCT ) )
                         continue;
 
                     if( shape->GetLayer() != aLayer )
                         continue;
 
+                    if( shape->IsAnnotationProxy() )    // Pad number (and net name) box
+                        return (FP_SHAPE*) item;
+
                     SHAPE_POLY_SET drawPoly;
-                    shape->TransformShapeWithClearanceToPolygon( drawPoly, aLayer, 0, maxError,
-                                                                 ERROR_INSIDE );
+                    shape->TransformShapeToPolygon( drawPoly, aLayer, 0, maxError, ERROR_INSIDE );
                     drawPoly.BooleanIntersection( padPoly, SHAPE_POLY_SET::PM_FAST );
 
                     if( !drawPoly.IsEmpty() )
@@ -715,8 +771,30 @@ void PAD_TOOL::recombinePad( PAD* aPad )
                 return nullptr;
             };
 
-    BOARD_COMMIT commit( frame() );
-    PCB_LAYER_ID layer;
+    auto findMatching =
+            [&]( FP_SHAPE* aShape ) -> std::vector<FP_SHAPE*>
+            {
+                std::vector<FP_SHAPE*> matching;
+
+                for( BOARD_ITEM* item : footprint->GraphicalItems() )
+                {
+                    FP_SHAPE* other = dynamic_cast<FP_SHAPE*>( item );
+
+                    if( !other || ( other->GetFlags() & SKIP_STRUCT ) )
+                        continue;
+
+                    if( aPad->GetLayerSet().test( other->GetLayer() )
+                            && aShape->Compare( other ) == 0 )
+                    {
+                        matching.push_back( other );
+                    }
+                }
+
+                return matching;
+            };
+
+    PCB_LAYER_ID           layer;
+    std::vector<FP_SHAPE*> mergedShapes;
 
     if( aPad->IsOnLayer( F_Cu ) )
         layer = F_Cu;
@@ -727,89 +805,116 @@ void PAD_TOOL::recombinePad( PAD* aPad )
 
     while( FP_SHAPE* fpShape = findNext( layer ) )
     {
-        commit.Modify( aPad );
-
-        // We've found an intersecting item.  First convert the pad to a custom-shape
-        // pad (if it isn't already)
+        // We've found an intersecting item to combine.
         //
-        if( aPad->GetShape() == PAD_SHAPE::RECT || aPad->GetShape() == PAD_SHAPE::CIRCLE )
+        fpShape->SetFlags( SKIP_STRUCT );
+
+        // First convert the pad to a custom-shape pad (if it isn't already)
+        //
+        if( !aIsDryRun )
         {
-            aPad->SetAnchorPadShape( aPad->GetShape() );
+            aCommit.Modify( aPad );
+
+            if( aPad->GetShape() == PAD_SHAPE::RECT || aPad->GetShape() == PAD_SHAPE::CIRCLE )
+            {
+                aPad->SetAnchorPadShape( aPad->GetShape() );
+            }
+            else if( aPad->GetShape() != PAD_SHAPE::CUSTOM )
+            {
+                // Create a new minimally-sized circular anchor and convert existing pad
+                // to a polygon primitive
+                SHAPE_POLY_SET existingOutline;
+                aPad->TransformShapeToPolygon( existingOutline, layer, 0, maxError, ERROR_INSIDE );
+
+                aPad->SetAnchorPadShape( PAD_SHAPE::CIRCLE );
+
+                if( aPad->GetSizeX() > aPad->GetSizeY() )
+                    aPad->SetSizeX( aPad->GetSizeY() );
+
+                aPad->SetOffset( VECTOR2I( 0, 0 ) );
+
+                PCB_SHAPE* shape = new PCB_SHAPE( nullptr, SHAPE_T::POLY );
+                shape->SetFilled( true );
+                shape->SetStroke( STROKE_PARAMS( 0, PLOT_DASH_TYPE::SOLID ) );
+                shape->SetPolyShape( existingOutline );
+                shape->Move( - aPad->GetPosition() );
+                shape->Rotate( VECTOR2I( 0, 0 ), - aPad->GetOrientation() );
+
+                aPad->AddPrimitive( shape );
+            }
+
+            aPad->SetShape( PAD_SHAPE::CUSTOM );
         }
-        else if( aPad->GetShape() != PAD_SHAPE::CUSTOM )
-        {
-            // Create a new minimally-sized circular anchor and convert existing pad
-            // to a polygon primitive
-            SHAPE_POLY_SET existingOutline;
-            aPad->TransformShapeWithClearanceToPolygon( existingOutline, layer, 0, maxError,
-                                                        ERROR_INSIDE );
-
-            aPad->SetAnchorPadShape( PAD_SHAPE::CIRCLE );
-            if( aPad->GetSizeX() > aPad->GetSizeY() )
-                aPad->SetSizeX( aPad->GetSizeY() );
-
-            aPad->SetOffset( wxPoint( 0, 0 ) );
-
-            PCB_SHAPE* shape = new PCB_SHAPE( nullptr, SHAPE_T::POLY );
-            shape->SetFilled( true );
-            shape->SetWidth( 0 );
-            shape->SetPolyShape( existingOutline );
-            shape->Move( - aPad->GetPosition() );
-            shape->Rotate( wxPoint( 0, 0 ), - aPad->GetOrientation() );
-
-            aPad->AddPrimitive( shape );
-        }
-
-        aPad->SetShape( PAD_SHAPE::CUSTOM );
 
         // Now add the new shape to the primitives list
         //
-        PCB_SHAPE* pcbShape = new PCB_SHAPE;
+        mergedShapes.push_back( fpShape );
 
-        pcbShape->SetShape( fpShape->GetShape() );
-        pcbShape->SetFilled( fpShape->IsFilled() );
-        pcbShape->SetWidth( fpShape->GetWidth() );
-
-
-        switch( pcbShape->GetShape() )
+        if( !aIsDryRun )
         {
-        case SHAPE_T::SEGMENT:
-        case SHAPE_T::RECT:
-        case SHAPE_T::CIRCLE:
-            pcbShape->SetStart( fpShape->GetStart() );
-            pcbShape->SetEnd( fpShape->GetEnd() );
-            break;
+            PCB_SHAPE* pcbShape = new PCB_SHAPE;
 
-        case SHAPE_T::ARC:
-            pcbShape->SetStart( fpShape->GetStart() );
-            pcbShape->SetEnd( fpShape->GetEnd() );
-            pcbShape->SetCenter( fpShape->GetCenter() );
-            break;
+            pcbShape->SetShape( fpShape->GetShape() );
+            pcbShape->SetFilled( fpShape->IsFilled() );
+            pcbShape->SetStroke( fpShape->GetStroke() );
 
-        case SHAPE_T::BEZIER:
-            pcbShape->SetStart( fpShape->GetStart() );
-            pcbShape->SetEnd( fpShape->GetEnd() );
-            pcbShape->SetBezierC1( fpShape->GetBezierC1() );
-            pcbShape->SetBezierC2( fpShape->GetBezierC2() );
-            break;
+            switch( pcbShape->GetShape() )
+            {
+            case SHAPE_T::SEGMENT:
+            case SHAPE_T::RECT:
+            case SHAPE_T::CIRCLE:
+                pcbShape->SetStart( fpShape->GetStart() );
+                pcbShape->SetEnd( fpShape->GetEnd() );
+                break;
 
-        case SHAPE_T::POLY:
-            pcbShape->SetPolyShape( fpShape->GetPolyShape() );
-            break;
+            case SHAPE_T::ARC:
+                pcbShape->SetStart( fpShape->GetStart() );
+                pcbShape->SetEnd( fpShape->GetEnd() );
+                pcbShape->SetCenter( fpShape->GetCenter() );
+                break;
 
-        default:
-            UNIMPLEMENTED_FOR( pcbShape->SHAPE_T_asString() );
+            case SHAPE_T::BEZIER:
+                pcbShape->SetStart( fpShape->GetStart() );
+                pcbShape->SetEnd( fpShape->GetEnd() );
+                pcbShape->SetBezierC1( fpShape->GetBezierC1() );
+                pcbShape->SetBezierC2( fpShape->GetBezierC2() );
+                break;
+
+            case SHAPE_T::POLY:
+                pcbShape->SetPolyShape( fpShape->GetPolyShape() );
+                break;
+
+            default:
+                UNIMPLEMENTED_FOR( pcbShape->SHAPE_T_asString() );
+            }
+
+            pcbShape->Move( - aPad->GetPosition() );
+            pcbShape->Rotate( VECTOR2I( 0, 0 ), - aPad->GetOrientation() );
+            pcbShape->SetIsAnnotationProxy( fpShape->IsAnnotationProxy());
+            aPad->AddPrimitive( pcbShape );
+
+            aCommit.Remove( fpShape );
         }
 
-        pcbShape->Move( - aPad->GetPosition() );
-        pcbShape->Rotate( wxPoint( 0, 0 ), - aPad->GetOrientation() );
-        aPad->AddPrimitive( pcbShape );
+        // See if there are other shapes that match and mark them for delete.  (KiCad won't
+        // produce these, but old footprints from other vendors have them.)
+        for( FP_SHAPE* other : findMatching( fpShape ) )
+        {
+            other->SetFlags( SKIP_STRUCT );
+            mergedShapes.push_back( other );
 
-        fpShape->SetFlags( STRUCT_DELETED );
-        commit.Remove( fpShape );
+            if( !aIsDryRun )
+                aCommit.Remove( other );
+        }
     }
 
-    commit.Push(_("Recombine pads") );
+    for( BOARD_ITEM* item : footprint->GraphicalItems() )
+        item->ClearFlags( SKIP_STRUCT );
+
+    if( !aIsDryRun )
+        aPad->ClearFlags( ENTERED );
+
+    return mergedShapes;
 }
 
 
