@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2019 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -34,9 +34,9 @@
 #include <layer_ids.h>
 #include <geometry/shape_poly_set.h>
 #include <zone_settings.h>
+#include <teardrop/teardrop_types.h>
 
 
-class EDA_RECT;
 class LINE_READER;
 class PCB_EDIT_FRAME;
 class BOARD;
@@ -84,66 +84,61 @@ public:
         return !GetIsRuleArea();
     }
 
-    NETCLASS* GetNetClass() const override
-    {
-        if( GetIsRuleArea() )
-            return nullptr;
-
-        return BOARD_CONNECTED_ITEM::GetNetClass();
-    }
-
-    wxString GetNetClassName() const override
-    {
-        if( GetIsRuleArea() )
-            return "UNDEFINED";
-
-        return BOARD_CONNECTED_ITEM::GetNetClassName();
-    }
-
     /**
      * Copy aZone data to me
      */
     void InitDataFromSrcInCopyCtor( const ZONE& aZone );
 
     /**
-     * @return a wxPoint, position of the first point of the outline
+     * For rule areas which exclude footprints (and therefore participate in courtyard conflicts
+     * during move).
      */
-    wxPoint GetPosition() const override;
-    void SetPosition( const wxPoint& aPos ) override {}
+    bool IsConflicting() const;
+
+    /**
+     * @return a VECTOR2I, position of the first point of the outline
+     */
+    VECTOR2I GetPosition() const override;
+    void     SetPosition( const VECTOR2I& aPos ) override {}
 
     /**
      * @param aPriority is the priority level.
      */
-    void SetPriority( unsigned aPriority ) { m_priority = aPriority; }
+    void SetAssignedPriority( unsigned aPriority ) { m_priority = aPriority; }
 
     /**
      * @return the priority level of this zone.
      */
-    unsigned GetPriority() const { return m_priority; }
+    unsigned GetAssignedPriority() const { return m_priority; }
+
+    bool HigherPriority( const ZONE* aOther ) const;
+
+    bool SameNet( const ZONE* aOther ) const;
 
     void GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITEM>& aList ) override;
+    wxString GetFriendlyName() const override;
 
     void SetLayerSet( LSET aLayerSet ) override;
-    virtual LSET GetLayerSet() const override;
+    virtual LSET GetLayerSet() const override { return m_layerSet; }
 
     wxString GetZoneName() const { return m_zoneName; }
     void SetZoneName( const wxString& aName ) { m_zoneName = aName; }
 
-    bool Matches( const wxFindReplaceData& aSearchData, void* aAuxData ) const override
+    bool Matches( const EDA_SEARCH_DATA& aSearchData, void* aAuxData ) const override
     {
         return BOARD_ITEM::Matches( GetZoneName(), aSearchData );
     }
 
     /**
-     * @return an EDA_RECT that is the bounding box of the zone outline.
+     * @return the bounding box of the zone outline.
      */
-    const EDA_RECT GetBoundingBox() const override;
+    const BOX2I GetBoundingBox() const override;
 
     /**
-     * ONLY TO BE USED BY CLIENTS WHICH SET UP THE CACHE!
+     * Used to preload the zone bounding box cache so we don't have to worry about mutex-locking
+     * it each time.
      */
-    const EDA_RECT GetCachedBoundingBox() const { return m_bboxCache; }
-    void CacheBoundingBox() { m_bboxCache = GetBoundingBox(); }
+    void CacheBoundingBox();
 
     /**
      * Return any local clearances set in the "classic" (ie: pre-rule) system.  These are
@@ -162,15 +157,13 @@ public:
      */
     bool IsOnCopperLayer() const override;
 
-    /**
-     * Test if this zone shares a common layer with the given layer set.
-     */
-    bool CommonLayerExists( const LSET aLayerSet ) const;
-
     virtual void SetLayer( PCB_LAYER_ID aLayer ) override;
 
     virtual PCB_LAYER_ID GetLayer() const override;
 
+    // Return the first layer in GUI sequence.
+    PCB_LAYER_ID GetFirstLayer() const;
+
     virtual bool IsOnLayer( PCB_LAYER_ID ) const override;
 
     virtual void ViewGetLayers( int aLayers[], int& aCount ) const override;
@@ -200,7 +193,6 @@ public:
     }
 
     int GetThermalReliefSpokeWidth() const { return m_thermalReliefSpokeWidth; }
-    int GetThermalReliefSpokeWidth( PAD* aPad, wxString* aSource = nullptr ) const;
 
     /**
      * Compute the area currently occupied by the zone fill.
@@ -209,6 +201,12 @@ public:
      */
     double CalculateFilledArea();
 
+    /**
+     * Compute the area of the zone outline (not the filled area).
+     * @return the currently calculated area
+     */
+    double CalculateOutlineArea();
+
     /**
      * This area is cached from the most recent call to CalculateFilledArea().
      *
@@ -219,6 +217,16 @@ public:
         return m_area;
     }
 
+    /**
+     * This area is cached from the most recent call to CalculateOutlineArea().
+     *
+     * @return the outline area
+     */
+    double GetOutlineArea()
+    {
+        return m_outlinearea;
+    }
+
     std::mutex& GetLock()
     {
         return m_lock;
@@ -226,10 +234,10 @@ public:
 
     int GetFillFlag( PCB_LAYER_ID aLayer )
     {
-        return m_fillFlags.count( aLayer ) ? m_fillFlags[ aLayer ] : false;
+        return m_fillFlags.test( aLayer );
     }
 
-    void SetFillFlag( PCB_LAYER_ID aLayer, bool aFlag ) { m_fillFlags[ aLayer ] = aFlag; }
+    void SetFillFlag( PCB_LAYER_ID aLayer, bool aFlag ) { m_fillFlags.set( aLayer, aFlag ); }
 
     bool IsFilled() const { return m_isFilled; }
     void SetIsFilled( bool isFilled ) { m_isFilled = isFilled; }
@@ -237,7 +245,6 @@ public:
     bool NeedRefill() const { return m_needRefill; }
     void SetNeedRefill( bool aNeedRefill ) { m_needRefill = aNeedRefill; }
 
-    ZONE_CONNECTION GetPadConnection( PAD* aPad, wxString* aSource = nullptr ) const;
     ZONE_CONNECTION GetPadConnection() const { return m_PadConnection; }
     void SetPadConnection( ZONE_CONNECTION aPadConnection ) { m_PadConnection = aPadConnection; }
 
@@ -256,8 +263,8 @@ public:
     int GetHatchGap() const { return m_hatchGap; }
     void SetHatchGap( int aStep ) { m_hatchGap = aStep; }
 
-    double GetHatchOrientation() const { return m_hatchOrientation; }
-    void SetHatchOrientation( double aStep ) { m_hatchOrientation = aStep; }
+    EDA_ANGLE GetHatchOrientation() const { return m_hatchOrientation; }
+    void SetHatchOrientation( const EDA_ANGLE& aStep ) { m_hatchOrientation = aStep; }
 
     int GetHatchSmoothingLevel() const { return m_hatchSmoothingLevel; }
     void SetHatchSmoothingLevel( int aLevel ) { m_hatchSmoothingLevel = aLevel; }
@@ -299,24 +306,9 @@ public:
     }
 
     ///
-    // Like HitTest but selects the current corner to be operated on
-    void SetSelectedCorner( const wxPoint& aPosition, int aAccuracy );
-
     int GetLocalFlags() const { return m_localFlgs; }
     void SetLocalFlags( int aFlags ) { m_localFlgs = aFlags; }
 
-    std::vector<SEG>& FillSegments( PCB_LAYER_ID aLayer )
-    {
-        wxASSERT( m_FillSegmList.count( aLayer ) );
-        return m_FillSegmList.at( aLayer );
-    }
-
-    const std::vector<SEG>& FillSegments( PCB_LAYER_ID aLayer ) const
-    {
-        wxASSERT( m_FillSegmList.count( aLayer ) );
-        return m_FillSegmList.at( aLayer );
-    }
-
     SHAPE_POLY_SET* Outline() { return m_Poly; }
     const SHAPE_POLY_SET* Outline() const { return m_Poly; }
 
@@ -330,20 +322,20 @@ public:
     /**
      * Test if a point is near an outline edge or a corner of this zone.
      *
-     * @param aPosition the wxPoint to test
+     * @param aPosition the VECTOR2I to test
      * @return true if a hit, else false
      */
-    bool HitTest( const wxPoint& aPosition, int aAccuracy = 0 ) const override;
+    bool HitTest( const VECTOR2I& aPosition, int aAccuracy = 0 ) const override;
 
     /**
-     * Test if the given wxPoint is within the bounds of a filled area of this zone.
+     * Test if the given VECTOR2I is within the bounds of a filled area of this zone.
      *
      * @param aLayer is the layer to test on
-     * @param aRefPos A wxPoint to test
+     * @param aRefPos A VECTOR2I to test
      * @param aAccuracy Expand the distance by which the areas are expanded for the hittest
      * @return true if a hit, else false
      */
-    bool HitTestFilledArea( PCB_LAYER_ID aLayer, const wxPoint &aRefPos, int aAccuracy = 0 ) const;
+    bool HitTestFilledArea( PCB_LAYER_ID aLayer, const VECTOR2I& aRefPos, int aAccuracy = 0 ) const;
 
     /**
      * Test if the given point is contained within a cutout of the zone.
@@ -356,18 +348,13 @@ public:
     bool HitTestCutout( const VECTOR2I& aRefPos, int* aOutlineIdx = nullptr,
                         int* aHoleIdx = nullptr ) const;
 
-    bool HitTestCutout( const wxPoint& aRefPos, int* aOutlineIdx = nullptr,
-                        int* aHoleIdx = nullptr ) const
-    {
-        return HitTestCutout( VECTOR2I( aRefPos.x, aRefPos.y ), aOutlineIdx, aHoleIdx );
-    }
-
     /**
      * Some intersecting zones, despite being on the same layer with the same net, cannot be
      * merged due to other parameters such as fillet radius.  The copper pour will end up
      * effectively merged though, so we need to do some calculations with them in mind.
      */
-    void GetInteractingZones( PCB_LAYER_ID aLayer, std::vector<ZONE*>* aZones ) const;
+    void GetInteractingZones( PCB_LAYER_ID aLayer, std::vector<ZONE*>* aSameNetCollidingZones,
+                              std::vector<ZONE*>* aOtherNetIntersectingZones ) const;
 
     /**
      * Convert solid areas full shapes to polygon set
@@ -376,11 +363,10 @@ public:
      * Arcs (ends of segments) are approximated by segments
      *
      * @param aLayer is the layer of the zone to retrieve
-     * @param aCornerBuffer = a buffer to store the polygons
+     * @param aBuffer = a buffer to store the polygons
      * @param aError = Maximum error allowed between true arc and polygon approx
      */
-    void TransformSolidAreasShapesToPolygon( PCB_LAYER_ID aLayer, SHAPE_POLY_SET& aCornerBuffer,
-                                             int aError = ARC_HIGH_DEF ) const;
+    void TransformSolidAreasShapesToPolygon( PCB_LAYER_ID aLayer, SHAPE_POLY_SET& aBuffer ) const;
 
     /**
      * Convert the outlines shape to a polygon with no holes
@@ -389,11 +375,11 @@ public:
      * Used in filling zones calculations
      * Circles (vias) and arcs (ends of tracks) are approximated by segments.
      *
-     * @param aCornerBuffer is a buffer to store the polygon
+     * @param aBuffer is a buffer to store the polygon
      * @param aClearance is the min clearance around outlines
      * @param aBoardOutline is the board outline (if a valid one exists; nullptr otherwise)
      */
-    void TransformSmoothedOutlineToPolygon( SHAPE_POLY_SET& aCornerBuffer, int aClearance,
+    void TransformSmoothedOutlineToPolygon( SHAPE_POLY_SET& aBuffer, int aClearance,
                                             int aError, ERROR_LOC aErrorLoc,
                                             SHAPE_POLY_SET* aBoardOutline ) const;
 
@@ -403,65 +389,45 @@ public:
      * Circles and arcs are approximated by segments
      *
      * @param aLayer is the layer of the filled zone to retrieve
-     * @param aCornerBuffer is a buffer to store the polygon
-     * @param aClearanceValue is the clearance around the pad
+     * @param aBuffer is a buffer to store the polygon
+     * @param aClearance is the clearance around the pad
      * @param aError is the maximum deviation from true circle
-     * @param ignoreLineWidth is used for edge cut items where the line width is only
-     * for visualization
+     * @param ignoreLineWidth is used for edge cut items where the line width is only for
+     *                        visualization
      */
-    void TransformShapeWithClearanceToPolygon( SHAPE_POLY_SET& aCornerBuffer,
-                                               PCB_LAYER_ID aLayer, int aClearanceValue,
-                                               int aError, ERROR_LOC aErrorLoc,
-                                               bool ignoreLineWidth = false ) const override;
+    void TransformShapeToPolygon( SHAPE_POLY_SET& aBuffer, PCB_LAYER_ID aLayer,
+                                  int aClearance, int aError, ERROR_LOC aErrorLoc,
+                                  bool ignoreLineWidth = false ) const override;
 
     /**
-     * Test if the given wxPoint is near a corner.
+     * Test if the given VECTOR2I is near a corner.
      *
-     * @param  refPos     is the wxPoint to test.
+     * @param  refPos     is the VECTOR2I to test.
      * @param  aAccuracy  increase the item bounding box by this amount.
-     * @param  aCornerHit [out] is the index of the closest vertex found, useless when return
-     *                    value is false.
+     * @param  aCornerHit [out, optional] is the index of the closest vertex found when return
+     *                    value is true
      * @return true if some corner was found to be closer to refPos than aClearance; false
      *         otherwise.
      */
-    bool HitTestForCorner( const wxPoint& refPos, int aAccuracy,
-                           SHAPE_POLY_SET::VERTEX_INDEX& aCornerHit ) const;
+    bool HitTestForCorner( const VECTOR2I& refPos, int aAccuracy,
+                           SHAPE_POLY_SET::VERTEX_INDEX* aCornerHit = nullptr ) const;
 
     /**
-     * Test if the given wxPoint is near a corner.
-     * @param  refPos     is the wxPoint to test.
-     * @param  aAccuracy  increase the item bounding box by this amount.
-     * @return true if some corner was found to be closer to refPos than aClearance; false
-     *         otherwise.
-     */
-    bool HitTestForCorner( const wxPoint& refPos, int aAccuracy ) const;
-
-    /**
-     * Test if the given wxPoint is near a segment defined by 2 corners.
+     * Test if the given VECTOR2I is near a segment defined by 2 corners.
      *
-     * @param  refPos     is the wxPoint to test.
+     * @param  refPos     is the VECTOR2I to test.
      * @param  aAccuracy  increase the item bounding box by this amount.
-     * @param  aCornerHit [out] is the index of the closest vertex found, useless when return
-     *                    value is false.
+     * @param  aCornerHit [out, optional] is the index of the closest vertex found when return
+     *                    value is true.
      * @return true if some edge was found to be closer to refPos than aClearance.
      */
-    bool HitTestForEdge( const wxPoint& refPos, int aAccuracy,
-                         SHAPE_POLY_SET::VERTEX_INDEX& aCornerHit ) const;
+    bool HitTestForEdge( const VECTOR2I& refPos, int aAccuracy,
+                         SHAPE_POLY_SET::VERTEX_INDEX* aCornerHit = nullptr ) const;
 
     /**
-     * Test if the given wxPoint is near a segment defined by 2 corners.
-     *
-     * @param  refPos     is the wxPoint to test.
-     * @param  aAccuracy  increase the item bounding box by this amount.
-     * @return true if some edge was found to be closer to refPos than aClearance.
+     * @copydoc BOARD_ITEM::HitTest(const BOX2I& aRect, bool aContained, int aAccuracy) const
      */
-    bool HitTestForEdge( const wxPoint& refPos, int aAccuracy ) const;
-
-    /**
-     * @copydoc BOARD_ITEM::HitTest(const EDA_RECT& aRect,
-     *                              bool aContained = true, int aAccuracy) const
-     */
-    bool HitTest( const EDA_RECT& aRect, bool aContained = true, int aAccuracy = 0 ) const override;
+    bool HitTest( const BOX2I& aRect, bool aContained = true, int aAccuracy = 0 ) const override;
 
     /**
      * Removes the zone filling.
@@ -477,7 +443,7 @@ public:
      *
      * @param offset is moving vector
      */
-    void Move( const wxPoint& offset ) override;
+    void Move( const VECTOR2I& offset ) override;
 
     /**
      * Move the outline Edge.
@@ -485,15 +451,14 @@ public:
      * @param offset is moving vector
      * @param aEdge is start point of the outline edge
      */
-    void MoveEdge( const wxPoint& offset, int aEdge );
+    void MoveEdge( const VECTOR2I& offset, int aEdge );
 
     /**
-     * Move the outlines.
+     * Rotate the outlines.
      *
      * @param aCentre is rot centre
-     * @param aAngle is in 0.1 degree
      */
-    void Rotate( const wxPoint& aCentre, double aAngle ) override;
+    void Rotate( const VECTOR2I& aCentre, const EDA_ANGLE& aAngle ) override;
 
     /**
      * Flip this object, i.e. change the board side for this object
@@ -501,7 +466,7 @@ public:
      *
      * @param aCentre is the rotation point.
      */
-    virtual void Flip( const wxPoint& aCentre, bool aFlipLeftRight ) override;
+    virtual void Flip( const VECTOR2I& aCentre, bool aFlipLeftRight ) override;
 
     /**
      * Mirror the outlines relative to a given horizontal axis the layer is not changed.
@@ -509,7 +474,7 @@ public:
      * @param aMirrorRef is axis position
      * @param aMirrorLeftRight mirror across Y axis (otherwise mirror across X)
      */
-    void Mirror( const wxPoint& aMirrorRef, bool aMirrorLeftRight );
+    void Mirror( const VECTOR2I& aMirrorRef, bool aMirrorLeftRight );
 
     /**
      * @return the class name.
@@ -573,7 +538,7 @@ public:
         return m_Poly->CVertex( index );
     }
 
-    void SetCornerPosition( int aCornerIndex, const wxPoint& new_pos )
+    void SetCornerPosition( int aCornerIndex, const VECTOR2I& new_pos )
     {
         SHAPE_POLY_SET::VERTEX_INDEX relativeIndices;
 
@@ -610,7 +575,7 @@ public:
      *                          even if it is duplicated.
      * @return true if the corner was added, false if error (aHoleIdx > hole count -1)
      */
-    bool AppendCorner( wxPoint aPosition, int aHoleIdx, bool aAllowDuplication = false );
+    bool AppendCorner( VECTOR2I aPosition, int aHoleIdx, bool aAllowDuplication = false );
 
     ZONE_BORDER_DISPLAY_STYLE GetHatchStyle() const { return m_borderStyle; }
     void SetHatchStyle( ZONE_BORDER_DISPLAY_STYLE aStyle ) { m_borderStyle = aStyle; }
@@ -634,12 +599,18 @@ public:
     /**
      * @return a reference to the list of filled polygons.
      */
-    const SHAPE_POLY_SET& GetFilledPolysList( PCB_LAYER_ID aLayer ) const
+    const std::shared_ptr<SHAPE_POLY_SET>& GetFilledPolysList( PCB_LAYER_ID aLayer ) const
     {
         wxASSERT( m_FilledPolysList.count( aLayer ) );
         return m_FilledPolysList.at( aLayer );
     }
 
+    SHAPE_POLY_SET* GetFill( PCB_LAYER_ID aLayer )
+    {
+        wxASSERT( m_FilledPolysList.count( aLayer ) );
+        return m_FilledPolysList.at( aLayer ).get();
+    }
+
     /**
      * Create a list of triangles that "fill" the solid areas used for instance to draw
      * these solid areas on OpenGL.
@@ -651,15 +622,7 @@ public:
      */
     void SetFilledPolysList( PCB_LAYER_ID aLayer, const SHAPE_POLY_SET& aPolysList )
     {
-        m_FilledPolysList[aLayer] = aPolysList;
-    }
-
-    /**
-     * Set the list of filled polygons.
-     */
-    void SetRawPolysList( PCB_LAYER_ID aLayer, const SHAPE_POLY_SET& aPolysList )
-    {
-        m_RawPolysList[aLayer] = aPolysList;
+        m_FilledPolysList[aLayer] = std::make_shared<SHAPE_POLY_SET>( aPolysList );
     }
 
     /**
@@ -688,12 +651,6 @@ public:
 
     unsigned int GetCornerRadius() const { return m_cornerRadius; }
 
-    bool GetFilledPolysUseThickness() const { return m_fillVersion == 5; }
-    bool GetFilledPolysUseThickness( PCB_LAYER_ID aLayer ) const;
-
-    int GetFillVersion() const { return m_fillVersion; }
-    void SetFillVersion( int aVersion ) { m_fillVersion = aVersion; }
-
     /**
      * Remove a cutout from the zone.
      *
@@ -708,27 +665,32 @@ public:
      * If the zone outline is empty, this is the main outline.  Otherwise it is a hole
      * inside the main outline.
      */
-    void AddPolygon( std::vector< wxPoint >& aPolygon );
+    void AddPolygon( std::vector<VECTOR2I>& aPolygon );
 
     void AddPolygon( const SHAPE_LINE_CHAIN& aPolygon );
 
-    void SetFillSegments( PCB_LAYER_ID aLayer, const std::vector<SEG>& aSegments )
-    {
-        m_FillSegmList[aLayer] = aSegments;
-    }
-
-    SHAPE_POLY_SET& RawPolysList( PCB_LAYER_ID aLayer )
-    {
-        wxASSERT( m_RawPolysList.count( aLayer ) );
-        return m_RawPolysList.at( aLayer );
-    }
-
-    wxString GetSelectMenuText( EDA_UNITS aUnits ) const override;
+    wxString GetItemDescription( UNITS_PROVIDER* aUnitsProvider ) const override;
 
     BITMAPS GetMenuImage() const override;
 
     EDA_ITEM* Clone() const override;
 
+    /**
+     * @return true if the zone is a teardrop area
+     */
+    bool IsTeardropArea() const { return m_teardropType != TEARDROP_TYPE::TD_NONE; }
+
+    /**
+     * Set the type of teardrop if the zone is a teardrop area
+     * for non teardrop area, the type must be TEARDROP_TYPE::TD_NONE
+     */
+    void SetTeardropAreaType( TEARDROP_TYPE aType ) { m_teardropType = aType; }
+
+    /**
+     * @return the type of the teardrop ( has meaning only if the zone is a teardrop area)
+     */
+    TEARDROP_TYPE GetTeardropAreaType() const { return m_teardropType; }
+
     /**
      * Accessors to parameters used in Rule Area zones:
      */
@@ -738,8 +700,6 @@ public:
     bool GetDoNotAllowTracks() const     { return m_doNotAllowTracks; }
     bool GetDoNotAllowPads() const       { return m_doNotAllowPads; }
     bool GetDoNotAllowFootprints() const { return m_doNotAllowFootprints; }
-    bool IsKeepout() const;
-    bool KeepoutAll() const;
 
     void SetIsRuleArea( bool aEnable )           { m_isRuleArea = aEnable; }
     void SetDoNotAllowCopperPour( bool aEnable ) { m_doNotAllowCopperPour = aEnable; }
@@ -771,21 +731,21 @@ public:
     /**
      * Set all hatch parameters for the zone.
      *
-     * @param  aHatchStyle   is the style of the hatch, specified as one of HATCH_STYLE possible
-     *                       values.
-     * @param  aHatchPitch   is the hatch pitch in iu.
-     * @param  aRebuildHatch is a flag to indicate whether to re-hatch after having set the
+     * @param  aBorderHatchStyle   is the style of the hatch, specified as one of HATCH_STYLE
+                                   possible values.
+     * @param  aBorderHatchPitch   is the hatch pitch in iu.
+     * @param  aRebuildBorderHatch is a flag to indicate whether to re-hatch after having set the
      *                       previous parameters.
      */
-    void SetBorderDisplayStyle( ZONE_BORDER_DISPLAY_STYLE aHatchStyle, int aHatchPitch,
-                                bool aRebuildHatch );
+    void SetBorderDisplayStyle( ZONE_BORDER_DISPLAY_STYLE aBorderHatchStyle, int aBorderHatchPitch,
+                                bool aRebuilBorderdHatch );
 
     /**
      * Set the hatch pitch parameter for the zone.
      *
      * @param aPitch is the hatch pitch in iu.
      */
-    void SetHatchPitch( int aPitch );
+    void SetBorderHatchPitch( int aPitch );
 
     /**
      * Clear the zone's hatch.
@@ -800,9 +760,6 @@ public:
 
     const std::vector<SEG>& GetHatchLines() const { return m_borderHatchLines; }
 
-    bool   GetHV45() const { return m_hv45; }
-    void   SetHV45( bool aConstrain ) { m_hv45 = aConstrain; }
-
     /**
      * Build the hash value of m_FilledPolysList, and store it internally in m_filledPolysHash.
      * Used in zone filling calculations, to know if m_FilledPolysList is up to date.
@@ -816,9 +773,17 @@ public:
 
 #if defined(DEBUG)
     virtual void Show( int nestLevel, std::ostream& os ) const override { ShowDummy( os ); }
+
+    void SetFillPoly( PCB_LAYER_ID aLayer, SHAPE_POLY_SET* aPoly )
+    {
+        m_FilledPolysList[ aLayer ] = std::make_shared<SHAPE_POLY_SET>( *aPoly );
+        SetFillFlag( aLayer, true );
+    }
+
 #endif
 
-    virtual void SwapData( BOARD_ITEM* aImage ) override;
+protected:
+    virtual void swapData( BOARD_ITEM* aImage ) override;
 
 protected:
     SHAPE_POLY_SET*       m_Poly;                ///< Outline of the zone.
@@ -841,6 +806,12 @@ protected:
      */
     bool m_isRuleArea;
 
+    /* A zone outline can be a teardrop zone with different rules for priority
+     * (always bigger priority than copper zones) and never removed from a
+     * copper zone having the same netcode
+     */
+    TEARDROP_TYPE m_teardropType;
+
     /* For keepout zones only:
      * what is not allowed inside the keepout ( pads, tracks and vias )
      */
@@ -886,7 +857,7 @@ protected:
     ZONE_FILL_MODE   m_fillMode;
     int              m_hatchThickness;          // thickness of lines (if 0 -> solid shape)
     int              m_hatchGap;                // gap between lines (0 -> solid shape
-    double           m_hatchOrientation;        // orientation in degrees of grid lines
+    EDA_ANGLE        m_hatchOrientation;        // orientation of grid lines
     int              m_hatchSmoothingLevel;     // 0 = no smoothing
                                                 // 1 = fillet
                                                 // 2 = arc low def
@@ -901,12 +872,6 @@ protected:
 
     int              m_localFlgs;               // Variable used in polygon calculations.
 
-    /**
-     * Segments used to fill the zone (#m_FillMode ==1 ), when fill zone by segment is used.
-     * In this case the segments have #m_ZoneMinThickness width.
-     */
-    std::map<PCB_LAYER_ID, std::vector<SEG> > m_FillSegmList;
-
     /* set of filled polygons used to draw a zone as a filled area.
      * from outlines (m_Poly) but unlike m_Poly these filled polygons have no hole
      * (they are all in one piece)  In very simple cases m_FilledPolysList is same
@@ -915,12 +880,10 @@ protected:
      * connecting "holes" with external main outline.  In complex cases an outline
      * described by m_Poly can have many filled areas
      */
-    std::map<PCB_LAYER_ID, SHAPE_POLY_SET> m_FilledPolysList;
-    std::map<PCB_LAYER_ID, SHAPE_POLY_SET> m_RawPolysList;
+    std::map<PCB_LAYER_ID, std::shared_ptr<SHAPE_POLY_SET>> m_FilledPolysList;
 
     /// Temp variables used while filling
-    EDA_RECT                               m_bboxCache;
-    std::map<PCB_LAYER_ID, bool>           m_fillFlags;
+    LSET                                   m_fillFlags;
 
     /// A hash value used in zone filling calculations to see if the filled areas are up to date
     std::map<PCB_LAYER_ID, MD5_HASH>       m_filledPolysHash;
@@ -932,9 +895,8 @@ protected:
     /// For each layer, a set of insulated islands that were not removed
     std::map<PCB_LAYER_ID, std::set<int>> m_insulatedIslands;
 
-    bool                      m_hv45;              // constrain edges to horiz, vert or 45º
-
     double                    m_area;              // The filled zone area
+    double                    m_outlinearea;       // The outline zone area
 
     /// Lock used for multi-threaded filling on multi-layer zones
     std::mutex m_lock;
@@ -956,4 +918,9 @@ public:
     double ViewGetLOD( int aLayer, KIGFX::VIEW* aView ) const override;
 };
 
+#ifndef SWIG
+DECLARE_ENUM_TO_WXANY( ZONE_CONNECTION )
+DECLARE_ENUM_TO_WXANY( ZONE_FILL_MODE )
+#endif
+
 #endif  // ZONE_H
